! $Id: avhrr_calnav_routines.f90,v 1.44 2010/08/29 00:23:55 heidinger Exp $
module AVHRR_CALNAV_ROUTINES
!-------------------------------------------------------------------------
!@***h* PATMOSX/avhrr_calnav_routines.f90
!   
! NAME
!   avhrr_calnav_routines.f90
! 
! LOCATION
!   .../patmosx/src/avhrr_calnav_routines.f90
! 
! PURPOSE
!   AVHRR CALIBRATION AND NAVIGATION ROUTINES
! 
! DESCRIPTION
!   EXPERMINENTAL VERSION = READS LEVEL-1b one segment at a time - not 2 scans
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!  Routines in this module and their purpose:
!
!  DEFINE_1B_DATA - based on level 1b data type, define some parameters
!                   needed to read and unpack the avhrr data
!  DETERMINE_FILE_TYPE - interrogate header and determine the file type
!  UNPACK_AVHRR_HEADER_RECORD - unpack level 1b header for pre-klm data
!  UNPACK_AVHRR_DATA_RECORD - unpack level 1b data for pre-klm data
!  LAGRANGIAN_ANCHOR_INTERP - perform Lagrangian interpolatation of anchors 
!  LINEAR_ANCHOR_INTERP - perform linear interpolation of anchors
!  COMPUTE_ANGLE_ANCHORS - compute satzen and relaz anchors for pre-klm data
!  UNPACK_AVHRR_HEADER_RECORD_KLM - unpack level 1b header for klm+ data
!  UNPACK_AVHRR_DATA_RECORD_KLM - unpack level 1b data for klm+ data
!  i4word_to_string - convert a 4 byte integer to a string
!  READ_AVHRR_INSTR_CONSTANTS - read avhrr instrument constant files
!  READ_CLAVRXORB_DEFAULT_OPTIONS - Opens default file if necessary
!  QC_CLAVRXORB_OPTIONS - quality control options
!  REF_CAL - perform the reflectance calibration
!  REF_CAL_COEFFS - compute the reflectance calibration coefficients
!  THERM_CAL - perform the thermal calibration
!  COMPUTE_NEW_THERM_CAL_COEF - compute thermal calibration coefficients
!  WRITE_HEADER_1BX - write a header to the level 1bx files
!  READ_AVHRR_LEVEL1B_DATA - read level 1b data records
!  READ_AVHRR_LEVEL1B_HEADER - read level 1b header records
!  COMPUTE_GLINTZEN_SCATANGLE - compute the glint and scattering angles
!  READ_CLAVRXORB_COMMANDLINE_OPTIONS - gathers command line options
!  					and checks the options for errors
!  HELPER - Displays what each command line option is
!  CALCULATE_ASC_DES - Calculates ascending/descending flag if error presennt
! 			in level1b file
!
!@*****
!-------------------------------------------------------------------------
 use AVHRR_PIXEL_COMMON
 use CALIBRATION_CONSTANTS
 use CONSTANTS
 use FILE_UTILITY
 use NUMERICAL_ROUTINES
 use PLANCK

  implicit none

  public:: READ_AVHRR_INSTR_CONSTANTS, &
           READ_CLAVRXORB_DEFAULT_OPTIONS,&
           WRITE_HEADER_1BX,&
           READ_AVHRR_LEVEL1B_DATA,  &
           READ_AVHRR_LEVEL1B_HEADER, &
           DETERMINE_FILE_TYPE, &
           DETERMINE_AVHRR_1, &
           DEFINE_1B_DATA, &
           READ_CLAVRXORB_COMMANDLINE_OPTIONS, &
           QC_CLAVRXORB_OPTIONS, &
           HELPER, &
           COMPUTE_NEW_SPACECRAFT_ID, &
           CALCULATE_ASC_DES

  private::  &
           LAGRANGIAN_ANCHOR_INTERP, &
           LINEAR_ANCHOR_INTERP, &
           GNOMIC_ANCHOR_INTERP, &
           COMPUTE_ANGLE_ANCHORS, &
           i4word_to_string, &
           REF_CAL,  &
           REF_CAL_COEFFS,  &
           THERM_CAL, &
           COMPUTE_NEW_THERM_CAL_COEF,  &
           COMPUTE_GLINTZEN_SCATANGLE, &
           MAKE_I4WORD, &
           MAKE_I2WORD

  private:: UNPACK_AVHRR_HEADER_RECORD, &
            UNPACK_AVHRR_HEADER_RECORD_KLM

  private:: UNPACK_AVHRR_DATA_RECORD,  &
            UNPACK_AVHRR_DATA_RECORD_KLM, &
            UNPACK_AVHRR_DATA_RECORD_AAPP

!--- variable declaration
  integer(kind=int2), dimension(5,10), private:: space_count_temp
  integer(kind=int2), dimension(3:5,10),private:: blackbody_count
  integer(kind=int2), dimension(103),private:: telemetry_word
  integer(kind=int1), public, save:: iprt=0, jcal = 0
  integer(kind=int4), parameter, private:: ncal = 10 
  integer(kind=int2), private, save:: sum_prt
  real(kind=real4), private:: mean_prt
  real(kind=real4), private:: Nbb_3b
  real(kind=real4), private:: Nbb_4
  real(kind=real4), private:: Nbb_5
  real(kind=real4), dimension(ncal), private:: T_prt
  logical, public, save:: valid_therm_cal
  logical, public, save:: spinup_new_therm_cal
  logical, public, save:: valid_ref_cal
  real(kind=real4), dimension(ncal),private,save:: space_count_temp_1
  real(kind=real4), dimension(ncal),private,save:: space_count_temp_2
  real(kind=real4), dimension(ncal),private,save:: space_count_temp_3
  real(kind=real4), dimension(ncal),private,save:: space_count_temp_4
  real(kind=real4), dimension(ncal),private,save:: space_count_temp_5
  real(kind=real4), dimension(ncal),private,save:: bb_count_3
  real(kind=real4), dimension(ncal),private,save:: bb_count_4
  real(kind=real4), dimension(ncal),private,save:: bb_count_5
  real(kind=real4),public,save::mean_space_count_1=0.0
  real(kind=real4),public,save::mean_space_count_2=0.0
  real(kind=real4),public,save::mean_space_count_3=0.0
  real(kind=real4),public,save::mean_space_count_4=0.0
  real(kind=real4),public,save::mean_space_count_5=0.0
  real(kind=real4),public,save::mean_T_prt=0.0
  real(kind=real4),public,save::mean_bb_count_3=0.0
  real(kind=real4),public,save::mean_bb_count_4=0.0
  real(kind=real4),public,save::mean_bb_count_5=0.0
  real(kind=real4), private, parameter:: beta=0.2
  integer,public, save:: calc_asc_des

  character(24), parameter, private :: MOD_PROMPT = " AVHRR_CALNAV_ROUTINES: "

  !--- values of iostatus indicative of an eof.  These values are not standard
  !--- on all fortran compilers.  These values come from xlf90, lahey, intel,
  !--- g95 and gfortran - there must be a more elegant solution
  integer, parameter, private:: num_eof_ios = 6
  integer, dimension(num_eof_ios), parameter, private::  &
           eof_iostatus = (/-1,-1,36,213,80,5002/)

  contains

!----------------------------------------------------------------------
! DEFINE_1B_DATA - based on the gac and klm flags, this routine defines
!                  the parameters necessary to read in the header and
!                  data records from the various 1B data types. This allows
!                  defines quantities to allow the clavr cloud mask to be
!                  written into the klm 1b files.
!
! input
!      gac - T if gac, F if hrpt or lac
!      klm - T if NOAA-klm, F if NOAA-J or earlier
!      aapp - T if AAPP, F if NESDIS
! output
!      num_pix - the number of pixels per scanline
!      l1b_rec_length - the number of bytes per record of the 1b file
!      nrec_header - the number of records used for the header
!      nword_clavr_start - byte number of first clavr byte on 1b
!      nword_clavr - number of bytes used for clavr cloud mask on 1b
!-------------------------------------------------------------------------------
 subroutine DEFINE_1B_DATA(gac,klm,aapp,nrec_header,nword_clavr_start,nword_clavr)

 integer(kind=int4), intent(in):: gac
 integer(kind=int4), intent(in):: klm
 integer(kind=int4), intent(in):: aapp
 integer(kind=int4), intent(out):: nword_clavr
 integer(kind=int4), intent(out):: nword_clavr_start
 integer(kind=int4), intent(out):: nrec_header

  if (gac == sym%YES) then
      num_pix = 409              !number of pixels per scan line
      if (klm == sym%YES) then
       l1b_rec_length  = 4608
       nrec_header = 1
       nword_clavr_start = 4049
       nword_clavr = 2*52
      else
       l1b_rec_length  = 3220
       nrec_header = 2
       nword_clavr_start = 0
       nword_clavr = 0
      endif
   else
     if (klm == sym%YES) then
       num_pix = 2048
       nrec_header = 1
       l1b_rec_length = 15872
       nword_clavr_start = 14977
       nword_clavr = 2*256
     else
      num_pix = 2048
      nrec_header = 1
      l1b_rec_length = 14800
      nword_clavr_start = 0
      nword_clavr = 0
      endif
   endif

!---- aapp - assumed to be 1 km  and no clavr bits
   if (aapp == sym%YES) then
       num_pix = 2048
       nrec_header = 1
       l1b_rec_length = 22016
       nword_clavr_start = 0
       nword_clavr = 0
    endif

!--- number of geolocation anchors
  num_anchors = 51

 end subroutine DEFINE_1B_DATA
!======================================================================
! Determine what type of file based on the header
!======================================================================
!-------------------------------------------------------
! read header to get data type and version of 1b data
! inorder to get klm and gac flags which determine
! the data format for subsequent read statements
! note, data_type and ver_1b are read in again later
!-------------------------------------------------------
   subroutine DETERMINE_FILE_TYPE(file_1b_local,gac,klm,aapp,ver_1b,data_type,byte_swap_1b)

    character(len=*), intent(in):: file_1b_local
    integer(kind=int4), intent(out):: gac
    integer(kind=int4), intent(out):: klm
    integer(kind=int4), intent(out):: aapp
    integer(kind=int2), intent(out):: ver_1b
    integer(kind=int2), intent(out):: data_type
    integer(kind=int4), intent(out):: byte_swap_1b

    integer(kind=int1), dimension(100):: header_buffer_temp
    integer:: word_start
    integer:: bytes_per_word
    integer:: number_of_words
    integer:: number_of_words_read

    !--- use c-routine
    bytes_per_word = 1
    word_start = 0
    number_of_words = 100
    call mreadf_int(file_1b_local//CHAR(0),word_start,bytes_per_word, &
                    number_of_words,number_of_words_read,header_buffer_temp)

    !--- use instrinsic fortran read
!   word_start = 1
!   read(unit=lun_level1b,pos=word_start) header_buffer_temp


    !--- look at first byte and determine klm flag - A. Jelenak Suggestion  - check this for AAPP!
     if (header_buffer_temp(1) <= 10) then
        klm = sym%NO
     elseif (header_buffer_temp(1) >= 65) then
        klm = sym%YES
     else
        print *, EXE_PROMPT, MOD_PROMPT, "Error diagnosing klm from first byte, assuming klm = 0"
        klm = sym%NO
     endif

!--- assuming klm, unpack ver1b, valid values are (1-5)
     ver_1b = MAKE_I2WORD(header_buffer_temp(5:6),sym%UNSIGNED,sym%NOSWAP)

!--- if version number unrealistic, assume aapp
     aapp = sym%NO
     byte_swap_1b = sym%NO
     if (klm == sym%YES) then
        if ((ver_1b > 10).or.(ver_1b < 1)) then
          aapp = sym%YES
          byte_swap_1b = sym%YES
          ver_1b = MAKE_I2WORD(header_buffer_temp(5:6),sym%UNSIGNED,byte_swap_1b)
        endif
     endif

!--- read spacedraft id
     sc_id = MAKE_I2WORD(header_buffer_temp(73:74),sym%UNSIGNED,byte_swap_1b)

!--- fix noaa-15 bug
     if ((ver_1b == 1) .and. (sc_id == 4)) then
       ver_1b = 2
     endif

!--- assign pre-klm to version 1
     if (klm == sym%NO) then
       ver_1b = 1
     endif

!--- based on klm flag, choose bytes to construct data_type
     if (klm == sym%YES) then
         data_type = MAKE_I2WORD(header_buffer_temp(77:78),sym%UNSIGNED,byte_swap_1b)
     else
         data_type = ishft(header_buffer_temp(2),-4)
     endif

!--- based on data_type, set gac flag
     gac = sym%NO
     if (data_type == 2) then
       gac = sym%YES
     endif

end subroutine DETERMINE_FILE_TYPE

!==================================================================================================
!--- determine if this data is from AVHRR/1 which has no channel 5
!==================================================================================================
subroutine DETERMINE_AVHRR_1(year, klm_flag, avhrr_1_flag)

    integer(kind=int2), intent(in):: year       !year of this data-set
    integer(kind=int4), intent(in):: klm_flag   !klm flag (yes/no)
    integer(kind=int4), intent(out):: avhrr_1_flag  !AVHRR/1 flag (yes/no)

     !--- initialize to no
     avhrr_1_flag = sym%NO

     !--- check sc_id for pre-klm data
     if (klm_flag == sym%NO) then
      if (sc_id == 1 .and. year < 1985) avhrr_1_flag = sym%YES   !TIROS-N (NOAA-11 repeat) 
      if (sc_id == 2) avhrr_1_flag = sym%YES   !NOAA-6
      if (sc_id == 6) avhrr_1_flag = sym%YES   !NOAA-8
      if (sc_id == 8) avhrr_1_flag = sym%YES   !NOAA-10
     endif

end subroutine DETERMINE_AVHRR_1

!----------------------------------------------------------------------------
! This routine reads in a specifed number of scans from a level 1b file
! it calibrates, navigates and quality controls the data
!
! this processes the level1b data two scanline lines at a time
!
!----------------------------------------------------------------------------
  subroutine READ_AVHRR_LEVEL1B_DATA(file_1b_local, & 
                                     klm, &
                                     aapp, &
                                     therm_cal_1b, &
                                     time_since_launch,  &
                                     nrec_header, &
                                     iseg)

    character(len=*), intent(in):: file_1b_local
    integer, intent(in):: klm
    integer, intent(in):: aapp
    integer, intent(in):: therm_cal_1b
    real(kind=real4), intent(in):: time_since_launch
    integer, intent(in):: nrec_header
    integer, intent(in):: iseg
    integer:: iline
    integer:: irec_start
    integer:: irec_end
    integer:: irec
    integer:: error_code

    integer(kind=int4):: word_start
    integer(kind=int4):: word_end
    integer(kind=int4):: number_of_words
    integer(kind=int4):: number_of_words_read
    integer(kind=int4):: bytes_per_word

   !--- initialize calc_asc_des flag

    calc_asc_des = sym%NO

   !--- based on segment number, compute starting and ending record in level-1b
   !--- for this segment
   irec_start = nrec_header + (iseg-1)*num_scans_per_segment + 1
   irec_end =   irec_start + num_scans_per_segment - 1

   !--- initialize number of scans-read index
   num_scans_read = 0

   !-------------------------------------------------------------------
   ! read one segment of data from level1b file
   !-------------------------------------------------------------------
   irec = irec_start
   iline = j_min

   !------
   !-- try to read in segment as a whole
   !----

   !--- initialize
   segment_buffer = 0


   !--- use external c-routine for read
   bytes_per_word = 1
   word_start = l1b_rec_length* ( 1 + (iseg-1)*num_scans_per_segment)
   number_of_words = l1b_rec_length*num_scans_per_segment
   call mreadf_int(file_1b_local//CHAR(0),word_start,bytes_per_word, &
                   number_of_words,number_of_words_read,segment_buffer)
   !--- update number of scans read
   num_scans_read = number_of_words_read / l1b_rec_length

!  !--- use instrinsic fortran read
!  word_start = l1b_rec_length* ( 1 + (iseg-1)*num_scans_per_segment) + 1
!  read(unit=lun_level1b,pos=word_start,iostat=ios_l1b) segment_buffer
!  if (ios_l1b == 0) then
!     num_scans_read = num_scans_per_segment
!  else
!     print *, "error on read from l1b", ios_l1b
!  stop
!  endif
 
  

   !------------------------------------------------------------------------------------------------------------
   !--- UNPACK Level-1b Data
   !------------------------------------------------------------------------------------------------------------
   segment_loop: do iline = j_min, j_min + num_scans_read - 1

       !extract level-1b information for this scanline
       word_start = (iline-1)*l1b_rec_length + 1
       word_end = word_start +  l1b_rec_length - 1

       buffer = segment_buffer(word_start:word_end)

       !--- unpack the level-1b data
       if (aapp == sym%YES) then
           call UNPACK_AVHRR_DATA_RECORD_AAPP(iline)
       else
         if (klm == sym%YES) then
           call UNPACK_AVHRR_DATA_RECORD_KLM(iline)
         else
           call UNPACK_AVHRR_DATA_RECORD(iline)
         endif
       endif

       !----------------------------------------------------------------------
       ! generate new calibration coefficients
       !----------------------------------------------------------------------

       if (therm_cal_1b == sym%NO) then

          call COMPUTE_NEW_THERM_CAL_COEF(iline)

          !--- if sufficient scans lines have not been processed, this is fatal
          if (spinup_new_therm_cal .eqv. .true.) then
            fatal(iline) = sym%YES
          endif
        else
           valid_therm_cal = .true.
       endif

       !--- compute reflectance coefficients
       call REF_CAL_COEFFS(time_since_launch,iline)

       !---- determine if valid coefficients are made
       if (valid_therm_cal .eqv. .false.) then
         fatal(iline) = sym%YES
       endif
       if (valid_ref_cal .eqv. .false.) then
         fatal(iline) = sym%YES
       endif

      !------------------------------------------------------------------------------
      ! Compute latitude and longitude
      !------------------------------------------------------------------------------
      if (maxval(abs(lat_anchor_1b(:,iline))) > 85.0) then
         call GNOMIC_ANCHOR_INTERP(lon_anchor_1b(:,iline),lat_anchor_1b(:,iline), &
                                   lon_1b(:,iline),lat_1b(:,iline))
      else
         call LAGRANGIAN_ANCHOR_INTERP(lat_anchor_1b(:,iline),lat_1b(:,iline))
         call LAGRANGIAN_ANCHOR_INTERP(lon_anchor_1b(:,iline),lon_1b(:,iline))
      endif

      !--- constrain lat and lon to physical values
      if ((minval(lat_1b(:,iline)) < -90.0).or.(maxval(lat_1b(:,iline)) > 90.0)) then
           lat_1b(:,iline) = min(90.0,max(-90.0,lat_1b(:,iline)))
      endif
      if ((minval(lon_1b(:,iline)) < -180.0).or.(maxval(lon_1b(:,iline)) > 180.0)) then
           lon_1b(:,iline) = min(180.0,max(-180.0,lon_1b(:,iline)))
      endif 

      !----------------------------------------------------------------
      !--- Compute viewing geometry
      !----------------------------------------------------------------

!     !--- compute anchor points for pre-klm series (not included in level-1b)
!     if (klm == sym%NO) then
        call COMPUTE_ANGLE_ANCHORS(scan_day(iline), &
                                   scan_time(iline), &
                                   lon_anchor_1b(:,iline), &
                                   lat_anchor_1b(:,iline), &
                                   satzen_anchor(:,iline),&
                                   solzen_anchor(:,iline), &
                                   relaz_anchor(:,iline),  &
                                   solaz_anchor(:,iline))
!     else
!       !--- modify klm relaz to expected convention
!       relaz_anchor(:,iline) = 180.0 - abs(relaz_anchor(:,iline))
!     endif

      !--- compute glint zenith angles
      call COMPUTE_GLINTZEN_SCATANGLE(solzen_anchor(:,iline),satzen_anchor(:,iline), &
                           relaz_anchor(:,iline),glintzen_anchor(:,iline), &
                           scatangle_anchor(:,iline))

      !--- interpolate pixel values from anchor values
      call LINEAR_ANCHOR_INTERP(satzen_anchor(:,iline),satzen(:,iline))
      call LINEAR_ANCHOR_INTERP(solzen_anchor(:,iline),solzen(:,iline))
      call LINEAR_ANCHOR_INTERP(relaz_anchor(:,iline),relaz(:,iline))
      call LINEAR_ANCHOR_INTERP(solaz_anchor(:,iline),solaz(:,iline))
      call LINEAR_ANCHOR_INTERP(glintzen_anchor(:,iline),glintzen(:,iline))
      call LINEAR_ANCHOR_INTERP(scatangle_anchor(:,iline),scatangle(:,iline))

      !--- constrain sensor zenith
      if ((minval(satzen(:,iline)) < 0.0).or.(maxval(satzen(:,iline)) > 89.9)) then
          satzen(:,iline) = min(89.9,max(0.0,satzen(:,iline)))
      endif

      !------------------------------------------------------------------------------
      ! check for bad scan lines here
      !------------------------------------------------------------------------------
       bad_scan_flag(iline) = sym%NO
       error_code = 0

       !--- any scan with a fatal error is considered bad
       if (fatal(iline) == sym%YES) then
         bad_scan_flag(iline) = sym%YES
         error_code = 1
       endif

       !--- if in spinup mode, set all scans as bad
       if (error_code == 0) then
         if ((therm_cal_1b == sym%NO) .and. (spinup_new_therm_cal .eqv. .true.)) then
           bad_scan_flag(iline) = sym%YES
           error_code = 2
         endif
       endif

       !---- check for missing navigation
       if ( (sum(lat_anchor_1b(:,iline)) == 0.00).or. (sum(lon_anchor_1b(:,iline)) == 0.00)) then
           bad_scan_flag(iline) = sym%YES
           error_code = 3
       endif

       !--- check erroneous asc/des flag 
       !--- erroneous values are common on AVHRR/1 data
       !--- so this is no longer considered a fatal error
       if ((ascend(iline) < 0) .or. (ascend(iline)  > 1)) then
           calc_asc_des = sym%YES
       endif

       !---- check angle anchors on klm data
       if (klm == sym%YES) then
         if ((minval(satzen_anchor(:,iline)) < 0.0) .or. (maxval(satzen_anchor(:,iline)) > 70.0)) then
           bad_scan_flag(iline) = sym%YES
           error_code  = 5
         endif
         if ((minval(solzen_anchor(:,iline)) < 0.0) .or. (maxval(solzen_anchor(:,iline)) > 180.0)) then
           bad_scan_flag(iline) = sym%YES
           error_code = 5
         endif
         if ((minval(relaz_anchor(:,iline)) < -180.0) .or. (maxval(relaz_anchor(:,iline)) > 180.0)) then
           bad_scan_flag(iline) = sym%YES
           error_code = 5
         endif
       endif


       !----- report bad scan code to standard output
       if (bad_scan_flag(iline) == sym%YES) then
         write(unit=6,fmt="(a,a,a,i6,a,i2)") EXE_PROMPT, MOD_PROMPT,  &
               "BAD SCAN: Numbers = ", scan_number(iline), " error code = ", error_code
       endif 

       !---- set geolocation to missing for bad-scans to avoid ancil-data interp
       if (bad_scan_flag(iline) == sym%YES) then
               lat_1b(:,iline) = missing_value_real4
               lon_1b(:,iline) = missing_value_real4
       endif

end do segment_loop

         !------------------------------------------------------------------------------
         ! Calculate ascending/descending flag if needed for this segment
         !------------------------------------------------------------------------------

         if (calc_asc_des == sym%YES) then
            print *, "CALLING CALCULATE_ASC_DES"
            call CALCULATE_ASC_DES(j_min,num_scans_read)
         endif

         !------------------------------------------------------------------------------
         ! Calibrate segment
         !------------------------------------------------------------------------------

         !--- REFLECTANCE CALIBRATION (1,2,3a)
         call REF_CAL(num_scans_read)

         !--- RADIANCE CALIBRATION (3b,4,5)
         call THERM_CAL(num_scans_read)

end subroutine READ_AVHRR_LEVEL1B_DATA


!========================================================================
! Read in the AVHRR Level1b header
!
! input and output passed through avhrr_pixel_common
!
! handles gac,lac, hrpt for pre and post klm data - and aapp
!========================================================================
 subroutine READ_AVHRR_LEVEL1B_HEADER(file_1b_local)

   character(len=*), intent(in):: file_1b_local
   integer(kind=int4):: word_start
   integer(kind=int4):: number_of_words
   integer(kind=int4):: number_of_words_read
   integer(kind=int4):: bytes_per_word

    !--- use external c
    bytes_per_word = 1
    word_start = 0
    number_of_words = l1b_rec_length
    call mreadf_int(file_1b_local//CHAR(0),word_start,bytes_per_word, &
                    number_of_words,number_of_words_read,header_buffer)

!   !--- use internal fortran
!   word_start = 1
!   read(unit=lun_level1b,pos=word_start), header_buffer

    if (gac == sym%YES) then

     if (klm == sym%YES) then

      call UNPACK_AVHRR_HEADER_RECORD_KLM(sc_id,data_type,start_year, &
                   start_day,start_time,num_scans,end_year,end_day,end_time, &
                   tip_parity,aux_sync,ramp_auto_cal,proc_block_id,ver_1b)

     else

      call UNPACK_AVHRR_HEADER_RECORD(sc_id,data_type,start_year, &
                   start_day,start_time,num_scans,end_year,end_day,end_time, &
                   tip_parity,aux_sync,ramp_auto_cal,proc_block_id,ver_1b)

      !--- pre klm used a 2 digit year
      if (end_year > 50) then
       end_year = end_year + 1900
      else
        end_year = end_year + 2000
      endif
      if (start_year > 50) then
        start_year = start_year + 1900
      else
        start_year = start_year + 2000
      endif
                                                                                                                                          
     endif

   else     !if not gac

     if (klm == sym%YES) then

      call UNPACK_AVHRR_HEADER_RECORD_KLM(sc_id,data_type,start_year, &
                  start_day,start_time,num_scans,end_year,end_day,end_time, &
                  tip_parity,aux_sync,ramp_auto_cal,proc_block_id,ver_1b)

     else

      call UNPACK_AVHRR_HEADER_RECORD(sc_id,data_type,start_year, &
                start_day,start_time,num_scans,end_year,end_day,end_time, &
                tip_parity,aux_sync,ramp_auto_cal,proc_block_id,ver_1b)

!--- pre klm used a 2 digit year
      if (end_year > 50) then
        end_year = end_year + 1900
      else
        end_year = end_year + 2000
      endif
      if (start_year > 50) then
        start_year = start_year + 1900
      else
        start_year = start_year + 2000
      endif
     endif
   endif

end subroutine READ_AVHRR_LEVEL1B_HEADER

!---------------------------------------------------------------------
! when creating a 1bx file, this routine write the original level 1b
! header to it
!---------------------------------------------------------------------
 subroutine WRITE_HEADER_1BX(file_1b)

    character(len=*), intent(in):: file_1b
    character(len=120):: file_1bx
    integer:: ios0,erstat

    file_1bx = trim(file_1b)
    if (file_1bx(1:3) .eq. 'NSS') then
       file_1bx(1:3) = 'TCL'
    endif
    print *,EXE_PROMPT, "PROCESS_AVHRR unit 11 out 1bx==>",dir_1bx
    print *,EXE_PROMPT, "creating file ",trim(file_1bx)

    open(unit=lun_level1b,file=trim(dir_1bx)//trim(file_1bx),  &
         status="replace",form="unformatted", &
         access="direct",recl=l1b_rec_length,action="write",iostat=ios0)

         if (ios0 /= 0) then
           erstat = 12
           print *, EXE_PROMPT, "Error opening 1Bx file, ios0 = ",ios0
           stop 12
         endif

      write(unit=lun_level1b,rec=1,iostat=ios0) header_buffer

      if (ios0 /= 0) then
       print *, EXE_PROMPT, "Error writing 1Bx file header, ios0 = ",ios0
       stop 13
      endif

end subroutine WRITE_HEADER_1BX
!*******************************************************************


!-----------------------------------------------------------
! REFLECTANCE CALIBRATION ROUTINE
!
! comments given for channel 1 only but apply to channel 2,3a
!
! input
!   time_temp_since_launch - the time_temp in year_temps since the launch
!   ref_cal_1b - flag controlling use of 1b coefficients
!
!  output
!   none - only through common memory
!
! Parameters defined in 'avhrr_pixel_common.f' module
!  ch1_gain_low - calibration slope or gain in low count region
!                  with degradation accounted for
!  ch1_gain_high - calibration slope or gain in high count region
!                  with degradation accounted for
!
! internal parameters
!  alb1_switch - the albedo at count = count1_switch
!
! arrays used from common memory
!  chan_counts_temp(1,:,:) - two scanlines of channel 1 counts
!
! arrays modified in common memory
!  alb1_temp - channel 1 albedo for scanline pair, not this is
!         not corrected for sun-earth distance or for solar zenith angle
!
!  variables used from common memory
!   ch1_gain_low_0 - calibration slope or gain in low count region at launch
!   ch1_gain_high_0 - calibration slope or gain in high count region at launch
!   ch1_switch_count - count separating low from high count region
!   ch1_dark_count_cal - the dark count used for calibration - may differ from the
!                         the true dark count for any given slope,intercept pair
!
! Discussion
!  This code computes the coefficients to compute the reflectance calibration 
!  do to the following (Method 1)
!   R_low = slope_low * (C - C_dark)
!   R_high = R_switch + slope_high*(C - C_switch)
! 
! The method in the POD guide is as follows (Method 2)
!   R_low = Slope_low * C + intercept_low
!   R_high = Slope_high * C + intercept_high
!
! If ref_cal_1b is true, then the coefficients read in from the 1b 
! file are converted into forms for Method 1 (done in REF_CAL()).
! This transformation is exact
!
! Revision History
! Coded January 2003 - A. Heidinger
!
! 2004-09-03 - A. Jelenak
!     Original REF_CAL sub split into two to allow computation of channel gain
!     coeffs and their storage in the OBS file. REF_CAL_COEFFS sub does this
!     task now, and REF_CAL just applies them to the chan_counts_temp() array.
!
! 2004-12-04 - A. Heidinger
!     Added capability to use 1b reflectance coeffcients
!
! 2008-12-24 - A. Heidinger
!     Now operates on single scan-line
!
! Note, if therm_cal_1b == sym%NO, this routine must be called
!       after compute_new_therm_cal_coeffs
!-----------------------------------------------------------
 subroutine REF_CAL_COEFFS(time_temp_since_launch,iline)

   real(kind=real4), intent(in):: time_temp_since_launch
   integer, intent(in):: iline

   valid_ref_cal = .true.

   !--- compute coefficient if using level-1b values
   if (ref_cal_1b == sym%YES) then

    ch1_gain_low = vis_slope_1(1,iline)
    ch1_dark_count_cal = -1.0*vis_intercept_1(1,iline)/vis_slope_1(1,iline)
    ch1_gain_high = vis_slope_2(1,iline)
    ch1_switch_count_cal = vis_intersection(1,iline)

    ch2_gain_low = vis_slope_1(2,iline)
    ch2_dark_count_cal = -1.0*vis_intercept_1(2,iline)/vis_slope_1(2,iline)
    ch2_gain_high = vis_slope_2(2,iline)
    ch2_switch_count_cal = vis_intersection(2,iline)

    ch3a_gain_low = vis_slope_1(3,iline)
    ch3a_dark_count_cal = -1.0*vis_intercept_1(3,iline)/vis_slope_1(3,iline)
    ch3a_gain_high = vis_slope_2(3,iline)
    ch3a_switch_count_cal = vis_intersection(3,iline)

    !---- compute reflectances at switch count (define consistent with)
    alb1_switch = vis_slope_2(1,iline) * vis_intersection(1,iline) + vis_intercept_2(1,iline)
    alb2_switch = vis_slope_2(2,iline) * vis_intersection(2,iline) + vis_intercept_2(2,iline)
    alb3a_switch = vis_slope_2(3,iline) * vis_intersection(3,iline) + vis_intercept_2(3,iline)

   else

     !--- adjust gains to account for degradation
     if (therm_cal_1b == sym%YES) then
       ch1_dark_count_cal = ch1_dark_count
       ch2_dark_count_cal = ch2_dark_count
       ch3a_dark_count_cal = ch3a_dark_count
     else
        ch1_dark_count_cal =  scan_space_counts(1,iline) 
        ch2_dark_count_cal = scan_space_counts(2,iline)
        ch3a_dark_count_cal = scan_space_counts(3,iline)
     endif

     !-- set switch counts to values in instrument files
     ch1_switch_count_cal = ch1_switch_count
     ch2_switch_count_cal = ch2_switch_count
     ch3a_switch_count_cal = ch3a_switch_count

     ch1_gain_low = ch1_gain_low_0*(100.0+ch1_degrad_low_1*time_temp_since_launch+ &
                                  ch1_degrad_low_2*time_temp_since_launch**2)/100.0
     ch1_gain_high = ch1_gain_high_0*(100.0+ch1_degrad_high_1*time_temp_since_launch+ &
                                  ch1_degrad_high_2*time_temp_since_launch**2)/100.0
     ch2_gain_low = ch2_gain_low_0*(100.0+ch2_degrad_low_1*time_temp_since_launch+ &
                                  ch2_degrad_low_2*time_temp_since_launch**2)/100.0
     ch2_gain_high = ch2_gain_high_0*(100.0+ch2_degrad_high_1*time_temp_since_launch+ &
                                  ch2_degrad_high_2*time_temp_since_launch**2)/100.0
     ch3a_gain_low = ch3a_gain_low_0*(100.0 + ch3a_degrad_low_1 * time_temp_since_launch + &
                                  ch3a_degrad_low_2 * time_temp_since_launch**2)/100.0
     ch3a_gain_high = ch3a_gain_high_0*(100.0+ch3a_degrad_high_1*time_temp_since_launch + &
                                  ch3a_degrad_high_2 * time_temp_since_launch**2)/100.0


     !---- compute reflectances at switch count
     alb1_switch = ch1_gain_low * (ch1_switch_count_cal - ch1_dark_count_cal)
     alb2_switch = ch2_gain_low * (ch2_switch_count_cal - ch2_dark_count_cal)
     alb3a_switch = ch3a_gain_low * (ch3a_switch_count_cal - ch3a_dark_count_cal)

     !--- test for valid dark count
     if (abs(ch1_dark_count_cal - ch1_dark_count) > 5) then
        valid_ref_cal = .false.
     endif

   endif

   !---- determine if coefficients are valid
   if ((ch1_gain_low < 0.0) .or. (ch1_gain_high < 0.0)) then
        valid_ref_cal = .false.
   endif

end subroutine REF_CAL_COEFFS

!======================================================================
! Apply the reflectance coefficients to produce a reflectance
!======================================================================
subroutine REF_CAL(num_scans_read)

 integer, intent(in):: num_scans_read
 integer:: iline

 !--- channel 1
  where (chan_counts(1,:,:) <= ch1_switch_count_cal)
    alb1 = ch1_gain_low*(chan_counts(1,:,:) - ch1_dark_count_cal) 
  elsewhere
    alb1 = alb1_switch + ch1_gain_high*(chan_counts(1,:,:) - ch1_switch_count_cal) 
  end where

!--- channel 2
 where (chan_counts(2,:,:) <= ch2_switch_count_cal)
    alb2 = ch2_gain_low*(chan_counts(2,:,:) - ch2_dark_count_cal) 
 elsewhere
    alb2 = alb2_switch + ch2_gain_high*(chan_counts(2,:,:) - ch2_switch_count_cal) 
 end where

!--- channel 3a
 where (chan_counts(3,:,:) <= ch3a_switch_count_cal)
    alb3a = ch3a_gain_low*(chan_counts(3,:,:) - ch3a_dark_count_cal) 
 elsewhere
    alb3a = alb3a_switch + ch3a_gain_high*(chan_counts(3,:,:) - ch3a_switch_count_cal) 
 end where

 do iline = j_min,j_min + num_scans_read - 1
  !--- check for a bad scan, if so then skip
  if (bad_scan_flag(iline) == sym%YES) then
    alb1(:,iline) = missing_value_real4 
    alb2(:,iline) = missing_value_real4 
    alb3a(:,iline) = missing_value_real4 
  endif

  !--- set alb3a to missing for ch3a = off
  if (ch3a_on(iline) == sym%NO) then
    alb3a(:,iline) = missing_value_real4 
  endif

 enddo
  
 end subroutine REF_CAL

!-----------------------------------------------------------
! THERMAL CALIBRATION
!
! variables
!    therm_cal_1b - use 1b cal. coefficients (true)
!    klm - logical flag set to true if this klm data
!    ch3a_on_temp - integer flag set to 0 if ch3a is off
!    count3 - vector of channel 3 counts
!    count4 - vector of channel 4 counts
!    count5 - vector of channel 5 counts
!    d_coef - the "d" coefficient as illustrated in klm-POD
!    a_coef - the "a" coefficient as illustrated in klm-POD
!    b_coef - the "b" coefficient as illustrated in klm-POD
!    linear_slope - slope in linear radiance computation
!    linear_intercept - intercept in linear radiance computation
!    nu_3b, a1_3b, a2_3b - radiance to temperature conversion parameters
!                        (see klm-PODUG for example)
!    nu_4, a1_4, a2_4 - radiance to temperature conversion parameters
!    nu_5, a1_5, a2_5 - radiance to temperature conversion parameters
!
!
! output 
!   rad3b_temp, rad4_temp, rad5_temp - vector of channel radiance in mW/m^2/str/cm^-1
!   bt3b_temp, bt4_temp, bt5_temp - vector of brightness temperature in K
!
! Some Notes
! 1- pre-klm data is first converted to a radiance assuming a linear calibration
!    then the non-linear adjustment is made
! 2- Does not incorporate the correct procedure for data
!    before NOAA-14.  Need to review original CLAVR-1 for this.
! 3 - klm data, the non-linear correction is added to the linear radiance 
!     but for non-klm data is the non-linear correction is additive (see pods)
!     this is reflected in the difference is the pre-klm and klm coefs.
!
! Revision History
!  Coded January 2003 - A. Heidinger
!        February 2003 - added therm_cal_1b argument and new calibration
!
!
! some clarification
!  the klm-POD nomenclature has changed.  
!
!---------------------------------------------------------------------------
subroutine THERM_CAL(num_scans_read)

  integer, intent(in):: num_scans_read
  integer(kind=int4):: i
  integer(kind=int4):: j

  do j = j_min, j_min + num_scans_read - 1


    if (bad_scan_flag(j) == sym%YES) then
       cycle
    endif 

    !----  convert counts to radiance
    if (therm_cal_1b == sym%YES) then

       if (klm == sym%YES) then       !apply level-1b non-linear correction

         if (ch3a_on(j) == sym%NO) then
           rad3b(:,j) = ir_coef_1_1b(3,j) + ir_coef_2_1b(3,j)*chan_counts(3,:,j) +  &
                        ir_coef_3_1b(3,j)*chan_counts(3,:,j)**2
         endif

         rad4(:,j) = ir_coef_1_1b(4,j) + ir_coef_2_1b(4,j)*chan_counts(4,:,j) +  &
                       ir_coef_3_1b(4,j)*chan_counts(4,:,j)**2
         rad5(:,j) = ir_coef_1_1b(5,j) + ir_coef_2_1b(5,j)*chan_counts(5,:,j) +  &
                        ir_coef_3_1b(5,j)*chan_counts(5,:,j)**2

       else                         !compute level-1b linear correction for pre-klm

         !--- Eqn 7.1.2.4-7
         rad3b(:,j) = ir_linear_slope_1b(3,j)*chan_counts(3,:,j) + ir_linear_intercept_1b(3,j)
         rad4(:,j) = ir_linear_slope_1b(4,j)*chan_counts(4,:,j) + ir_linear_intercept_1b(4,j)
         rad5(:,j) = ir_linear_slope_1b(5,j)*chan_counts(5,:,j) + ir_linear_intercept_1b(5,j)

       endif

    else    !generate linear radiance using internally computed coefficients

      if (ch3a_on(j) == sym%NO) then
         rad3b(:,j) = ir_linear_slope_new(3,j)*chan_counts(3,:,j) + ir_linear_intercept_new(3,j)
      endif
      rad4(:,j) = ir_linear_slope_new(4,j)*chan_counts(4,:,j) + ir_linear_intercept_new(4,j)
      rad5(:,j) = ir_linear_slope_new(5,j)*chan_counts(5,:,j) + ir_linear_intercept_new(5,j)

    endif

   !--- apply non-linear correction where appropriate (non-klm or any if thermal cal redone)
   if ((klm == sym%NO) .or. (therm_cal_1b == sym%NO)) then 

     if (ch3a_on(j) == sym%NO) then
       rad3b(:,j) = rad3b(:,j) + b0_3b + b1_3b*rad3b(:,j) + b2_3b*rad3b(:,j)**2
     endif
     rad4(:,j) = rad4(:,j) + b0_4 + b1_4*rad4(:,j) + b2_4*rad4(:,j)**2
     rad5(:,j) = rad5(:,j) + b0_5 + b1_5*rad5(:,j) + b2_5*rad5(:,j)**2
   endif

   !---- convert radiance to brightness temperature

    do i = 1, num_pix

        if ((rad3b(i,j) > 0.0) .and. (ch3a_on(j) == 0)) then
          bt3b(i,j) = PLANCK_TEMP_FAST(3,rad3b(i,j)) 
        else
          rad3b(i,j) = missing_value_real4
          bt3b(i,j) = missing_value_real4
        endif

        if (rad4(i,j) > 0.0) then
          bt4(i,j) = PLANCK_TEMP_FAST(4,rad4(i,j)) 
        else
          rad4(i,j) = missing_value_real4
          bt4(i,j) = missing_value_real4
        endif

        if (rad5(i,j) > 0.0) then
          bt5(i,j) = PLANCK_TEMP_FAST(5,rad5(i,j)) 
        else
          rad5(i,j) = missing_value_real4
          bt5(i,j) = missing_value_real4
        endif

    end do

  enddo

  !----- additional quality check on bt4
  do j = j_min, j_min + num_scans_read - 1
    if (minval(bt4(:,j)) < 0.0) then
      bad_scan_flag(j) = sym%YES
    endif
  enddo

end subroutine THERM_CAL

!--------------------------------------------------------------------------
! compute parameters needed to derive new thermal calibration
! coefficients using a method similar to that described in the
! PODUG for HPRT users.  
!
! The methodology is 
!  average all space, blackbody counts and PRT's rom one scan together
!  average all quantities together for ncal scanlines
!  every ncal scanlines, update a running mean of all these values
!  the running mean is defined as
!   new_running_mean = (1-beta)*old_running_mean + new_value 
!
!  experimentaion has shown ncal = 10 and beta = 1/5 result in smooth curves
!  the variation 1b slope and intercept is approximated with ncal=5, beta = 1.0
!  unsure if smoother is better (TBD).
!
! Author: Andrew Heidinger
! Date: February 2003
!
! Revision - added check for missing space and blackbody counts - noticed
!           this occured for D95001 - D95005 data for NOAA-14
!
!  - modified for one scan at a time processing - akh
!--------------------------------------------------------------------------
subroutine COMPUTE_NEW_THERM_CAL_COEF(iline)

 integer, intent(in):: iline
 integer:: iword
 integer:: ibyte
 integer:: j
 integer:: ipix
 integer:: ichan
 integer(kind=int4):: i4word
 real(kind=real4):: mean_space_count_1_temp
 real(kind=real4):: mean_space_count_2_temp
 real(kind=real4):: mean_space_count_3_temp
 real(kind=real4):: mean_space_count_4_temp
 real(kind=real4):: mean_space_count_5_temp
 real(kind=real4):: mean_bb_count_3_temp
 real(kind=real4):: mean_bb_count_4_temp
 real(kind=real4):: mean_bb_count_5_temp
 real(kind=real4):: space_count_temp_4_test

 valid_therm_cal = .true.

!-----------------------------------------------------------------------
! unpack PRT readings
!-----------------------------------------------------------------------

!-- reset prt readings

   sum_prt = 0

  if (klm == sym%YES) then
!-----------------------------------------------------------------------
! unpack telemetry words for klm
!-----------------------------------------------------------------------
    ibyte = 1091 
    sum_prt = MAKE_I2WORD(buffer(ibyte:ibyte+1),sym%SIGNED,byte_swap_1b) +  &
              MAKE_I2WORD(buffer(ibyte+2:ibyte+3),sym%SIGNED,byte_swap_1b) + &
              MAKE_I2WORD(buffer(ibyte+4:ibyte+5),sym%SIGNED,byte_swap_1b)
  else
!-----------------------------------------------------------------------
! unpack telemetry words for pre-klm
!-----------------------------------------------------------------------
    iword = 1
    do ibyte = 309,448,4
      i4word = MAKE_I4WORD(buffer(ibyte:ibyte+3),sym%UNSIGNED,byte_swap_1b)
      do j = 1,3
       telemetry_word(iword) = i4word / (1024**(3-j))
       i4word = i4word - telemetry_word(iword)*(1024**(3-j))
       iword = iword + 1
       if (iword > 103) then
        exit
       endif
      enddo
       if (iword > 103) then
        exit
       endif
    enddo

    !--- prt readings
    sum_prt = telemetry_word(18)+telemetry_word(19)+telemetry_word(20)

  endif

  !--- increment prt counter
  iprt = iprt + 1

    !---- check for reset of PRTs
    if (sum_prt < 50) then
     iprt = 0
    endif

!--------------------------------------------------
! test to see if any coefficients can be made
!--------------------------------------------------
    if ((iprt == 0) .and. (mean_T_prt <= 0.0)) then
        print *, EXE_PROMPT, "Error: PRT Temperatures bad "
        valid_therm_cal = .false.
        return
    endif


!---- check to see if out of cycle
    if (iprt > 4) then
      print *, EXE_PROMPT, "Error: PRTs out of order "
      valid_therm_cal = .false.
      iprt = 0    ! added akh
      return
    endif

!----------------------------------------------------
!---- unpack target view
!----------------------------------------------------
if (klm == sym%YES) then

!--- the internal target data from klm
  do j = 1,10
   blackbody_count(3,j) = MAKE_I2WORD(buffer(1101+6*(j-1):1102+6*(j-1)),sym%SIGNED,byte_swap_1b)
   blackbody_count(4,j) = MAKE_I2WORD(buffer(1103+6*(j-1):1104+6*(j-1)),sym%SIGNED,byte_swap_1b)
   blackbody_count(5,j)  = MAKE_I2WORD(buffer(1105+6*(j-1):1106+6*(j-1)),sym%SIGNED,byte_swap_1b)
  enddo
else

!--- the internal target data from pre-klm
    ichan = 3
    ipix = 1
    do iword = 23,52,1
       blackbody_count(ichan, ipix) = telemetry_word(iword)
       ichan = ichan + 1
       if (ichan > 5) then
          ichan = 3
          ipix = ipix + 1
       endif
     enddo

endif

!------------------------------------------------------
!---- unpack space view
!------------------------------------------------------

if (klm == sym%YES) then

  !---space data from klm data
  do j = 1,10
   space_count_temp(1,j)  = MAKE_I2WORD(buffer(1161+10*(j-1):1162+10*(j-1)),sym%SIGNED,byte_swap_1b)
   space_count_temp(2,j) = MAKE_I2WORD(buffer(1163+10*(j-1):1164+10*(j-1)),sym%SIGNED,byte_swap_1b)
   space_count_temp(3,j) = MAKE_I2WORD(buffer(1165+10*(j-1):1166+10*(j-1)),sym%SIGNED,byte_swap_1b)
   space_count_temp(4,j) = MAKE_I2WORD(buffer(1167+10*(j-1):1168+10*(j-1)),sym%SIGNED,byte_swap_1b)
   space_count_temp(5,j)  = MAKE_I2WORD(buffer(1169+10*(j-1):1170+10*(j-1)),sym%SIGNED,byte_swap_1b)
  enddo

else

    !---space data from pre-klm data
    ichan = 1
    ipix = 1
    do iword = 53,102,1
       space_count_temp(ichan, ipix) = telemetry_word(iword)
       ichan = ichan + 1
       if (ichan > 5) then
          ichan = 1
          ipix = ipix + 1
       endif
    enddo

endif



   !---- check for rapid change in ch4 space count (ie during a moon view)
   space_count_temp_4_test = sum(space_count_temp(4,:))/10.0
   if ((mean_space_count_4 > 0) .and.  &
       (abs(space_count_temp_4_test - mean_space_count_4) > 4.0))  then
       print *, EXE_PROMPT, "Error: rapid change in ch4 space count "
       valid_therm_cal = .false.
       return
   endif

   !---- check for bad space or blackbody counts
   if ((minval(space_count_temp(4:5,:)) < 50) .or. (minval(blackbody_count(4:5,:)) < 50)) then
       print *, EXE_PROMPT, "Error: ch4 -ch5 space count threshold exceeded "
       valid_therm_cal = .false.
       return
   endif

    !----- assume a valid calibration can be made
    if (valid_therm_cal .eqv. .true.) then
     jcal = jcal + 1
    endif

    !--- compute blackbody temperature if valid PRT reading
    if ((iprt > 0) .and. (valid_therm_cal .eqv. .true.)) then
       mean_prt = sum_prt/3.0
       T_prt(jcal) = prt_coef(0,iprt) +  &
                     prt_coef(1,iprt)*mean_prt + &
                     prt_coef(2,iprt)*mean_prt**2 + &
                     prt_coef(3,iprt)*mean_prt**3 + &
                     prt_coef(4,iprt)*mean_prt**4

    endif

    !--- if no PRT reading, fill in with mean value
    if ((iprt == 0) .and. (valid_therm_cal .eqv. .true.)) then
       T_prt(jcal) = mean_T_prt
    endif

    !--- average space and bb counts for this scan
    if (jcal > 0) then
       space_count_temp_1(jcal) = sum(space_count_temp(1,:))/10.0
       space_count_temp_2(jcal) = sum(space_count_temp(2,:))/10.0
       space_count_temp_3(jcal) = sum(space_count_temp(3,:))/10.0
       space_count_temp_4(jcal) = sum(space_count_temp(4,:))/10.0
       space_count_temp_5(jcal) = sum(space_count_temp(5,:))/10.0

       bb_count_3(jcal) = sum(blackbody_count(3,:))/10.0
       bb_count_4(jcal) = sum(blackbody_count(4,:))/10.0
       bb_count_5(jcal) = sum(blackbody_count(5,:))/10.0
    endif

!-------------------------------------------------------------------
! compute running means of space, blackbody and prt temps
!-------------------------------------------------------------------

!--- if initial spinup period, sum all to get mean

    if (spinup_new_therm_cal .eqv. .true.) then

      if (jcal > 0) then
       mean_T_prt = sum(T_prt(1:jcal))/jcal
       mean_space_count_1 = sum(space_count_temp_1(1:jcal))/jcal
       mean_space_count_2 = sum(space_count_temp_2(1:jcal))/jcal
       mean_space_count_3 = sum(space_count_temp_3(1:jcal))/jcal
       mean_space_count_4 = sum(space_count_temp_4(1:jcal))/jcal
       mean_space_count_5 = sum(space_count_temp_5(1:jcal))/jcal
       mean_bb_count_3 = sum(bb_count_3(1:jcal))/jcal
       mean_bb_count_4 = sum(bb_count_4(1:jcal))/jcal
       mean_bb_count_5 = sum(bb_count_5(1:jcal))/jcal
      else
       mean_T_prt = missing_value_real4
       mean_space_count_1 = missing_value_real4
       mean_space_count_2 = missing_value_real4
       mean_space_count_3 = missing_value_real4
       mean_space_count_4 = missing_value_real4
       mean_space_count_5 = missing_value_real4
       mean_bb_count_3 = missing_value_real4
       mean_bb_count_4 = missing_value_real4
       mean_bb_count_5 = missing_value_real4
       valid_therm_cal = .false.
      endif

      if (jcal == ncal) then
        spinup_new_therm_cal = .false.
        jcal = 0
        T_prt = 0.0
        space_count_temp_1 = 0.0
        space_count_temp_2 = 0.0
        space_count_temp_3 = 0.0
        space_count_temp_4 = 0.0
        space_count_temp_5 = 0.0
        bb_count_3 = 0.0
        bb_count_4 = 0.0
        bb_count_5 = 0.0
      endif

    else

      !--- after ncal lines, update space and bb count values 
      if (jcal == ncal) then

       !--- compute running-mean of the black-body temperature
       mean_T_prt = (1.0-beta)*mean_T_prt + beta*(sum(T_prt)/ncal)

       !--- compute instantaneous mean of the space counts
       mean_space_count_1_temp = sum(space_count_temp_1)/ncal
       mean_space_count_2_temp = sum(space_count_temp_2)/ncal
       mean_space_count_3_temp = sum(space_count_temp_3)/ncal
       mean_space_count_4_temp = sum(space_count_temp_4)/ncal
       mean_space_count_5_temp = sum(space_count_temp_5)/ncal

       !--- compute instantaneous mean of the black-body counts
       mean_bb_count_3_temp = sum(bb_count_3)/ncal
       mean_bb_count_4_temp = sum(bb_count_4)/ncal
       mean_bb_count_5_temp = sum(bb_count_5)/ncal

       !--- compute running mean of space counts
       mean_space_count_1 = (1.0-beta)*mean_space_count_1 + beta*mean_space_count_1_temp
       mean_space_count_2 = (1.0-beta)*mean_space_count_2 + beta*mean_space_count_2_temp
       mean_space_count_3 = (1.0-beta)*mean_space_count_3 + beta*mean_space_count_3_temp
       mean_space_count_4 = (1.0-beta)*mean_space_count_4 + beta*mean_space_count_4_temp
       mean_space_count_5 = (1.0-beta)*mean_space_count_5 + beta*mean_space_count_5_temp

       !--- compute running mean of black-body counts
       mean_bb_count_3 = (1.0-beta)*mean_bb_count_3 + beta*mean_bb_count_3_temp
       mean_bb_count_4 = (1.0-beta)*mean_bb_count_4 + beta*mean_bb_count_4_temp
       mean_bb_count_5 = (1.0-beta)*mean_bb_count_5 + beta*mean_bb_count_5_temp


       !--- handle ch3 differently due to ch3a/ch3b switching
       !--- if a switch detected, reset running means with instantataneous values

       if ((ch3a_on(iline) == 1) .and. (mean_space_count_3 > 50)) then
          mean_space_count_3 = mean_space_count_3_temp
          mean_bb_count_3 =  mean_bb_count_3_temp
       endif

       if ((ch3a_on(iline) == 0) .and. (mean_space_count_3 < 950)) then
          mean_space_count_3 = mean_space_count_3_temp
          mean_bb_count_3 =  mean_bb_count_3_temp
       endif

       jcal = 0
       T_prt = 0.0
       space_count_temp_1 = 0.0
       space_count_temp_2 = 0.0
       space_count_temp_3 = 0.0
       space_count_temp_4 = 0.0
       space_count_temp_5 = 0.0
       bb_count_3 = 0.0
       bb_count_4 = 0.0
       bb_count_5 = 0.0

      endif

    endif


    !---  look for space counts that invalid for ch3a/3b
    if ((mean_space_count_3 > 50.0) .and. (mean_space_count_3 < 950.0)) then
             valid_therm_cal = .false.
    endif


!   !--- ch3b cal test
!   write(unit=6,fmt="(I6,2I3,8f10.4)") scan_number(iline), ch3a_on(iline),  &
!             ascend(iline), &
!             lat_anchor_1b(25,iline), &
!             mean_space_count_3, &
!             mean_bb_count_3, mean_space_count_4, mean_bb_count_4, mean_T_prt

    if (valid_therm_cal .eqv. .true.) then

       !--- blackbody radiance
       Nbb_3b = PLANCK_RAD_FAST(3,mean_T_prt)
       Nbb_4 = PLANCK_RAD_FAST(4,mean_T_prt)
       Nbb_5 = PLANCK_RAD_FAST(5,mean_T_prt)

       !--- compute new linear slopes
       if (ch3a_on(iline) == 0) then
        ir_linear_slope_new(3,iline) = (space_rad_3b - Nbb_3b) / &
                                       (mean_space_count_3 - mean_bb_count_3)
        ir_linear_intercept_new(3,iline) = space_rad_3b - &
                  ir_linear_slope_new(3,iline)*mean_space_count_3
       endif


       ir_linear_slope_new(4,iline) = (space_rad_4 - Nbb_4) / &
                                (mean_space_count_4 - mean_bb_count_4)
       ir_linear_intercept_new(4,iline) = space_rad_4 - &
                  ir_linear_slope_new(4,iline)*mean_space_count_4

       ir_linear_slope_new(5,iline) = (space_rad_5 - Nbb_5) / &
                                (mean_space_count_5 - mean_bb_count_5)
       ir_linear_intercept_new(5,iline) = space_rad_5 - &
                  ir_linear_slope_new(5,iline)*mean_space_count_5

     else
        valid_therm_cal = .false.
     endif

     !--- store these into public arrays for access by other routines
     scan_space_counts(1,iline) = mean_space_count_1
     scan_space_counts(2,iline) = mean_space_count_2
     scan_space_counts(3,iline) = mean_space_count_3
     scan_space_counts(4,iline) = mean_space_count_4
     scan_space_counts(5,iline) = mean_space_count_5
     scan_bb_counts(3,iline) = mean_bb_count_3
     scan_bb_counts(4,iline) = mean_bb_count_4
     scan_bb_counts(5,iline) = mean_bb_count_5

end subroutine COMPUTE_NEW_THERM_CAL_COEF

!----------------------------------------------------------------
! read the avhrr constants into memory
!-----------------------------------------------------------------
subroutine READ_AVHRR_INSTR_CONSTANTS(instr_const_file)
 character(len=*), intent(in):: instr_const_file
 integer:: ios0, erstat
 integer:: instr_const_lun

 instr_const_lun = GET_LUN()

 open(unit=instr_const_lun,file=trim(instr_const_file),status="old",position="rewind",action="read",iostat=ios0)

 erstat = 0
 if (ios0 /= 0) then
    erstat = 19
    print *, EXE_PROMPT, "Error opening AVHRR constants file, ios0 = ", ios0
    stop 19
 endif

  read(unit=instr_const_lun,fmt="(a3)") sat_name
  read(unit=instr_const_lun,fmt=*) solar_3b
  read(unit=instr_const_lun,fmt=*) ew_3b
  read(unit=instr_const_lun,fmt=*) space_rad_3b, b0_3b, b1_3b, b2_3b
  read(unit=instr_const_lun,fmt=*) space_rad_4, b0_4, b1_4, b2_4
  read(unit=instr_const_lun,fmt=*) space_rad_5, b0_5, b1_5, b2_5
  read(unit=instr_const_lun,fmt=*) nu_3b
  read(unit=instr_const_lun,fmt=*) a1_3b
  read(unit=instr_const_lun,fmt=*) a2_3b
  read(unit=instr_const_lun,fmt=*) nu_4
  read(unit=instr_const_lun,fmt=*) a1_4
  read(unit=instr_const_lun,fmt=*) a2_4
  read(unit=instr_const_lun,fmt=*) nu_5
  read(unit=instr_const_lun,fmt=*) a1_5
  read(unit=instr_const_lun,fmt=*) a2_5
  read(unit=instr_const_lun,fmt=*) ch1_dark_count
  read(unit=instr_const_lun,fmt=*) ch2_dark_count
  read(unit=instr_const_lun,fmt=*) ch3a_dark_count
  read(unit=instr_const_lun,fmt=*) ch1_gain_low_0,ch1_degrad_low_1, ch1_degrad_low_2
  read(unit=instr_const_lun,fmt=*) ch1_gain_high_0,ch1_degrad_high_1, ch1_degrad_high_2
  read(unit=instr_const_lun,fmt=*) ch2_gain_low_0,ch2_degrad_low_1, ch2_degrad_low_2
  read(unit=instr_const_lun,fmt=*) ch2_gain_high_0,ch2_degrad_high_1, ch2_degrad_high_2
  read(unit=instr_const_lun,fmt=*) ch3a_gain_low_0,ch3a_degrad_low_1, ch3a_degrad_low_2
  read(unit=instr_const_lun,fmt=*) ch3a_gain_high_0,ch3a_degrad_high_1, ch3a_degrad_high_2
  read(unit=instr_const_lun,fmt=*) launch_date
  read(unit=instr_const_lun,fmt=*) ch1_switch_count
  read(unit=instr_const_lun,fmt=*) ch2_switch_count
  read(unit=instr_const_lun,fmt=*) ch3a_switch_count
  read(unit=instr_const_lun,fmt=*) prt_coef(:,1)
  read(unit=instr_const_lun,fmt=*) prt_coef(:,2)
  read(unit=instr_const_lun,fmt=*) prt_coef(:,3)
  read(unit=instr_const_lun,fmt=*) prt_coef(:,4)
  read(unit=instr_const_lun,fmt=*) prt_weight(:)
  read(unit=instr_const_lun,fmt=*) b1_day_mask,b2_day_mask,b3_day_mask,b4_day_mask
  close(unit=instr_const_lun)

!-- convert solar flux in channel 3b to mean with units mW/m^2/cm^-1
  solar_3b_nu = 1000.0 * solar_3b / ew_3b

!----- define planck constants here

end subroutine READ_AVHRR_INSTR_CONSTANTS

!-----------------------------------------------------------------
! functions to convert 1 byte integers into 2 or 4 byte integers
! if the integeris signed (isign = 0) then the first byte
! remains signed
!---------------------------------------------------------------- 
 function MAKE_I2WORD(byte1,isign,iswap) result(i2word)
  integer(kind=int1), intent(in), dimension(:):: byte1
  integer(kind=int4), intent(in):: isign
  integer(kind=int4), intent(in):: iswap
  integer(kind=int2):: i2word
  integer(kind=int2), dimension(:), allocatable:: byte2
  integer:: i,n

  n = size(byte1)

  allocate(byte2(n))

!--- swap bytes if necessary
  if (iswap == sym%YES) then
   do i = 1, n
    byte2(i) = byte1(n-i+1) 
   enddo
  else
   byte2 = byte1
  endif

!--- account for unsigned integers
   if ((byte2(1) < 0) .and. (isign == 0)) then
     byte2(1) = byte2(1) + 256
   endif
   do i = 2,n
    if ((byte2(i) < 0)) then
     byte2(i) = byte2(i) + 256
    end if
   end do

!--- make word
   i2word = byte2(1)*256 + byte2(2)

   deallocate(byte2)

 end function MAKE_I2WORD

!-----
 function MAKE_I4WORD(byte1,isign,iswap) result(i4word)
  integer(kind=int1), intent(in), dimension(:):: byte1
  integer(kind=int4), intent(in):: isign
  integer(kind=int4), intent(in):: iswap
  integer(kind=int4):: i4word
  integer(kind=int2), dimension(:), allocatable:: byte2
  integer:: i,n

   n = size(byte1)

   allocate(byte2(n))

!--- swap bytes if necessary
  if (iswap == sym%YES) then
   do i = 1, n
    byte2(i) = byte1(n-i+1) 
   enddo
  else
   byte2 = byte1
  endif

!--- account for unsigned integers
   if ((byte2(1) < 0) .and. (isign == sym%NO)) then
     byte2(1) = byte2(1) + 256
   endif
   do i = 2,n
    if ((byte2(i) < 0)) then
     byte2(i) = byte2(i) + 256
    end if
   end do
   i4word = 0
   do i = n,1,-1
     i4word = i4word + byte2(i)*(256**(n-i))
   enddo

   deallocate(byte2)

 end function MAKE_I4WORD


 function i4word_to_string(i4word,n) result(char_word)
    integer(kind=int4), intent(in):: i4word
    integer, intent(in):: n
    character(len=n):: char_word
    integer(kind=int4):: i4temp
    integer:: i

    i4temp = i4word
    char_word = ""
    do i = 1,n
     char_word(i:i) = char(i4temp/(10**(n-i))+48)
     i4temp = i4temp - 10**(n-i)*(i4temp / (10**(n-i)))
    enddo
 end function i4word_to_string

!======================================================================
! unpack avhrr header record - routine should work for lac and gac on
! all pre-klm sensors
!
! Modified December 2008 for segment processing
!======================================================================
subroutine UNPACK_AVHRR_HEADER_RECORD(sc_id,data_type,start_year_temp,&
              start_day_temp,start_time_temp,num_scans,end_year_temp,end_day_temp,end_time_temp, &
              tip_parity,aux_sync,ramp_auto_cal,proc_block_id,ver_1b)

   integer(kind=int2), intent(out):: sc_id,data_type,start_year_temp,start_day_temp, &
                                     end_year_temp,end_day_temp, &
                                     tip_parity,aux_sync,ramp_auto_cal,ver_1b
   integer(kind=int4), intent(out):: start_time_temp,end_time_temp,num_scans
   character(len=*), intent(out):: proc_block_id
   integer(kind=int4):: i4word
   integer(kind=int2), dimension(2):: data_byte
   integer:: i
     
   !-----------------------------------------------------------------------
   ! begin executable code
   !-----------------------------------------------------------------------

   sc_id = header_buffer(1)
   data_type = ishft(header_buffer(2),-4)

   tip_parity = 0
   aux_sync = 0
   ramp_auto_cal = 0
   ver_1b = 1

   !----------------------------------------------------------------------
   ! unpack starting time_temp
   !----------------------------------------------------------------------
   data_byte(1) = header_buffer(3)
   data_byte(2) = header_buffer(4)
   where (data_byte < 0)
         data_byte = data_byte + 256
   end where
   i4word =  data_byte(1)*256 + data_byte(2)
   start_year_temp = i4word / (2**9)
   start_day_temp = i4word - start_year_temp * (2**9)

   i4word = MAKE_I4WORD(header_buffer(5:8),sym%UNSIGNED,byte_swap_1b)
   start_time_temp = i4word - ((i4word/2**27)*(2**27))

   !---------------------------------------------------------------------
   ! unpack processing block id
   !---------------------------------------------------------------------
   do i = 1,len(proc_block_id)
    proc_block_id(i:i) =  char(header_buffer(17+(i-1)))
   end do 
       
   !----------------------------------------------------------------------
   ! unpack  number of scan lines
   !----------------------------------------------------------------------
   num_scans = MAKE_I4WORD(header_buffer(9:10),sym%UNSIGNED,byte_swap_1b)

   !----------------------------------------------------------------------
   ! unpack end time
   !----------------------------------------------------------------------
   data_byte(1) = header_buffer(11)
   data_byte(2) = header_buffer(12)
   where (data_byte < 0)
       data_byte = data_byte + 256
   end where
   i4word =  data_byte(1)*256 + data_byte(2)

   end_year_temp = i4word / (2**9)
   end_day_temp = i4word - start_year_temp * (2**9)

   i4word = MAKE_I4WORD(header_buffer(13:16),sym%UNSIGNED,byte_swap_1b)
   end_time_temp = i4word - ((i4word/2**27)*(2**27))

   !----------------------------------------------------------------------
   ! unpack quality indicators
   !-----------------------------------------------------------------------
!  tip_parity = MAKE_I2WORD(header_buffer(139:140),sym%UNSIGNED,byte_swap_1b)
!  aux_sync = MAKE_I2WORD(header_buffer(141:142),sym%UNSIGNED,byte_swap_1b)
!  ramp_auto_cal =  MAKE_I2WORD(header_buffer(187:188),sym%UNSIGNED,byte_swap_1b)

 end subroutine UNPACK_AVHRR_HEADER_RECORD
!======================================================================
! NOAA 15+ unpack avhrr header record - routine should work for lac and gac
!
! Modified January 2005 for NOAA-N
!
! Modified December 2008 for segment processing
!======================================================================
subroutine UNPACK_AVHRR_HEADER_RECORD_KLM(sc_id,data_type,start_year_temp,&
              start_day_temp,start_time_temp,num_scans,end_year_temp,end_day_temp,end_time_temp, &
              tip_parity,aux_sync,ramp_auto_cal,proc_block_id,ver_1b)

   integer(kind=int2), intent(out):: sc_id,data_type,start_year_temp,start_day_temp, &
                                     end_year_temp,end_day_temp, &
                                     tip_parity,aux_sync,ramp_auto_cal,ver_1b
   integer(kind=int4), intent(out):: start_time_temp,end_time_temp,num_scans
   character(len=*), intent(out):: proc_block_id
   integer:: i

!-----------------------------------------------------------------------
! begin executable code
!-----------------------------------------------------------------------
      sc_id = MAKE_I2WORD(header_buffer(73:74),sym%UNSIGNED,byte_swap_1b)
      data_type = MAKE_I2WORD(header_buffer(77:78),sym%UNSIGNED,byte_swap_1b)

      do i = 1,len(proc_block_id)
       proc_block_id(i:i) =  char(header_buffer(66+(i-1)))
      end do 

!----------------------------------------------------------------------
! unpack 1b version number
!----------------------------------------------------------------------
      ver_1b = MAKE_I2WORD(header_buffer(5:6),sym%UNSIGNED,byte_swap_1b)

     !--- correct for noaa-15 bug in version number
     if ((ver_1b == 1) .and. (sc_id== 4)) then
       ver_1b = 2
     endif  

     !----------------------------------------------------------------------
     ! unpack starting time_temp
     !----------------------------------------------------------------------
     start_year_temp = MAKE_I2WORD(header_buffer(85:86),sym%UNSIGNED,byte_swap_1b)
     start_day_temp = MAKE_I2WORD(header_buffer(87:88),sym%UNSIGNED,byte_swap_1b)
     start_time_temp = MAKE_I4WORD(header_buffer(89:92),sym%UNSIGNED,byte_swap_1b)
   
    !----------------------------------------------------------------------
    ! unpack  number of scan lines
    ! note, the POD stores this number as a two byte unsigned integer
    !----------------------------------------------------------------------
    num_scans = MAKE_I4WORD(header_buffer(129:130),sym%UNSIGNED,byte_swap_1b)

    !----------------------------------------------------------------------
    ! unpack end time_temp
    !----------------------------------------------------------------------
    end_year_temp = MAKE_I2WORD(header_buffer(97:98),sym%UNSIGNED,byte_swap_1b)
    end_day_temp = MAKE_I2WORD(header_buffer(99:100),sym%UNSIGNED,byte_swap_1b)
    end_time_temp = MAKE_I4WORD(header_buffer(101:104),sym%UNSIGNED,byte_swap_1b)

    !----------------------------------------------------------------------
    ! unpack quality indicators - check this
    !-----------------------------------------------------------------------
    tip_parity = MAKE_I2WORD(header_buffer(139:140),sym%UNSIGNED,byte_swap_1b)
    aux_sync = MAKE_I2WORD(header_buffer(141:142),sym%UNSIGNED,byte_swap_1b)
    ramp_auto_cal =  MAKE_I2WORD(header_buffer(187:188),sym%UNSIGNED,byte_swap_1b)

 end subroutine UNPACK_AVHRR_HEADER_RECORD_KLM

!---------------------------------------------------------------------
! EXPLANATION OF LAC TO GAC GEOLOCATION CONVERSION:
!
! A LAC scan line is converted to GAC line in the following way:
!
!  1. GAC processing starts from the 3rd LAC pixel; 1st and 2nd LAC pixels are
!     not used.
!
!  2. The 1st GAC pixel is formed from the 3rd, 4th, 5th and 6th LAC pixel.
!
!  3. The 7th LAC pixel is skipped and the processing for the 2nd GAC pixel uses
!     the 8th, 9th, 10th and 11th LAC pixel.
!
!  4. LAC pixels contributing to a GAC pixel can be described with a formula:
!
!        l, l+1, l+2, l+3 for l = 5*g-2 where g = 1,..,409
!
!     `l' and `g' being LAC and GAC pixel index, respectively, in a scan line.
!
!  5. Geolocation of the third LAC pixel of the four used for a GAC pixel is
!     assigned to that GAC pixel. For the 1st and 2nd GAC pixels these are the
!     5th and 10th LAX pixels. Or in short:
!
!        l = 5*g (g=1,..,409)
!
!  6. The 2048th LAC pixel is not used for GAC pixel processing. The processing
!     ends with the 2047th LAX pixels, for a total of 2045 LAX pixels used
!     (2045/5=409).
!
! This subroutine performs a five-point Lagrangian interpolation for 2048 LAC
! pixels regardless of the actual length of the scan line (GAC or LAC). If it's
! a GAC scan line, the geolocations of the 2nd and 3rd LAC pixels for each GAC
! pixel are averaged to yield the final GAC pixel's geolocation.
!---------------------------------------------------------------------
 subroutine LAGRANGIAN_ANCHOR_INTERP(anchor,y)
   real (kind=ipre), dimension(:), intent(in):: anchor
   real (kind=ipre), dimension(:), intent(out):: y
   real (kind=ipre), dimension(2048):: temp_y
   real(kind=ipre), dimension(:), allocatable:: temp_anchor
   real (kind=ipre):: x,L,xa_i,xa_j
   integer:: m,i,j,ia,ia1,ia2,ipix,num_anchor,num_skip,num_start
   integer:: discon

   m = 2    !2 means 5 point interp
   num_anchor = size(anchor,1)  !should be 51
   allocate(temp_anchor(num_anchor))

! I'm leaving `num_skip' and `num_start' here as useful constants although they
! are not used in this subroutine.
   if (num_pix == 409) then
      ! It's a GAC scan line.
      num_start = 5 !number of pixels between anchors
      num_skip = 8  !pixel number for first anchor point
   else
      ! Must be a LAC scan line.
      num_start = 25
      num_skip = 40
   endif

!--- initialize
   y = 0.0
   temp_y = 0.0
   temp_anchor = anchor

!-------- check for a discontinuity (ie. crossing the 180W meridian)
    discon = 0
    if ((minval(temp_anchor) < -140.0).and.(maxval(temp_anchor) > 140.0)) then
     discon = 1
     where (temp_anchor < 0.0)
        temp_anchor =  temp_anchor + 360.0
     endwhere
    endif

!-- perform 2m+1 point Lagrangian interpolation
      do ipix = 1, 2048
         ia = (ipix-25)/40+1
         ia1 =  min(num_anchor,max(1,ia-m))
         ia2 =  max(1,min(num_anchor,ia+m))
         x = ipix
         do i = ia1,ia2
           L = 1.0
           xa_i = (i-1)*(40)+25
           do j = ia1,ia2
            xa_j = (j-1)*(40)+25
            if (i /= j) L = L*(x-xa_j)/(xa_i-xa_j)
          enddo
          temp_y(ipix) = temp_y(ipix)+L*temp_anchor(i)
         enddo
      enddo

! If it's a GAC scan line perform averaging of `temp_y' for the 2nd and 3rd LAC
! pixel going into every GAC pixel for the final value of `y'.
if (num_pix == 409) then
   do ipix = 1, 409
      y(ipix) = 0.5*(temp_y(5*ipix)+temp_y(5*ipix-1))
   enddo
else
   y = temp_y
endif

!----- account for any discontinuities
   where(y > 180.0)
    y = y - 360.0
   endwhere
   where(y < -180.0)
    y = 360.0 + y
   endwhere

   deallocate(temp_anchor)

 end subroutine LAGRANGIAN_ANCHOR_INTERP

!---------------------------------------------------------------------
! gnomic navigation anchor interpolation routine
! note, this is only valid for lat and lon, do not apply to angles
!---------------------------------------------------------------------
 subroutine GNOMIC_ANCHOR_INTERP(lon_anchor_deg,lat_anchor_deg,longitude,latitude)

   real (kind=ipre), dimension(:), intent(in):: lon_anchor_deg,lat_anchor_deg
   real (kind=ipre), dimension(:), intent(out):: longitude,latitude
   real (kind=ipre), dimension(:), allocatable:: k,x_anchor,y_anchor, &
                                                 lon_anchor,lat_anchor
   real (kind=ipre), dimension(num_pix):: x,y
   real (kind=ipre):: yy,xx,xa1,xa2,rho,v,v_prev,c,lat_center,lon_center,offset
   integer:: ia1,ia2,ipix,num_anchor,num_skip,num_start


   num_anchor = size(lon_anchor_deg)  !should be 51
   num_skip = num_pix / num_anchor
   num_start = (num_pix - num_skip*(num_anchor-1))/2 + 1

   allocate(k(num_anchor))
   allocate(x_anchor(num_anchor))
   allocate(y_anchor(num_anchor))
   allocate(lon_anchor(num_anchor))
   allocate(lat_anchor(num_anchor))

!--- initialize
     latitude = 0.0
     longitude = 0.0

!--- convert anchors to radians
     lat_anchor = lat_anchor_deg * dtor
     lon_anchor = lon_anchor_deg * dtor

!--- pick center for projection (in radians)
     lat_center = lat_anchor(num_anchor/2)
     lon_center = lon_anchor(num_anchor/2)

!--- convert to gnomic space
     k = sin(lat_center) * sin(lat_anchor) +  &
         cos(lat_center) * cos(lat_anchor) * cos(lon_anchor - lon_center)
     k = 1.0 / k

     x_anchor = k * cos(lat_anchor) * sin(lon_anchor - lon_center)
     y_anchor = k * (cos(lat_center) * sin(lat_anchor) -  &
                     sin(lat_center) * cos(lat_anchor) * cos(lon_anchor - lon_center))

!--- linear interp in gnomic space
      do ipix = 1, num_pix

!--- determine two closest anchor points
         ia1 = max(1,min(num_anchor-1,(ipix - num_start)/num_skip + 1))
         ia2 = ia1 + 1

!--- determine weights of two closest anchor points
         xa1 = (ia1-1)*(num_skip) + num_start
         xa2 = (ia2-1)*(num_skip) + num_start

!--- determine point to interpolate to
         xx = ipix

!--- interp x at xx
         yy = (xx-xa1)*(x_anchor(ia2)-x_anchor(ia1))/(xa2-xa1) +  &
                   x_anchor(ia1)
         x(ipix) = yy
         
!--- interp y at xx
         yy = (xx-xa1)*(y_anchor(ia2)-y_anchor(ia1))/(xa2-xa1) +  &
                   y_anchor(ia1)
         y(ipix) = yy

      enddo

!--- convert other quantities
      v_prev = 0.0
      offset = 0
      do ipix = 1,num_pix

!--- some needed constants
        rho = sqrt(x(ipix)**2 + y(ipix)**2)

!--- handle center point where rho = 0.0
        if (rho == 0.0) then 
           latitude(ipix) = lat_center
           longitude(ipix) = lon_center
           cycle
        endif
        c = atan(rho)

!--- v is the longitude in radians relative to the center longitude
        v = atan( x(ipix) * sin(c) /  &
          (rho * cos(lat_center) * cos(c) - y(ipix) * sin(lat_center) * sin(c)) )

!--- look for a jump and determine offset
        if (ipix == 1) then
          v_prev = v
        endif

!       dv = v - v_prev
!       offset = 0.0
!       if (abs(dv) > 0.9*pi) then
!         if (dv < 0.0) then
!           offset = pi
!         else
!           offset = -1.0*pi
!         endif
!       endif

!       if (ipix == 205) then 
!          print *, dv, 0.9*pi, offset
!       endif
!       v = v + offset

        latitude(ipix) = asin(cos(c) * sin(lat_center) + (y(ipix)*sin(c)*cos(lat_center) / rho))
        longitude(ipix) = lon_center + v

        v_prev = v

      enddo

!--- convert to degrees
      latitude = latitude / dtor
      longitude = longitude / dtor
     

!----- account for discontinuity if there was one
  where (longitude > 180.0)
    longitude = longitude - 360.0
  endwhere
  where (longitude < -180.0)
    longitude = longitude + 360.0
  endwhere


  deallocate(k)
  deallocate(x_anchor)
  deallocate(y_anchor)
  deallocate(lon_anchor)
  deallocate(lat_anchor)

  end subroutine GNOMIC_ANCHOR_INTERP

!---------------------------------------------------------------------
! linear interpolation routine
!---------------------------------------------------------------------
 subroutine LINEAR_ANCHOR_INTERP(anchor,y)
   real (kind=ipre), dimension(:), intent(in):: anchor
   real (kind=ipre), dimension(:), intent(out):: y
   real (kind=ipre), dimension(:), allocatable:: temp_anchor
   real (kind=ipre):: x,xa1,xa2
   integer:: ia1,ia2,ipix,num_anchor,num_skip,num_start
   integer:: discon

   num_anchor = size(anchor)  !should be 51
   num_skip = num_pix / num_anchor
   num_start = (num_pix - num_skip*(num_anchor-1))/2 + 1

   allocate(temp_anchor(num_anchor))

!--- initialize
     y = 0.0
     temp_anchor = anchor

!-------- check for a discontinuity (ie. crossing dateline)
    discon = 0
    if ((minval(temp_anchor) < -175.0).and.(maxval(temp_anchor) > 175.0)) then
     discon = 1
     where( temp_anchor < 0.0)
        temp_anchor =  temp_anchor + 360.0
     endwhere
    endif

!-----------------------------------------------------------
      do ipix = 1, num_pix
         ia1 = max(1,min(num_anchor-1,(ipix - num_start)/num_skip + 1))
         ia2 = ia1 + 1

         xa1 = (ia1-1)*(num_skip) + num_start
         xa2 = (ia2-1)*(num_skip) + num_start

         x = ipix

         y(ipix) = (x-xa1)*(temp_anchor(ia2)-temp_anchor(ia1))/(xa2-xa1) +  &
                   temp_anchor(ia1)

      enddo

!----- account for discontinuity if there was one
  if (discon == 1) then
   where (y > 180.0)
    y = y - 360.0
   endwhere
  endif

  deallocate(temp_anchor)

  end subroutine LINEAR_ANCHOR_INTERP

!-------------------------------------------------------------------------------
! COMPUTE_ANGLE_ANCHORS
!
! input
!   jday_temp -  Julian Day
!   time_temp - time_temp of day_temp
!   lon_temp - vector of longitude (degrees)
!   lat_temp - vector of latitude (degrees)
!
! output
!   satzen_temp - vector of satellite view zenith angles (degrees)
!   solzen_temp - vector of solar zenith angles (degrees)
!   relaz_temp - vector of relative azimuth angles
!   solaz_temp - vector of solar azimuth angles
! 
! Revision History
!   Jan 2003 - Coded A. Heidinger (Based on CLAVR-1 code)   
!
! note - relaz is the relative azimuth defined as this
!   relaz = 180 means the sun is behind you (backscatter)
!   relaz = 0 mean the sun is infront of you (forward scatter)
!   this is convention used in noaa-klm relaz but not in clavr-1
!-------------------------------------------------------------------------------
   subroutine COMPUTE_ANGLE_ANCHORS(jday_temp,   &
                                    time_temp,   &
                                    lon_temp,    &
                                    lat_temp,    &
                                    satzen_temp, &
                                    solzen_temp, &
                                    relaz_temp,  &
                                    solaz_temp)


     integer(kind=int2), intent(in):: jday_temp
     integer(kind=int4), intent(in):: time_temp
     real(kind=ipre), dimension(:), intent(in):: lon_temp
     real(kind=ipre), dimension(:), intent(in):: lat_temp
     real(kind=ipre), dimension(:), intent(out):: satzen_temp
     real(kind=ipre), dimension(:), intent(out):: solzen_temp
     real(kind=ipre), dimension(:), intent(out):: relaz_temp
     real(kind=ipre), dimension(:), intent(out):: solaz_temp
     integer:: i
     integer:: n
     integer:: jday_temp4
     real(kind=real4):: frac_time_temp
     real(kind=real4):: lat_temp_center
     real(kind=real4):: lon_temp_center
     real(kind=real4):: solzen_temp_center
     real(kind=real8):: cosgeo
     real(kind=real8):: psix
     real(kind=real8):: geox
     real(kind=real8):: numor
     real(kind=real8):: denom
     real(kind=real8):: cos_solzen_temp


      !--- convert time to fraction hours
      frac_time_temp = time_temp / (60.0*60.0*1000.0)

      !--- determine size of the arrays to process
      n = size(lon_temp)

      !--- make a local int4 julian day copy
      jday_temp4 = jday_temp

     !-------------------------------------------------
     ! solar zenith angle
     !-------------------------------------------------
      do i = 1, n
       call POSSOL(jday_temp4, &
                   frac_time_temp, &
                   lon_temp(i), &
                   lat_temp(i), &
                   solzen_temp(i), &
                   solaz_temp(i))
      enddo

      !-------------------------------------------------
      !---- relative azimuth and satellite zenith angle
      !-------------------------------------------------
      do i = 1,n
       lat_temp_center = lat_temp(26)
       lon_temp_center = lon_temp(26)
       solzen_temp_center = solzen_temp(26)
       cosgeo = cos(lat_temp(i)*dtor)*cos(lat_temp_center*dtor)*cos((lon_temp(i)-lon_temp_center)*dtor) + &
                sin(lat_temp(i)*dtor)*sin(lat_temp_center*dtor)
       cosgeo = max(-1.0_real8,min(1.0_real8,cosgeo))
       geox = acos(cosgeo)
       cos_solzen_temp = cos(solzen_temp(i)*dtor)
       numor= cosgeo*cos_solzen_temp - cos(solzen_temp_center*dtor)
       denom = sin(geox)* sqrt(1.0-cos_solzen_temp**2)
       if (denom == 0.0) then
            psix = 1.0
       else
            psix = numor/denom
       endif
       psix = max(-1.0_real8,min(1.0_real8,psix))
       relaz_temp(i) = acos(psix)/dtor
       satzen_temp(i) = geox/dtor + abs(26-i)*2.16512
      enddo

 end subroutine COMPUTE_ANGLE_ANCHORS

!-------------------------------------------------------------------------------
! to use for gac data:
!  1. set gac to true
!  2. read the AVHRR 1B data records directly into buffer
!
! to use for lac data:
!  1. set gac to false (0)
!  2. combine the two  data-records from the AVHRR 1B file  into buffer
!-------------------------------------------------------------------------------
 subroutine UNPACK_AVHRR_DATA_RECORD(iline)

   integer, intent(in):: iline
   integer(kind=int4):: i4word
   integer(kind=int2):: i2word
   integer(kind=int4):: num_data_bytes
   integer(kind=int4):: num_chan
   integer:: ichan
   integer:: ipix
   integer:: j
   integer:: ibyte
   real, dimension(5):: slope
   real, dimension(5):: intercept
   integer(kind=int2), dimension(2):: data_byte

   !-------------------------------------------------------------------------------
   ! set parameters for gac or lac processing
   !-------------------------------------------------------------------------------
   if (gac == sym%YES) then
      num_data_bytes = 2728
    else
      num_data_bytes = 6952 + 6704
   endif
   num_chan = size(chan_counts,1)

   !----------------------------------------------------------------
   ! unpack scan line number
   !----------------------------------------------------------------
   scan_number(iline) = MAKE_I4WORD(buffer(1:2),sym%UNSIGNED,byte_swap_1b)

   !----------------------------------------------------------------------
   ! unpack time_temp code
   !----------------------------------------------------------------------
   data_byte(1) = buffer(3)
   data_byte(2) = buffer(4)
   where (data_byte < 0)
         data_byte = data_byte + 256
   end where
   i4word =  data_byte(1)*256 + data_byte(2)
   scan_year(iline) = i4word / (2**9)
   scan_day(iline) = i4word - scan_year(iline) * (2**9)

   i4word = MAKE_I4WORD(buffer(5:8),sym%UNSIGNED,byte_swap_1b)
   scan_time(iline) = i4word - ((i4word/2**27)*(2**27))

   !----------------------------------------------------------------------
   ! unpack quality indicators - not done fully
   !----------------------------------------------------------------------
   i2word = buffer(9)
   if (i2word < 0) then
      i2word = i2word + 256
   end if
   fatal(iline) = ishft(i2word, -7)

   i2word = buffer(9)
   if (i2word < 0) then
      i2word = i2word + 256
   end if
   ascend(iline) = ishft(ishft(i2word, 6),-7)

   !--- this needs to be verified
   i2word = buffer(10)
   if (i2word < 0) then
      i2word = i2word + 256
   end if
   resync(iline) = ishft(ishft(i2word, 0),-7)
   
!----------------------------------------------------------------------
! unpack calibration coefficients
!----------------------------------------------------------------------
    ichan = 1
    do ibyte = 13,52,8
     slope(ichan) = MAKE_I4WORD(buffer(ibyte:ibyte+3),sym%SIGNED,byte_swap_1b) / (2.0**30)
     intercept(ichan) = MAKE_I4WORD(buffer(ibyte+4:ibyte+7),sym%SIGNED,byte_swap_1b) / (2.0**22)
     ichan = ichan + 1
    enddo

!----------------------------------------------
! mimic noaa klm type calibration coefficients
!----------------------------------------------
    vis_slope_1(1,iline) = slope(1)   !high gain is same as low gain
    vis_slope_1(2,iline) = slope(2)   !high gain is same as low gain
    vis_slope_1(3,iline) = 0.0        !no channel 3a
    vis_slope_2(:,iline) = vis_slope_1(:,iline)   !high gain is same as low gain
    vis_intercept_2(:,iline) = vis_intercept_1(:,iline)   !high gain intercept same as low
    vis_intersection(:,iline) = 1024     !these values are ignored

    ir_linear_slope_1b(3,iline) = slope(3)
    ir_linear_slope_1b(4,iline) = slope(4)
    ir_linear_slope_1b(5,iline) = slope(5)

    ir_linear_intercept_1b(3,iline) = intercept(3)
    ir_linear_intercept_1b(4,iline) = intercept(4)
    ir_linear_intercept_1b(5,iline) = intercept(5)

    !----------------------------------------------------------------------
    ! unpack geolocation coefficients
    !----------------------------------------------------------------------
    num_loc = buffer(53)

    !----------------------------------------------------------------------
    ! unpack solar zenith angle data - treating as unsigned integer
    !----------------------------------------------------------------------
    do ibyte = 54,104
        j = ibyte - 53
        if (buffer(ibyte) < 0) then
             solzen_anchor(j,iline) = (256.0 + buffer(ibyte)) / 2.0
        else
             solzen_anchor(j,iline) = buffer(ibyte) / 2.0
        endif
    enddo

    !----------------------------------------------------------------------
    ! unpack earth location
    !----------------------------------------------------------------------
    j = 1
    do ibyte = 105,308,4
        lat_anchor_1b(j,iline) = MAKE_I2WORD(buffer(ibyte:ibyte+1),sym%SIGNED,byte_swap_1b) / 128.0
        lon_anchor_1b(j,iline) = MAKE_I2WORD(buffer(ibyte+2:ibyte+3),sym%SIGNED,byte_swap_1b) / 128.0
        j = j + 1
    enddo

    !-- make sure longitudes are correct ( ?? check this)
    where (lon_anchor_1b(:,iline) > 180)
        lon_anchor_1b(:,iline) = lon_anchor_1b(:,iline) - 360.0
    end where

    !----------------------------------------------------------------------
    ! unpack channel  data
    !----------------------------------------------------------------------
    ichan = 1
    ipix = 1

    do ibyte = 449,448+num_data_bytes,4

      i4word = MAKE_I4WORD(buffer(ibyte:ibyte+3),sym%UNSIGNED,byte_swap_1b)

      do j = 1,3

        if (ipix > num_pix) then
            exit
        endif

        chan_counts(ichan,ipix,iline) = ishft(ishft(i4word,2+10*(j-1)),-22)

        ichan = ichan + 1

        if (ichan > num_chan) then
          ichan = 1
          ipix = ipix + 1
        endif

      enddo

      if (ipix > num_pix) then
            exit
      endif

    enddo

  end subroutine UNPACK_AVHRR_DATA_RECORD

!-------------------------------------------------------------------------------
! to use for gac data:
!  1. set gac to true (1)
!  2. read the AVHRR 1B data records directly into buffer
!
! to use for lac data:
!  1. set gac to false (0)
!  2. combine the two  data-records from the AVHRR 1B file  into buffer
!
!
! Modifications
!  January 2005 - Modified for NOAA-N
!
!  Modified for one scan-line at a time
!-------------------------------------------------------------------------------
 subroutine UNPACK_AVHRR_DATA_RECORD_KLM(iline)

   integer, intent(in):: iline
   integer(kind=int4):: i4word
   integer(kind=int2):: i2word
   integer(kind=int4):: num_data_bytes
   integer(kind=int4):: num_chan
   integer(kind=int4):: num_clavr_bytes
   integer(kind=int4):: num_digtel_bytes
   integer(kind=int4):: num_house_bytes
   integer(kind=int4):: start_pixel
   integer(kind=int4):: end_pixel
   integer:: ichan
   integer:: ipix
   integer:: i
   integer:: j
   integer:: ibyte
   integer:: iword
   integer:: start_word
   integer:: end_word
   integer:: word_clavr_start
   integer(kind=int1):: onebyte

!-------------------------------------------------------------------------------
! set parameters for gac or lac processing
!-------------------------------------------------------------------------------
   if (gac == sym%YES) then
      num_data_bytes = 4000 - 1265 + 1
      num_digtel_bytes =  4016 - 4001 + 1
      num_house_bytes =  4048 - 4017 + 1
      num_clavr_bytes = 4160 - 4049 + 1
    else
      num_data_bytes = 14928 - 1265
      num_digtel_bytes = 14944 - 14929
      num_house_bytes = 14976 - 14971
      num_clavr_bytes = 15496 - 14977
   endif
   num_chan = size(chan_counts,1)

   valid_therm_cal = .true.

   !----------------------------------------------------------------
   ! unpack scan line number
   !----------------------------------------------------------------
   scan_number(iline) = MAKE_I4WORD(buffer(1:2),sym%UNSIGNED,byte_swap_1b)

   !----------------------------------------------------------------------
   ! unpack time_temp code
   !----------------------------------------------------------------------
   scan_year(iline) = MAKE_I2WORD(buffer(3:4),sym%UNSIGNED,byte_swap_1b)
   scan_day(iline) = MAKE_I2WORD(buffer(5:6),sym%UNSIGNED,byte_swap_1b)
   scan_time(iline) = MAKE_I4WORD(buffer(9:12),sym%UNSIGNED,byte_swap_1b)

   !----------------------------------------------------------------------
   ! unpack quality indicators - not done fully
   !----------------------------------------------------------------------
   i2word  = buffer(25)
   if (i2word < 0) then
      i2word = i2word + 256
   end if
   fatal(iline) = ishft(i2word, -7)

   i2word  = buffer(13)
   if (i2word < 0) then
      i2word = i2word + 256
   end if
   ascend(iline) = ishft(i2word, -7)

   i2word  = buffer(14)
   if (i2word < 0) then
      i2word = i2word + 256
   end if
   ch3a_on(iline) = ishft(ishft(i2word, 6),-6)
   ch3a_on_pixel_mask(:,iline) = ch3a_on(iline)

   i2word  = buffer(28)
   if (i2word < 0) then
      i2word = i2word + 256
   end if
   resync(iline) = ishft(ishft(i2word, 6),-7)

!---------------------------------------------------------------------
! unpack space view
!---------------------------------------------------------------------
  do j = 1,10
   space_count_temp(1,j) = MAKE_I2WORD(buffer(1161+10*(j-1):1162+10*(j-1)),sym%SIGNED,byte_swap_1b)
   space_count_temp(2,j) = MAKE_I2WORD(buffer(1163+10*(j-1):1164+10*(j-1)),sym%SIGNED,byte_swap_1b)
   space_count_temp(3,j) = MAKE_I2WORD(buffer(1165+10*(j-1):1166+10*(j-1)),sym%SIGNED,byte_swap_1b)
   space_count_temp(4,j) = MAKE_I2WORD(buffer(1167+10*(j-1):1168+10*(j-1)),sym%SIGNED,byte_swap_1b)
   space_count_temp(5,j) = MAKE_I2WORD(buffer(1169+10*(j-1):1170+10*(j-1)),sym%SIGNED,byte_swap_1b)
  enddo
  
!----------------------------------------------------------------------
! unpack calibration coefficients
!----------------------------------------------------------------------
  do ichan = 1, 3

    vis_slope_1(ichan,iline) = MAKE_I4WORD(buffer(49+(ichan-1)*60:52+(ichan-1)*60), &
                                     sym%SIGNED,byte_swap_1b) / (10.0**7)

    vis_intercept_1(ichan,iline) = MAKE_I4WORD(buffer(53+(ichan-1)*60:56+(ichan-1)*60), &
                                         sym%SIGNED,byte_swap_1b) / (10.0**6)

    vis_slope_2(ichan,iline) = MAKE_I4WORD(buffer(57+(ichan-1)*60:60+(ichan-1)*60), &
                                     sym%SIGNED,byte_swap_1b) / (10.0**7)

    vis_intercept_2(ichan,iline) = MAKE_I4WORD(buffer(61+(ichan-1)*60:64+(ichan-1)*60), &
                                     sym%SIGNED,byte_swap_1b) / (10.0**6)

    vis_intersection(ichan,iline) = MAKE_I4WORD(buffer(65+(ichan-1)*60:68+(ichan-1)*60), &
                                             sym%SIGNED,byte_swap_1b)

 enddo

 do ichan = 1,3
    ir_coef_1_1b(ichan+2,iline) = MAKE_I4WORD(buffer(229+(ichan-1)*24:232+(ichan-1)*24), &
                                              sym%SIGNED,byte_swap_1b)/(10.0**6)
    ir_coef_2_1b(ichan+2,iline) = MAKE_I4WORD(buffer(233+(ichan-1)*24:236+(ichan-1)*24), &
                                              sym%SIGNED,byte_swap_1b) / (10.0**6)
    ir_coef_3_1b(ichan+2,iline) = MAKE_I4WORD(buffer(237+(ichan-1)*24:240+(ichan-1)*24), &
                                              sym%SIGNED,byte_swap_1b) / (10.0**6)
 enddo

!-----------------------------------------------------------------------------
! handling NOAA-N change to scaling of ir_coef_3_1b for ch4 and ch5 
! the scaling changed from 10**6 to 10**7 - account for extra
! factor of 10 in NOAA-N format here
!------------------------------------------------------------------------------
  if (ver_1b >= 3) then
    ir_coef_3_1b(4,iline) = ir_coef_3_1b(4,iline) / 10.0
    ir_coef_3_1b(5,iline) = ir_coef_3_1b(5,iline) / 10.0
  endif

!----------------------------------------------------------------------
! unpack number of anchor points
!----------------------------------------------------------------------
!   num_loc = buffer_temp(53)
    num_loc = 51

!----------------------------------------------------------------------
! unpack angle data
!----------------------------------------------------------------------
      do j = 1, 51
        ibyte = 329 + (j-1)*6
        solzen_anchor(j,iline) = MAKE_I2WORD(buffer(ibyte:ibyte+1),sym%SIGNED,byte_swap_1b)/(10.0**2)
        satzen_anchor(j,iline) = MAKE_I2WORD(buffer(ibyte+2:ibyte+3),sym%SIGNED,byte_swap_1b)/(10.0**2)
        relaz_anchor(j,iline) = MAKE_I2WORD(buffer(ibyte+4:ibyte+5),sym%SIGNED,byte_swap_1b)/(10.0**2)
      enddo

!----------------------------------------------------------------------
! unpack earth location
!----------------------------------------------------------------------
      j = 1
      do j = 1,51
        ibyte = 641 + (j-1)*8 
        lat_anchor_1b(j,iline) = MAKE_I4WORD(buffer(ibyte:ibyte+3),sym%SIGNED,byte_swap_1b)/(10.0**4)
        lon_anchor_1b(j,iline) = MAKE_I4WORD(buffer(ibyte+4:ibyte+7),sym%SIGNED,byte_swap_1b)/(10.0**4)
      enddo

!-- make sure longitudes are correct ( ?? check this)
      where (lon_anchor_1b(:,iline) > 180)
         lon_anchor_1b(:,iline) = lon_anchor_1b(:,iline) - 360.0
      end where

!----------------------------------------------------------------------
! unpack channel  data
!----------------------------------------------------------------------
   ichan = 1
   ipix = 1

   do ibyte = 1265,1264+num_data_bytes,4

      i4word = MAKE_I4WORD(buffer(ibyte:ibyte+3),sym%UNSIGNED,byte_swap_1b)

      do j = 1,3
       if (ipix > num_pix) then
           exit
       endif

!--- old
!      chan_counts(ichan,ipix,iline) = i4word/ (1024**(3-j))
!      i4word = i4word - chan_counts(ichan,ipix,iline)*(1024**(3-j))
!--- new
       chan_counts(ichan,ipix,iline) = ishft(ishft(i4word,2+10*(j-1)),-22)

       ichan = ichan + 1
        if (ichan > num_chan) then
          ichan = 1
          ipix = ipix + 1
        endif
      enddo

      if (ipix > num_pix) then
            exit
      endif

   enddo

!-------------------------------------------------------------------------
! read in cloud mask 
!-----------------------------------------------------------------------
    ipix = 0
    word_clavr_start = 1264 + num_data_bytes + num_digtel_bytes + num_house_bytes + 1
 
!------ clavr status flag
    onebyte = buffer(word_clavr_start + 3)
    clavr_status_1b(iline) = ishft(ishft(onebyte,7),-7)

!--- cloud mask
    start_word = word_clavr_start + 8
    end_word = start_word + num_clavr_bytes

    do iword = start_word, end_word
      onebyte = buffer(iword)
      ipix = 1
      start_pixel = (iword - start_word)*4 + 1
      end_pixel = min(num_pix, start_pixel+3_int2)

      do ipix = start_pixel, end_pixel
         if (ipix > num_pix) then
           exit
         endif
         i = (ipix - start_pixel) + 1
         cld_mask_1b(ipix,iline) = ishft(ishft(onebyte,2*(i-1)),-6)
      enddo
    enddo

  end subroutine UNPACK_AVHRR_DATA_RECORD_KLM

!----------------------------------------------------------------------------------------
!  Read data records for AAPP
!----------------------------------------------------------------------------------------
 subroutine UNPACK_AVHRR_DATA_RECORD_AAPP(iline)

   integer, intent(in):: iline
   integer(kind=int2):: i2word
   integer(kind=int4):: num_chan
   integer(kind=int4):: num_clavr_bytes
   integer(kind=int4):: num_digtel_bytes
   integer(kind=int4):: num_house_bytes
   integer:: ichan
   integer:: ipix
   integer:: j
   integer:: ibyte
   integer(kind=int4):: first_data_byte
   integer(kind=int1):: onebyte

   !-------------------------------------------------------------------------------
   ! set parameters for gac or lac processing
   !-------------------------------------------------------------------------------
   if (gac == sym%YES) then
      print *, EXE_PROMPT, "ERROR: Cannot handle GAC AAPP, stopping"
      stop
   endif

   first_data_byte = 1265
   num_digtel_bytes = 14944 - 14929
   num_house_bytes = 14976 - 14971
   num_clavr_bytes = 0

   num_chan = size(chan_counts,1)

   valid_therm_cal = .true.

   !----------------------------------------------------------------
   ! unpack scan line number
   !----------------------------------------------------------------
   scan_number(iline) = make_i4word(buffer(1:2),sym%UNSIGNED,byte_swap_1b)
 
   !----------------------------------------------------------------------
   ! unpack time_temp code
   !----------------------------------------------------------------------
   scan_year(iline) = make_i2word(buffer(3:4),sym%UNSIGNED,byte_swap_1b)
   scan_day(iline) = make_i2word(buffer(5:6),sym%UNSIGNED,byte_swap_1b)
   scan_time(iline) = make_i4word(buffer(9:12),sym%UNSIGNED,byte_swap_1b)

   !----------------------------------------------------------------------
   ! unpack quality indicators - not done fully
   !----------------------------------------------------------------------
   i2word  = buffer(25)
   if (i2word < 0) then
      i2word = i2word + 256
   end if
   fatal(iline) = ishft(i2word, -7)

   !--- ch3a/b flag - note difference in position nesdis (due to byte swapping)
   i2word  = buffer(14)
   if (i2word < 0) then
      i2word = i2word + 256
   end if
   ascend(iline) = ishft(i2word, -7)

   !--- ch3a/b flag - note difference in position and meaning from nesdis
   i2word  = buffer(13)
   if (i2word < 0) then
      i2word = i2word + 256
   end if
   onebyte = ishft(ishft(i2word, 6),-6)

   ch3a_on(iline) = onebyte
   if (onebyte == 0) then 
      ch3a_on(iline) = 1
   endif
   if (onebyte == 1) then 
      ch3a_on(iline) = 0
   endif
   ch3a_on_pixel_mask(:,iline) = ch3a_on(iline)

   i2word  = buffer(28)
   if (i2word < 0) then
      i2word = i2word + 256
   end if
   resync(iline) = ishft(ishft(i2word, 6),-7)

   !---------------------------------------------------------------------
   ! unpack space view
   !---------------------------------------------------------------------
   do j = 1,10
    space_count_temp(1,j) = make_i2word(buffer(1161+10*(j-1):1162+10*(j-1)),sym%SIGNED,byte_swap_1b)
    space_count_temp(2,j) = make_i2word(buffer(1163+10*(j-1):1164+10*(j-1)),sym%SIGNED,byte_swap_1b)
    space_count_temp(3,j) = make_i2word(buffer(1165+10*(j-1):1166+10*(j-1)),sym%SIGNED,byte_swap_1b)
    space_count_temp(4,j) = make_i2word(buffer(1167+10*(j-1):1168+10*(j-1)),sym%SIGNED,byte_swap_1b)
    space_count_temp(5,j) = make_i2word(buffer(1169+10*(j-1):1170+10*(j-1)),sym%SIGNED,byte_swap_1b)
  enddo

!----------------------------------------------------------------------
! unpack calibration coefficients
!----------------------------------------------------------------------
!--- note switch in scale factors from nesdis
!--- in AAPP, the operational and test coefficients are always zero, so
!--- here we read in the prelaunch

  do ichan = 1, 3
    vis_slope_1(ichan,iline) = make_i4word(buffer(89+(ichan-1)*60:92+(ichan-1)*60), &
                                           sym%SIGNED,byte_swap_1b) / (10.0**10)
    vis_intercept_1(ichan,iline) = make_i4word(buffer(93+(ichan-1)*60:96+(ichan-1)*60), & 
                                               sym%SIGNED,byte_swap_1b) / (10.0**7)
    vis_slope_2(ichan,iline) = make_i4word(buffer(97+(ichan-1)*60:100+(ichan-1)*60), &
                                           sym%SIGNED,byte_swap_1b) / (10.0**10)
    vis_intercept_2(ichan,iline) = make_i4word(buffer(101+(ichan-1)*60:104+(ichan-1)*60), &
                                               sym%SIGNED,byte_swap_1b) / (10.0**7)
    vis_intersection(ichan,iline) = make_i4word(buffer(105+(ichan-1)*60:108+(ichan-1)*60), &
                                                sym%SIGNED,byte_swap_1b)
 enddo

 !--- note switch from nesdis in the order
 do ichan = 1,3
    ir_coef_3_1b(ichan+2,iline) = make_i4word(buffer(229+(ichan-1)*24: &
                          232+(ichan-1)*24),sym%SIGNED,byte_swap_1b)/(10.0**9)
    ir_coef_2_1b(ichan+2,iline) = make_i4word(buffer(233+(ichan-1)*24: &
                          236+(ichan-1)*24),sym%SIGNED,byte_swap_1b) / (10.0**6)
    ir_coef_1_1b(ichan+2,iline) = make_i4word(buffer(237+(ichan-1)*24: &
                          240+(ichan-1)*24),sym%SIGNED,byte_swap_1b) / (10.0**6)
 enddo

 !----------------------------------------------------------------------
 ! unpack number of anchor points
 !----------------------------------------------------------------------
 !   num_loc = buffer(53)
     num_loc = 51

 !----------------------------------------------------------------------
 ! unpack angle data
 !----------------------------------------------------------------------
      do j = 1, 51
        ibyte = 329 + (j-1)*6
        solzen_anchor(j,iline) = make_i2word(buffer(ibyte:ibyte+1),sym%SIGNED,byte_swap_1b)/(10.0**2)
        satzen_anchor(j,iline) = make_i2word(buffer(ibyte+2:ibyte+3),sym%SIGNED,byte_swap_1b)/(10.0**2)
        relaz_anchor(j,iline) = make_i2word(buffer(ibyte+4:ibyte+5),sym%SIGNED,byte_swap_1b)/(10.0**2)
      enddo

 !----------------------------------------------------------------------
 ! unpack earth location
 !----------------------------------------------------------------------
      j = 1
      do j = 1,51
        ibyte = 641 + (j-1)*8
        lat_anchor_1b(j,iline) = make_i4word(buffer(ibyte:ibyte+3),sym%SIGNED,byte_swap_1b)/(10.0**4)
        lon_anchor_1b(j,iline) = make_i4word(buffer(ibyte+4:ibyte+7),sym%SIGNED,byte_swap_1b)/(10.0**4)
      enddo

 !----------------------------------------------------------------------
 ! unpack channel  data
 !----------------------------------------------------------------------

 !--- AAPP has each observation stored as two byte integers - not packed
   first_data_byte = 1265

   do ipix = 1, 2048
     do ichan = 1,5
       ibyte = first_data_byte + (ipix-1)*10 + (ichan-1)*2
       chan_counts(ichan,ipix,iline) = make_i2word(buffer(ibyte:ibyte+1),sym%UNSIGNED,byte_swap_1b) 
     enddo
   enddo

  end subroutine UNPACK_AVHRR_DATA_RECORD_AAPP

!---------------------------------------------------------------------
! COMPUTE THE GLINT ZENITH AND THE SCATTERING ANGLE
!
! input
!  solzen - vector of solar zenith angle (degrees)
!  satzen - vector of satellite view zenith angle (degrees)
!  relaz - vector of relative azimuth angle (degrees)
!
! output
!  glintzen - vector of glint zenith angle (degrees)
!  scatangle - vector of glint zenith angle (degrees)
!
! revision history:
!    january 2003 - coded by A. Heidinger
!---------------------------------------------------------------------
  subroutine COMPUTE_GLINTZEN_SCATANGLE(solzen,satzen,relaz,glintzen,scatangle)
   real(kind=real4), dimension(:), intent(in):: solzen
   real(kind=real4), dimension(:), intent(in):: satzen
   real(kind=real4), dimension(:), intent(in):: relaz
   real(kind=real4), dimension(:), intent(out):: glintzen
   real(kind=real4), dimension(:), intent(out):: scatangle

!--- compute the cosine of the glint zenith angle
   glintzen = cos(solzen*dtor)*cos(satzen*dtor) +    &
              sin(solzen*dtor)*sin(satzen*dtor)*cos(relaz*dtor)

!--- compute the cosine of the scattering angle
   scatangle = -1.0 * (cos(solzen*dtor)*cos(satzen*dtor) -    &
                      sin(solzen*dtor)*sin(satzen*dtor)*cos(relaz*dtor))

!--- constrain it to fall within expected bounds
   glintzen = max(-1.0,min(glintzen,1.0))
   scatangle = max(-1.0,min(scatangle,1.0))

!--- convert to angle (degrees)
   glintzen = acos(glintzen) / dtor
   scatangle = acos(scatangle) / dtor

  end subroutine COMPUTE_GLINTZEN_SCATANGLE


!------------------------------------------------------------------
! Read Parameters from AVHRR INPUT files and check them for errors
!
! parameters are pased in avhrr_pixel_common public memory
!------------------------------------------------------------------
  subroutine READ_CLAVRXORB_DEFAULT_OPTIONS(file_default)
    character(len=*), intent(in):: file_default
    integer::ios0
    integer::erstat
    integer:: default_lun
    
    print *, EXE_PROMPT, "DEFAULT FILE READ IN"
    print *, EXE_PROMPT, "Default file to be read in: ", trim(file_default)

    default_lun = GET_LUN()

    open(unit=default_lun,file=trim(file_default), &
          iostat = ios0, &
          action="read", &
          status="old", &
          position="rewind")

    erstat = 0
    if (ios0 /= 0) then    !avhrr_input_check
      erstat = 1
      print *, EXE_PROMPT, "error opening AVHRR default control file, ios0 = ", ios0
      stop 1
   else
      read(unit=default_lun,fmt=*) ref_cal_1b
      read(unit=default_lun,fmt=*) therm_cal_1b
      read(unit=default_lun,fmt=*) bx_file_flag
      read(unit=default_lun,fmt=*) nav_flag
      read(unit=default_lun,fmt=*) nav_file_flag
      read(unit=default_lun,fmt=*) cmr_file_flag
      read(unit=default_lun,fmt=*) obs_file_flag
      read(unit=default_lun,fmt=*) geo_file_flag
      read(unit=default_lun,fmt=*) cld_file_flag
      read(unit=default_lun,fmt=*) sst_file_flag
      read(unit=default_lun,fmt=*) rtm_file_flag
      read(unit=default_lun,fmt=*) ash_file_flag
      read(unit=default_lun,fmt=*) level2_file_flag
      read(unit=default_lun,fmt=*) level3_flag
      read(unit=default_lun,fmt=*) cloud_mask_1b
      read(unit=default_lun,fmt=*) cloud_mask_bayesian_flag
      read(unit=default_lun,fmt=*) sst_flag
      read(unit=default_lun,fmt=*) cld_flag
      read(unit=default_lun,fmt=*) aer_flag
      read(unit=default_lun,fmt=*) erb_flag
      read(unit=default_lun,fmt=*) ash_flag
      read(unit=default_lun,fmt=*) use_sst_anal_default
      read(unit=default_lun,fmt=*) sst_anal_opt
      read(unit=default_lun,fmt=*) dlat
      read(unit=default_lun,fmt=*) level3_format
      read(unit=default_lun,fmt=*) data_comp_flag
      read(unit=default_lun,fmt=*) subset_pixel_hdf_flag
      read(unit=default_lun,fmt=*) nwp_flag
      read(unit=default_lun,fmt=*) rtm_flag
      read(unit=default_lun,fmt=*) modis_clr_alb_flag
      read(unit=default_lun,fmt=*) prob_clear_res_flag
      read(unit=default_lun,fmt=*) lrc_flag
      read(unit=default_lun,fmt=*) process_undetected_cloud_flag
      read(unit=default_lun,fmt=*) diag_flag
      read(unit=default_lun,fmt=*) asc_flag_diag
      read(unit=default_lun,fmt=*) lat_min_diag
      read(unit=default_lun,fmt=*) lat_max_diag
      read(unit=default_lun,fmt="(a)") ancil_data_dir
      read(unit=default_lun,fmt="(a)") gfs_data_dir
      read(unit=default_lun,fmt="(a)") ncep_data_dir
      read(unit=default_lun,fmt="(a)") oisst_data_dir
      read(unit=default_lun,fmt="(a)") snow_data_dir
      read(unit=default_lun,fmt=*) smooth_nwp
      read(unit=default_lun,fmt=*) use_seebor
      read(unit=default_lun,fmt=*) read_hires_sfc_type
      read(unit=default_lun,fmt=*) read_land_mask
      read(unit=default_lun,fmt=*) read_coast_mask
      read(unit=default_lun,fmt=*) read_surface_elevation
      read(unit=default_lun,fmt=*) read_volcano_mask
      read(unit=default_lun,fmt=*) solzen_min_limit, solzen_max_limit
      read(unit=default_lun,fmt=*) read_snow_mask
    endif

    close(unit=default_lun)

 end subroutine READ_CLAVRXORB_DEFAULT_OPTIONS
 
 subroutine READ_CLAVRXORB_COMMANDLINE_OPTIONS()
!------------------------------------------------------------------
! Command-line input option variables 
!------------------------------------------------------------------  
  
  character(len=30) :: fargv
  character(len=30) :: junk
  character(len=1) :: temp_string
  logical:: back
  character(len=300):: default_temp
  real :: int_temp
  integer :: fargc
  integer :: i
  integer :: Temp_Scans_Arg !--- temporary integer for number of scanlines
  integer*4 :: iargc

  temp_string = '.'  !--- tempoary string to search for in angle commandline

  !---- SET DEFAULT OPTIONS
  
  use_default = sym%YES
  default_temp="./clavrxorb_default_options"
  file_list = "./clavrxorb_file_list"
  
  !--- set yes/no options to no as default
  ref_cal_1b = sym%NO
  therm_cal_1b = sym%NO
  bx_file_flag = sym%NO
  nav_flag = 0
  prob_clear_res_flag = 0
  lrc_flag = sym%NO
  process_undetected_cloud_flag = sym%NO
  nav_file_flag = sym%NO 
  cmr_file_flag = sym%NO
  cld_file_flag = sym%NO
  sst_file_flag = sym%NO
  obs_file_flag = sym%NO
  geo_file_flag = sym%NO
  rtm_file_flag = sym%NO
  ash_file_flag = sym%NO
  level2_file_flag = sym%NO
  level3_flag = sym%NO
  sst_flag = sym%NO
  cld_flag = sym%NO
  aer_flag = sym%NO
  erb_flag = sym%NO
  ash_flag = sym%NO
  data_comp_flag = 0
  diag_flag = 0
  subset_pixel_hdf_flag = 0
  use_seebor = 0
  smooth_nwp = 0
  read_hires_sfc_type = 0
  read_land_mask = 0
  read_coast_mask = 0
  read_surface_elevation = 0
  read_volcano_mask = 0
  cloud_mask_1b = 0
  cloud_mask_bayesian_flag = 0
  cloud_type_ir_flag = 0
  use_sst_anal_default = 0 ! Do not use OISST analisys
  sst_anal_opt = 0 !determine OISST file
  modis_clr_alb_flag = 0 ! do not use clear-sky MODIS albedo maps
  read_snow_mask = 0
  output_scaled_reflectances = sym%NO !default is to output ref / cossolzen
  ncdc_level2_flag = sym%NO           !default is to output full level2
  num_scans_per_segment = 200
  Temp_Scans_Arg = 0
 
  
  !--- Default is equal angle @ 0.5 degree resolution
  level3_format = 1
  dlat = 0.5
  
  !--- default solar zenith limits
  solzen_min_limit= 0 
  solzen_max_limit= 180.0
  
  !--- default latitude limits
  lat_min_diag = -90.0
  lat_max_diag =  90.0

  !--- default NWP and RTM options
  nwp_flag = 1     !--- use GFS
  rtm_flag = 1     !--- use PFAST
  
  !--- default ancillary data directory
  ancil_data_dir = './data/'
  gfs_data_dir = './data/gfs_archive/'
  ncep_data_dir = './data/ncep-reanalysis/'
  oisst_data_dir = './data/oisst_archive/'
  snow_data_dir = './data/snow/'
 
  fargc = iargc()
  
  
  !--- first we will check to see if the default file is used
  !--- also check to see if the help file is to be displayed
  
  do i=1, fargc
        call getarg(i,fargv)
        if (trim(fargv) == "-help" .or. &
             trim(fargv) == "-h") then
          call HELPER()
          stop
        !No default file used
        elseif (trim(fargv) == "-no_default") then 
          use_default = sym%NO
        !Different default file used
        elseif (trim(fargv) == "-default") then
          call getarg(i+1,default_temp)
          default_temp=trim(default_temp)
       endif
  enddo
  
  
  !---- If the default file is used, read it in first, then
  !---- check for other command line changes to the options
  

  if(use_default == sym%YES)  then
     call READ_CLAVRXORB_DEFAULT_OPTIONS(default_temp)
  endif
  if(use_default == sym%NO)  then
          print *, EXE_PROMPT, "Using standard defaults and command line options"
  endif 
 
  do i=1, fargc
        call getarg(i,fargv)
    
        !Change ref_cal_1b flag 
        if (trim(fargv) == "-ref_cal_1b") then 
          ref_cal_1b = sym%YES
        elseif (trim(fargv) == "-no_ref_cal_1b") then 
          ref_cal_1b = sym%NO
        
        !Change therm_cal_1b flag
        elseif(trim(fargv) == "-therm_cal_1b") then
          therm_cal_1b = sym%YES
        elseif(trim(fargv) == "-no_therm_cal_1b") then
          therm_cal_1b = sym%NO
        
        !Change bx_file_flag
        elseif(trim(fargv) == "-bx_file_flag") then
          bx_file_flag = sym%YES
        elseif(trim(fargv) == "-no_bx_file_flag") then
          bx_file_flag = sym%NO

        !Change Nav type
        elseif(trim(fargv) == "-l1bnav") then
          nav_flag = 0
          
        !Change Nav output flag
        elseif(trim(fargv) == "-nav_file") then
          nav_file_flag = sym%YES
        elseif(trim(fargv) == "-no_nav_file") then
          nav_file_flag = sym%NO

        !Change cmr output flag
        elseif(trim(fargv) == "-cmr_file") then
          cmr_file_flag = sym%YES
        elseif(trim(fargv) == "-no_cmr_file") then
          cmr_file_flag = sym%NO

        !Change obs output flag
        elseif(trim(fargv) == "-obs_file") then
          obs_file_flag = sym%YES
        elseif(trim(fargv) == "-no_obs_file") then
          obs_file_flag = sym%NO

        !Change geo output flag
        elseif(trim(fargv) == "-geo_file") then
          geo_file_flag = sym%YES
        elseif(trim(fargv) == "-no_geo_file") then
          geo_file_flag = sym%NO

        !Change cld output flag
        elseif(trim(fargv) == "-cld_file") then
          cld_file_flag = sym%YES 
        elseif(trim(fargv) == "-no_cld_file") then
          cld_file_flag = sym%NO

        !Change SST output flag
        elseif(trim(fargv) == "-sst_file") then
          sst_file_flag = sym%YES 
        elseif(trim(fargv) == "-no_sst_file") then
          sst_file_flag = sym%NO

        !Change RTM output flag
        elseif(trim(fargv) == "-rtm_file") then
          rtm_file_flag = sym%YES
        elseif(trim(fargv) == "-no_rtm_file") then
          rtm_file_flag = sym%NO

        !Change ash output flag
        elseif(trim(fargv) == "-ash_file") then
          ash_file_flag = sym%YES
        elseif(trim(fargv) == "-no_ash_file") then
          ash_file_flag = sym%NO

        !Change PATMOSX output flag
        elseif(trim(fargv) == "-level2_file") then
          level2_file_flag = sym%YES
        elseif(trim(fargv) == "-no_level2_file") then
          level2_file_flag = sym%NO

        !Change level3 output flag
        elseif(trim(fargv) == "-level3_file") then
          level3_flag = sym%YES
        elseif(trim(fargv) == "-no_level3_file") then
          level3_flag = sym%NO

        !Change cloud mask
        elseif(trim(fargv) == "-cloud_mask_1b") then
          cloud_mask_1b = sym%YES
        elseif(trim(fargv) == "-cloud_mask_calc") then
          cloud_mask_1b = sym%NO

        !Change cloud mask bayesian otion
        elseif(trim(fargv) == "-cloud_mask_bayesian") then
          cloud_mask_bayesian_flag = sym%YES
        elseif(trim(fargv) == "-no_cloud_mask_bayesian") then
          cloud_mask_bayesian_flag = sym%NO

        !Change sst flag
        elseif(trim(fargv) == "-sst_flag") then
          sst_flag = sym%YES
        elseif(trim(fargv) == "-no_sst_flag") then
          sst_flag = sym%NO

        !Change cld flag
        elseif(trim(fargv) == "-cld_flag") then
          cld_flag =  sym%YES
        elseif(trim(fargv) == "-no_cld_flag") then
          cld_flag =  sym%NO

        !Change aerosol flag
        elseif(trim(fargv) == "-aer_flag") then
          aer_flag = sym%YES
        elseif(trim(fargv) == "-no_aer_flag") then
          aer_flag = sym%NO

        !Change erb flag
        elseif(trim(fargv) == "-erb_flag") then
          erb_flag = sym%YES
        elseif(trim(fargv) == "-no_erb_flag") then
          erb_flag = sym%NO

        !Change ash flag
        elseif(trim(fargv) == "-ash_flag") then
          ash_flag = sym%YES
        elseif(trim(fargv) == "-no_ash_flag") then
          ash_flag = sym%NO

        !Use or not use OISST analysis
        elseif(trim(fargv) == "-use_sst_anal") then
          use_sst_anal_default = sym%YES
        elseif(trim(fargv) == "-no_use_sst_anal") then
          use_sst_anal_default = sym%NO

        !Determin file or use oisst.current
        elseif(trim(fargv) == "-use_oisst_cur") then
          sst_anal_opt = sym%YES
        elseif(trim(fargv) == "-determine_oisst_file") then
          sst_anal_opt = sym%NO

        !Change grid resolution
        elseif(trim(fargv) == "-grid_res") then
          call getarg(i+1,junk)
          int_temp = scan(junk,temp_string, back) ! check for decimal point
          if(int_temp > 0.0) read(junk,'(f6.3)') dlat
          if(int_temp == 0.0) read(junk,'(f6.0)') dlat

        !Grid format (equal angle or area)
        elseif(trim(fargv) == "-equal_area") then
               level3_format = 0
        elseif(trim(fargv) == "-equal_angle") then
               level3_format = 1

        !Change data compression flag
        elseif(trim(fargv) == "-no_data_comp") then
          data_comp_flag=0
        elseif(trim(fargv) == "-data_comp_gzip") then
          data_comp_flag=1
        elseif(trim(fargv) == "-data_comp_szip") then
          data_comp_flag=2

        !Subset pixel HDF
        elseif(trim(fargv) == "-subset_pixel_hdf") then
          subset_pixel_hdf_flag = sym%YES
        elseif(trim(fargv) == "-no_subset_pixel_hdf") then
          subset_pixel_hdf_flag = sym%NO

        !Change NWP Flag
        elseif(trim(fargv) == "-no_nwp") then
          nwp_flag = 0
        elseif(trim(fargv) == "-gfs_nwp") then
          nwp_flag = 1
        elseif(trim(fargv) == "-ncep_nwp") then
          nwp_flag = 2

        !Change RTM
        elseif(trim(fargv) == "-crtm") then
          print *,  EXE_PROMPT, "Only PFAST RTM supported, stopping"
          stop
        elseif(trim(fargv) == "-pfast") then
          rtm_flag = 1

        !Use/don't use MODIS clear reflectance
        elseif(trim(fargv) == "-use_modis_clr_alb") then
          modis_clr_alb_flag = sym%YES
        elseif(trim(fargv) == "-no_modis_clr_alb") then
          modis_clr_alb_flag = sym%NO

        !Change probably clear restoral flag
        elseif(trim(fargv) == "-prob_clear_res") then
          prob_clear_res_flag = sym%YES
        elseif(trim(fargv) == "-no_prob_clear_res") then
          prob_clear_res_flag = sym%NO

        !Change lrc flag
        elseif(trim(fargv) == "-lrc") then
          lrc_flag = sym%YES 
        elseif(trim(fargv) == "-no_lrc") then
          lrc_flag = sym%NO

        !Change crc flag
        elseif(trim(fargv) == "-process_undetected") then
          process_undetected_cloud_flag = sym%YES 
        elseif(trim(fargv) == "-no_process_undetected") then
          process_undetected_cloud_flag = sym%NO

        !Diagnostic output flag
        elseif(trim(fargv) == "-diag") then
          diag_flag = sym%YES
        elseif(trim(fargv) == "-no_diag") then
          diag_flag = sym%NO

        !Change which node will be output 
        elseif(trim(fargv) == "-asc_diag") then
          asc_flag_diag = 0
        elseif(trim(fargv) == "-des_diag") then
          asc_flag_diag = 1


        !Change lat max/min for diagnositc output
        elseif(trim(fargv) == "-lat_min_diag") then
           call getarg(i+1,junk)
           int_temp = scan(junk,temp_string, back) 
           if(int_temp > 0.0) read(junk,'(f6.3)') lat_min_diag
           if(int_temp == 0.0) read(junk,'(f6.0)') lat_min_diag
        elseif(trim(fargv) == "-lat_max_diag") then
          call getarg(i+1,junk)
          int_temp = scan(junk,temp_string, back) 
          if(int_temp > 0.0) read(junk,'(f6.3)') lat_max_diag
          if(int_temp == 0.0) read(junk,'(f6.0)') lat_max_diag

        !Change ancillary data directory
        elseif(trim(fargv) == "-ancil_data_dir") then
          call getarg(i+1,ancil_data_dir)
          ancil_data_dir=trim(ancil_data_dir)

        !Change GFS data directory
        elseif(trim(fargv) == "-gfs_data_dir") then
          call getarg(i+1,gfs_data_dir)
          gfs_data_dir=trim(gfs_data_dir)

        !Change NCEP reanalysis data directory
        elseif(trim(fargv) == "-ncep_data_dir") then
          call getarg(i+1,ncep_data_dir)
          ncep_data_dir=trim(ncep_data_dir)

        !Change OISST data directory
        elseif(trim(fargv) == "-oisst_data_dir") then
          call getarg(i+1,oisst_data_dir)
          oisst_data_dir=trim(oisst_data_dir)

        !Change SNOW data directory
        elseif(trim(fargv) == "-snow_data_dir") then
          call getarg(i+1,snow_data_dir)
          snow_data_dir=trim(snow_data_dir)

        !Smooth/not smooth NWP data
        elseif(trim(fargv) == "-smooth_nwp") then
          smooth_nwp = sym%YES
        elseif(trim(fargv) == "-no_smooth_nwp") then
          smooth_nwp = sym%NO

        !Use/not use SEEBOR emissivity
        elseif(trim(fargv) == "-use_seebor") then
          use_seebor = sym%YES
        elseif(trim(fargv) == "-no_seebor") then
          use_seebor = sym%NO

        !Change which surface type flag 
        elseif(trim(fargv) == "-high_res") then
          read_hires_sfc_type = 1
        elseif(trim(fargv) == "-low_res") then
          read_hires_sfc_type = 0

        !Read/not read land mask
        elseif(trim(fargv) == "-read_land_mask") then
          read_land_mask = sym%YES
        elseif(trim(fargv) == "-no_land_mask") then
          read_land_mask = sym%NO

        !Read/not read coast mask
        elseif(trim(fargv) == "-read_coast_mask") then
          read_coast_mask = sym%YES
        elseif(trim(fargv) == "-no_coast_mask") then
          read_coast_mask = sym%NO

        !Read/not read surface elevation
        elseif(trim(fargv) == "-no_surface_elevation") then
          read_surface_elevation = 0
        elseif(trim(fargv) == "-high_surface_elevation") then
          read_surface_elevation = 1
        elseif(trim(fargv) == "-low_surface_elevation") then
          read_surface_elevation = 2

        !Read/not read volcano mask
        elseif(trim(fargv) == "-read_volcano_mask") then
          read_volcano_mask = sym%YES
        elseif(trim(fargv) == "-no_volcano_mask") then
          read_volcano_mask = sym%NO

        !read snow mask
        elseif(trim(fargv) == "-snow") then
          read_snow_mask = sym%YES
        elseif(trim(fargv) == "-no_snow") then
          read_snow_mask = sym%NO

        !Output scaled reflectances or not
        elseif(trim(fargv) == "-output_scaled_ref") then
          output_scaled_reflectances = sym%YES
        elseif(trim(fargv) == "-no_output_scaled_ref") then
          output_scaled_reflectances = sym%NO

        ! change the number of scanlines per segment
        elseif(trim(fargv) == "-ncdc_level2") then
          ncdc_level2_flag = sym%YES

        ! change the number of scanlines per segment
        elseif(trim(fargv) == "-lines_per_seg") then
        call getarg(i+1,junk)
        read(junk,'(i4)') Temp_Scans_Arg
        if(Temp_Scans_Arg > 1) read(junk,'(i4)') num_scans_per_segment

        !Change solar zenith angle limits
        elseif(trim(fargv) == "-solzen_min_limit") then
          call getarg(i+1,junk)
          int_temp = scan(junk,temp_string, back) 
          if(int_temp > 0.0) read(junk,'(f6.3)') solzen_min_limit
          if(int_temp == 0.0) read(junk,'(f6.0)') solzen_min_limit

    elseif(trim(fargv) == "-solzen_max_limit") then
       call getarg(i+1,junk)
       int_temp = scan(junk,temp_string, back) 
       if(int_temp > 0.0) read(junk,'(f6.3)') solzen_max_limit
       if(int_temp == 0.0) read(junk,'(f6.0)') solzen_max_limit
    elseif (trim(fargv) == "-filelist") then
       call getarg(i+1,file_list)
       file_list=trim(file_list)
    endif
  enddo

 end subroutine READ_CLAVRXORB_COMMANDLINE_OPTIONS

!-------------------------------------------------------------------------------
!--- QC options and modify as needed
!-------------------------------------------------------------------------------
 subroutine QC_CLAVRXORB_OPTIONS()

      integer:: erstat

!---- Since the NWP controls everything, we first check if an NWP is being used
!----   before anything else is checked.

    if ((nwp_flag /= 0) .and. (nwp_flag /= 1) .and. (nwp_flag /= 2)) then
       print *,  EXE_PROMPT, "unrecognized value for nwp_flag: ", nwp_flag
       stop "6-nwp_flag"
    endif
    if (nwp_flag == 1) then
       print *,  EXE_PROMPT, "GFS data will be used"
    else if (nwp_flag == 2) then
       print *,  EXE_PROMPT, "NCEP Reanalysis data will be used"
    endif
    if (nwp_flag == 0) then
       print *,  EXE_PROMPT, "No choice made for NWP data, will not run algoritms or orbital level3 files"
       cld_flag = sym%NO
       aer_flag = sym%NO
       erb_flag = sym%NO
       ash_flag = sym%NO
       sst_flag = sym%NO
       cld_file_flag = sym%NO
       cmr_file_flag = sym%NO
       ash_file_flag = sym%NO
       sst_file_flag = sym%NO
       rtm_file_flag = sym%NO
       cloud_mask_bayesian_flag = sym%NO
       level3_flag = sym%NO
       cloud_mask_1b = sym%NO ! this is to determine if the lut's are being read in
    endif



      if (cloud_mask_bayesian_flag == sym%YES) then
       print *, EXE_PROMPT, "Bayesian cloud mask will be generated"
      endif

      if (cloud_type_ir_flag == sym%YES) then
        print *, EXE_PROMPT, "IR Cloud Typing is turned not available in this version, ", &
                             "cloud_type_ir_flag being turned off"
        cloud_type_ir_flag = sym%NO
      endif

      if (ref_cal_1b == sym%YES) then
       print *, EXE_PROMPT, "Reflectance Calibration within 1b will be used"
      endif

      if (therm_cal_1b == sym%YES) then
       print *, EXE_PROMPT, "Thermal Calibration within 1b will be used"
      endif

      if (nav_flag == 1) then
        print *,  EXE_PROMPT, "CLEVERNAV geolocation no longer supported, using REPOSNX"
        nav_flag = 2
      endif
      if (nav_flag == 2) then
            print *,  EXE_PROMPT, "REPOSNX geolocation adjustment done"
      endif

      if (nav_file_flag == sym%YES) then
        print *,  EXE_PROMPT, "nav file will be created"
      endif

      if (cld_file_flag == sym%YES) then
        print *,  EXE_PROMPT, "cld file will be created"
      endif

      if (cmr_file_flag == sym%YES) then
        print *, EXE_PROMPT, "cmr file will be created"
      endif

      if (sst_file_flag == sym%YES) then
        print *, EXE_PROMPT, "sst file will be created"
      endif

      if (ash_file_flag == sym%YES) then
        print *, EXE_PROMPT, "vol ash file will be created"
      endif

      if (sst_flag == sym%NO) then
        print *, EXE_PROMPT, "SST product will not be created"
      endif

      if (cld_flag == sym%NO) then
        print *, EXE_PROMPT, "Cloud products will not be created"
      endif

      if (aer_flag == sym%NO) then
        print *, EXE_PROMPT, "Aerosol products will not be created"
      endif

      if (erb_flag == sym%NO) then
        print *, EXE_PROMPT, "Radiative Flux products will not be created"
      endif
      
      if (obs_file_flag == sym%YES) then
        print *, EXE_PROMPT, "obs file will be created"
      endif

      if (geo_file_flag == sym%YES) then
        print *, EXE_PROMPT, "geo file will be created"
      endif

      if (rtm_file_flag == sym%YES) then
        print *, EXE_PROMPT, "rtm file will be created"
      endif

      if (level3_flag == sym%YES) then
        print *, EXE_PROMPT, "level3 file will be created"
      endif

    if (cloud_mask_1b == sym%YES) then
       print *,  EXE_PROMPT, "Cloud mask results will be read in from precomputed 1bx file"
    endif

      if (bx_file_flag == 1) then
         if (nav_flag == 1) then
            print *,  EXE_PROMPT, "Cannot create 1bx file and use external re-navigation"
            stop "3a"
         else
            print *, EXE_PROMPT, "1bx file will be created"
         endif
      endif


    if ((cloud_mask_1b == 1) .and. (bx_file_flag == 1)) then
       erstat = 6
       print *,  EXE_PROMPT, "Cannot read cloud mask from both 1bx and write out a 1bx file, stopping"
       stop 6
    endif

    if (rtm_flag /=1) then
       print *,  EXE_PROMPT, "Only PFAST RTM implemented, stopping"
       stop
    endif


 end subroutine QC_CLAVRXORB_OPTIONS
 
 
!--------------------------------------------------------------------------
! This subroutine outputs what each command line options
! are available to override the default file options or to set
! options different from the default options
!--------------------------------------------------------------------------
subroutine HELPER()
 
  print "(a,'help: option list')",EXE_PROMPT
  print *,"  This is a list of all of the command line options to override"
  print *,"     the file list options"
  print *," "

  print *,"  -default (default_temp)"
  print *, "  This option allows you to set which default file you want to use."
  print *, "  Initial default file is clavrx_default_options"
  print *," "

  print *,"  -filelist (file_list)"
  print *, "  This option allows you to set which clavrxorb_file_list file you want to use."
  print *, "  Initial default file is the clavrxorb_file_list in the current directory"
  print *," "

  print *,"  -lines_per_seg (num_scans_per_segment)"
  print *, "  specify the number of scanlines per segment (default is 200)"
  print *," "
 
  print *,"  -ref_cal_1b"
  print *,"  Use the reflectance cal in level 1b file."
  print *," "

  print *,"  -no_ref_cal_1b"
  print *,"  Do not us the reflectance cal in level 1b file."
  print *," "

  print *,"  -therm_cal_1b"
  print *,"   Use the thermal calibration in the level 1b file"
  print *," "

  print *,"  -no_therm_cal_1b"
  print *,"   Do not use the thermal calibration in the level 1b file"
  print *," "
  
  print *,"  -bx_file_flag"
  print *, "  Fill in clavr-x bytes in the level 1b file"
  print *," "

  print *,"  -no_bx_file_flag"
  print *, "  Do not fill in clavr-x bytes in the level 1b file"
  print *," "
  
  print *,"  -l1bnav"
  print *, "  Use the navigation data from level 1b file"
  print *," "

  print *,"  -clevernav"
  print *, "  Use the navigation data from a clevernav"
  print *," "

  print *,"  -nav_file"
  print *, "   Output a navigation file."
  print *," "
  print *,"  -no_nav_file"
  print *, "   Do not output a navigation file."
  print *," "

  print *,"  -cmr_file"
  print *, "   Output cmr file."
  print *," "
  print *,"  -no_cmr_file"
  print *, "   Do not output cmr file."
  print *," "
  
  print *,"  -obs_file"
  print *, "   Output obs file."
  print *," "
  print *,"  -no_obs_file"
  print *, "   Do not output obs file."
  print *," "
  
  print *,"  -geo_file"
  print *, "   Output geolocation file. "
  print *," "
  print *,"  -no_geo_file"
  print *, "   Output geolocation file. "
  print *," "

  print *,"  -cld_file"
  print *, "   Run and output cloud algorithms."
  print *," "
  print *,"  -no_cld_file"
  print *, "   Do not output cloud algorithms."
  print *," "

  print *,"  -sst_file"
  print *, "   Calculate and output SST data. "
  print *," "
  print *,"  -no_sst_file"
  print *, "   Do not output SST data. "
  print *," "

  print *,"  -ash_file"
  print *, "   Output VolAsh data file. "
  print *," "
  print *,"  -no_ash_file"
  print *, "   Do not output VolAsh data file. "
  print *," "

  print *,"  -rtm_file"
  print *, "   Output rtm data file. "
  print *," "
  print *,"  -no_rtm_file"
  print *, "   Do not output rtm data file. "
  print *," "

  print *,"  -level2_file"
  print *, "   Make Level-2 output."
  print *," "
  print *,"  -no_level2_file"
  print *, "   Don't make Level-2 output."
  print *," "

  print *,"  -level3_file"
  print *, "   Make level3 output."
  print *," "
  print *,"  -no_level3_file"
  print *, "   Don't make level3 output."
  print *," "
  
  print *,"  -cloud_mask_1b"
  print *, "   Read cloud mask from level 1b file."
  print *," "
  print *,"  -cloud_mask_calc"
  print *, "   recalculate cloud mask."
  print *," "
  
  print *,"  -cloud_type_ir"
  print *, "   Use ir cloud type for processing"
  print *," "
  print *,"  -no_cloud_type_ir"
  print *, "   Don't use ir cloud type for processing"
  print *," "
  
  print *,"  -sst_flag"
  print *, "   Calculate SST. "
  print *," "
  print *,"  -no_sst_flag"
  print *, "   Don't calculate SST. "
  print *," "
    
  print *,"  -cld_flag"
  print *, "   Run cloud algorithms. "
  print *," "
  print *,"  -no_cld_flag"
  print *, "   Don't run cloud algorithms. "
  print *," "
  
  print *,"  -aer_flag"
  print *, "   Run aerosol algorithms. "
  print *," "
  print *,"  -no_aer_flag"
  print *, "   Don't run aerosol algorithms. "
  print *," "
  
  print *,"  -erb_flag"
  print *, "   Run radiative flux algorithms. "
  print *," "
  print *,"  -no_erb_flag"
  print *, "   Don't run radiative flux algorithms. "
  print *," "

!  print *,"  -ash_flag"
!  print *, "   Run Vol Ash algorithms. "
!  print *," "
!  print *,"  -no_ash_flag"
!  print *, "   Don't run Vol Ash algorithms. "
!  print *," "

  print *,"  -use_sst_anal"
  print *, "   Use OISST analysis. "
  print *," "
  print *,"  -no_use_sst_anal"
  print *, "   Don't use OISST analysis. "
  print *," "
  
  print *,"  -use_oisst_cur"
  print *, "   Use the oisst.current OISST analysis."
  print *," "
  print *,"  -determine_oisst_file"
  print *, "   Determine which OISST analysis file to use."
  print *," "
  
  print *,"  -grid_res (dlat)"
  print *, "   Input the level3 resolution. "
  print *," "
  
  print *,"  -equal_area"
  print *, "   Level3 output as equal area."
  print *," "

  print *,"  -equal_angle"
  print *, "   Level3 output as equal angle. "
  print *," "

  print *,"  -no_data_comp"
  print *, "   No data compression"
  print *," "
  print *,"  -data_comp_gzip"
  print *, "   GZIP Data compression"
  print *," "
  print *,"  -data_comp_szip"
  print *, "   SZIP Data compression"
  print *," "

  print *,"  -subset_pixel_hdf"
  print *, "  Subset HDF pixel data. " 
  print *," "
  print *,"  -no_subset_pixel_hdf"
  print *, "  Don't subset HDF pixel data. " 
  print *," "
  
  print *,"  -no_nwp"
  print *, "  Do not use NWP data. No algorithms will be processed. Also, no orbital data will be processed"
  print *," "

  print *,"  -gfs_nwp"
  print *, "  Use GFS data for NWP dataset. "
  print *," "
  
  print *,"  -ncep_nwp"
  print *, "  Use NCEP Reanalysis data"
  print *," "
  
  print *,"  -crtm"
  print *, "  Use CRTM. "
  print *, "  NOT AVAILABLE CURRENTLY. ONLY PFAST AVAILABLE" 
  print *," "
  
  print *,"  -pfast"
  print *, "  Use PFAST RTM. "
  print *," "
  
  print *,"  -use_clear_composite"
  print *,"    Use this option to use clear composite refs."
  print *," "
  print *,"  -no_clear_composite"
  print *,"    Use this option to NOT use clear composite refs."
  print *," "

  print *,"  -use_modis_clr_alb"
  print *,"    Use this option to use MODIS clear reflectance data."
  print *," "
  print *,"  -no_modis_clr_alb"
  print *,"    Use this option to NOT use MODIS clear reflectance data."
  print *," "
 
  print *,"  -prob_clear_res"
  print *, "  Restore probably clear pixels. "
  print *," "
  print *,"  -no_prob_clear_res"
  print *, "  Don't restore probably clear pixels. "
  print *," "

  print *,"  -lrc"
  print *, "  Include local radiative center calcs.  "
  print *," "
  print *,"  -no_lrc"
  print *, "  Don't include local radiative center calcs.  "
  print *," "
  
  print *,"  -crc"
  print *, "  Include cloud radiative center calcs.  "
  print *," "
  print *,"  -no_crc"
  print *, "  Don't include cloud radiative center calcs.  "
  print *," "

  print *,"  -diag"
  print *, "  Output binary diagnositic files. "
  print *," "
  print *,"  -no_diag"
  print *, "  Don't output binary diagnositic files. "
  print *," "
  
  print *,"  -asc_diag"
  print *, "  Outputs ascending node in diagnositic files. "
  print *," "
  print *,"  -des_diag"
  print *, "  Outputs descending node in diagnositic files."
  print *," "
  
  print *,"  -lat_min_diag (lat_min_diag)"
  print *, "  minimum latitude for diagnostic output (degrees)"
  print *," "
  print *,"  -lat_max_diag (lat_max_diag)"
  print *, "  maximum latitude for diagnostic output (degrees)"
  print *," "

  print *,"  -ancil_data_dir (ancil_data_dir)"
  print *, "  change the location of the ancillary data directory"
  print *," "
  print *,"  -gfs_data_dir (gfs_data_dir)"
  print *, "  change the location of the gfs data directory"
  print *," "
  print *,"  -ncep_data_dir (ncep_data_dir)"
  print *, "  change the location of the gfs data directory"
  print *," "
  print *,"  -oisst_data_dir (oisst_data_dir)"
  print *, "  change the location of the oisst data directory"
  print *," "
  print *,"  -snow_data_dir (snow_data_dir)"
  print *, "  change the location of the snow data directory"
  print *," "
  
  print *,"  -smooth_nwp"
  print *, "  Smooth the NWP fields. "
  print *," "
  print *,"  -no_smooth_nwp"
  print *, "  Don't smooth the NWP fields. "
  print *," "
  
  print *,"  -use_seebor"
  print *, "  Use Seebor emissivity dataset. "
  print *," "
  print *,"  -no_seebor"
  print *, "  Don't use Seebor emissivity dataset. "
  print *," "
  
  print *,"  -high_res"
  print *, "  Use high resolution surface type map. "
  print *," "
  print *,"  -low_res"
  print *, "  Use low resolution (8km) surface type map. "
  print *," "
  
  print *,"  -read_land_mask"
  print *, "  Read land mask in. "
  print *," "
  print *,"  -no_land_mask"
  print *, "  Don't read land mask in. "
  print *," "
  
  print *,"  -read_coast_mask"
  print *, "  Read coast mask in. "
  print *," "
  print *,"  -no_coast_mask"
  print *, "  Don't read coast mask in. "
  print *," "
  
  print *,"  -no_surface_elevation"
  print *, "  Don't read surface elevation map in. "
  print *," "
  print *,"  -high_surface_elevation"
  print *, "  Use 1km res GLOBE global elevation map. "
  print *," "
  print *,"  -low_surface_elevation"
  print *, "  Use 8km res GLOBE global elevation map. "
  print *," "

  
  print *,"  -read_volcano_mask"
  print *, "  Read volcano map in. "
  print *," "
  print *,"  -no_volcano_mask"
  print *, "  Don't read volcano map in. "
  print *," "
  
  print *,"  -solzen_min_limit (solzen_min_limit)"
  print *, "  Solar zenith angle minimum limit"
  print *," "
  
  print *,"  -solzen_max_limit (solzen_max_limit)"
  print *, "  Solar zenith angle maximum limit."
  print *," "  

  end subroutine HELPER

!-------------------------------------------------------------------
!-- define a sc_id that is unique value for each satellite
!-------------------------------------------------------------------
 subroutine COMPUTE_NEW_SPACECRAFT_ID(sc_id,sc_id_new,sc_id_char)
  integer(kind=int2), intent(in):: sc_id
  integer(kind=int4), intent(out):: sc_id_new
  character(len=6), intent(out):: sc_id_char

  sc_id_new = 0
  if (klm == sym%NO) then
    if(sc_id == 1 .and. avhrr_1_flag == sym%YES) then  !TIROS-N
       sc_id_new = 5
       sc_id_char = 'tirosn'
    endif
    if(sc_id == 2 .and. avhrr_1_flag == sym%YES) then  !NOAA-6
       sc_id_new = 6
       sc_id_char = 'noaa06'
    endif
    if(sc_id == 4 .and. klm == sym%NO) then  !NOAA-7
       sc_id_new = 7
       sc_id_char = 'noaa07'
    endif
    if(sc_id == 6 .and. avhrr_1_flag == sym%YES) then  !NOAA-8
       sc_id_new = 8      
       sc_id_char = 'noaa08'
    endif
    if(sc_id == 7 .and. avhrr_1_flag == sym%NO) then  !NOAA-9
       sc_id_new = 9      
       sc_id_char = 'noaa09'
    endif
    if(sc_id == 8 .and. avhrr_1_flag == sym%YES) then  !NOAA-10
       sc_id_new = 10      
       sc_id_char = 'noaa10'
    endif
    if(sc_id == 1 .and. avhrr_1_flag == sym%NO) then  !NOAA-11
       sc_id_new = 11      
       sc_id_char = 'noaa11'
    endif
    if(sc_id == 5 .and. klm==sym%NO) then             !NOAA-12
       sc_id_new = 12     
       sc_id_char = 'noaa12'
    endif
    if(sc_id == 3 .and. klm == sym%NO) then           !NOAA-14
       sc_id_new = 14     
       sc_id_char = 'noaa14'   
    endif
  else
    if(sc_id == 4 .and. klm == sym%YES) then          !NOAA-15
       sc_id_new = 15      
       sc_id_char = 'noaa15'   
    endif
    if(sc_id == 2 .and. klm == sym%YES) then          !NOAA-16
       sc_id_new = 16      
       sc_id_char = 'noaa16'   
    endif
    if(sc_id == 6 .and. klm == sym%YES) then          !NOAA-17
       sc_id_new = 17      
       sc_id_char = 'noaa17'   
    endif
    if(sc_id == 7 .and. klm == sym%YES) then          !NOAA-18
       sc_id_new = 18      
       sc_id_char = 'noaa18'   
    endif
    if(sc_id == 8 .and. klm == sym%YES) then         !NOAA-19
       sc_id_new = 19      
       sc_id_char = 'noaa19'   
    endif
    if(sc_id == 12 .and. klm == sym%YES) then         !Metop-A
       sc_id_new =  2     
       sc_id_char = 'metopa'   
    endif
    if(sc_id == 8 .and. klm == sym%YES) then          !NOAA-19
       sc_id_new = 19      
       sc_id_char = 'noaa19'   
    endif
  endif

  end subroutine COMPUTE_NEW_SPACECRAFT_ID

!---------------------------------------------------------
! compute the ascending/descending node flag 
!---------------------------------------------------------
  subroutine CALCULATE_ASC_DES(j_min,num_scans_read)

        integer, intent(in):: j_min
        integer, intent(in):: num_scans_read
        real:: diff
        real:: diff_before
        real:: diff_after
        integer:: j

        do j = j_min+1, num_scans_read -1
        
         if (bad_scan_flag(j) == sym%YES) then

            if (j > j_min + 1) then
              ascend(j) = ascend(j-1)
            endif

            cycle ! Cycle if there is missing navigation

         endif

         diff = lat_anchor_1b(25,j) - lat_anchor_1b(25,j+1)

         ascend(j) = 1
         if (diff <= 0) ascend(j) = 0

         if (bad_scan_flag(j+1) == sym%YES .and.  &
              bad_scan_flag(j-1) == sym%NO) then
              ascend(j) = ascend(j-1)
         endif

         diff_before = lat_anchor_1b(25,j-1) - lat_anchor_1b(25,j)
         diff_after = lat_anchor_1b(25,j+1) - lat_anchor_1b(25,j+2)

         ! check to see if current one is a fluke in diff
         if (diff_after >= 0 .and. ascend(j-1) == 1 .and.  &
              diff <= 0 .and. ascend(j) == 0)  then
              ascend(j) = 1
         endif
         if (ascend(j-1) == 0 .and. diff_after <= 0 .and.  &
             diff >= 0 .and. ascend(j) == 1)  then
             ascend(j) = 0
         endif

         enddo

        ascend(j_min) = ascend(j_min+1)
        ascend(num_scans_read) = ascend(num_scans_read - 1)

  end subroutine CALCULATE_ASC_DES


!----------------------------------------------------------- ! end of module !----------------------------------------------------------- end 
end module AVHRR_CALNAV_ROUTINES
