! $Id: avhrr_pixel_routines.f90,v 1.62 2010/08/29 00:10:50 heidinger Exp $
module AVHRR_PIXEL_ROUTINES
!-------------------------------------------------------------------------
!@***h* PATMOSX/avhrr_pixel_routines.f90
! 
! NAME
!   avhrr_pixel_routines.f90
! 
! LOCATION
!   /patmosx/src/avhrr_pixel_routines.f90
! 
! PURPOSE
!   AVHRR_PIXEL_ROUTINES
! 
! DESCRIPTION
!   this module houses routines for computing some needed pixel-level arrays
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!
! EXTERNALS
! Public routines used in this module:
! READ_AVHRR_ALGO_CONSTANTS - read coefficients from AVHRR algo files
! COMPUTE_PIXEL_ARRAYS - compute some commonly used arrays
! OPERATIONAL_SST - compute an sst using the NESDIS oper. equations
! COMPUTE_TSFC - combine sst, lst and ist into a single surface
!                    temperature and derive an estimate of total prec. water
! COMPUTE_ERB - compute outgoing longwave radiation (OLR)
! ATMOS_CORR - perform atmospheric correction
! NORMALIZE_REFLECTANCES - divide reflectances by cosine solar zenith angle
! CH3B_ALB - compute the channel 3b reflectance
! COMPUTE_SPATIAL_UNIFORMITY - compute metrics of radiance and reflectance
!                              spatial uniformity
! CONVERT_COUNTS_SINGLE_GAIN - convert dual gain counts to single gain for 
!                              calibration diagnostics
! SPECTRAL_CORRECT_NDVI - apply a spectral correct to ndvi to look like NOAA14
! ASSIGN_CLEAR_SKY_QUALITY_FLAGS - assign quality flags to clear-sky products
! COMPUTE_DATA_MASK - compute data mask used in cloud mask - holds flags 
!                     telling processing path (day/land/coast/...)
! CONVERT_TIME - compute a time in hours based on millisecond time in level1b
! COMPUTE_SNOW_FIELD - based on snow information, make a snow field.
! COMPUTE_GLINT - derive a glint mask
! COMPUTE_INOUE_CLOUD_TYPE - compute cloud type based on Toshiro Inoue's scheme
! ---- constants needed in this module
!
!@*****
!------------------------------------------------------------------------
! AVHRR PIXEL ROUTINES
!
! this module houses routines for computing some needed pixel-level arrays
!
! Author: Andrew Heidinger, NOAA/NESDIS
!
! Public routines used in this module:
! READ_AVHRR_ALGO_CONSTANTS - read coefficients from AVHRR algo files
! COMPUTE_PIXEL_ARRAYS - compute some commonly used arrays
! OPERATIONAL_SST - compute an sst using the NESDIS oper. equations
! COMPUTE_TSFC - derive pixel-level surface temperature 
! COMPUTE_ERB - compute outgoing longwave radiation (OLR)
! ATMOS_CORR - perform atmospheric correction
! NORMALIZE_REFLECTANCES - divide reflectances by cosine solar zenith angle
! CH3B_ALB - compute the channel 3b reflectance
! COMPUTE_SPATIAL_UNIFORMITY - compute metrics of radiance and reflectance
!                              spatial uniformity
! CONVERT_COUNTS_SINGLE_GAIN - convert dual gain counts to single gain for 
!                              calibration diagnostics
! SPECTRAL_CORRECT_NDVI - apply a spectral correct to ndvi to look like NOAA14
! ASSIGN_CLEAR_SKY_QUALITY_FLAGS - assign quality flags to clear-sky products
! COMPUTE_DATA_MASK - compute data mask used in cloud mask - holds flags 
!                     telling processing path (day/land/coast/...)
! CONVERT_TIME - compute a time in hours based on millisecond time in level1b
! COMPUTE_SNOW_FIELD - based on snow information, make a snow field.
! COMPUTE_GLINT - derive a glint mask
! COMPUTE_INOUE_CLOUD_TYPE - compute cloud type based on Toshiro Inoue's scheme
!---- constants needed in this module
! 
!-------------------------------------------------------------------------
 use CONSTANTS
 use ALGORITHM_CONSTANTS
 use AVHRR_PIXEL_COMMON
 use NUMERICAL_ROUTINES
 use NWP_COMMON
 use PLANCK
 use CLOUD_MASK_ROUTINES
 use LAND_SFC_PROPERTIES
 use FILE_UTILITY
 use SURFACE_PROPERTIES

 implicit none
 private
 public:: READ_AVHRR_ALGO_CONSTANTS,  &
          COMPUTE_PIXEL_ARRAYS, &
          OPERATIONAL_SST,&
          COMPUTE_TSFC,  &
          COMPUTE_ERB,  &
          ATMOS_CORR,&
          NORMALIZE_REFLECTANCES,  &
          CH3B_ALB,  &
          COMPUTE_SPATIAL_UNIFORMITY, &
          CONVERT_COUNTS_SINGLE_GAIN, &
          ASSIGN_CLEAR_SKY_QUALITY_FLAGS, &
          COMPUTE_DATA_MASK, &
          CONVERT_TIME, &
          COMPUTE_SNOW_FIELD, &
          SET_LOW_QUALITY_SCANS_TO_MISSING, &
          READ_MODIS_WHITE_SKY_ALBEDO,      &
          COMPUTE_CLEAR_SKY_SCATTER,        &
          COMPUTE_DCC_MASK,                 &
          COMPUTE_MASKED_SST,               &
          COMPUTE_GLINT,                    &
          COMPUTE_INOUE_CLOUD_TYPE,         &
          COMPUTE_CLOUD_FRACTION_3x3

!---- constants needed in this module

  real, parameter, private:: stefan_boltzmann_constant = 5.670e-08  !W/m^2/K^4
  real, parameter, private:: solar_constant = 1360.0   !W/m^2

!--- default modis white sky values for snow and water
  real, parameter, private:: alb1_sfc_white_sky_snow = 90.0   
  real, parameter, private:: alb1_sfc_white_sky_water = 2.0   
  real, parameter, private:: alb3a_sfc_white_sky_snow = 12.0   
  real, parameter, private:: alb3a_sfc_white_sky_water = 2.0   

!--- 11 micron emiss at trop threshold
  real, parameter, private:: emiss_11_trop_dcc_thresh = 0.99 

!--- include cloud mask thresholds
  INCLUDE "awg_cloud_mask_tests_thresholds.inc"

 contains

!======================================================================
! Check for bad pixels
!======================================================================
subroutine SET_LOW_QUALITY_SCANS_TO_MISSING(j1,nj)
   integer, intent(in):: j1, nj
   integer:: j2, ielem,iline

!----------------------------------------------------------------------
!--- assign bad pixel mask based on scanline fatal flag
!----------------------------------------------------------------------
   j2 = j1 + nj - 1

   do iline = j1,j2+j1-1

      !--- initialize
      solar_contamination_mask(:,iline) = sym%NO
      bad_pixel_mask(:,iline) = sym%NO
      space_mask(:,iline) = sym%NO_SPACE

     !--- check for a bad scan
     if (bad_scan_flag(iline) == sym%YES) then
       bad_pixel_mask(:,iline) = sym%YES
       space_mask(:,iline) = sym%SPACE
     else

      !--- call any scan with a ridiculous pixel as bad 
      !--- this is attempt data like NOAA-16 2004 023 where
      !--- large fractions of scans are bad but not flagged as so
      if (maxval(abs(bt4(:,iline) - bt5(:,iline))) > 20.0) then
            bad_pixel_mask(:,iline) = sym%YES
            space_mask(:,iline) = sym%YES
      endif

      !--- if not a bad scan, check pixels on this scan
      do ielem = 1, num_pix

        !--- missing geolocation
        if ((lat(ielem,iline) == missing_value_real4) .or. (lon(ielem,iline) == missing_value_real4)) then
           bad_pixel_mask(ielem,iline) = sym%YES
        endif

        !--- missing observations
        if (rad4(ielem,iline) == missing_value_real4) then
           bad_pixel_mask(ielem,iline) = sym%YES
        endif

        !--- check for solar zenith angle limits
        if ((solzen(ielem,iline) < solzen_min_limit) .or. (solzen(ielem,iline) > solzen_max_limit)) then
             bad_pixel_mask(ielem,iline) = sym%YES
        endif

        !--- check for solar contamination of nighttime data
        if ((solzen(ielem,iline) > 90.0) .and. (scatangle(ielem,iline) < 60.0)) then
            if (therm_cal_1b == sym%NO) then
              if (chan_counts(1,ielem,iline) - scan_space_counts(1,iline) > 2) then 
                 solar_contamination_mask(ielem,iline) = sym%YES
              endif
           else
             if (chan_counts(1,ielem,iline) - ch1_dark_count > 2) then 
              solar_contamination_mask(ielem,iline) = sym%YES
             endif 
           endif
        endif

        !--- consider solar contaminated pixels as bad pixels
       ! if (solar_contamination_mask(ielem,iline) == sym%YES) then
       !     bad_pixel_mask(ielem,iline) = sym%YES
       ! endif

        !--- call any bad pixel as being space (for ancil data interp)
        if (bad_pixel_mask(ielem,iline) == sym%YES) then
           space_mask(ielem,iline) = sym%SPACE
        endif

      enddo

     endif

   end do


end subroutine SET_LOW_QUALITY_SCANS_TO_MISSING


!--------------------------------------------------------------------------
! CONVERT TIME
!
! compute the utc time for each scan in fractional hours and compute the
! local time for each pixel in fractional hours
!
!--------------------------------------------------------------------------
subroutine CONVERT_TIME(j1,j2)

   integer, intent(in):: j1,j2
   integer:: i,j

!--- loop over scans
   do j = j1,j1+j2-1

!--- check for a bad scan
      if (bad_scan_flag(j) == sym%YES) then
        cycle
      endif

!--- convert ms time in level1b to utc time in hours
      utc_scan_time_hours(j) = scan_time(j) / 60.0 / 60.0/ 1000.0

!--- loop over pixels
    do i = 1, num_pix

!--- check for a bad pixel
      if (bad_pixel_mask(i,j) == sym%YES) then
        cycle
      endif

!--- compute local time based on utc and longitude
      pixel_local_time_hours(i,j) = utc_scan_time_hours(j) + lon(i,j) / 15.0

!--- constrain local time to be between 0 and 24 
      if (pixel_local_time_hours(i,j) > 24.0) then 
          pixel_local_time_hours(i,j) = pixel_local_time_hours(i,j) - 24.0
      endif

      if (pixel_local_time_hours(i,j) < 0.0) then 
          pixel_local_time_hours(i,j) = pixel_local_time_hours(i,j) + 24.0
      endif

    enddo

   enddo

end subroutine CONVERT_TIME

!--------------------------------------------------------------------------
! COMPUTE_PIXEL_ARRAYS
!
! compute quantities needed for other routines, (ie cloud mask)
!
! input - none
!
! output - only into shared memory
!   seczen - secant of zenith angle
!   sst_mask - an sst used for cloud masking only
!   bt45 - brightness temperature difference between bt4 and bt5 
!   bt3b4 - brightness temperature difference between bt3b and bt4 
!   alb21 - ratio of alb2 / alb1
!   alb31 - ratio of alb3ab / alb1
!   ndvi - (alb2  - alb1)/(alb2 + alb1)
!   ndsi - (alb1 - alb3a)/(alb3a + alb1) 
!--------------------------------------------------------------------------
 subroutine COMPUTE_PIXEL_ARRAYS(j1,nj)

   integer, intent(in):: j1,nj
   integer(kind=int4):: j2,j

   j2 = j1 + nj - 1

   coszen(:,j1:j2) = cos(satzen(:,j1:j2)*dtor)

   seczen(:,j1:j2) = 1.0 / coszen(:,j1:j2)

   cossolzen(:,j1:j2) = cos(solzen(:,j1:j2)*dtor)

   airmass(:,j1:j2) = 1.0
   where(solzen(:,j1:j2) /= 0.0 .and. coszen(:,j1:j2) /= 0.0) 
    airmass(:,j1:j2) = 1.0/cossolzen(:,j1:j2) + 1.0/coszen(:,j1:j2)
   endwhere

!--- other useful arrays 

   bt45(:,j1:j2) = bt4(:,j1:j2) - bt5(:,j1:j2)

   alb21(:,j1:j2) = missing_value_real4
   where (alb1(:,j1:j2) > 0)
     alb21(:,j1:j2) = alb2(:,j1:j2) / alb1(:,j1:j2)
   endwhere

   ndvi(:,j1:j2) = missing_value_real4
   where((solzen(:,j1:j2) < 89.5) .and. (alb1(:,j1:j2)+alb2(:,j1:j2) > 0.0))
    ndvi(:,j1:j2)  = (alb2(:,j1:j2) - alb1(:,j1:j2))/(alb2(:,j1:j2) + alb1(:,j1:j2))
   endwhere

!--- SST for cloud masking - MCSST
   if (avhrr_1_flag == sym%NO) then
       sst_mask(:,j1:j2) = b1_day_mask*bt4(:,j1:j2) + b2_day_mask*(bt45(:,j1:j2)) +  &
                           b3_day_mask*(bt45(:,j1:j2))*(seczen(:,j1:j2)-1.0) + b4_day_mask + 273.15
   else
       sst_mask(:,j1:j2) = missing_value_real4
   endif
!--- ch3a/3b specific
   ndsi(:,j1:j2) = missing_value_real4
   bt3b4(:,j1:j2) = missing_value_real4
   bt3b5(:,j1:j2) = missing_value_real4
   alb31(:,j1:j2) = missing_value_real4
   do j = j1,j2
    if (ch3a_on(j) == 0) then
     bt3b4(:,j) = bt3b(:,j) - bt4(:,j)
     bt3b5(:,j) = bt3b(:,j) - bt5(:,j)
    endif
    if (ch3a_on(j) == 1) then
     where(solzen(:,j) < 89.5 .and. ndvi(:,j) /= missing_value_real4)
      ndsi(:,j) = (alb1(:,j)-alb3a(:,j))/(alb3a(:,j)+alb1(:,j))
      alb31(:,j) = alb3a(:,j)/alb1(:,j)
     endwhere
    endif
   enddo

 end subroutine COMPUTE_PIXEL_ARRAYS

!-------------------------------------------------------------------------------
!--- populate the snow array based on all available sources of snow data
!-------------------------------------------------------------------------------
 subroutine COMPUTE_SNOW_FIELD(j1,nj)

   integer, intent(in):: j1,nj
   integer(kind=int4):: i,j, j2, inwp,jnwp

   j2 = j1 + nj - 1
   snow = sym%NO_SNOW

j_loop:    do j = j1,j2
                                                                                                                                         
!--- check for bad scans
     if (bad_scan_flag(j) == sym%YES) then
       cycle
     endif
                                                                                                                                         
i_loop:    do i = 1, num_pix

      !--- save nwp indices for convenience
      inwp = i_nwp(i,j)
      jnwp = j_nwp(i,j)

      !--- if hires available, use it and ignore other sources
      if (read_snow_mask == sym%YES .and.     &
          failed_hires_snow_mask_flag == sym%NO) then

           snow(i,j) = snow_hires(i,j)

      else   !use nwp and/or sst analysis

       if (nwp_flag > 0) then
        if ((inwp > 0) .and. (jnwp > 0)) then
         if (weasd_nwp(inwp, jnwp) > 0.1) then  !this is snow depth
          snow(i,j) = sym%SNOW
         endif
         if (ice_nwp(inwp, jnwp) > 0.1) then    !this is a fraction (0-1)
          snow(i,j) = sym%SEA_ICE
         endif
        endif
       endif

       !--- no sea-ice in ncep reanalysis
       if ((nwp_flag == 2).and.(use_sst_anal == sym%NO)) then
           if ((snow(i,j) == sym%SNOW) .and. (land(i,j) /= sym%LAND)) then
              snow(i,j) = sym%SEA_ICE
           endif
       endif

       !--- check for sea-ice from sst analysis (OISST in this case)
       !--- and replace values from nw
       if ((land_mask(i,j) == sym%NO) .and.  &
          (use_sst_anal == sym%YES) .and. &
          (sst_anal_cice(i,j) > 0.10)) then     
         snow(i,j) = sym%SEA_ICE
       endif

      endif

     !-------------------------------------------------------
     ! use some basic tests to modify snow
     ! only do this when hires snow field is unavailable
     !-------------------------------------------------------

      if (read_snow_mask == sym%NO .or.     &        !hires snow check
          failed_hires_snow_mask_flag == sym%YES) then

       !-- can't be snow if warm
       if (bt4(i,j) > 277.0) then
          snow(i,j) = sym%NO_SNOW
       endif

       !--- some day-specific tests
       if (solzen(i,j) < 75.0) then  ! day check

        !--- conditions where snow is not possible
        if (alb1(i,j) < 10.0) then
           snow(i,j) = sym%NO_SNOW
        endif

       endif           !day check

      endif            !hires snow check

    !--- store in cloud test vector location
    cld_test_vector(First_Ancil_Test+7,i,j) = sym%NO
    if (snow(i,j) /= sym%NO_SNOW) then
       cld_test_vector(First_Ancil_Test+7,i,j) = sym%YES
    endif

    end do i_loop
  end do   j_loop

 end subroutine COMPUTE_SNOW_FIELD

!------------------------------------------------------------------
! Compute a surface temperature by using the observed 
! 11 micron radiance, the rtm calculations and the surface emissivity
!
! July 2009 - made AVHRR/1 compliant
!
! Author: Andrew Heidinger
! 
!------------------------------------------------------------------
 subroutine COMPUTE_TSFC(jmin,jmax)

  integer, intent(in):: jmin
  integer, intent(in):: jmax

  integer:: iline
  integer:: ielem
  integer:: xnwp
  integer:: ynwp

  real:: rad11
  real:: rad11_atm
  real:: trans11_atm
  real:: emiss_sfc11
  real:: rad11_sfc
  real:: B11_sfc

!------------------------------------------------------------------
! Loop over pixels and derive surface temp
!------------------------------------------------------------------

!--- initialize
  tsfc = missing_value_real4
  tsfc_qf = 0

  line_loop: do iline=jmin, jmax - jmin + 1
    element_loop: do ielem= 1, num_pix

!--- check for a bad pixel pixel
      if (bad_pixel_mask(ielem,iline) == sym%YES) then
        cycle
      endif

!--- aliases for visual convenience
       rad11 = rad4(ielem,iline)
       xnwp = i_nwp(ielem,iline)                    !nwp latitude cell
       ynwp = j_nwp(ielem,iline)                    !nwp longitude cell
       rad11_atm = rad_atm_ch4_rtm(ielem,iline)     !11 micron atmospheric radiance
       trans11_atm = trans_atm_ch4_rtm(ielem,iline) !11 micron atmospheric transmittance
       emiss_sfc11 = sfc_emiss_4(ielem,iline)       !11 micron surface emissivity

!--- compute the radiance coming off the surface
        rad11_sfc = (rad11 - rad11_atm) / trans11_atm

!--- adjust for surface emissivity - this is now the black body emission at Tsfc
        B11_sfc = rad11_sfc / emiss_sfc11

!--- compute to a temperature
        tsfc(ielem,iline) = planck_temp_fast(4,B11_sfc)

!--- if water and not AVHRR/1, use sst
    if ((avhrr_1_flag == sym%NO) .and. &
        (land_mask(ielem,iline) == sym%NO) .and.  &
        (snow(ielem,iline) == sym%NO_SNOW)) then   !water check

        if (sst_flag == sym%NO) then
            tsfc(ielem,iline) = sst_mask(ielem,iline)
        else
            tsfc(ielem,iline) = sst_oper(ielem,iline)
        endif
    endif   !end water check

    if (avhrr_1_flag == sym%YES) then
        sst_mask(ielem,iline) = tsfc(ielem,iline)
        sst_oper(ielem,iline) = tsfc(ielem,iline)
    endif

    end do element_loop
  end do line_loop

end subroutine COMPUTE_TSFC
!------------------------------------------------------------
! atmospheric correction
!
! note, optical depths taken from
!  Ignatov and Stowe, 2002, vol 59, JAS, pg 313-334
!  I used the MLW values
!-----------------------------------------------------------
subroutine ATMOS_CORR(sc_id_new,nwp_flag,j1,nj)

   integer, intent(in):: j1,nj
   integer, intent(in):: sc_id_new
   integer, intent(in):: nwp_flag
   real, dimension(3):: tau_aer_corr, g_aer_corr,wo_aer_corr,tau_gas
   real:: tau_total, trans_total,tau_scat_total
   integer:: i,j,k,j2
   real:: ref_ss
   real:: ref_sfc
   real:: albedo_view
   real:: albedo_sun

   j2 = j1 + nj - 1

      !---- define channel properties
      tau_aer_corr(1) = 0.00
      wo_aer_corr(1) = 0.8
      g_aer_corr(1) = 0.6
      tau_aer_corr(2) = 0.01
      wo_aer_corr(2) = 0.8
      g_aer_corr(2) = 0.6
      tau_aer_corr(3) = 0.01
      wo_aer_corr(3) = 0.8
      g_aer_corr(3) = 0.6


      !--- initialize
      alb1_sfc = missing_value_real4
      alb1_clear = missing_value_real4
      alb2_sfc = missing_value_real4
      alb2_clear = missing_value_real4
      alb3a_sfc = missing_value_real4
      alb3a_clear = missing_value_real4
      ndvi_sfc = missing_value_real4
      rsr = missing_value_real4

     !---------- loop over scan lines
line_loop:    do j = j1,j2

      !--- check for bad scans
      if (bad_scan_flag(j) == sym%YES) then
        cycle
      endif

element_loop:  do i = 1, num_pix

     !-------- before correction if day-time
     if (cossolzen(i,j) > cossolzen_min_solar) then        !day check

     !--- compute gas optical depths - use nwp if available
     !--- loop through each channel and compute terms needed for solar atmospheric correction
      do k = 1,3 

       !--- compute gas optical depths - use nwp if available
       if (nwp_flag > 0) then
         tau_h2o_corr(sc_id_new,k) = h2o_tau_coef(sc_id_new,k,1) +  &
                      h2o_tau_coef(sc_id_new,k,2)*tpw_rtm(i,j) + & 
                      h2o_tau_coef(sc_id_new,k,3)*(tpw_rtm(i,j)**2)

         tau_h2o_corr(sc_id_new,k) = max(0.0,tau_h2o_corr(sc_id_new,k))

       endif
       tau_gas(k) = tau_h2o_corr(sc_id_new,k) + tau_o3_corr(sc_id_new,k) + tau_o2_corr(sc_id_new,k) +  &
                 tau_co2_corr(sc_id_new,k) + tau_ch4_corr(sc_id_new,k)

      if (k == 1) then
           if (alb1_sfc_white_sky(i,j) /= missing_value_real4) then
                   albedo_view = alb1_sfc_white_sky(i,j) / 100.0
           else
                   albedo_view = ch1_sfc_alb_umd(sfc_type(i,j)) / 100.0
           endif
      endif
      if (k == 2) then
             albedo_view = ch2_sfc_alb_umd(sfc_type(i,j)) / 100.0
      endif
      if (k == 3) then
           if (alb3a_sfc_white_sky(i,j) /= missing_value_real4) then
                   albedo_view = alb3a_sfc_white_sky(i,j) / 100.0
           else
                   albedo_view = ch3a_sfc_alb_umd(sfc_type(i,j)) / 100.0
           endif
      endif

      !-- assume lambertian
      albedo_sun = albedo_view

      call COMPUTE_CLEAR_SKY_SCATTER(tau_aer_corr(k), &
                                     wo_aer_corr(k), &
                                     g_aer_corr(k), &
                                     tau_ray_corr(sc_id_new,k), &
                                     tau_gas(k), &
                                     scatangle(i,j), &
                                     coszen(i,j), &
                                     cossolzen(i,j), &
                                     albedo_view, &
                                     albedo_sun, &
                                     ref_ss)

      tau_total = tau_aer_corr(k) + tau_ray_corr(sc_id_new,k) + tau_gas(k)
      trans_total = exp(-tau_total*airmass(i,j))
      if (k == 1) then
         alb1_sfc(i,j) = (alb1(i,j) - ref_ss) / trans_total
         alb1_clear(i,j) = ref_ss + trans_total*100.0*albedo_view
      endif
      if (k == 2) then
         alb2_sfc(i,j) = (alb2(i,j) - ref_ss) / trans_total
         alb2_clear(i,j) = ref_ss + trans_total*100.0*albedo_view
      endif
      if (k == 3) then
         alb3a_sfc(i,j) = (alb3a(i,j) - ref_ss) / trans_total
         alb3a_clear(i,j) = ref_ss + trans_total*100.0*albedo_view
      endif

    end do

    !--- toc ndvi
    ndvi_sfc(i,j)  = (alb2_sfc(i,j) - alb1_sfc(i,j))/(alb2_sfc(i,j) + alb1_sfc(i,j))

    !--- remote sensing reflectance - for ocean applications (Smyth, Tyrrell and Tarrant, 2004 GRL, 31)
    rsr(i,j) = (alb1_sfc(i,j) - alb2_sfc(i,j)) / exp( -0.5*tau_ray_corr(sc_id_new,1) * airmass(i,j))

   endif             !day check
   
!--- thermal atmospheric correction - use rtm parameters
     if (trans_atm_ch3b_rtm(i,j) > 0.0) then
      rad3b_sfc(i,j) = max(0.0,(rad3b(i,j) - rad_atm_ch3b_rtm(i,j))/trans_atm_ch3b_rtm(i,j))
     else
      rad3b_sfc(i,j) = missing_value_real4
     endif

     if (trans_atm_ch4_rtm(i,j) > 0.0) then

      rad4_sfc(i,j) = max(0.0,(rad4(i,j) - rad_atm_ch4_rtm(i,j))/trans_atm_ch4_rtm(i,j))

      if (rad4_sfc(i,j) > 0.0) then
       bt4_sfc(i,j) = PLANCK_TEMP_FAST(4,rad4_sfc(i,j))
      else
       bt4_sfc(i,j) = missing_value_real4
      endif
     else
      rad4_sfc(i,j) = missing_value_real4
      bt4_sfc(i,j) = missing_value_real4
     endif

     if (trans_atm_ch5_rtm(i,j) > 0.0) then
       rad5_sfc(i,j) = max(0.0,(rad5(i,j) - rad_atm_ch5_rtm(i,j))/trans_atm_ch5_rtm(i,j))
     else
      rad5_sfc(i,j) = missing_value_real4
     endif

!--- channel 3b albedo correction
     if (ch3a_on(j) == 1) then
         rad3b_ems_sfc(i,j) = missing_value_real4
         ems3b_sfc(i,j) = missing_value_real4
         alb3b_sfc(i,j) = missing_value_real4
     else
       if (bt4_sfc(i,j) > 180.0) then
          rad3b_ems_sfc(i,j) = PLANCK_RAD_FAST(3,bt4_sfc(i,j))
          ems3b_sfc(i,j) = rad3b_sfc(i,j) / rad3b_ems_sfc(i,j)
       else
         rad3b_ems_sfc(i,j) = missing_value_real4
         ems3b_sfc(i,j) = missing_value_real4
       endif
       if ((solzen(i,j)<90.0).and.(rad3b_ems_sfc(i,j)>0.0).and.(rad3b_sfc(i,j)>0.0)) then
         alb3b_sfc(i,j) = 100.0*pi*(rad3b_sfc(i,j)-rad3b_ems_sfc(i,j)) /  &
                  ((solar_3b_nu*cossolzen(i,j))/(sun_earth_distance**2) - &
                  pi*rad3b_ems_sfc(i,j) )
       endif

!----- during the night, derive alb3b_sfc from ems3b_sfc
       if ((solzen(i,j)>=90.0).and.(ems3b_sfc(i,j)/=missing_value_real4)) then
         alb3b_sfc(i,j) = 100.0*(1.0-ems3b_sfc(i,j))
       endif

     endif

!--- constrain values
     if (alb3b_sfc(i,j) /= missing_value_real4) then
      alb3b_sfc(i,j) = max(-50.0,min(100.0,alb3b_sfc(i,j)))
     endif

   end do element_loop

 end do line_loop

end subroutine ATMOS_CORR

!------------------------------------------------------------------
! derive the earth radiation budget parameters 
!------------------------------------------------------------------
 subroutine COMPUTE_ERB(j1,nj)

  integer, intent(in):: j1
  integer, intent(in):: nj
  integer:: j2

  j2 = j1 + nj - 1

  !--- OLR------------------------------------------------------------------
  ! this algorithm assumes that the window channels on avhrr are best used
  ! to derive the olr component in the 8.5 to 12 micron window region.
  ! A seperate regression is used to predict the broadband olr from the
  ! window-component of the olr
  !--------------------------------------------------------------------------
   olr(:,j1:j2) = missing_value_real4
   olr_qf(:,j1:j2) = 0

   where(bt4(:,j1:j2) > 100.0 .and. bt4(:,j1:j2) < 350.0)
    !-- this makes the window eff flux temp (K) (win_olr = sigma*win_temp^4)
    olr(:,j1:j2) = win_0 + win_1*bt5(:,j1:j2) + win_2*bt5(:,j1:j2)*seczen(:,j1:j2) + &
                   win_3*(bt45(:,j1:j2))*seczen(:,j1:j2)
    !--- this make the olr temp (K)
    olr(:,j1:j2) = olr_0 + olr_1*olr(:,j1:j2) + olr_2*(bt45(:,j1:j2)) +  &
          olr_3*(bt45(:,j1:j2))*seczen(:,j1:j2) + olr_4*olr(:,j1:j2)*seczen(:,j1:j2)
    !--- this make the olr  flux (W/m^2)
    olr(:,j1:j2) = stefan_boltzmann_constant * (olr(:,j1:j2)**4)
    olr_qf(:,j1:j2) = 3
   endwhere

!--- SWAR
   swar(:,j1:j2) = missing_value_real4
   swar_qf(:,j1:j2) = 0
   where(solzen(:,j1:j2) <= 80.0 .and. alb1(:,j1:j2) > 0.0)
     swar(:,j1:j2) = ((100.0 - alb1(:,j1:j2))/100.0) * (solar_constant*cossolzen(:,j1:j2)/(sun_earth_distance**2))
     swar_qf(:,j1:j2) = 3
   endwhere

 end subroutine COMPUTE_ERB 

!======================================================================
! Normalize the reflectances by the solar zenith angle cosine
!======================================================================
 subroutine NORMALIZE_REFLECTANCES(sun_earth_distance,j1,nj)
  real(kind=real4), intent(in):: sun_earth_distance
  integer, intent(in):: j1
  integer, intent(in):: nj
  integer:: i,j,j2

  j2 = j1 + nj - 1

  do j = j1,j2

!--- check for bad scans
      if (bad_scan_flag(j) == sym%YES) then
        alb1(:,j) = missing_value_real4
        alb2(:,j) = missing_value_real4
        alb3a(:,j) = missing_value_real4
        cycle
      endif

     do i = 1, num_pix

      if (cossolzen(i,j) > 0.0) then
       alb1(i,j) = (alb1(i,j) / cossolzen(i,j)) * (sun_earth_distance**2)
       alb2(i,j) = (alb2(i,j) / cossolzen(i,j)) * (sun_earth_distance**2)
       alb3a(i,j) = (alb3a(i,j)/cossolzen(i,j)) * (sun_earth_distance**2)
      else
       alb1(i,j) = missing_value_real4
       alb2(i,j) = missing_value_real4
       alb3a(i,j) = missing_value_real4
      endif

     end do

     !--- set alb3a to missing for ch3a = off 
     if (ch3a_on(j) == sym%NO) then
       alb3a(:,j) = missing_value_real4
     endif

   end do

end subroutine NORMALIZE_REFLECTANCES
!----------------------------------------------------------------------
! Compute Channel3b albedo
!
! input
!    ch3a_on - 0 if ch3b is on, 1 if ch3a is on
!    sun_earth_distance - sun-earth distance factor
!    rad3b - vector of ch3b radiance (mW/m^2/str/cm^1)
!    bt4 - vector of ch4 brightness temp. (K)
!    bt5 - vector of ch5 brightness temp. (K)
!    solzen - vector of solar zenith angles (degrees)
!
!  output (passed through shared memory)
!    alb3b - ch3b albedo computed using standard ch4 based method
!    ems3b - ch3b emissivity computed using standrad ch3 based method
!
!  internal
!    rad3b_ems - ch3b emission radiance (mW/m^2/str/cm^-1)
!
! Note, alb3b_sfc is computed in ATMOS_CORR
!
! Revision History
!   January 2003 - A. Heidinger
!-----------------------------------------------------------------------
subroutine CH3B_ALB(sun_earth_distance,j1,j2)
  real(kind=real4), intent(in):: sun_earth_distance
  integer, intent(in):: j1, j2
  integer:: i,j

  do j = j1,j1+j2-1

!--- check for bad scans
      if (bad_scan_flag(j) == sym%YES) then
        cycle
      endif

!---- compute channel 3b albedo
    if (ch3a_on(j) == 0) then   !start ch3a_on check

!----------------------------------------------------------------------------
!--- standard alb3b computation
!---------------------------------------------------------------------------
      do i = 1, num_pix

        if (bt4(i,j) > 180.0) then
           rad3b_ems(i,j) = PLANCK_RAD_FAST(3,bt4(i,j))
           ems3b(i,j) = rad3b(i,j) / rad3b_ems(i,j)
        else
           rad3b_ems(i,j) = missing_value_real4
           ems3b(i,j) = missing_value_real4
        endif

        if ((solzen(i,j)<90.0).and.(rad3b_ems(i,j)>0.0).and.(rad3b(i,j)>0.0)) then
           alb3b(i,j) = 100.0*pi*(rad3b(i,j)-rad3b_ems(i,j)) /  &
                        ((solar_3b_nu*cossolzen(i,j))/(sun_earth_distance**2) - &
                          pi*rad3b_ems(i,j) )
        endif

!---- at night, compute albedo as 1 - emissivity
        if ((solzen(i,j)>=90.0).and.(ems3b(i,j)/=missing_value_real4)) then
          alb3b(i,j) = 100.0*(1.0-ems3b(i,j))
        endif

!--- constrain values
           if (alb3b(i,j) /= missing_value_real4) then
               alb3b(i,j) = max(-50.0,min(100.0,alb3b(i,j)))
           endif
!--- compute ratio of alb3b to alb1
           if ((alb3b(i,j) /= missing_value_real4) .and.  &
               (alb1(i,j) /= missing_value_real4) .and. &
               (alb1(i,j) /= 0.0 )) then

               alb31(i,j) = alb3b(i,j) / alb1(i,j)

           else 

               alb31(i,j) = missing_value_real4

           endif

      enddo

     endif   !--end of ch3a_on check

   end do

!--- constrain values

end subroutine CH3B_ALB

!---------------------------------------------------------------
! routine to compute SST using operational nesdis equations
!
! output: 
! sst_oper (sst in K)
!
! input: 
!  seczen - secant of viewing angle
!  sst_mask - sst from mcsst used for cloud mask
!  sst_clim - climatological sst
!  ch3a_on - ch3a or ch3b flag
!  bt3b - channel 3b brightness temperature
!  bt4 - channel 3b brightness temperature
!  bt5 - channel 3b brightness temperature
!---------------------------------------------------------------
 subroutine OPERATIONAL_SST(j1,j2)
  integer, intent(in):: j1,j2
  integer:: i, j
  real:: sst_back

  sst_oper(:,j1:j2) = missing_value_real4

scan_loop:  do j = j1,j1+j2-1

!--- check for bad scans
      if (bad_scan_flag(j) == sym%YES) then
        cycle
      endif

pixel_loop:  do i = 1, num_pix

!--- choose background sst for nlsst
     if (sst_anal(i,j) > 265.0) then
       sst_back = sst_anal(i,j)
     else
       sst_back = sst_mask(i,j)
     endif

!--- apply equations
    if (ch3a_on(j) == 0) then
     if (solzen(i,j) < 90.0) then
      sst_oper(i,j) = a1_day*bt4(i,j) + a2_day*(sst_back-273.15)*bt45(i,j) +  &
               a3_day*bt45(i,j)*(seczen(i,j)-1.0) + a4_day + 273.15
     else
      sst_oper(i,j) = a1_night*bt4(i,j) + a2_night*bt3b(i,j) + a3_night*bt5(i,j) + &
               a4_night*(bt3b(i,j)-bt5(i,j))*(seczen(i,j)-1.0)  + &
               a5_night*(seczen(i,j)-1.0) + a6_night + 273.15
     endif
    else
      sst_oper(i,j) = a1_day*bt4(i,j) + a2_day*(sst_back-273.15)*bt45(i,j) +  &
               a3_day*bt45(i,j)*(seczen(i,j)-1.0) + a4_day + 273.15
    endif

   enddo pixel_loop

  enddo scan_loop

end subroutine OPERATIONAL_SST
!----------------------------------------------------------------
! read the avhrr algorithm constants into memory
!-----------------------------------------------------------------
subroutine READ_AVHRR_ALGO_CONSTANTS(algo_const_file)
 character(len=*), intent(in):: algo_const_file
 integer:: ios
 integer:: algo_lun

 algo_lun = GET_LUN()

 open(unit=algo_lun,file=trim(algo_const_file),status="old",position="rewind",action="read",iostat = ios)

  if (ios /= 0) then
    print *, "Error opening algorithm constant file, iostat = ", ios
    stop
  endif

  !nlsst
  read(unit=algo_lun,fmt=*) a1_day,a2_day,a3_day,a4_day

  !triple sst
  read(unit=algo_lun,fmt=*) a1_night,a2_night,a3_night,a4_night,a5_night,a6_night

  !-- olr
  read(unit=algo_lun,fmt=*) win_0,win_1,win_2,win_3
  read(unit=algo_lun,fmt=*) olr_0,olr_1,olr_2,olr_3,olr_4

  close(unit=algo_lun)

end subroutine READ_AVHRR_ALGO_CONSTANTS

!-----------------------------------------------------------
! this converts the dual gain reflectance counts to single
! gain for the klm sensors
!-----------------------------------------------------------
subroutine CONVERT_COUNTS_SINGLE_GAIN(klm,j1,j2) 
 integer, intent(in):: j1,j2
 integer, intent(in):: klm
 integer:: i, j

if (klm == 0) then
  chan_counts_sg(:,:,j1:j2) = chan_counts(:,:,j1:j2)
  do j = j1,j1+j2-1
    chan_counts_sg(1,:,j) = chan_counts_sg(1,:,j) - scan_space_counts(1,j)
    chan_counts_sg(2,:,j) = chan_counts_sg(2,:,j) - scan_space_counts(2,j)
    chan_counts_sg(3,:,j) = chan_counts_sg(3,:,j) - scan_space_counts(3,j)
  enddo
else

  do j = j1, j1+j2-1

!--- check for bad scans
      if (bad_scan_flag(j) /= 0) then
        cycle
      endif

  do i = 1, num_pix

!--- channel 1
     if (chan_counts(1,i,j) < ch1_switch_count) then
       chan_counts_sg(1,i,j) = scan_space_counts(1,j) + 0.5*(chan_counts(1,i,j) - scan_space_counts(1,j))
     else
       chan_counts_sg(1,i,j) = (scan_space_counts(1,j) + 0.5*(ch1_switch_count - scan_space_counts(1,j))) + &
                                 3.0*(chan_counts(1,i,j) - ch1_switch_count)/2.0
     endif
     chan_counts_sg(1,i,j) = chan_counts_sg(1,i,j) - scan_space_counts(1,j)

!--- channel 2
     if (chan_counts(2,i,j) < ch2_switch_count) then
       chan_counts_sg(2,i,j) = scan_space_counts(2,j) + 0.5*(chan_counts(2,i,j) - scan_space_counts(2,j))
     else
       chan_counts_sg(2,i,j) = (scan_space_counts(2,j) + 0.5*(ch2_switch_count - scan_space_counts(2,j))) + &
                                 3.0*(chan_counts(2,i,j) - ch2_switch_count)/2.0
     endif
     chan_counts_sg(2,i,j) = chan_counts_sg(2,i,j) - scan_space_counts(1,j)

!--- channel 3a
    if (ch3a_on(j) == 1) then

     if (chan_counts(3,i,j) < ch3a_switch_count) then
       chan_counts_sg(3,i,j) = scan_space_counts(3,j) + 0.25*(chan_counts(3,i,j) - scan_space_counts(3,j))
     else
       chan_counts_sg(3,i,j) = (scan_space_counts(3,j) + 0.25*(ch3a_switch_count - scan_space_counts(3,j))) + &
                                 1.75*(chan_counts(3,i,j) - ch3a_switch_count)
     endif
     chan_counts_sg(3,i,j) = chan_counts_sg(3,i,j) - scan_space_counts(1,j)

    endif

    enddo
   enddo

endif

end subroutine CONVERT_COUNTS_SINGLE_GAIN


!------------------------------------------------------------------------
! Routine to assign clear-sky quality flags
!------------------------------------------------------------------------
  subroutine ASSIGN_CLEAR_SKY_QUALITY_FLAGS(jmin,jmax)

    integer, intent(in):: jmin,jmax
    integer:: i, j, i1, i2, j1, j2, n, max_mask

!-- determine size of box for spatial filter
 n = 1
                                                                                                                                                
!--- initialize
 sst_qf = 0
 tsfc_qf = 0
 ndvi_qf = 0
 rsr_qf = 0


 do j = jmin, jmin+jmax-1
                                                                                                                                                
    !--- check for bad scans
    if (bad_scan_flag(j) > 0) then
        cycle
    endif
                                                                                                                                                
   !--- determine y-dimensions of array to check
   j1 = max(jmin,j-n)
   j2 = min(jmax,j+n)
                                                                                                                                                
    do i = 1, num_pix
                                                                                                                                                
      !--- determine x-dimensions of array to check
      i1 = max(1,i-n)
      i2 = min(num_pix,i+n)
        
      !--- initial cloud mask based
      if (cld_mask(i,j) == sym%CLEAR) then
         sst_qf(i,j) = 3
         tsfc_qf(i,j) = 3
         sst_qf(i,j) = 3
         ndvi_qf(i,j) = 3
         rsr_qf(i,j) = 3
      endif
      if (cld_mask(i,j) == sym%PROB_CLEAR) then
         sst_qf(i,j) = 2
         tsfc_qf(i,j) = 2
         ndvi_qf(i,j) = 2
         rsr_qf(i,j) = 2
      endif
      if (cld_mask(i,j) == sym%PROB_CLOUDY) then
         sst_qf(i,j) = 1
         tsfc_qf(i,j) = 1
         ndvi_qf(i,j) = 1
         rsr_qf(i,j) = 1
      endif
      if (cld_mask(i,j) == sym%CLOUDY) then
         sst_qf(i,j) = 0
         tsfc_qf(i,j) = 0
         ndvi_qf(i,j) = 0
         rsr_qf(i,j) = 0
      endif

      !--- assign sst over land to be low quality
      if (land_mask(i,j) == sym%YES) then  
        sst_qf(i,j) = 0
      endif

      !--- assign sst over snow/ice to be low quality
      if (snow(i,j) /= sym%NO_SNOW)    then !sea ice check
       sst_qf(i,j) = 0
      endif

      !--- assign ndvi over water to be low quality
      if (land_mask(i,j) == sym%NO) then
        ndvi_qf(i,j) = 0
      endif

      !--- assign ndvi at high angles to be low quality
      if (solzen(i,j) > 75.0) then
       ndvi_qf(i,j) = min(1,int(ndvi_qf(i,j)))
      endif

      !--- modifcations of rsr quality
      if (land_mask(i,j) == sym%YES) then    !ocean only
        rsr_qf(i,j) = 0
      endif
      if (solzen(i,j) > 75.0) then    !sufficient light
        rsr_qf(i,j) = min(1,int(rsr_qf(i,j)))
      endif
      if (glintzen(i,j) < Glint_Zen_Thresh) then   !outside glint
       rsr_qf(i,j) = min(1,int(rsr_qf(i,j)))
      endif
      if (snow(i,j) /= sym%NO_SNOW) then    !snow
       rsr_qf(i,j) = min(1,int(rsr_qf(i,j)))
      endif


      !--- aot 
       if (aer_flag == sym%YES) then

        aot_qf(i,j) = 0
        if ((cld_mask(i,j) == sym%CLEAR) .and. (solzen(i,j) < 70.00) .and. (snow(i,j) == sym%NO_SNOW)) then
          aot_qf(i,j) = 1
          if (glintzen(i,j) > Glint_Zen_Thresh) then
            if (relaz(i,j) > 90.0) then
              aot_qf(i,j) = 3
            else
             aot_qf(i,j) = 2 - 1
           endif
         endif
        endif
                                                                                                                              
        !--- assign aerosol over snow to be of low quality
        if (snow(i,j) /= sym%NO_SNOW) then
         aot_qf(i,j) = 0
        endif

        !--- assign high quality pixels around a cloudy results as qf = 2
        max_mask = maxval(cld_mask(i1:i2,j1:j2))
      if (max_mask >= 2) then
        if (aot_qf(i,j) == 3) then
          aot_qf(i,j) = 2
        endif
        if (sst_qf(i,j) == 3) then
          sst_qf(i,j) = 2
        endif
        if (ndvi_qf(i,j) == 3) then
          ndvi_qf(i,j) = 2
        endif
        if (tsfc_qf(i,j) == 3) then
          tsfc_qf(i,j) = 2
        endif
      endif

     !--- forcing the reporting of aerosol for this condition (A. Evan)
     if ((dust(i,j) == 1) .and. (bt45(i,j) < -0.5)) then
         aot_qf(i,j) = 3
     endif
  endif ! end of Aerosol  QF


    end do
  end do

  end subroutine ASSIGN_CLEAR_SKY_QUALITY_FLAGS

!----------------------------------------------------------------------------
! compute spatial uniformity used for each pixel
!
!  note this can handle nx2 uniformity calculations.  If n /= 2, then make
!  sure this routine is called for each pixel in the scanline.  However,
!  because we still process two scanlines, we only do this once for each
!  scanline in the pair
!
! note, this routine attempts to return the local standard deviation of area
! surrounding the pixels.  This is done using the following approximation
! sigma = (max - min) / sqrt(n)  where n is the number of pixels that comprise
! the region.  For a 2x2, n=4, for 3x2, n=6 and so forth.
!
!  Now written assuming 3x3 uniformity except at edges of segments
!---------------------------------------------------------------------------
subroutine COMPUTE_SPATIAL_UNIFORMITY(jmin,jmax)
                                                                                                           
  integer, intent(in):: jmin,jmax
  integer:: nbox

  integer:: nx
  integer:: ny
  integer, dimension(:,:), allocatable:: i_max
  integer, dimension(:,:), allocatable:: j_max
  integer, dimension(:,:), allocatable:: i_min
  integer, dimension(:,:), allocatable:: j_min

  integer:: uni_land_mask_flag_yes
  integer:: uni_land_mask_flag_no
  integer:: ielem
  integer:: iline
 

!----------------------------------------------------------------------
! determine size of input arrays
!----------------------------------------------------------------------
   nx = size(bt4,1)
   ny = size(bt4,2)

!----------------------------------------------------------------------
! allocate memory for temporary arrays
!----------------------------------------------------------------------
   allocate(i_max(nx,ny))
   allocate(j_max(nx,ny))
   allocate(i_min(nx,ny))
   allocate(j_min(nx,ny))

   i_max = 0
   j_max = 0
   i_min = 0
   j_min = 0

!----------------------------------------------------------------------
! set flag to enforce uniformity in land mask for these computations
!----------------------------------------------------------------------
   uni_land_mask_flag_yes  = sym%YES
   uni_land_mask_flag_no  = sym%NO

!----------------------------------------------------------------------
! compute 3x3 metric
!----------------------------------------------------------------------
   nbox = 1   !1 gives a 3x3 box

   !--- bt4 
   call COMPUTE_SPATIAL_UNIFORMITY_NxN_WITH_INDICES( &
                                      !bt4,nbox, uni_land_mask_flag_yes, &
                                       bt4,nbox, uni_land_mask_flag_no, &
                                       bad_pixel_mask, land_mask, &
                                       1,num_pix,jmin,jmax, &
                                       bt4_mean_3x3,bt4_min_3x3, &
                                       bt4_max_3x3,bt4_std_3x3, &
                                       i_max, j_max, i_min, j_min)
    !--- store indices
    i_max_bt4_3x3 = i_max
    j_max_bt4_3x3 = j_max
    i_min_bt4_3x3 = i_min
    j_min_bt4_3x3 = j_min

   !--- alb1
   call COMPUTE_SPATIAL_UNIFORMITY_NxN_WITH_INDICES( &
                                       alb1,nbox,  &
                                      !uni_land_mask_flag_yes, &
                                       uni_land_mask_flag_no, &
                                       bad_pixel_mask, land_mask, &
                                       1,num_pix,jmin,jmax, &
                                       alb1_mean_3x3,alb1_min_3x3, &
                                       alb1_max_3x3,alb1_std_3x3, &
                                       i_max, j_max, i_min, j_min)
   !--- ems3b
   call COMPUTE_SPATIAL_UNIFORMITY_NxN_WITH_INDICES( &
                                       ems3b,nbox,  &
                                      !uni_land_mask_flag_yes, &
                                       uni_land_mask_flag_no, &
                                       bad_pixel_mask, land_mask, &
                                       1,num_pix,jmin,jmax, &
                                       ems3b_mean_3x3,ems3b_min_3x3, &
                                       ems3b_max_3x3,ems3b_std_3x3, &
                                       i_max, j_max, i_min, j_min)
   !--- zsfc
   call COMPUTE_SPATIAL_UNIFORMITY_NxN_WITH_INDICES( & 
                                       zsfc,nbox,  &
                                       uni_land_mask_flag_no, &
                                       bad_pixel_mask, land_mask, &
                                       1,num_pix,jmin,jmax, &
                                       zsfc_mean_3x3,zsfc_min_3x3, &
                                       zsfc_max_3x3,zsfc_std_3x3, &
                                       i_max, j_max, i_min, j_min)

   !--- alb1 clear white sky from MODIS
   call COMPUTE_SPATIAL_UNIFORMITY_NxN_WITH_INDICES( &
                                       alb1_sfc_white_sky,nbox, &
                                       uni_land_mask_flag_no, &
                                       bad_pixel_mask, land_mask, &
                                       1,num_pix,jmin,jmax, &
                                       alb1_sfc_white_sky_mean_3x3, &
                                       alb1_sfc_white_sky_min_3x3, &
                                       alb1_sfc_white_sky_max_3x3, &
                                       alb1_sfc_white_sky_std_3x3, &
                                       i_max, j_max, i_min, j_min)

   !--- fill in wholes in white sky albedo
   where(alb1_sfc_white_sky == missing_value_real4 .and. &
           alb1_sfc_white_sky_mean_3x3 /= missing_value_real4) 
           alb1_sfc_white_sky = alb1_sfc_white_sky_mean_3x3
   end where

   !--- bt3b
   call COMPUTE_SPATIAL_UNIFORMITY_NxN_WITH_INDICES( &
                                       bt3b,nbox,  & 
                                       !uni_land_mask_flag_yes, &
                                       uni_land_mask_flag_no, &
                                       bad_pixel_mask, land_mask, &
                                       1,num_pix,jmin,jmax, &
                                       bt3b_mean_3x3,bt3b_min_3x3, &
                                       bt3b_max_3x3,bt3b_std_3x3, &
                                       i_max, j_max, i_min, j_min)

   !--- bt45
   call COMPUTE_SPATIAL_UNIFORMITY_NxN_WITH_INDICES( &
                                       bt45,nbox,  &
                                       !uni_land_mask_flag_yes, &
                                       uni_land_mask_flag_no, &
                                       bad_pixel_mask, land_mask, &
                                       1,num_pix,jmin,jmax, &
                                       bt45_mean_3x3,bt45_min_3x3, &
                                       bt45_max_3x3,bt45_std_3x3, &
                                       i_max, j_max, i_min, j_min)

!----- store bt45 at maximum bt4 in surrounding pixels
  line_loop: do iline=jmin, jmax - jmin + 1
    element_loop: do ielem= 1, num_pix
       if ((i_max_bt4_3x3(ielem,iline) > 0) .and. &
           (j_max_bt4_3x3(ielem,iline) > 0)) then
        bt45_bt4_max_3x3(ielem,iline) = bt45(i_max_bt4_3x3(ielem,iline),j_max_bt4_3x3(ielem,iline)) 
       endif
    end do element_loop
  end do line_loop


!--- deallocate memory allocated in this routine
  deallocate(i_max)
  deallocate(j_max)
  deallocate(i_min)
  deallocate(j_min)

end subroutine COMPUTE_SPATIAL_UNIFORMITY
!----------------------------------------------------------------------
!--- Compute a mask identifying oceanic glint
!----------------------------------------------------------------------
subroutine COMPUTE_GLINT(jmin,jmax)
  integer, intent(in):: jmin,jmax
  integer:: i, j

    line_loop: do j = jmin,jmax

     !--- check for bad scans
     if (bad_scan_flag(j) /= 0) then
        cld_test_vector(1,:,j) = sym%NO
        cycle
     endif

     element_loop: do i = 1, num_pix

     !- initialize to no
     glint_mask(i,j) = sym%NO

     !--- skip bad pixels
     if (bad_pixel_mask(i,j) == sym%YES) then
             cycle
     endif

     if ((land_mask(i,j) == sym%NO)) then

       !- turn on in geometric glint cone and sufficient alb1
       if ((glintzen(i,j) < Glint_Zen_Thresh)) then

          glint_mask(i,j) = sym%YES

          !-turn off if cold
          if (sst_mask(i,j) < tsfc_rtm(i,j) - 5.0) then 
           glint_mask(i,j) = sym%NO
          endif

          !-turn off if non-uniform in reflectance
          if (alb1_std_3x3(i,j) > min(1.0,0.20*alb1_mean_3x3(i,j))) then 
           glint_mask(i,j) = sym%NO
          endif

          !--- ch3b glint modifications
          if (ch3a_on(j) == sym%NO) then

             !-turn on if high alb31
             if (alb31(i,j) > 1.00) then 
                glint_mask(i,j) = sym%YES
             endif

             !-turn off if low alb31
             if (alb31(i,j) < 0.60) then
               glint_mask(i,j) = sym%NO
             endif 
              
          endif   !ch3a check


       endif  !glintzen check

     endif    !land check

     enddo element_loop
   enddo line_loop

 end subroutine COMPUTE_GLINT
!------------------------------------------------------------------------
! compute the elements of cld_test_vector that hold information pertaining to the
! type of pixel this is and how the cloud mask treats it
!------------------------------------------------------------------------------
 subroutine COMPUTE_DATA_MASK(jmin,jmax)
  integer, intent(in):: jmin,jmax
  integer:: i, j

    do j = jmin, jmax - jmin + 1

      !--- check for bad scans
      if (bad_scan_flag(j) /= 0) then
        cld_test_vector(1,:,j) = sym%NO
        cycle
      endif

     do i = 1, num_pix

     !--- assume all data on good scans are valid - cloud mask can change this
     cld_test_vector(1,i,j) = sym%YES

     !--- day
     if (solzen(i,j) < 90.0) then  !day
       cld_test_vector(2,i,j) = sym%YES
     endif

     !terminator
     if (solar_contamination_mask(i,j) == sym%YES) then
       cld_test_vector(3,i,j) = sym%YES
     endif

     !--- land
     if (land_mask(i,j) == sym%YES) then      !land
       cld_test_vector(4,i,j) = sym%YES
     endif

     !--- coast flag
     cld_test_vector(5,i,j) = coast_mask(i,j)

     !--- glint flag
     cld_test_vector(6,i,j) = glint_mask(i,j)

     !--- desert flag
     cld_test_vector(7,i,j) = desert_mask(i,j)    !desert

     !--- snow from input
     if (snow(i,j) /= sym%NO_SNOW) then
        cld_test_vector(8,i,j) = sym%YES
     endif

    enddo
  enddo

end subroutine COMPUTE_DATA_MASK

!----------------------------------------------------------------------
! Read MODIS white sky albedoes
!----------------------------------------------------------------------
 subroutine READ_MODIS_WHITE_SKY_ALBEDO(modis_alb_0_66_id, &
                                        modis_alb_1_64_id, &
                                        modis_alb_0_66_str, &
                                        modis_alb_1_64_str)

    integer(kind=4), intent(in):: modis_alb_0_66_id
    integer(kind=4), intent(in):: modis_alb_1_64_id
    TYPE(land_grid_description), intent(in) :: modis_alb_0_66_str
    TYPE(land_grid_description), intent(in) :: modis_alb_1_64_str


    !--- 0.66 micron
    call read_land_sfc_hdf(modis_alb_0_66_id, modis_alb_0_66_str, lat, &
                          lon, space_mask, two_byte_temp)
    alb1_sfc_white_sky = 0.1*two_byte_temp

    alb1_sfc_white_sky = 1.10*alb1_sfc_white_sky   !EMPIRICAL ADJUSTMENT

    where(two_byte_temp == 32767)
             alb1_sfc_white_sky = missing_value_real4
    endwhere

    !--- 1.64 micron
    call read_land_sfc_hdf(modis_alb_1_64_id, modis_alb_1_64_str, lat, &
                          lon, space_mask, two_byte_temp)
    alb3a_sfc_white_sky = 0.1*two_byte_temp
    where(two_byte_temp == 32767)
             alb3a_sfc_white_sky = missing_value_real4
    endwhere

!--- modify for snow and water
!   where(snow /= sym%NO_SNOW .and. alb1_sfc_white_sky < 50.0)
!           alb1_sfc_white_sky = alb1_sfc_white_sky_snow
!   endwhere
!   where(snow /= sym%NO_SNOW .and. alb3a_sfc_white_sky > 16.0)
!           alb3a_sfc_white_sky = alb3a_sfc_white_sky_snow
!    endwhere

    !--- modify for water
    where(snow == sym%NO_SNOW .and. land_mask == sym%NO)
            alb1_sfc_white_sky = alb1_sfc_white_sky_water
            alb3a_sfc_white_sky = alb3a_sfc_white_sky_water
    endwhere

 end subroutine READ_MODIS_WHITE_SKY_ALBEDO

!------------------------------------------------------------
! compute the single scater and aerosol reflectance
! this assumes that the gas is mixed in with scattering
!-----------------------------------------------------------
subroutine COMPUTE_CLEAR_SKY_SCATTER(tau_aer, &
                                     wo_aer, &
                                     g_aer, &
                                     tau_ray, &
                                     tau_gas, &
                                     scatangle, &
                                     coszen, &
                                     cossolzen, &
                                     cloud_albedo_view, &
                                     cloud_albedo_sun, &
                                     ref_ss)

   real, intent(in):: tau_aer
   real, intent(in):: wo_aer
   real, intent(in):: g_aer
   real, intent(in):: tau_ray
   real, intent(in):: tau_gas
   real, intent(in):: scatangle
   real, intent(in):: coszen
   real, intent(in):: cossolzen
   real, intent(in):: cloud_albedo_view
   real, intent(in):: cloud_albedo_sun
   real, intent(out):: ref_ss

   real:: airmass
   real:: P_aer
   real:: P_ray
   real:: tau_total
   real:: tau_scat_total
   real:: trans_total
   real:: tau_iso_total
   real:: trans_iso_total_view
   real:: trans_iso_total_sun
   real:: tau_iso_scat_total
   real:: mu
   real:: Pf
   real:: wo
   real:: ref_ss_a
   real:: ref_ss_b
   real:: ref_ss_c

      !--- compute cosine of scattering angle
      mu = cos(scatangle*dtor)

      !-- compute Rayleigh phase function
      airmass = 1.0/coszen + 1.0/cossolzen
      P_ray = 0.75*(1.0 + mu**2)

      !--- compute total transmission
      tau_total = tau_aer + tau_ray + tau_gas
      trans_total = exp(-tau_total*airmass)

      tau_iso_total = (1.0-g_aer)*tau_aer + tau_ray + tau_gas
      trans_iso_total_view = exp(-tau_iso_total/coszen)
      trans_iso_total_sun = exp(-tau_iso_total/cossolzen)

      !--- compute total scattering optical depth
      tau_scat_total = wo_aer*tau_aer + tau_ray
      tau_iso_scat_total = wo_aer*(1.0-g_aer)*tau_aer + tau_ray

      !--- single scatter albedo
      wo = (wo_aer*tau_aer + tau_ray)/ ( tau_total )

      !aerosol phase function (Henyey-Greenstein)
      P_aer = (1.0 - g_aer**2)/( (1.0 + g_aer**2 - 2.0*g_aer*mu)**(1.5) )

      !--- compute effective phase function
      Pf = (wo_aer*tau_aer*P_aer + tau_ray*P_ray)/(tau_scat_total)

      !--- compute single scatter reflectance (0-100%)
      ref_ss_a = wo*Pf/(4.0*airmass*coszen*cossolzen) * (1.0 - trans_total )


      ref_ss_b = (tau_iso_scat_total / (2.0*cossolzen)) *trans_iso_total_view * cloud_albedo_view

      ref_ss_c = (tau_iso_scat_total / (2.0*coszen)) *trans_iso_total_sun * cloud_albedo_sun

      ref_ss = 100.0*(ref_ss_a + ref_ss_b + ref_ss_c)

end subroutine COMPUTE_CLEAR_SKY_SCATTER
!=============================================================================
!-- Compute Deep Convective Cloud Mask
!=============================================================================
subroutine COMPUTE_DCC_MASK(jmin,jmax)

  integer, intent(in):: jmin
  integer, intent(in):: jmax

  integer:: ielem
  integer:: iline

  dcc_mask = missing_value_int1

  line_loop: do iline=jmin, jmax - jmin + 1
    element_loop: do ielem= 1, num_pix

     if (bad_pixel_mask(ielem,iline) == sym%YES) then
        cycle
     endif

     dcc_mask(ielem,iline) = sym%NO

     if (emiss_11_tropo_rtm(ielem,iline) > emiss_11_trop_dcc_thresh) then
       dcc_mask(ielem,iline) = sym%YES

       !-- set cloud type to overshooting for dcc events
       cld_type(ielem,iline) = sym%OVERSHOOTING_TYPE
       cld_type_ir(ielem,iline) = sym%OVERSHOOTING_TYPE

     endif

    end do element_loop
  end do line_loop

 end subroutine COMPUTE_DCC_MASK
!======================================================================
! Professor Toshiro Inoue's Split Window Cloud Type
!
! In this scheme, there are 17 cloud types
!  11+12+15+16 = dense cirrus
!  9+10+13+14 = cumulonimbus
!  4+8 = cirrus
!  3+7 = N-type
!  1+2+5+6 = low level cumulus
!
! See Figure 1 of Inoue et al., 2009: Life Cycle of Deep Convective ...
! Journal of Met Soc Japan, Vol 87A, ! pp 381391, 2009
!======================================================================
subroutine COMPUTE_INOUE_CLOUD_TYPE(jmin,jmax)

  integer, intent(in):: jmin
  integer, intent(in):: jmax

  integer:: ielem
  integer:: iline
  integer:: ct

  real:: TBB
  real:: BTD

  real, parameter:: TBB_Threshold_1 = 213.0
  real, parameter:: TBB_Threshold_2 = 235.0
  real, parameter:: TBB_Threshold_3 = 253.0
  real, parameter:: TBB_Threshold_4 = 273.0
  real, parameter:: BTD_Threshold_1 = 0.5
  real, parameter:: BTD_Threshold_2 = 1.0
  real, parameter:: BTD_Threshold_3 = 2.0

  cld_type_inoue = 0

  line_loop: do iline=jmin, jmax - jmin + 1
    element_loop: do ielem= 1, num_pix

    !--- initialize
    ct = 0

    !--- filter bad pixels
    if (bad_pixel_mask(ielem,iline) == sym%YES) then
           cld_type_inoue(ielem,iline) = 0
           cycle
    endif

    !--- local aliases
    TBB = bt4(ielem,iline)
    BTD = bt45(ielem,iline)

    !--- opaque class
    if (TBB <= TBB_Threshold_1) then
            ct = 17
            cld_type_inoue(ielem,iline) = ct
            cycle
    endif

    !--- assign non-opaque codes based on TBB and BTD
    if (BTD <= BTD_Threshold_1) ct = 1
    if (BTD >= BTD_Threshold_1 .and. BTD < BTD_Threshold_2) ct = 2
    if (BTD >= BTD_Threshold_2 .and. BTD < BTD_Threshold_3) ct = 3
    if (BTD >= BTD_Threshold_3) ct = 4

    if (TBB > TBB_Threshold_1 .and. TBB <= TBB_Threshold_2) then
        ct = ct + 12
    endif
    if (TBB > TBB_Threshold_2 .and. TBB <= TBB_Threshold_3) then
        ct = ct + 8
    endif
    if (TBB > TBB_Threshold_3 .and. TBB <= TBB_Threshold_4) then
        ct = ct + 4
    endif

    cld_type_inoue(ielem,iline) = ct

    enddo element_loop
  enddo line_loop

end subroutine COMPUTE_INOUE_CLOUD_TYPE

!===========================================================================
!-- Compute Cloud and Land Masked SST
!===========================================================================
subroutine COMPUTE_MASKED_SST(jmin,jmax)

  integer, intent(in):: jmin
  integer, intent(in):: jmax

  integer:: ielem
  integer:: iline

  sst_masked = missing_value_real4

  line_loop: do iline=jmin, jmax - jmin + 1
    element_loop: do ielem= 1, num_pix

      if (sst_qf(ielem,iline) == 3) then
         if (sst_flag == sym%NO) then
           sst_masked(ielem,iline) = sst_mask(ielem,iline)
         else
           sst_masked(ielem,iline) = sst_mask(ielem,iline)
         endif
      endif

    end do element_loop
  end do line_loop

 end subroutine COMPUTE_MASKED_SST

!------------------------------------------------------------------------------
! compute cloud fraction over a 3x3 array using the Bayesian probability
!------------------------------------------------------------------------------
 subroutine COMPUTE_CLOUD_FRACTION_3x3(jmin,jmax)
                                                                                                           
  integer, intent(in):: jmin,jmax
  integer :: i
  integer :: j
  integer :: i1
  integer :: i2
  integer :: j1
  integer :: j2
  integer :: ii
  integer :: jj
  integer :: ngood
  integer :: ncloud
  integer,parameter :: n=1
  real:: cloud_fraction_uncer_temp

  !--- initialize
  cloud_fraction_3x3 = missing_value_real4

  line_loop: do j=jmin, jmax - jmin + 1

    j1 = max(jmin,j-n)
    j2 = min(jmax,j+n)

    element_loop: do i = 1, num_pix

      i1 = max(1,i-n)
      i2 = min(num_pix,i+n)

      !--- check for a bad pixel pixel
      if (bad_pixel_mask(i,j) == sym%YES) then
        cycle
      endif

      !--- compute cloud amount
      ngood = count(posterior_cld_probability(i1:i2,j1:j2) /= missing_value_real4)        

      !--- see if there are any valid points
      if (ngood == 0 ) then
         cycle
      endif

      ncloud = count(posterior_cld_probability(i1:i2,j1:j2) >= 0.5)        
      cloud_fraction_3x3(i,j) = real(ncloud)/real(ngood)

      !--- compute the uncertainty
      cloud_fraction_uncer_temp = 0.0
      do ii = i1,i2
         do jj = j1,j2  
           if (posterior_cld_probability(ii,jj) >= 0.5) then
             cloud_fraction_uncer_temp = 1.0 - posterior_cld_probability(ii,jj)
           else
             cloud_fraction_uncer_temp = posterior_cld_probability(ii,jj)
           endif
           cloud_fraction_uncer_3x3(i,j) = cloud_fraction_uncer_3x3(i,j) +   &
                                           cloud_fraction_uncer_temp
         enddo
      enddo

      cloud_fraction_uncer_3x3(i,j) = cloud_fraction_uncer_3x3(i,j)  / ngood

    end do element_loop
 end do line_loop

 end subroutine COMPUTE_CLOUD_FRACTION_3x3

!-----------------------------------------------------------
! end of module
!-----------------------------------------------------------
end module AVHRR_PIXEL_ROUTINES
