! $Id: cld_hght_11_12_1dvar.f90,v 1.26 2010/02/22 18:34:42 heidinger Exp $
module CLOUD_HEIGHT_11_12_1DVAR
!---------------------------------------------------------------------
!@***h* PATMOSX/cld_hght_11_12_1dvar.f90
! 
! NAME
!   cld_hght_11_12_1dvar.f90
! 
! LOCATION
!   /patmosx/src/cld_hght_11_12_1dvar.f90
! 
! PURPOSE
!   This module holds the algorithms to estimate cloud properties
! 
! DESCRIPTION
!  This module holds the algorithms to estimate cloud properties
!  accomplished via multiple passes
!  pass 1 = non-multi-layer ncc pixels
!  pass 2 = single layer water cloud pixels
!    <---- interp water cloud values to all pixels
!  pass 3 = lrc multi-layer clouds
!  pass 4 = all remaining clouds
!
!  pass 0 = treat all pixels as single layer ncc pixels (only done if lrc_flag=no)
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!  reference:
!   Heidinger, A.K., and M.J. Pavolonis, 2009: Gazing at Cirrus Clouds for 25 Years
!   through a Split Window. Part I: Methodology. J. Appl. Meteor. Climatol., 48,
!   1100–1116.
!
!
! MODIFICATION HISTORY
!  v5 changes:
!      1) multi-layer clouds are now processed using a dynamically
!         determined lower cloud boundary condition
!      2) profiles are now interpolated to each pixel
!
!@*****
!------------------------------------------------------------------------
! This module holds the algorithms to estimate cloud properties
!
! author: Andrew Heidinger, NOAA/NESDIS
!
! reference:
!  Heidinger, A.K., and M.J. Pavolonis, 2009: Gazing at Cirrus Clouds for 25 Years
!  through a Split Window. Part I: Methodology. J. Appl. Meteor. Climatol., 48,
!  1100–1116.
!
! v5 changes:
!      1) multi-layer clouds are now processed using a dynamically
!         determined lower cloud boundary condition
!      2) profiles are now interpolated to each pixel
!
! accomplished via multiple passes
! pass 1 = non-multi-layer ncc pixels
! pass 2 = single layer water cloud pixels
!   <---- interp water cloud values to all pixels
! pass 3 = lrc multi-layer clouds
! pass 4 = all remaining clouds
!
! pass 0 = treat all pixels as single layer ncc pixels (only done if lrc_flag=no)
!
!----------------------------------------------------------------------
  use CONSTANTS
  use AVHRR_PIXEL_COMMON
  use NUMERICAL_ROUTINES
  use SURFACE_PROPERTIES
  use NWP_COMMON
  use PLANCK
  use RT_UTILITIES

  implicit none

  public:: SPLIT_WIN_CLOUD_BETA     !estimate Tc, ec, beta from T4,T4-T5
  public:: OPAQUE_CLOUD_HEIGHT      !use opaque values as fill for AVHRR/1
  private:: SPATIALLY_INTERPOLATE_LOWER_CLOUD_POSITION  
  private:: SPATIALLY_INTERPOLATE_RTM_PROFILES
  private:: KNOWING_P_COMPUTE_T_Z
  private:: KNOWING_T_COMPUTE_P_Z
  private:: KNOWING_Z_COMPUTE_T_P
  private:: GENERIC_PROFILE_INTERPOLATION

!--- include parameters
  INCLUDE 'cld_hght_11_12_1dvar.inc'

  !--- interpolated profiles
  real, private, dimension(nlevels_rtm):: Temp_Prof_RTM
  real, private, dimension(nlevels_rtm):: Press_Prof_RTM
  real, private, dimension(nlevels_rtm):: Hght_Prof_RTM
  real, private, dimension(nlevels_rtm):: Atm_Rad_Prof_11um_RTM
  real, private, dimension(nlevels_rtm):: Atm_Rad_Prof_12um_RTM
  real, private, dimension(nlevels_rtm):: Atm_Trans_Prof_11um_RTM
  real, private, dimension(nlevels_rtm):: Atm_Trans_Prof_12um_RTM
  integer, private:: Inver_Level_RTM
  integer, private:: Sfc_Level_RTM
  integer, private:: Tropo_Level_RTM

  CONTAINS

!------------------------------------------------------------------------------
! Split window retrieval of cloud temperature and cloud emissivity (11 micron)
!
! Author: Andrew Heidinger, NOAA
!
! Assumptions
!        1) No scattering
!        2) cloud is 1 km thick and emission varies linear with emissivity
!        3) single layer cloud for cloud type /= 6
!        4) for cloud type = 6, an opaque cloud 200 mb above the surface lies below 
!
!  Limitations
!        1) sensitivity to Tc is low for thin clouds
!        2) little emissivity sensitivity for low clouds
!
! input to the retrieval
!  y(1) = t4 - 11 micron brightness temperature
!  y(2) = t4-t5 - the split window temperature
!
! the output of the retrieval
!  x(1) - the cloud temperature
!  x(2) - the 11 micron emissivity at nadir
!  x(3) - the beta ratio for 11 and 12 microns
!
! This routine uses a 1d-var retrieval approach as outlined in Rodger (1976)
!
! input to the 1d-var approach
!  y - the vector of observations
!  x_ap - the a apriori estimates of x
!  Sa - the error covariance matric of x_ap
!  Sy - the error covariance of y (included calibration, forward model)
!
! internal matrices in the 1d-var approach
!  f- the forward model estimate of y
!  K - the kernel matrix - df/dx
!
! output from the 1d-var approach
!  x - the vector of parameters to retrieve
!  Sx - the error covariance of x
!  A - observation weight matrix - 1.0 means total reliance on obs.
!
! the quality flags are determined as follows
!  3 - estimated error < 1/3 a priori error
!  2 - estimated error < 2/3 a priori error
!  1 - any other converged retrieval
!  0 - a failed or unattempted retrieval
!
! modification history
!
! July 2006 - Added beta as an element of x
! October 2006 - Added cloud lapse rate to make Tc more related to true 
!                cloud-top temperature
!
!------------------------------------------------------------------------------
  subroutine  SPLIT_WIN_CLOUD_BETA(jmin,jmax)
                              
  integer, intent(in):: jmin
  integer, intent(in):: jmax
  integer:: ielem
  integer:: iline
  integer:: i
  integer:: ifail
  integer:: isingular
  integer:: ilev
  integer:: inwp
  integer:: jnwp
  integer:: inwp_x
  integer:: jnwp_x
  integer:: pp
  integer:: ivza
  integer:: ilrc
  integer:: jlrc
  integer:: iter
  integer:: idiag_output
  integer:: ierror
  integer:: ipass
  integer:: ipass_min
  integer:: ipass_max
  integer:: ilev_min_inversion
  integer:: ilev_max_inversion
  real:: adjusted_pc
  real:: adjusted_zc

  real:: conv_test
  real:: conv_crit

  real:: Tc_temp
  real:: rad_ac_4
  real:: trans_ac_4
  real:: rad_ac_5
  real:: trans_ac_5
  real:: rad_clear_4
  real:: rad_clear_5
  real:: dB_dT_4
  real:: dB_dT_5
  real:: rad_4
  real:: rad_5
  real:: Bc_4
  real:: Bc_5
  real:: emiss_4
  real:: emiss_5
  real:: dB_dTc_4
  real:: dB_dTc_5
  real:: demiss5_demiss4
  real:: tsfc_est
  real:: tau_ap
  real:: Pc_temp
  real:: Zc_temp
  real:: trans_4
  real:: trans_5
  real:: Tc_ap_uncer
  real:: ec_ap_uncer
  real:: beta_ap_uncer
  real:: T11_clr_uncer
  real:: T11_12_clr_uncer
  real:: T11_12_cld_uncer
  real:: T11_cld_uncer
  real:: Tc_ap_cirrus
  real:: Tc_ap_opaque
  real:: delta_tc_tsfc
  real:: r4_dummy
  real:: lat_x
  real:: lon_x
  integer:: cloud_type
  integer:: cloud_phase
  integer:: undetected_cloud
  integer(kind=int1), dimension(:,:), pointer :: processing_order

  !--- 1d-var retrieval arrays
  real, dimension(num_obs):: y
  real, dimension(num_obs):: f
  real, dimension(num_param):: x
  real, dimension(num_param):: x_ap
  real, dimension(num_param):: delta_x
  real, dimension(num_obs,num_param):: K
  real, dimension(num_param,num_param):: Sa
  real, dimension(num_param,num_param):: Sa_inv
  real, dimension(num_param,num_param):: Sx
  real, dimension(num_param,num_param):: Sx_inv
  real, dimension(num_param,num_param):: E
  real, dimension(num_obs,num_obs):: Sy
  real, dimension(num_obs,num_obs):: Sy_inv

!-----------------------------------------------------------------------
! BEGIN EXECUTABLE CODE
!-----------------------------------------------------------------------
  

  !--- turn on diagnostic output
  idiag_output = sym%NO

  !--- set convergence criterion
  conv_crit = num_param / 2.0
  conv_crit = num_param - 1.0

  !----------- make identity matrix
  E = 0.0
  do i = 1,num_param
    E(i,i) = 1.0
  enddo

  !--- initialize output
  Tc_sw =  missing_value_real4
  ec_sw =  missing_value_real4
  beta_sw =  missing_value_real4
  Pc_sw =  missing_value_real4
  Zc_sw =  missing_value_real4
  Tc_sw_qf = 0
  ec_sw_qf = 0
  beta_sw_qf = 0
  cld_layer_sw = 0

  !--------------------------------------------------------------------------
  ! determine processing order of pixels
  !--------------------------------------------------------------------------
  processing_order => temp_mask
  processing_order = 0

  if (lrc_flag == sym%NO) then

      ipass_min = 0
      ipass_max = 0

  else 

      ipass_min = 1
      ipass_max = 4

      !--- loop through pixels, determine processing order
      line_loop_0: do iline = jmin,jmax + jmin - 1
        element_loop_0:   do ielem = 1, num_pix

        if (bad_pixel_mask(ielem,iline) == sym%YES) then
            cycle
        endif

        ilrc = i_lrc(ielem,iline)
        jlrc = j_lrc(ielem,iline)
        cloud_type = cld_type(ielem,iline)

        !-- on pass 1, do single layer lrc's
        if ((ielem == ilrc) .and. (iline == jlrc) .and. &
            (cloud_type /= sym%OVERLAP_TYPE)) then
             processing_order(ielem,iline) = 1
        endif

        !-- on pass 2, do non-lrc water clouds
        if ((ielem /= ilrc) .and. (iline /= jlrc) .and. &
            (cloud_type == sym%FOG_TYPE .or. &
            cloud_type == sym%WATER_TYPE .or. &
            cloud_type == sym%MIXED_TYPE .or. &
            cloud_type == sym%SUPERCOOLED_TYPE)) then
            processing_order(ielem,iline) = 2
        endif

        !-- on pass 3, do lrc overlap clouds
        if ((ielem == ilrc) .and. (iline == jlrc) .and. &
            (cloud_type == sym%OVERLAP_TYPE)) then
             processing_order(ielem,iline) = 3
        endif

        !--  on pass-4 do remaining
        if (processing_order(ielem,iline) == 0) then
           processing_order(ielem,iline) = 4
        endif

       end do element_loop_0
     end do line_loop_0
   endif

  !--------------------------------------------------------------------------
  ! perform multiple passes if using lrc results
  !--------------------------------------------------------------------------

  pass_loop: do ipass = ipass_min, ipass_max

   !--------------------------------------------------------------------------
   ! on the third pass, spatially interpolate water cloud temperature
   !--------------------------------------------------------------------------
   if ((ipass == 0) .or. (ipass == 3)) then
     call SPATIALLY_INTERPOLATE_LOWER_CLOUD_POSITION(ipass,jmin,jmax)
   endif

   !--------------------------------------------------------------------------
   ! loop over pixels in scanlines
   !--------------------------------------------------------------------------
   line_loop: do iline = jmin,jmax + jmin - 1

    element_loop:   do ielem = 1, num_pix

    !--- check if pixel should be processd in this path
    if (ipass /= processing_order(ielem,iline)) then
          cycle
    endif

    !--- check for a bad pixel
    if (bad_pixel_mask(ielem,iline) == sym%YES) then
          cycle
    endif

    !--- for convenience, save nwp indices to local variables
    inwp = i_nwp(ielem,iline)
    jnwp = j_nwp(ielem,iline)
    inwp_x = i_nwp_x(ielem,iline)
    jnwp_x = j_nwp_x(ielem,iline)
    lat_x = lat_nwp_fac(ielem,iline)
    lon_x = lon_nwp_fac(ielem,iline)
    ivza = ivza_rtm(ielem,iline)
    ilrc = i_lrc(ielem,iline)
    jlrc = j_lrc(ielem,iline)

    !-----------------------------------------------------------------------
    !  spatially interpolate nwp profiles to this pixel's location
    !-----------------------------------------------------------------------
    call  SPATIALLY_INTERPOLATE_RTM_PROFILES(smooth_nwp, inwp,jnwp, ivza, &
                                           inwp_x,jnwp_x,lat_x,lon_x)

    !----------------------------------------------------------------------
    !--- select cloud type to use
    !----------------------------------------------------------------------
    if (cloud_type_ir_flag == sym%YES) then
      cloud_type = cld_type_ir(ielem,iline)
    else
      cloud_type = cld_type(ielem,iline)
    endif
   !----------------------------------------------------------------------
   ! determine cloud phase from cloud type for convienience
   !----------------------------------------------------------------------
   cloud_phase = sym%UNKNOWN_PHASE

   if ( (cloud_type  == sym%FOG_TYPE) .or. &
       (cloud_type  == sym%WATER_TYPE) .or. &
       (cloud_type  == sym%SUPERCOOLED_TYPE)) then
       cloud_phase = sym%WATER_PHASE
   endif

   if ( (cloud_type  == sym%CIRRUS_TYPE) .or. &
        (cloud_type  == sym%OVERLAP_TYPE) .or. &
        (cloud_type  == sym%OPAQUE_ICE_TYPE)) then
     cloud_phase = sym%ICE_PHASE
   endif

  !-----------------------------------------------------------------------
  !----- data quality check
  !-----------------------------------------------------------------------
  if ((bt4(ielem,iline) < 170.0) .or. &
     (bt4(ielem,iline) > 340.0) .or. &
     (tmpsfc_nwp(inwp,jnwp) < 180.0) .or. &
     (tmpsfc_nwp(inwp,jnwp) > 340.0) .or. &
     (ttropo_rtm(ielem,iline) < 160.0) .or. &
     (ttropo_rtm(ielem,iline) > 270.0)) then

     Tc_sw(ielem,iline) =  missing_value_real4
     ec_sw(ielem,iline) =  missing_value_real4
     beta_sw(ielem,iline) =  missing_value_real4
     Pc_sw(ielem,iline) =  missing_value_real4
     Zc_sw(ielem,iline) =  missing_value_real4
     Tc_sw_qf(ielem,iline) = 0
     ec_sw_qf(ielem,iline) = 0
     beta_sw_qf(ielem,iline) = 0
     cld_layer_sw(ielem,iline) = 0

 else  !if passed data check then proceed with retrieval

   !---------------------------------------------------------------------
   ! select to do retrievals for all pixels or just cloudy ones
   !---------------------------------------------------------------------
   undetected_cloud = sym%NO

   if ((cld_mask(ielem,iline) == sym%CLEAR) .or.  &
       (cld_mask(ielem,iline) == sym%PROB_CLEAR)) then

       if (process_undetected_cloud_flag == sym%NO) then
               cycle
       else
              undetected_cloud = sym%YES
       endif

   endif

   !-----------------------------------------------------------------------
   ! assign values to y and x_ap
   !----------------------------------------------------------------------

   !--- y - the observation vector
   y(1) = bt4(ielem,iline)
   y(2) = bt4(ielem,iline) - bt5(ielem,iline)
 
   !------------ measurement and forward model error
   !--- specify clear sky model errors
   ! these are based on patmos-x clear data and are the 
   ! approx average of des+asc from August 2006 NOAA-18

   if (sfc_type(ielem,iline) == sym%WATER_SFC) then
     T11_clr_uncer = T11_clr_uncer_water
     T11_12_clr_uncer = T11_12_clr_uncer_water
   else
     T11_clr_uncer = T11_clr_uncer_land
     T11_12_clr_uncer = T11_12_clr_uncer_land
   endif

   !--------------------------------------------------------------------
   ! pick a priori conditions
   !--------------------------------------------------------------------

  !--- calipso values (not multiplier on uncer values)
  Tc_ap_cirrus = ttropo_rtm(ielem,iline) + Tc_ap_tropo_offset_cirrus
  Tc_ap_opaque = Tc_opaque_cloud(ielem,iline)
  if (Tc_ap_opaque == missing_value_real4) then
     Tc_ap_opaque = bt4(ielem,iline)
  endif


  !--- logic for unmasked or untyped pixels (undetected cloud)
  if (undetected_cloud == sym%YES) then
         if (Tc_ap_opaque < 260.0) then
             cloud_type = sym%CIRRUS_TYPE
         else
             cloud_type = sym%FOG_TYPE
         endif
  endif

  !---- select Tc and ec apriori based on cloud type
  if (cloud_type == sym%FOG_TYPE) then
     x_ap(1) = Tc_ap_opaque
     tau_ap = tau_ap_fog_type  
     Tc_ap_uncer = Tc_ap_uncer_opaque
     ec_ap_uncer = 2.0*ec_ap_uncer_opaque
  elseif (cloud_type == sym%WATER_TYPE) then
     x_ap(1) = Tc_ap_opaque
     tau_ap = tau_ap_water_type
     Tc_ap_uncer = Tc_ap_uncer_opaque
     ec_ap_uncer = ec_ap_uncer_opaque
  elseif (cloud_type == sym%SUPERCOOLED_TYPE .or. &
          cloud_type == sym%MIXED_TYPE) then
     x_ap(1) = Tc_ap_opaque
     tau_ap = tau_ap_mixed_type
     Tc_ap_uncer = Tc_ap_uncer_opaque
     ec_ap_uncer = ec_ap_uncer_opaque
   elseif (cloud_type == sym%OPAQUE_ICE_TYPE) then
     x_ap(1) = Tc_ap_opaque
     tau_ap = tau_ap_opaque_ice_type
     Tc_ap_uncer = Tc_ap_uncer_opaque
     ec_ap_uncer = ec_ap_uncer_opaque
   elseif (cloud_type == sym%OVERSHOOTING_TYPE) then
     x_ap(1) = Tc_ap_opaque
     tau_ap = tau_ap_opaque_ice_type
     Tc_ap_uncer = Tc_ap_uncer_opaque
     ec_ap_uncer = ec_ap_uncer_opaque
   elseif (cloud_type == sym%CIRRUS_TYPE) then
     x_ap(1) = Tc_ap_cirrus
     tau_ap = tau_ap_cirrus_type
     Tc_ap_uncer = Tc_ap_uncer_cirrus
     ec_ap_uncer = ec_ap_uncer_cirrus
   elseif (cloud_type == sym%OVERLAP_TYPE) then
     x_ap(1) = Tc_ap_cirrus
     tau_ap = tau_ap_overlap_type
     Tc_ap_uncer = Tc_ap_uncer_cirrus
     ec_ap_uncer = ec_ap_uncer_cirrus
   endif

   !---- determine beta apriori and fit parameters based on phase (derived from type)
          
   !--- water phase clouds
   IF ((cloud_type == sym%FOG_TYPE) .or. &
      (cloud_type == sym%WATER_TYPE) .or. &
      (cloud_type == sym%SUPERCOOLED_TYPE)) THEN
      x_ap(3) = beta_ap_water
      beta_ap_uncer = beta_ap_uncer_water
   ELSE
   !--- ice phase clouds
       x_ap(3) = beta_ap_ice
       beta_ap_uncer = beta_ap_uncer_ice
  END IF

  !--- convert tau_ap to emissivity
  x_ap(2) = 1.0 - exp(-tau_ap/coszen(ielem,iline))  !slow!

  !--- now compute Sa
  Sa = 0.0
  Sa(1,1) = Tc_ap_uncer
  Sa(2,1) = 0.0 
  Sa(2,2) = ec_ap_uncer
  Sa(1,2) = 0.0 
  Sa(3,3) = beta_ap_uncer

  !--- modify a priori values based on lrc
  if ((ilrc /= missing_value_int4) .and. &
      (jlrc /= missing_value_int4)) then

        if ((Tc_sw(ilrc,jlrc) /= missing_value_real4) .and. &
            (ec_sw(ilrc,jlrc) > 0.00) .and. &
            (ec_sw(ilrc,jlrc) <= 1.0)) then

          !-- use lrc value but weight uncertainty
          x_ap(1) = Tc_sw(ilrc,jlrc)
          Sa(1,1) = 5.0 + (1.0-ec_sw(ilrc,jlrc))*Tc_ap_uncer

       endif
  endif

  !--- square the individual elements to convert to variances (not a matmul)
  Sa = Sa**2

  !--- compute inverse of Sa matrix
  call INVERT_3x3(Sa,Sa_inv,isingular)
  if (isingular == 1) then
    print *, "Cloud Height warning ==> Singular Sa in Split Window", &
           ielem,iline, cloud_type
    ifail = sym%YES
    exit
  endif

!----------------------------------------------------------------------------
! compute clear-sky radiances
!-----------------------------------------------------------------------------

!---------------------------------------------------------------------------
!--- modify clear radiances to simulate that from an opaque cloud at 200 mb
!--- above the surface when a multi-layer situation is suspected
!---------------------------------------------------------------------------

 if (cloud_type == sym%OVERLAP_TYPE .and.  &
     Pc_lower_cloud(ielem,iline) /= missing_value_real4) then

  rad_ac_4 = GENERIC_PROFILE_INTERPOLATION(Zc_lower_cloud(ielem,iline), &
                            Hght_Prof_RTM,Atm_Rad_Prof_11um_RTM)

  trans_ac_4 = GENERIC_PROFILE_INTERPOLATION(Zc_lower_cloud(ielem,iline), &
                            Hght_Prof_RTM,Atm_Trans_Prof_11um_RTM)

  rad_ac_5 = GENERIC_PROFILE_INTERPOLATION(Zc_lower_cloud(ielem,iline), &
                            Hght_Prof_RTM,Atm_Rad_Prof_12um_RTM)

  trans_ac_5 = GENERIC_PROFILE_INTERPOLATION(Zc_lower_cloud(ielem,iline), &
                            Hght_Prof_RTM,Atm_Trans_Prof_12um_RTM)

  Bc_4 = PLANCK_RAD_FAST(4,Tc_lower_cloud(ielem,iline))
  Bc_5 = PLANCK_RAD_FAST(5,Tc_lower_cloud(ielem,iline))

  tsfc_est = Tc_lower_cloud(ielem,iline)

  rad_clear_4 = rad_ac_4 + trans_ac_4*Bc_4
  rad_clear_5 = rad_ac_5 + trans_ac_5*Bc_5

 else

!---------------------------------------------------------------
! if not multi-layer, use existing clear sky radiances
!---------------------------------------------------------------

  tsfc_est = tsfc_rtm(ielem,iline) 
  rad_clear_4 = rad_clear_ch4_rtm_unbiased(ielem,iline)
  rad_clear_5 = rad_clear_ch5_rtm_unbiased(ielem,iline)

 endif

!----------------------------------------------------------------
! if no inversion is present, check to see if clear radiance
! is warmer than observed radiance, if not then do not proceeed
!----------------------------------------------------------------
!if ((rtm(inwp,jnwp)%inversion_level == 0) .and. (rad_clear_4 < rad4(ielem,iline))) then

if (( Inver_Level_RTM == 0) .and. (rad_clear_4 < rad4(ielem,iline))) then

   !-- consider this pixel a failed retrieval
   ifail = sym%YES

   !--- assign default values for this result
   Tc_sw(ielem,iline) = x_ap(1)   !missing_value_real4
   ec_sw(ielem,iline) = x_ap(2)   !missing_value_real4
   beta_sw(ielem,iline) = x_ap(3) !missing_value_real4
   tau_sw(ielem,iline) = tau_ap   !missing_value_real4
   Tc_sw_qf(ielem,iline) = 1
   ec_sw_qf(ielem,iline) = 1
   beta_sw_qf(ielem,iline) = 1

   call KNOWING_T_COMPUTE_P_Z(Pc_sw(ielem,iline),Tc_sw(ielem,iline),Zc_sw(ielem,iline),ilev,ierror)
   if (ierror == 0) then
    Zc_sw(ielem,iline) = max(0.0,Zc_sw(ielem,iline))
    Pc_sw(ielem,iline) = min(psfc_nwp(inwp,jnwp),Pc_sw(ielem,iline))
   endif
   
   !-- go to next pixel
   cycle

endif

!-----------------------------------------------------------------
! start of retrieval loop
!-----------------------------------------------------------------
iter = 0
conv_test = huge(conv_test)
ifail = sym%NO

!---- assign x to the first guess
x = x_ap

retrieval_loop: do

 iter = iter + 1

 !--- check for traditional convergence
 if ( conv_test < conv_crit) then
   ifail = sym%NO
   exit
 endif

 !--- check for exceeding allowed number of interactions
 if (iter > iter_max) then
   ifail = sym%YES
   exit
  endif
 
  !---------------------------------------------------------------------
  ! estimate above cloud radiances and transmissions
  !---------------------------------------------------------------------
  Tc_temp = x(1)

  !----------------------------------------------------------------------
  ! need to consider ways of preventing stratospheric solutions here
  !----------------------------------------------------------------------
  call KNOWING_T_COMPUTE_P_Z(Pc_temp,Tc_temp,Zc_temp,ilev,ierror)

  !--- compute above-cloud terms
  rad_ac_4 = GENERIC_PROFILE_INTERPOLATION(Zc_temp, &
                            Hght_Prof_RTM,Atm_Rad_Prof_11um_RTM)

  trans_ac_4 = GENERIC_PROFILE_INTERPOLATION(Zc_temp, &
                            Hght_Prof_RTM,Atm_Trans_Prof_11um_RTM)

  rad_ac_5 = GENERIC_PROFILE_INTERPOLATION(Zc_temp, &
                            Hght_Prof_RTM,Atm_Rad_Prof_12um_RTM)

  trans_ac_5 = GENERIC_PROFILE_INTERPOLATION(Zc_temp, &
                            Hght_Prof_RTM,Atm_Trans_Prof_12um_RTM)

 !--------------------------------------------------
 ! call forward models
 !--------------------------------------------------
 
 !--- compute planck emission for cloud temperature
 Bc_4 = PLANCK_RAD_FAST(4,Tc_temp,dB_dT = dB_dTc_4)
 Bc_5 = PLANCK_RAD_FAST(5,Tc_temp,dB_dT = dB_dTc_5)

 !-------------------------------------------------
 ! determine beta (only on first interation)
 !-------------------------------------------------

 !----- compute channel emissivities
 emiss_4 = min(x(2),0.999999)
 demiss5_demiss4 = x(3) * (1.0-emiss_4)**(x(3)-1.0)
 emiss_5 = 1.0 - (1.0-emiss_4)**x(3)

 !--- define transmission as complement of emissivity
 trans_4 = (1.0 - emiss_4)
 trans_5 = (1.0 - emiss_5)

 !---------------- forward model for channel 4
 rad_4 = emiss_4*rad_ac_4 + trans_ac_4 * emiss_4 * Bc_4 +  trans_4 * rad_clear_4

 f(1) = PLANCK_TEMP_FAST(4,rad_4,dB_dT = dB_dT_4)

 !---------------- forward model for channel 5
 rad_5 = emiss_5*rad_ac_5 + trans_ac_5 * emiss_5 * Bc_5 +  trans_5 * rad_clear_5

 f(2) = f(1) - PLANCK_TEMP_FAST(5,rad_5,dB_dT = dB_dT_5)

 !------ compute kernel matrix
 K(1,1) = (trans_ac_4 * emiss_4 * dB_dTc_4) / dB_dT_4                                !dT_4 / dT_c
 K(1,2) = (rad_ac_4 + trans_ac_4*Bc_4 - rad_clear_4)/dB_dT_4                         !dT_4 / demiss_c
 K(1,3) = 0.0                                                                        !dT_4 / dbeta
 K(2,1) = K(1,1) - trans_ac_5 * emiss_5 * dB_dTc_5 / dB_dT_5                         !d(T_4 - T_5) / dT_c
 K(2,2) = K(1,2) -  (rad_ac_5+trans_ac_5*Bc_5-rad_clear_5)*(demiss5_demiss4)/dB_dT_5 !d(T_4 - T_5) / demiss_4 
 K(2,3) = (rad_ac_5+trans_ac_5*Bc_5-rad_clear_5)/dB_dT_5  * &
          alog(1.0-emiss_4)*(1.0-emiss_5)                                             !d(T_4 - T_5) / dbeta

 !--- compute Sy 
 T11_cld_uncer = max(1.0, bt4_std_3x3(ielem,iline))
 T11_12_cld_uncer = max(0.5, bt45_std_3x3(ielem,iline))

 Sy = 0.0
 Sy(1,1) = T11_cal_uncer**2 + ((1.0-x(2))*T11_clr_uncer)**2 + (T11_cld_uncer)**2
 Sy(2,2) = T11_12_cal_uncer**2 + ((1.0-x(2))*T11_12_clr_uncer)**2 + (T11_12_cld_uncer)**2
 call INVERT_2x2(Sy,Sy_inv,isingular)
 if (isingular == 1) then
   print *, "Cloud Height warning ==> Singular Sy in Split Window ",ielem,iline
   ifail = sym%YES
   exit
 endif

 !---- compute next step
 Sx_inv = Sa_inv + matmul(transpose(K),matmul(Sy_inv,K)) !(Eq.102 Rodgers)
 call INVERT_3x3(Sx_inv,Sx,isingular)
 if (isingular == 1) then
   print *, "Cloud Height warning ==> Singular Sx in Split Window ",ielem,iline
   ifail = sym%YES
   exit
 endif

 delta_x = matmul(Sx,(matmul(transpose(K),matmul(Sy_inv,(y-f))) +  &
                      matmul(Sa_inv,x_ap-x) ))

 !--------------------------------------------------------------
 ! check for convergence
 !--------------------------------------------------------------

  !--- compute convergence metric
  conv_test = abs(sum(delta_x*matmul(Sx_inv,delta_x)))

  !--- control step size
  do pp = 1,num_param
     if (abs(delta_x(pp)) > abs(x(pp))/2.0) then
       delta_x(pp) = sign( x(pp)/2.0 , delta_x(pp) )
     endif
  end do

  if (abs(delta_x(1)) > 20.0) then
       delta_x(1) = sign( 20.0 , delta_x(1) )
  endif
  if (abs(delta_x(2)) > 0.2) then
       delta_x(2) = sign( 0.2 , delta_x(2) )
  endif
  if (abs(delta_x(3)) > 0.1) then
       delta_x(3) = sign( 0.1 , delta_x(3) )
  endif

  !--- check for non-traditional convergence
  if ((abs(delta_x(1)) < 0.1) .and. (iter > 1)) then
    ifail = sym%NO
    exit
  endif

  !---------------------------------------------------------
  ! update retrieved vector
  !---------------------------------------------------------
  x = x + delta_x

  !-------------------------------------------------------
  ! constrain to reasonable values
  !-------------------------------------------------------
  x(1) = max(min_allowable_Tc,min(tsfc_est,x(1)))     !should we do this?
  x(2) = max(0.0,min(x(2),1.0))
  x(3) = max(0.8,min(x(3),2.0))

end do retrieval_loop

if (ifail == sym%NO) then  !successful retrieval if statement

 Tc_sw(ielem,iline) = x(1)
 beta_sw(ielem,iline) = x(3)
 if (x(2) < 1.00) then
   tau_sw(ielem,iline) = -coszen(ielem,iline)*alog(1.0 - x(2))    !this is the split-window optical depth adjusted to nadir
   ec_sw(ielem,iline) = 1.0 - exp(-tau_sw(ielem,iline))
   tau_sw(ielem,iline) = 2.0 * tau_sw(ielem,iline)    !this is now a "visible" optical depth (assuming 2.0 scale factor)
 else
   tau_sw(ielem,iline) = 20.0
   ec_sw(ielem,iline) = 1.0
   beta_sw(ielem,iline) = 1.3
 endif

!------------------------------------------------------
!--- base quality flag on estimated error
!-- highest quality is sqrt(Sx) < sqrt(Sa)/3  which
!-- means that the estimated error is one third of the 
!-- a priori error
!---------------------------------------------------
!--- cloud temperature quality flag
 Tc_sw_qf(ielem,iline) = 3
 ec_sw_qf(ielem,iline) = 3
 beta_sw_qf(ielem,iline) = 3

else                           

 Tc_sw(ielem,iline) = x_ap(1)   !missing_value_real4
 ec_sw(ielem,iline) = x_ap(2)   !missing_value_real4
 beta_sw(ielem,iline) = x_ap(3) !missing_value_real4
 tau_sw(ielem,iline) = tau_ap   !missing_value_real4
 Tc_sw_qf(ielem,iline) = 1
 ec_sw_qf(ielem,iline) = 1
 beta_sw_qf(ielem,iline) = 1

endif                              !end successful retrieval if statement

if (undetected_cloud == sym%YES) then
 Tc_sw_qf(ielem,iline) = 2
 ec_sw_qf(ielem,iline) = 2
 beta_sw_qf(ielem,iline) = 2
endif

 !--- check for a valid retrieval
 if (ifail == sym%NO) then

  !-- check for valid Tc
  if (Tc_sw(ielem,iline) > 0.0) then

   !--- compute p and z based on retrieved t
   call KNOWING_T_COMPUTE_P_Z(Pc_sw(ielem,iline),Tc_sw(ielem,iline),Zc_sw(ielem,iline),ilev,ierror)

   !--- modify for bottom approach over water
   if ((cloud_type == sym%FOG_TYPE) .or. &
       (cloud_type == sym%wATER_TYPE) .or. &
       (cloud_type == sym%SUPERCOOLED_TYPE)) then

     !-----------------------------------------------------------------------
     ! determine level to stop for low-level inversions 
     !-----------------------------------------------------------------------
     call KNOWING_P_COMPUTE_T_Z(min_P_inversion,r4_dummy,r4_dummy,ilev_min_inversion)

     call KNOWING_P_COMPUTE_T_Z(psfc_nwp(inwp,jnwp) - delta_Psfc_inversion, &
                               r4_dummy, r4_dummy, ilev_max_inversion)

     ilev_max_inversion = max(1,min(int(Sfc_Level_RTM),ilev_max_inversion))
     ilev_min_inversion = max(1,min(ilev_max_inversion,ilev_min_inversion))

     !--- check for water surface
     if (sfc_type(ielem,iline) == sym%WATER_SFC) then

       !--- check for low level inversion
       if ((inversion_level_nwp(inwp,jnwp) >= ilev_min_inversion) .and. &
           (inversion_level_nwp(inwp,jnwp) <= ilev_max_inversion)) then

           !--- compute temp diff cloud and surface
           delta_tc_tsfc = tsfc_rtm(ielem,iline) - Tc_sw(ielem,iline)

           !--- estimate cloud height based on assumed lapse rate
           adjusted_Zc = delta_tc_tsfc / lapse_rate_ocean + zsfc(ielem,iline)

            !--- compute pressure knowing height
            call KNOWING_Z_COMPUTE_T_P(adjusted_Pc,r4_dummy,adjusted_Zc,ilev)

            !--- take values that are closer to the surface
            if (adjusted_Pc > Pc_sw(ielem,iline)) then
                 Pc_sw(ielem,iline) = adjusted_Pc
                 Zc_sw(ielem,iline) = adjusted_Zc
            endif

        endif   !low level inversion check

     endif  !water surface check

   endif  !water cloud check

  endif  !valid Tc check

    !------- determine cloud layer based on pressure
    cld_layer_sw(ielem,iline) = 0
    if (Pc_sw(ielem,iline) <= 440.0) then
     cld_layer_sw(ielem,iline) = 3
    elseif (Pc_sw(ielem,iline) < 680.0) then
      cld_layer_sw(ielem,iline) = 2
    else
       cld_layer_sw(ielem,iline) = 1
    endif

  endif  !valid retrieval check

  !--- ensure undetected cloud is not assigned a layer
  if (undetected_cloud == sym%YES) then
   cld_layer_sw(ielem,iline) = 0
  endif


 endif     ! ---------- end of data check

 end do element_loop

end do line_loop

end do pass_loop

processing_order => null()

end subroutine SPLIT_WIN_CLOUD_BETA

!----------------------------------------------------------------------
! Cloud Height for AVHRR/1.  Use precomputed opaque solutions
! for AVHRR/1 sensors.  Set quality flag to degraded (1)
!----------------------------------------------------------------------
  subroutine  OPAQUE_CLOUD_HEIGHT(jmin,jmax)
                              
  integer, intent(in):: jmin,jmax

  integer:: ielem
  integer:: iline

  !--- initialize output
  Tc_sw =  missing_value_real4
  ec_sw =  missing_value_real4
  beta_sw =  missing_value_real4
  Pc_sw =  missing_value_real4
  Zc_sw =  missing_value_real4
  Tc_sw_qf = 0
  ec_sw_qf = 0
  beta_sw_qf = 0
  cld_layer_sw = 0

  !--------------------------------------------------------------------------
  ! loop over pixels in scanlines
  !--------------------------------------------------------------------------
  line_loop: do iline = jmin,jmax + jmin - 1

    element_loop:   do ielem = 1, num_pix

    !--- check for a bad pixel
    if (bad_pixel_mask(ielem,iline) == sym%YES) then
          cycle
    endif

    if (Tc_opaque_cloud(ielem,iline) /= missing_value_real4) then
       Pc_sw(ielem,iline) = Pc_opaque_cloud(ielem,iline)
       Tc_sw(ielem,iline) = Tc_opaque_cloud(ielem,iline)
       Zc_sw(ielem,iline) = Zc_opaque_cloud(ielem,iline)
       ec_sw(ielem,iline) = 1.0
       Tc_sw_qf(ielem,iline) = 1
       ec_sw_qf(ielem,iline) = 1
       beta_sw_qf(ielem,iline) = 1

       !------- determine cloud layer based on pressure
       if (Pc_sw(ielem,iline) <= 440.0) then
         cld_layer_sw(ielem,iline) = 3
       elseif (Pc_sw(ielem,iline) < 680.0) then
         cld_layer_sw(ielem,iline) = 2
       else
         cld_layer_sw(ielem,iline) = 1
       endif

       !----- set beta passed on temp
       if (Tc_sw(ielem,iline) < 260.0) then
           beta_sw(ielem,iline) = beta_ap_ice
       else
           beta_sw(ielem,iline) = beta_ap_water
       endif

    endif

    end do element_loop

  end do line_loop

  
  end subroutine OPAQUE_CLOUD_HEIGHT

!-------------------------------------------------------------------------------
! Routine to spatially interpret water cloud temperature values to surrounding 
! pixels
!
! input:  interp_flag = 0 (do no interp, assume Zc=2km) /= 0 (do spatial interp)
!-------------------------------------------------------------------------------
  subroutine SPATIALLY_INTERPOLATE_LOWER_CLOUD_POSITION(interp_flag,jmin,jmax)

      integer, intent(in):: interp_flag    
      integer, intent(in):: jmin
      integer, intent(in):: jmax
      integer:: iline
      integer:: ielem
      integer:: inwp
      integer:: jnwp
      integer:: ilev
      integer:: num_elem
      integer:: num_line
      integer:: line_start
      integer:: line_end
      integer:: delem
      integer:: dline
      integer:: ielem_1
      integer:: ielem_2
      integer:: iline_1
      integer:: iline_2
      integer:: count_valid
      integer, dimension(:,:), allocatable:: mask

      num_elem = size(cld_type,1)
      num_line = size(cld_type,2)
      line_start = jmin
      line_end = jmax + jmin - 1

      if (interp_flag == 0) then 

       line_loop_1: do iline = line_start, line_end 
         element_loop_1: do ielem = 1, num_elem 
           if (bad_pixel_mask(ielem,iline) == sym%YES) then
               Pc_lower_cloud(ielem,iline) = missing_value_real4
               cycle
           endif
          inwp = i_nwp(ielem,iline)
          jnwp = j_nwp(ielem,iline)
          Pc_lower_cloud(ielem,iline) = psfc_nwp(inwp,jnwp) - Pc_lower_cloud_offset
        end do element_loop_1
       end do line_loop_1


      else    !if interp_flag > 0 then do a spatial interpolation

        !--- set box width
        delem = 1
        dline = 1

        allocate(mask(num_elem,num_line))

         mask = 0

         where((cld_type == sym%FOG_TYPE .or. &
             cld_type == sym%WATER_TYPE .or. &
             cld_type == sym%SUPERCOOLED_TYPE) .and. &
             Pc_sw /= missing_value_real4)
             mask = 1
         endwhere

         line_loop_2: do iline = line_start, line_end 
             element_loop_2: do ielem = 1, num_elem 

             if (cld_type(ielem,iline)  /= sym%OVERLAP_TYPE) then
                 cycle
             endif

             ielem_1 = min(num_elem,max(1,ielem-delem))
             ielem_2 = min(num_elem,max(1,ielem+delem))
             iline_1 = min(num_line,max(1,iline-dline))
             iline_2 = min(num_line,max(1,iline+dline))

             count_valid = sum(mask(ielem_1:ielem_2,iline_1:iline_2))

             if (count_valid > 0) then 
                 Pc_lower_cloud(ielem,iline) =  &
                              sum(Pc_sw(ielem_1:ielem_2,iline_1:iline_2)* &
                                  mask(ielem_1:ielem_2,iline_1:iline_2)) / count_valid
             endif
         
             end do element_loop_2
          end do line_loop_2

          !--- fill missing values with default value
          line_loop_3: do iline = line_start, line_end 
            element_loop_3: do ielem = 1, num_elem 
             if ((Pc_lower_cloud(ielem,iline) == missing_value_real4) .and. &
               (bad_pixel_mask(ielem,iline) == sym%NO)) then
               inwp = i_nwp(ielem,iline)
               jnwp = j_nwp(ielem,iline)
               Pc_lower_cloud(ielem,iline) = psfc_nwp(inwp,jnwp) - Pc_lower_cloud_offset
             endif
            end do element_loop_3
           end do line_loop_3

      deallocate(mask)

      endif   !end of interp_flag check

      !----------------------------------------------------------------
      !  Compute Height and Temperature
      !----------------------------------------------------------------
      line_loop_4: do iline = line_start, line_end 
         element_loop_4: do ielem = 1, num_elem 

            !-- if a bad pixel, set to missing
            if (bad_pixel_mask(ielem,iline) == sym%YES) then
               Pc_lower_cloud(ielem,iline) = missing_value_real4
               Tc_lower_cloud(ielem,iline) = missing_value_real4 
               Zc_lower_cloud(ielem,iline) = missing_value_real4 
               cycle
            endif

            !--- if not overlap, set to all missing 
            if (cld_type(ielem,iline) /= sym%OVERLAP_TYPE .or. &
                Pc_lower_cloud(ielem,iline) == missing_value_real4) then
                Pc_lower_cloud(ielem,iline) = missing_value_real4
                Zc_lower_cloud(ielem,iline) = missing_value_real4
                Tc_lower_cloud(ielem,iline) = missing_value_real4
                cycle
            endif

            !--- compute T and Z from P
            inwp = i_nwp(ielem,iline)
            jnwp = j_nwp(ielem,iline)
            call KNOWING_P_COMPUTE_T_Z_RTM(inwp,jnwp, &
                                           Pc_lower_cloud(ielem,iline), &
                                           Tc_lower_cloud(ielem,iline), &
                                           Zc_lower_cloud(ielem,iline),ilev)
        end do element_loop_4
       end do line_loop_4

   end subroutine SPATIALLY_INTERPOLATE_LOWER_CLOUD_POSITION
   !-----------------------------------------------------------------------
   ! spatially interpolate profiles which have the 101 level rtm spacing
   !-----------------------------------------------------------------------
   subroutine SPATIALLY_INTERPOLATE_RTM_PROFILES(smooth_nwp, inwp,jnwp, ivza, &
                                                 inwp_x,jnwp_x,lat_x,lon_x)

   integer, intent(in):: smooth_nwp
   integer, intent(in):: inwp
   integer, intent(in):: jnwp
   integer, intent(in):: ivza
   integer, intent(in):: inwp_x
   integer, intent(in):: jnwp_x
   real, intent(in):: lat_x
   real, intent(in):: lon_x

   !--- initialize with closest nwp values
   Temp_Prof_RTM = rtm(inwp,jnwp)%T_prof
   Hght_Prof_RTM = rtm(inwp,jnwp)%Z_prof
   Atm_Rad_Prof_11um_RTM = rtm(inwp,jnwp)%d(ivza)%rad_atm_clr4
   Atm_Rad_Prof_12um_RTM = rtm(inwp,jnwp)%d(ivza)%rad_atm_clr5
   Atm_Trans_Prof_11um_RTM = rtm(inwp,jnwp)%d(ivza)%trans_atm_clr4
   Atm_Trans_Prof_12um_RTM = rtm(inwp,jnwp)%d(ivza)%trans_atm_clr5
   Sfc_Level_RTM = rtm(inwp,jnwp)%sfc_level
   Inver_Level_RTM = rtm(inwp,jnwp)%inversion_level
   Tropo_Level_RTM = rtm(inwp,jnwp)%tropo_level

   !--- if interpolation wanted, see if profiles exist
   if (smooth_nwp == sym%YES .and. &
      rtm(inwp,jnwp)%flag == sym%YES .and. &
      rtm(inwp_x,jnwp)%flag == sym%YES .and. &
      rtm(inwp,jnwp_x)%flag == sym%YES .and. &
      rtm(inwp_x,jnwp_x)%flag == sym%YES) then

      call INTERPOLATE_PROFILE(rtm(inwp,jnwp)%T_prof, &    
                                   rtm(inwp,jnwp_x)%T_prof, &    
                                   rtm(inwp_x,jnwp)%T_prof, &    
                                   rtm(inwp_x,jnwp_x)%T_prof, &    
                                   lon_x, lat_x, &
                                   Temp_Prof_RTM)

      call INTERPOLATE_PROFILE(rtm(inwp,jnwp)%Z_prof, &    
                                   rtm(inwp,jnwp_x)%Z_prof, &    
                                   rtm(inwp_x,jnwp)%Z_prof, &    
                                   rtm(inwp_x,jnwp_x)%Z_prof, &    
                                   lon_x, lat_x, &
                                   Hght_Prof_RTM)

      Tropo_Level_RTM = nint(INTERPOLATE_NWP(real(rtm(inwp,jnwp)%tropo_level), &    
                                             real(rtm(inwp,jnwp_x)%tropo_level), &    
                                             real(rtm(inwp_x,jnwp)%tropo_level), &    
                                             real(rtm(inwp_x,jnwp_x)%tropo_level), &    
                                             lon_x, lat_x))

      Sfc_Level_RTM = nint(INTERPOLATE_NWP(real(rtm(inwp,jnwp)%sfc_level), &    
                                           real(rtm(inwp,jnwp_x)%sfc_level), &    
                                           real(rtm(inwp_x,jnwp)%sfc_level), &    
                                           real(rtm(inwp_x,jnwp_x)%sfc_level), &    
                                           lon_x, lat_x))

    !--- for rtm profiles, make sure profiles and angles exist
    if (smooth_nwp == sym%YES .and. &
        rtm(inwp,jnwp)%d(ivza)%flag == sym%YES .and. &
        rtm(inwp_x,jnwp)%d(ivza)%flag == sym%YES .and. &
        rtm(inwp,jnwp_x)%d(ivza)%flag == sym%YES .and. &
        rtm(inwp_x,jnwp_x)%d(ivza)%flag == sym%YES) then

        call INTERPOLATE_PROFILE( &
                            rtm(inwp,jnwp)%d(ivza)%rad_atm_clr4, &    
                            rtm(inwp,jnwp_x)%d(ivza)%rad_atm_clr4, &    
                            rtm(inwp_x,jnwp)%d(ivza)%rad_atm_clr4, &    
                            rtm(inwp_x,jnwp_x)%d(ivza)%rad_atm_clr4, &    
                            lon_x, lat_x, &
                            Atm_Rad_Prof_11um_RTM)

        call INTERPOLATE_PROFILE( &
                            rtm(inwp,jnwp)%d(ivza)%trans_atm_clr4, &    
                            rtm(inwp,jnwp_x)%d(ivza)%trans_atm_clr4, &    
                            rtm(inwp_x,jnwp)%d(ivza)%trans_atm_clr4, &    
                            rtm(inwp_x,jnwp_x)%d(ivza)%trans_atm_clr4, &    
                            lon_x, lat_x, &
                            Atm_Trans_Prof_11um_RTM)

        call INTERPOLATE_PROFILE( &
                            rtm(inwp,jnwp)%d(ivza)%rad_atm_clr5, &    
                            rtm(inwp,jnwp_x)%d(ivza)%rad_atm_clr5, &    
                            rtm(inwp_x,jnwp)%d(ivza)%rad_atm_clr5, &    
                            rtm(inwp_x,jnwp_x)%d(ivza)%rad_atm_clr5, &    
                            lon_x, lat_x, &
                            Atm_Rad_Prof_12um_RTM)

        call INTERPOLATE_PROFILE( &
                            rtm(inwp,jnwp)%d(ivza)%trans_atm_clr5, &    
                            rtm(inwp,jnwp_x)%d(ivza)%trans_atm_clr5, &    
                            rtm(inwp_x,jnwp)%d(ivza)%trans_atm_clr5, &    
                            rtm(inwp_x,jnwp_x)%d(ivza)%trans_atm_clr5, &    
                            lon_x, lat_x, &
                            Atm_Trans_Prof_12um_RTM)

     endif
   endif

   end subroutine SPATIALLY_INTERPOLATE_RTM_PROFILES

   !-----------------------------------------------------------------
   ! Interpolate within profiles knowing P to determine T and Z
   !-----------------------------------------------------------------
   subroutine KNOWING_P_COMPUTE_T_Z(P,T,Z,ilev)

     real, intent(in):: P
     real, intent(out):: T
     real, intent(out):: Z
     integer, intent(out):: ilev
     real:: dp, dt, dz

     !--- interpolate pressure profile
     call LOCATE(P_std_rtm,nlevels_rtm,P,ilev)
     ilev = max(1,min(nlevels_rtm-1,ilev))

     dp = P_std_rtm(ilev+1) - P_std_rtm(ilev)
     dt = Temp_Prof_RTM(ilev+1) - Temp_Prof_RTM(ilev)
     dz = Hght_Prof_RTM(ilev+1) - Hght_Prof_RTM(ilev)

     !--- perform interpolation
       if (dp /= 0.0) then
           T = Temp_Prof_RTM(ilev) + dt/dp * (P - P_std_rtm(ilev))
           Z = Hght_Prof_RTM(ilev) + dz/dp * (P - P_std_rtm(ilev))
       else
           T = Temp_Prof_RTM(ilev)
           Z = Hght_Prof_RTM(ilev)
       endif
   end subroutine KNOWING_P_COMPUTE_T_Z

   !-----------------------------------------------------------------
   ! Interpolate within profiles knowing Z to determine T and P
   !-----------------------------------------------------------------
   subroutine KNOWING_Z_COMPUTE_T_P(P,T,Z,ilev)

     real, intent(in):: Z
     real, intent(out):: T
     real, intent(out):: P
     integer, intent(out):: ilev
     real:: dp, dt, dz
     integer:: kstart
     integer:: kend
     integer:: nlevels_temp

     !--- compute region of atmosphere to look
     kstart = Tropo_Level_RTM
     kend = Sfc_Level_RTM
     nlevels_temp = kend - kstart + 1

     !--- interpolate pressure profile
     call LOCATE(Hght_Prof_RTM,nlevels_rtm,Z,ilev)
     ilev = max(1,min(nlevels_rtm-1,ilev))
     ilev = ilev + kstart - 1

     dp = P_std_rtm(ilev+1) - P_std_rtm(ilev)
     dt = Temp_Prof_RTM(ilev+1) - Temp_Prof_RTM(ilev)
     dz = Hght_Prof_RTM(ilev+1) - Hght_Prof_RTM(ilev)

     !--- perform interpolation
       if (dz /= 0.0) then
           T = Temp_Prof_RTM(ilev) + dt/dz * (Z - Hght_Prof_RTM(ilev))
           P = P_std_rtm(ilev) + dp/dz * (Z - Hght_Prof_RTM(ilev))
       else
           T = Temp_Prof_RTM(ilev)
           P = P_std_rtm(ilev)
       endif
   end subroutine KNOWING_Z_COMPUTE_T_P

   !-----------------------------------------------------------------
   ! Interpolate within profiles knowing T to determine P and Z
   !-----------------------------------------------------------------
   subroutine KNOWING_T_COMPUTE_P_Z(P,T,Z,klev,ierr)
     real, intent(in):: T
     real, intent(out):: P
     real, intent(out):: Z
     integer, intent(out):: klev
     integer, intent(out):: ierr
     real:: dp, dt, dz
     integer:: kstart
     integer:: kend
     integer:: nlevels_temp
     integer:: level_within_inversion_flag

     !--- initialization
     ierr = sym%NO
     Z = missing_value_real4
     P = missing_value_real4

     !--- test for existence of a valid solution with troposphere
     kstart = Tropo_Level_RTM
     kend = Sfc_Level_RTM
     nlevels_temp = kend - kstart + 1

     !--- check to see if warmer than max, than assume at surface
     if (T > maxval(Temp_Prof_RTM(kstart:kend))) then
         P = P_std_rtm(kend)
         Z = Hght_Prof_RTM(kend)
         klev = kend - 1
         ierr = sym%NO
         return
     endif

     !--- check to see if colder than min, than assume tropopause
     if (T < minval(Temp_Prof_RTM(kstart:kend))) then
         P = P_std_rtm(kstart)
         Z = Hght_Prof_RTM(kstart)
         klev = kstart + 1
         ierr = sym%NO
         return
     endif

     !--- if there is an inversion, look below first
     level_within_inversion_flag = 0
     if (Inver_Level_RTM > 0 .and. Inver_Level_RTM < Sfc_Level_RTM) then
         kstart = Inver_Level_RTM
         kend =  Sfc_Level_RTM
         nlevels_temp = kend - kstart + 1
         call LOCATE(Temp_Prof_RTM(kstart:kend),nlevels_temp,T,klev)
         if ((klev > 0) .and. (klev < nlevels_temp -1)) then
              klev = klev + kstart - 1
              level_within_inversion_flag = 1
         endif
      endif

    !--- if no solution within an inversion, look above
    if (level_within_inversion_flag == 0) then
        kstart = Tropo_Level_RTM
        kend = Sfc_Level_RTM
        nlevels_temp = kend - kstart + 1
        call LOCATE(Temp_Prof_RTM(kstart:kend),nlevels_temp,T,klev)
        klev = klev + kstart - 1
        klev = max(1,min(nlevels_rtm-1,klev))
    endif

    !-- if solution is above trop, set to trop values
    if (klev < Tropo_Level_RTM) then
        P = P_std_rtm(Tropo_Level_RTM)
        Z = Hght_Prof_RTM(Tropo_Level_RTM)

    else

        !--- determine derivatives
        dp = P_std_rtm(klev+1) - P_std_rtm(klev)
        dt = Temp_Prof_RTM(klev+1) - Temp_Prof_RTM(klev)
        dz = Hght_Prof_RTM(klev+1) - Hght_Prof_RTM(klev)

        if (dt /= 0.0) then
         P = P_std_rtm(klev) + dp/dt*(T-Temp_Prof_RTM(klev))
         Z = Hght_Prof_RTM(klev) + dz/dt*(T-Temp_Prof_RTM(klev))
        else
         P = P_std_rtm(klev) 
         Z = Hght_Prof_RTM(klev)
        endif

   !--- end above trop check
   endif

   end subroutine KNOWING_T_COMPUTE_P_Z

   !-----------------------------------------------------------------
   ! Interpolate within profiles knowing Z to determine above cloud
   ! radiative terms used in forward model
   !-----------------------------------------------------------------
   function GENERIC_PROFILE_INTERPOLATION(X_value,X_profile,Y_profile)  &
            result(Y_value) 

     real, intent(in):: X_value 
     real, dimension(:), intent(in):: X_profile
     real, dimension(:), intent(in):: Y_profile
     real:: Y_value

     integer:: ilev
     real:: dx
     integer:: nlevels


     nlevels = size(X_profile)

     !--- interpolate pressure profile
     call LOCATE(X_profile,nlevels,X_value,ilev)
     ilev = max(1,min(nlevels-1,ilev))

     dx = X_profile(ilev+1) - X_profile(ilev)

     !--- perform interpolation
     if (dx /= 0.0) then
        Y_value = Y_profile(ilev) +  &
                 (X_value - X_profile(ilev))  * &
                 (Y_profile(ilev+1) - Y_profile(ilev)) / dx
     else
          Y_value = Y_profile(ilev)
     endif
   end function GENERIC_PROFILE_INTERPOLATION

!----------------------------------------------------------------------
! End of Module
!----------------------------------------------------------------------

end module CLOUD_HEIGHT_11_12_1DVAR
