! $Id: cloud_tau_re_solar.f90,v 1.39 2010/07/30 21:16:25 awalther Exp $ 
!
MODULE CLOUD_TAU_RE_SOLAR_MODULE 

!----------------------------------------------------------------------
!@***h* PATMOSX/cloud_tau_re_solar.f90
! 
! NAME
!   cloud_tau_re_solar.f90
! 
! LOCATION
!   /patmosx/src/cloud_tau_re_solar.f90
! 
! PURPOSE
!   This module contains the routine to compute daytime microphysical 
!   properties (DCOMP)
! 
! DESCRIPTION
!  This function retrieves the cloud  microphysical properties optical depth,
!  effective radius and water path (liquid and solid)
!  This algorithm uses the 0.6 and 3.75 (1.6 for 3a periods) micron channels
!  with optimal estimation inversion to retrieve the cloud microphysics 
!
! AUTHOR
!   Andi Walther 
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!  Reference: The Cloud Application Team Cloud DAYTIME Microphysical 
!             properties ATBD (DCOMP)
!
! Calling Sequence:
!           USE BASELINE_CLOUD_MICRO_DAY
!           CALL BASELINE_CLOUD_MICRO_DAY_MAIN(ialgo)
!           where ialgo is the GEOCAT algorithm index. All other input/output
!           is passed via global module variables. 
!           Called after cloud mask, cloud type and cloud height
!
! EXTERNALS
!  Dependencies:
!   ALGORITHM_MODULE_USAGE --> a module that lists all of the modules that need
!                              to be included
!   ABI cloud mask --> the ABI cloud mask must be processed first and available
!                      in the sat%cldmask variable
!   ABI cloud type --> the ABI cloud type must be processed first and available
!                      in the sat%cldtype variable
!   ABI cloud pressure --> the ABI cloud top pressure must be processed first and available
!                      in the sat%cldp variable
!   ABI cloud temperature --> the ABI cloud top temperature must be processed first and available
!                      in the sat%cldt variable
!
!   HDF_DCOMP     ---> a module that contains all HDF routines to enable access to main amd anillary daya LUTs.
!                       This module is stored in DCOMP_LIB library path.
!
!                     Routines in HDF_DCOMP module those are called in this module:
!                             - POPULATE_ALL_LUT      --> called once at the first segment to populate data structure
!                             - GET_LUT_DATA          --> Access routine 
!
!        GEOCAT satellite, RTM,  and NWP structures must be
!        populated for each segment
!
! INPUTS_OUTPUTS
!  Inputs: All input passed through geocat structures 
!         (SATELLITE, NWP, RTM and temporal)
!
!  Outputs:   
!        COD = out2(ialgo)%cod_vis = cloud optical depth
!        CPS = out2(ialgo)%cldreff = cloud particle size
!        LWP = out2(ialgo)%cldlwp  = liquid water path
!        IWP = out2(ialgo)%cldiwp  = ice water path
!        QC = out2(ialgo)%qcflg1   = quality flags
!
! MODIFICATION HISTORY
!   Version        Date           Comment
!   - - - -      - - - -        - - - - - -
!     v1           2007          initial version Andrew Heidinger
!     v3           02/02/2009    transformed code to NOAA standard
!                  05/31/2009    bug fix, airmass was twice included in tau 
!                                calculations of h2o  (AW)
!                  02/18/2010    revised
!                  03/26/2010    major revision. outsource hdf routines 
!                                 and allocatable arrays (AW)!
!                  04/23/2010    new get_lut_data subroutine in dcomp_lib library
!
! ATTRIBUTES
!   Language : Fortran 90
!   Software Standards: NOAA/NESDIS STAR WI-12.1.1
!
!@*****
!----------------------------------------------------------------------
! This module contains the routine to compute daytime microphysical 
!      properties (DCOMP)
!
! Author: Andi Walther 
!
! Module name:
! Baseline_Cloud_Micro_Day
!
! Function:
!          This function retrieves the cloud  microphysical properties optical depth
!          ,effective radius and water path (liquid and solid)
!
! Description: This algorithm uses the 0.6 and 3.75 (1.6 for 3a periods) micron channels
!              with optimal estimation inversion to retrieve the 
!              cloud microphysics 
!
! Reference: The Cloud Application Team Cloud DAYTIME Microphysical 
!             properties ATBD (DCOMP)
!
! Calling Sequence:
!           USE BASELINE_CLOUD_MICRO_DAY
!           CALL BASELINE_CLOUD_MICRO_DAY_MAIN
!          
!           All other input/output is passed via global module variables. 
!           Called after cloud mask, cloud type and cloud height
!
! Inputs: All input passed through geocat structures 
!         (SATELLITE, NWP, RTM and temporal)
!
! Outputs:   
!        COD = cloud optical depth
!        CPS = cloud particle size
!        LWP = liquid water path
!        IWP = ice water path
!        QC = = quality flags
!
!
! Dependencies:
!   ALGORITHM_MODULE_USAGE --> a module that lists all of the modules that need
!                              to be included
!   ABI cloud mask --> the ABI cloud mask must be processed first and available
!                      in the sat%cldmask variable
!   ABI cloud type --> the ABI cloud type must be processed first and available
!                      in the sat%cldtype variable
!   ABI cloud pressure --> the ABI cloud top pressure must be processed first and available
!                      in the sat%cldp variable
!   ABI cloud temperature --> the ABI cloud top temperature must be processed first and available
!                      in the sat%cldt variable
!
!    HDF_DCOMP     ---> a module that contains all HDF routines to enable access to main amd anillary daya LUTs.
!                       This module is stored in DCOMP_LIB library path.
!
!                     Routines in HDF_DCOMP module those are called in this module:
!                             - POPULATE_ALL_LUT      --> called once at the first segment to populate data structure
!                             - GET_LUT_DATA          --> Access routine 
!
!        CLAVR-x satellite, RTM,  and NWP structures must be
!        populated for each segment
!
! Restrictions:
!
! History:
!
!   Version        Date           Comment
!   - - - -      - - - -        - - - - - -
!
!     v1           2007          initial version Andrew Heidinger
!     v3           02/02/2009    transformed code to NOAA standard
!                  05/31/2009    bug fix, airmass was twice included in tau 
!                                calculations of h2o  (AW)
!                  02/18/2010    revised
!                  03/26/2010    major revision. outsource hdf routines 
!                                 and allocatable arrays (AW)!
!                  04/23/2010    new get_lut_data subroutine in dcomp_lib library 
!                  
! CODE DESCRIPTION
!
!   Language : Fortran 90
!   Software Standards: NOAA/NESDIS STAR WI-12.1.1
!
!  
!-------------------------------------------------------------------------

! -- MODULES USED
USE ALGORITHM_MODULE_USAGE 
USE HDF_DCOMP

IMPLICIT NONE

PRIVATE :: ASSIGN_LOCAL_VAR
PRIVATE :: ATMOSPHERIC_CORRECTION
PRIVATE :: COD_FIX_CPS
PRIVATE :: COMPUTE_WATER_TRANS
PRIVATE :: TRANS_WV_FROM_COEFF
PRIVATE :: OPTIMAL_ESTIMATION
PRIVATE :: FORWARD_MODEL
PRIVATE :: FORWARD_MODEL_SINGLE_CHANNEL
PRIVATE :: FORWARD_MODEL_SINGLE_CHANNEL_EMISSIVITY
PRIVATE :: DERIVATION
PRIVATE :: INTERPOLATE_2D
PRIVATE :: INTERPOLATE_3D
PRIVATE :: INTERPOLATE_4D
PRIVATE :: INTERPOLATE_5D
PRIVATE :: GET_ALBEDO_TRANS_VIS

                 
!--- module main SUBROUTINE
PUBLIC:: CLOUD_TAU_RE_SOLAR
                             
!---------------------------------------------------------------
!  --- module-level variables
!---------------------------------------------------------------
 
! -- pointers to NWP data structures
REAL(KIND=Real4), DIMENSION(:), POINTER :: Temp_Prof_Nwp
REAL(KIND=Real4), DIMENSION(:), POINTER :: Press_Prof_Nwp
REAL(KIND=Real4), DIMENSION(:), POINTER :: Hgt_Prof_Nwp
REAL(KIND=Real4), DIMENSION(:), POINTER :: WV_Prof_Nwp
 
! -- NWP variables
REAL(KIND=Real4) :: Wv_Path_Nwp
REAL(KIND=Real4) :: Ozone_Path_Nwp
REAL(KIND=Real4) :: Sfc_Press_Nwp  
   
! -- Sun/satellite observation geometry variables
REAL(KIND=Real4) :: Sat_Zen
REAL(KIND=Real4) :: Sol_Zen
REAL(KIND=Real4) :: Cos_Sat_Zen
REAL(KIND=Real4) :: Cos_Sol_Zen
REAL(KIND=Real4) :: Rel_Azi
REAL(KIND=Real4) :: Scat_Zen
INTEGER(KIND=Int1) :: Is_Snow   ! - snow flag

! -- Other cloud products 
INTEGER(KIND=Int4) :: Cld_Phase
REAL(KIND=Real4)   :: Cld_Top_Temp
REAL(KIND=Real4)   :: Cld_Top_Press
REAL(KIND=Real4)   :: Cld_Top_Hgt

INTEGER::Idx_lev
REAL::Prof_Wgt

! -- other values
REAL(KIND=Real4) :: Refl_Variab      !- Spatial reflectance variability 
REAL(KIND=Real4) :: Alb_Sfc_Chn_IR   !- Albedo Surface IR Channel 
REAL(KIND=Real4) :: Alb_Sfc_Chn_VIS  !- Albedo Surface VIS Channel
 
!- reflectance variability for segment
REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE :: Vis_Stddev_3X3 

! -- Channel reflectance
REAL(KIND=Real4) :: Refl_VIS  ! reflectance visible channel defined between [0,1]
REAL(KIND=Real4) :: Refl_IR   ! reflectance IR channel defined between [0,1]

! --  Defines global variables
INTEGER(Kind=Int4) :: Fail_Flag  ! - Fail flag

INTEGER, PARAMETER :: NUM_CHN = 2.
INTEGER, PARAMETER :: NUM_OBS = 2.
INTEGER, PARAMETER :: NUM_PHASE = 2 ! -- number of allowable phases

! -- Angle LUT points and weights
INTEGER::Sol_Pos
INTEGER::Sol2_Pos
INTEGER::Sat_Pos
INTEGER::Azi_Pos
REAL(KIND=REAL4) :: Sol_Wgt
REAL(KIND=REAL4) :: Sol2_Wgt
REAL(KIND=REAL4) :: Sat_Wgt
REAL(KIND=REAL4) :: Azi_Wgt

INTEGER :: VIS_CHN_NUMBER 
INTEGER :: IR_CHN_NUMBER 

INTEGER,PARAMETER::VIS_CHN_IDENT_1 = 1
INTEGER,PARAMETER::IR_CHN_IDENT_3A = 2
INTEGER,PARAMETER::IR_CHN_IDENT_3B = 3

CHARACTER(LEN=1024):: Lut_Path  !- directory where LUTs are stored  
! -- look-up-tables structure 

CHARACTER(LEN=20)::sensor_name 

! - CLAVRX variables for IR chnnel at 3.75 micron
REAL(KIND=REAL4)::RTM_Rad_Clr_IR
REAL(KIND=REAL4)::emiss_sfc_7

REAL(KIND=REAL4)::RTM_Trans_Full_IR
REAL(KIND=REAL4)::RTM_Trans_AC_Zen_IR
REAL(KIND=REAL4)::RTM_Trans_AC_IR
REAL(KIND=REAL4)::RTM_Rad_AC_Zen_IR
REAL(KIND=REAL4)::Rad_To_Refl_Fac


!--- include parameters
INCLUDE 'cloud_tau_re_solar.inc'

CONTAINS
 
!---------------------------------------------------------------
! Baseline Cloud Daytime Microphysical Properties from GOES-R AWG 
!              Cloud Application Team
!
! Principal Author: Andi Walther
!
! SUBROUTINE NAME: BASELINE_CLOUD_MICRO_DAY_MAIN
!
! FUNCTION:  
!  Generates baseline cloud optical daytime properties 
!            
! DESCRIPTION:
!      This main routine contains 
!                  1.  Call of population routines for LUTs
!                  2.  Pixel loop
!                  3.  Populates output pointer variables 
!                  4.  Postprocessing
!
! Calling SEQUENCE:
!     CALL BASELINE_CLOUD_MICRO_DAY_MAIN 
!
! INPUTS: 
!        Algo_Idx:  Algorithm index in Geocat
!
!       All other input passed through geocat structures (satellite, NWP, 
!         rtm and temporal)
!    
!
! OUTPUT: see module description
!
! REFERENCES: 
!          see module description
!
! DEPENDENCIES: 
!          see module description 
!
!
! GLOBAL VARIABLES:
!
!   NUM_OBS : number of observation parameters
!   NUM_CHN : number of channels used
!   VIS_SDDEV_3x3
!
! LOCAL VARIABLES:
!  
!   Algo_Num  : algorithm 
!   Line_Idx  : line index
!   Elem_idx  : Element index
!
!====================================================================
SUBROUTINE CLOUD_TAU_RE_SOLAR(jmin,jmax,iseg,nseg)   
 
IMPLICIT NONE
  
!--- input
 
INTEGER, INTENT(in):: jmin  !-clavrx
INTEGER, INTENT(in):: jmax  !-clavrx
INTEGER, INTENT(in):: iseg  !-clavrx
INTEGER, INTENT(in):: nseg  !-clavrx

!--- local variables
INTEGER(Kind=Int4) :: Line_Idx              !- line index   loop variable
INTEGER(Kind=Int4) :: Elem_Idx              !- element index  loop variable
INTEGER :: Cycle_Status                     !- error status
 
REAL, DIMENSION(NUM_OBS):: Obs_Vec              !- observation vector
REAL, DIMENSION(NUM_CHN):: State_Vec            !- atmospheric state vector 
REAL(KIND=Real4) :: Quality                             !- quality parameter
REAL(KIND=Real4) ::Trans_Above_Cld_Chn_VIS  !- Atm trans above cloud VIS
REAL(KIND=Real4) ::Trans_Above_Cld_Chn_IR       !- Atm trans above cloud IR   
REAL(KIND=Real4) ::Trans_Below_Cld_Chn_VIS  !- trans below cloud VIS
REAL(KIND=Real4) ::Trans_Below_Cld_Chn_IR       !- trans below cloud IR
REAL(KIND=Real4) :: Refl_Sing_Scat_Chn_VIS  !- Back Scattering Reflectance VIS
INTEGER(KIND = int1) :: Atm_Level                       !- below or Above cloud
REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE:: Optional_Real4_1 !  dummy 
REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE:: Optional_Real4_2 !  dummy 
REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE:: Optional_Real4_3 !  dummy 

!--- local pointers to geocat output pointers
REAL(KIND=Real4),       DIMENSION(:,:),   POINTER:: COD      !- Cloud optical depth
REAL(KIND=Real4),       DIMENSION(:,:),   POINTER:: CPS      !- Cloud particle size
REAL(KIND=Real4),       DIMENSION(:,:),   POINTER:: LWP          !- Liquid water path
REAL(KIND=Real4),       DIMENSION(:,:),   POINTER:: IWP          !- Ice water path 
INTEGER(KIND=Int1), DIMENSION(:,:), POINTER:: QC_COD         !- Quality class
INTEGER(KIND=Int1), DIMENSION(:,:), POINTER:: QC_REF         !- Quality class

CHARACTER(LEN=2),DIMENSION(3):: Sensor_Channel_Num !- channel number as appeared in file name 
CHARACTER(LEN=10) :: NOAA_Sat_string
REAL(KIND=real4),DIMENSION(2)::diag  

REAL(KIND=real4)::cost
REAL(KIND=real4),DIMENSION(2)::Sx

!- CLAVRX 
REAL(KIND=real4)::cloud_albedo_VIS      
REAL(KIND=real4)::cloud_transmission_VIS_solar
REAL(KIND=real4)::cloud_transmission_VIS_view
REAL(KIND=real4)::cloud_spherical_albedo


!----------------------------------------------------------------------
! Executable Code  
!----------------------------------------------------------------------
 
!--- Set local pointers to output structures
COD   => tau_vis                                              ! CLAVRX
CPS   => re_vis                                               ! CLAVRX
LWP   => lwp_vis                                              ! CLAVRX
IWP   => iwp_vis                                              ! CLAVRX
QC_COD    => tau_vis_qf                                           ! CLAVRX
QC_REF    => re_vis_qf                                           ! CLAVRX
 
!--- Initialize to missing
COD       = MISSING_VALUE_REAL4
CPS      = MISSING_VALUE_REAL4
LWP       = MISSING_VALUE_REAL4
IWP       = MISSING_VALUE_REAL4
QC_COD        = 0
QC_REF        = 0

!-- CLAVRX 
Cloud_063um_transmission_solar = MISSING_VALUE_REAL4
Cloud_063um_transmission_view  = MISSING_VALUE_REAL4
Cloud_063um_albedo                 = MISSING_VALUE_REAL4
Cloud_063um_spherical_albedo   = MISSING_VALUE_REAL4
    
!---- Populate LUT structures at first segment
IF (Iseg == 1)  THEN 
  PRINT "(a,'Start DCOMP algorithm ')",EXE_PROMPT
  
  ! -  Sensor Name from CLAVRX
  Sensor_Name = 'AVHRR'
  NOAA_Sat_string = 'not_yet_set'
  ! --   Set LUT file
  Lut_Path = TRIM(ancil_data_dir)//"/luts/cld/"
  
  ! - Define Channel Sets 
  SELECT CASE (TRIM(Sensor_Name))
    
        CASE('AVHRR')
           VIS_CHN_NUMBER = VIS_CHN_IDENT_1
           IR_CHN_NUMBER = IR_CHN_IDENT_3B  ! initial
           Sensor_Channel_Num(1) = '1'
           Sensor_Channel_Num(2) = '3a'
           Sensor_Channel_Num(3) = '3b'
           
           IF (sc_id_new .LT. 15) Sensor_Channel_Num(2) = 'no'
           
           IF (sc_id_new .LT. 4) THEN
             IF (sc_id_new .EQ. 2 ) THEN
			   Sensor_Channel_Num(2) = '3a'
               NOAA_Sat_String = 'METOP-A'
             ENDIF
            ! IF (sc_id_new .EQ. 2 ) THEN
            !   NOAA_Sat_String = 'METOP-B'
             !    ENDIF
       ENDIF
           
           IF ( sc_id_new .GT. 4 ) THEN
             WRITE(unit=NOAA_sat_string,FMT="(a5,i2.2)") "NOAA-",sc_id_new
           ENDIF
                           
    !- ( SEVIRI 0.6/1.6)
    CASE('Meteosat-8') 
           VIS_CHN_NUMBER = 2
           IR_CHN_NUMBER = 5 
           
        !- ( SEVIRI 0.6/1.6)   
        CASE('Meteosat-9')  
           VIS_CHN_NUMBER = 2
           IR_CHN_NUMBER = 5 
           
         ! - ABI 0.6/2.2  
         CASE ('GOES-ABI')
           VIS_CHN_NUMBER = 2
           IR_CHN_NUMBER = 6 
           
         CASE DEFAULT
           VIS_CHN_NUMBER = 2
           IR_CHN_NUMBER = 6 
           
  END SELECT      
   
  
  
  ! - Put LUT data into memory in hdf_dcomp module
  CALL POPULATE_ALL_LUT(LUT_Path,sensor_name, Sensor_Channel_Num,NOAA_sat_string)
        
ENDIF 
     
!--- Spatial homogeneity parameter
!--- Calls a routine COMPUTE_SPATIAL_UNIFORMITY  in num_mod module
!  - it is done in the clavrx software

!CALL COMPUTE_SPATIAL_UNIFORMITY(1, 1, Sat%Space_Mask, Sat%Ref2, &
!                                 Optional_Real4_1, Optional_Real4_2 &
 !                                ,Optional_Real4_3, Vis_Stddev_3X3)
  
   
!--- Loop over line and elements
Line_Loop: DO Line_Idx=1, jmax+jmin-1
  
  IF (ch3a_on(Line_Idx) .GT. 0) THEN
    !use 1.6 micron
        IR_CHN_NUMBER = IR_CHN_IDENT_3A
  ELSE
    !use 3.75 micron total reflectance
        IR_CHN_NUMBER = IR_CHN_IDENT_3B
  ENDIF
        
  Element_Loop: DO Elem_Idx=1, num_pix
          
    !-- Set aliases and check validity 
    CALL ASSIGN_LOCAL_VAR(Elem_Idx,Line_Idx,Cycle_Status)
        IF (Cycle_Status == 1 ) CYCLE
		  !  cycle status =2 means observation but no cloud
		IF (Cycle_Status .EQ. 2 ) THEN
		  cloud_063um_transmission_solar(Elem_Idx,Line_Idx)  = 1.
          cloud_063um_transmission_view(Elem_Idx,Line_Idx)   = 1.
          cloud_063um_albedo(Elem_Idx,Line_Idx)                  = 0.
          cloud_063um_spherical_albedo(Elem_Idx,Line_Idx)    = 0.   
		  CYCLE
		
		ENDIF 
           
        !-- Atmospheric correction above cloud     
        Atm_Level = LAYER_ABOVE_CLOUD
        CALL ATMOSPHERIC_CORRECTION(Atm_Level,Trans_Above_Cld_Chn_VIS &
                ,Trans_Above_Cld_Chn_IR,Refl_Sing_Scat_Chn_VIS)  
          
        !-- Define observation vector for top of cloud
        Obs_Vec(1) = (Refl_VIS - Refl_Sing_Scat_Chn_VIS) &
                         / (Trans_Above_Cld_Chn_VIS)
        
        Obs_Vec(2) = Refl_IR / (Trans_Above_Cld_Chn_IR)
         
         
        
        !- Atmospheric correction below cloud 
        Atm_Level=LAYER_BELOW_CLOUD
        CALL ATMOSPHERIC_CORRECTION(Atm_Level , Trans_Below_Cld_Chn_VIS &
             , Trans_Below_Cld_Chn_IR , Refl_Sing_Scat_Chn_VIS)  
        
        ! -- Virtual surface albedo to correct atmosphere below cloud   
    Alb_Sfc_Chn_IR =  Alb_Sfc_Chn_IR * Trans_Below_Cld_Chn_IR
    Alb_Sfc_Chn_VIS =  Alb_Sfc_Chn_VIS * Trans_Below_Cld_Chn_VIS
         
    ! Inversion to find atmospheric state vector from observation vector 
         CALL OPTIMAL_ESTIMATION( Obs_Vec , State_Vec , Quality,diag,cost,Sx)  
         
        ! dcomp_atmos_correct_AC_VIS(Elem_Idx,Line_Idx) =   Refl_VIS/Obs_Vec(1)
        ! dcomp_atmos_correct_AC_IR(Elem_Idx,Line_Idx)  =   Refl_IR/Obs_Vec(2)
		
         
        ! Populate result pointers
        QC_COD(Elem_Idx,Line_Idx)  = Quality
		QC_REF(Elem_Idx,Line_Idx)  = Quality
        
        IF (Fail_Flag == Sym%NO) THEN
          QC_COD(Elem_Idx,Line_Idx)  = 3
		  QC_REF(Elem_Idx,Line_Idx)  = 3
          COD(Elem_Idx,Line_Idx)   = 10 ** State_Vec(1)
          CPS(Elem_Idx,Line_Idx)   = 10 ** State_Vec(2)
	  
	      error_cov_matrix_cod(Elem_Idx,Line_Idx)= (10 ** (State_Vec(1) + (Sx(1)/2.))) -  (10 ** (State_Vec(1) - (Sx(1)/2.))) 
          error_cov_matrix_ref(Elem_Idx,Line_Idx)= (10 ** (State_Vec(2) + (Sx(2)/2.))) -  (10 ** (State_Vec(2) - (Sx(2)/2.))) 
		  
		  cost_dcomp(Elem_Idx,Line_Idx) = cost 

          !-- use Bennartz and Heymsfield relations for LWP and IWP
          LWP(Elem_Idx,Line_Idx)         = &
                          MAX(0.0,0.833 * 0.65*COD(Elem_Idx,Line_Idx) &
                                   * CPS(Elem_Idx,Line_Idx))
                                   
           IWP(Elem_Idx,Line_Idx)         = &
                         MAX(0.0,(COD(Elem_Idx,Line_Idx)**(1.0/0.84))/0.065)
						 
		IF ((glintzen(Elem_Idx,Line_Idx) .LT. 25.) .AND. ((land_mask(Elem_Idx,Line_Idx) == sym%NO)) ) THEN
		    QC_REF(Elem_Idx,Line_Idx)  = 2
		
		END IF				 
                                 
                                 
          CALL GET_ALBEDO_TRANS_VIS(Cld_Phase &
                                     , State_Vec   &
                                     , Cloud_albedo_VIS &
                                     , cloud_transmission_VIS_solar &
                                     , cloud_transmission_VIS_view &
                                     , cloud_spherical_albedo )  
                                                         
          cloud_063um_transmission_solar(Elem_Idx,Line_Idx)  = cloud_transmission_VIS_solar
          cloud_063um_transmission_view(Elem_Idx,Line_Idx)   = cloud_transmission_VIS_view
          cloud_063um_albedo(Elem_Idx,Line_Idx)                  = cloud_albedo_VIS
          cloud_063um_spherical_albedo(Elem_Idx,Line_Idx)    = cloud_spherical_albedo                                                                                                    
    
    END IF      
          
  END DO Element_Loop
END DO Line_Loop





!--- Postprocessing
  

!- Nullify local pointers at last segment
IF (Iseg == Nseg-1) THEN
  COD            => Null()
  CPS            => Null()
  LWP            => Null()
  IWP            => Null()
  QC_REF         => Null()
  QC_COD         => Null()
  Press_Prof_Nwp => Null()
  Temp_Prof_Nwp  => Null()
  Hgt_Prof_Nwp   => Null()
  WV_Prof_Nwp    => Null()
END IF

END SUBROUTINE CLOUD_TAU_RE_SOLAR

!--- Other subroutines and functions


!===============================================================================
!
! SUBROUTINE Name: ASSIGN_LOCAL_VAR
!
! Function:    This function assigns framework/geocat variables to local variables 
!                   and checks for valid pixel
!
! Description:  This function populates module variables with data from the 
!                     environment for the current pixel.
!               The module variables were defined in module include file.  
!               It also checks whether this pixel is outside limits for 
!                 cloud mask, observation geometry, etc. 
!
! Calling Sequence: CALL DATA_ACCESS(Elem_Idx,Line_Idx,Cycle_Status)
!
! Inputs:  Elem_Idx  : element index
!          Line_Idx  : line index
!                              of the pixel 
!          
!
! Output:   Cycle_Status :  if set to 1 pixel will not be proceeded
!
!
! History:                AW 2009
!
!
!===============================================================================
SUBROUTINE ASSIGN_LOCAL_VAR (Elem_Idx, Line_Idx, Cycle_Status)

IMPLICIT NONE

!--- input variables
INTEGER(Kind=Int4), INTENT(In) :: Line_Idx              ! line index
INTEGER(Kind=Int4), INTENT(In) :: Elem_Idx              ! element index
 
!--- output variables
INTEGER,INTENT(Out):: Cycle_Status  !- If set to 1, cycle loop to next pixel 
   
!-- local
INTEGER(KIND=Int4):: Cld_Type_loc = 0  !- short lifetime cloud type
INTEGER(KIND=Int4):: Cld_Msk = 0   !- short lifetime cloud mask
INTEGER(KIND=Int4):: Snow_Mask     !- short lifetime snow mask

INTEGER:: X_NWP   !NWP column  number
INTEGER:: Y_NWP   !NWP row  number
  
! -- locals
INTEGER ::num_sol_zen    !- number solar zenith angles
INTEGER:: num_sat_Zen    !- number satellite angles
INTEGER:: num_rel_azi    !- number relative azimuth
REAL(Kind=Real4), DIMENSION(:),POINTER :: Sol_Zen_Vec_LUT !- solar zenith angle vector   
REAL(Kind=Real4), DIMENSION(:),POINTER :: Sat_Zen_Vec_LUT !- solar zenith angle vector
REAL(Kind=Real4), DIMENSION(:),POINTER :: Rel_Azi_Vec_LUT !- relative azimuth vector

REAL(KIND=real4)::cld_Type_Clvrx
REAL(KIND=real4)::ref_mean
REAL(KIND=real4)::ref_std
INTEGER::ivza
REAL(KIND=real4)::Radi_IR  
!-------------------------------------------------------------------------------
! Executable Code  
!-------------------------------------------------------------------------------
 
!--CHECK data 

Cycle_Status = 0
  
!--- check for space pixel
IF (bad_pixel_mask(Elem_Idx,Line_Idx) == Sym%SPACE) THEN
  Cycle_Status = 1              
  RETURN
END IF

!--- check maximal sensor geometry
IF (satzen(Elem_Idx,Line_Idx) .GE. SAT_ZEN_MAX) THEN
  Cycle_Status = 1
  RETURN
END IF

!--- check maximal solar geometry
IF (solzen(Elem_Idx,Line_Idx) >= SOL_ZEN_MAX) THEN
  Cycle_Status = 1
  RETURN
END IF

!--- check maximal relative azimuth 
! -- use 180.1 because azimuth precision could  
IF (Relaz(Elem_Idx,Line_Idx) .GT. 180.1) THEN
  Cycle_Status = 1
  RETURN
END IF

!--- set mask  and check for clear pixels         
Cld_Msk  = Cld_mask(Elem_Idx,Line_Idx)
        
IF (Cld_Msk == Sym%CLEAR .OR.  &
  Cld_Msk == Sym%PROB_CLEAR) THEN
  Cycle_Status = 2
  RETURN
END IF

!--- check for valid cloud height
IF (Pc_sw(Elem_Idx,Line_Idx) == MISSING_VALUE_REAL4) THEN
  Cycle_Status = 1
  RETURN
END IF

!CLV --- select cloud type to use
if (cloud_type_ir_flag == sym%YES) then
  Cld_type_Clvrx = cld_type_ir(Elem_Idx,Line_Idx)
else
  Cld_type_Clvrx = cld_type(Elem_Idx,Line_Idx)
endif

!--- check  phase
IF (Cld_type_Clvrx == Sym%UNKNOWN_TYPE) THEN
  Cycle_Status = 2
  RETURN
END IF
   
! -- SETTINGS

!--- determine phase for this algorithm 
Cld_Type_loc = Cld_type_Clvrx    !cloud type 
Cld_Phase = ICE_PHASE
IF ((Cld_Type_loc == Sym%FOG_TYPE) .OR.  &
    (Cld_Type_loc == Sym%WATER_TYPE) .OR. & 
    (Cld_Type_loc == Sym%SUPERCOOLED_TYPE) .OR. &
    (Cld_Type_loc == Sym%MIXED_TYPE)) THEN
  
  Cld_Phase = WATER_PHASE
END IF
    
!-- Set local NWP params
X_NWP  = i_NWP(Elem_Idx,Line_Idx)        ! - NWP longitude cell
Y_NWP  = j_NWP(Elem_Idx,Line_Idx)        ! - NWP latitude cell
 
Press_Prof_Nwp => P_std_nwp                              ! - pressure profile
Temp_Prof_Nwp  => T_prof_nwp(:,X_NWP,Y_NWP)          ! - temperature profile
Hgt_Prof_Nwp   => Z_prof_nwp(:,X_NWP,Y_NWP)              ! - height profile
WV_Prof_Nwp    => tpw_prof_nwp(:,X_NWP,Y_NWP)    ! - total water path profile
 
WV_Path_Nwp    = tpw_nwp(X_NWP,Y_NWP)        ! - Total water path
Sfc_Press_Nwp  = Psfc_nwp(x_nwp,y_nwp)           ! - Surface pressure
Ozone_Path_Nwp = ozone_nwp(x_nwp,y_nwp)      ! - Ozone

! -- set sun/satellite geometry variables
Sat_Zen     = Satzen(Elem_Idx,Line_Idx)            ! - satellite zenith angle 
Sol_Zen     = Solzen(Elem_Idx,Line_Idx)            ! - solar zenith angle 
Cos_Sat_Zen = Coszen(Elem_Idx,Line_Idx)    ! - cosine viewing zenith angle 
Cos_Sol_Zen = CosSolzen(Elem_Idx,Line_Idx)    ! - cosine solar zenith angle 
Rel_Azi     = Relaz(Elem_Idx,Line_Idx)             ! - relative solar azimuth angle 
Scat_Zen    = Scatangle(Elem_Idx,Line_Idx)       ! - scattering angle
Snow_Mask   = Snow(Elem_Idx,Line_Idx)     ! - snow mask
 
Is_Snow = sym%no
!print*,snow_mask
IF (Snow_Mask .eq. sym%snow .or. Snow_Mask .eq. sym%sea_ice) Is_Snow = sym%yes
  
! -- set other geocat products
Cld_Top_Temp  = Tc_sw(Elem_Idx,Line_Idx)     ! - cloud temperature
Cld_Top_Press = Pc_sw(Elem_Idx,Line_Idx)         ! - cloud pressure
Cld_Top_Hgt = Zc_sw(Elem_Idx,Line_Idx)       ! - cloud height

! -- set other values
ref_mean =  alb1_mean_3x3(Elem_Idx,Line_Idx) 
ref_std  =  alb1_std_3x3(Elem_Idx,Line_Idx)   !measure of reflectance variability
Refl_Variab = 0

IF (ref_mean > 0.0) THEN
  Refl_Variab = ref_std / ref_mean      ! - measure of reflectance variability
ENDIF   

!--  surface albedo 
          
Alb_Sfc_Chn_VIS =        ch1_sfc_alb_umd(sfc_type(Elem_Idx,Line_Idx))


IF (IR_CHN_NUMBER .EQ. IR_CHN_IDENT_3B ) THEN
   Alb_Sfc_Chn_IR =     ch3b_sfc_alb_umd(sfc_type(Elem_Idx,Line_Idx))
ELSE  
   Alb_Sfc_Chn_IR =     ch3a_sfc_alb_umd(sfc_type(Elem_Idx,Line_Idx)) 
ENDIF   


IF (Sfc_Type(Elem_Idx,Line_Idx) .NE. sym%WATER_SFC) THEN
   IF (modis_clr_alb_flag == sym%YES) THEN
     IF (alb1_sfc_white_sky(Elem_Idx,Line_Idx) > 0.0)  THEN
        Alb_Sfc_Chn_VIS =  max(alb1_sfc_white_sky(Elem_Idx,Line_Idx)/100.,0.02)
     END IF
     IF (alb3a_sfc_white_sky(Elem_Idx,Line_Idx) > 0.0)  THEN
        Alb_Sfc_Chn_IR = MAX(alb3a_sfc_white_sky(Elem_Idx,Line_Idx)/100.0,0.02)
     END IF
   END IF    
END IF 

IF ( Snow_Mask .EQ. sym%snow) then
   Alb_Sfc_Chn_VIS =   Alb_Sfc_Chn_VIS  * 0.8
endif
IF ( Snow_Mask .EQ. sym%sea_ice) then
   Alb_Sfc_Chn_VIS =   0.8
endif


!--- use sfc emissivitys for surface reflectance for 3.75 micron
IF (IR_CHN_NUMBER .EQ. IR_CHN_IDENT_3B ) THEN
  RTM_Rad_Clr_IR = rad_clear_ch3b_rtm(Elem_Idx,Line_Idx)  !clear 4 micron radiance
     
  emiss_sfc_7 = sfc_emiss_3b(Elem_Idx,Line_Idx)    !3.9 micron surface emissivity
  IF ((emiss_sfc_7 > 0.0) .and. (sfc_type(Elem_Idx,Line_Idx) > 0))  THEN   
    Alb_Sfc_Chn_IR = 1.0 - emiss_sfc_7
  ENDIF
ENDIF  


!--- check if cloud top pressure is lower than NWP surface pressure
IF (Cld_Top_Press .GT. Sfc_Press_Nwp) THEN
  Cycle_Status = 1
  RETURN
END IF

 call KNOWING_Z_COMPUTE_T_P_NWP(x_nwp,y_nwp,Cld_Top_Press,Cld_Top_Temp,Cld_Top_Hgt,idx_Lev)  

 Prof_Wgt = (Cld_Top_Hgt - Hgt_Prof_Nwp(idx_Lev)) /  (Hgt_Prof_Nwp(idx_Lev+1)-Hgt_Prof_Nwp(idx_Lev))

IF ( IR_CHN_NUMBER .EQ. IR_CHN_IDENT_3B) THEN
 
   ivza = ivza_rtm(Elem_Idx,Line_Idx) !viewing zenith angle bin
   RTM_Trans_Full_IR = trans_atm_ch3b_solar_rtm(Elem_Idx,Line_Idx)
   RTM_Trans_AC_Zen_IR = rtm(x_nwp,Y_NWP)%d(ivza)%trans_atm_clr3b(idx_Lev) +    &
                      Prof_Wgt*(rtm(X_NWP,Y_NWP)%d(ivza)%trans_atm_clr3b(idx_Lev+1) -  &
                            rtm(X_NWP,Y_NWP)%d(ivza)%trans_atm_clr3b(idx_Lev))
   RTM_Trans_AC_IR = (RTM_Trans_AC_Zen_IR)** (1.0/cos_sol_zen)
   
   RTM_Rad_AC_Zen_IR = rtm(X_NWP,Y_NWP)%d(ivza)%rad_atm_clr3b(idx_Lev) +    &
                        Prof_Wgt*(rtm(X_NWP,Y_NWP)%d(ivza)%rad_atm_clr3b(idx_Lev+1) -  &
                            rtm(X_NWP,Y_NWP)%d(ivza)%rad_atm_clr3b(idx_Lev))
  
 END IF

Rad_To_Refl_Fac =  pi / Cos_Sol_Zen / (solar_3b_nu/sun_earth_distance**2)

!--- set reflectance values range them  [0,1] instead of [0,100]
Refl_VIS = (alb1(Elem_Idx,Line_Idx))/100.  ! - seviri channel2 0.6 micron
IF (IR_CHN_NUMBER .EQ. IR_CHN_IDENT_3B) THEN
   Radi_IR = rad3b(Elem_Idx,Line_Idx)
   Refl_IR = Radi_IR * Rad_To_Refl_Fac
ELSE
  Refl_IR  = (alb3a(Elem_Idx,Line_Idx))/100. 
ENDIF
  
!-- set LUT points and weights for angles, those are fix within the retrieval loop of one pixel.


! - get needed LUT values
CALL GET_LUT_DATA(VIS_CHN_NUMBER,Cld_Phase &
           , num_sol_zen_lut = Num_Sol_Zen &
           , num_sat_zen_lut = Num_Sat_Zen &
           , num_rel_azi_lut = Num_Rel_Azi &
           , Sol_Zen_Vec_LUT = Sol_Zen_Vec_LUT &
           , Sat_Zen_Vec_LUT = Sat_Zen_Vec_LUT &
           , Rel_Azi_Vec_LUT = Rel_Azi_Vec_LUT )
           
!- set modul angular LUT position and weight variables         
Sol_Zen = MAX(0.,MIN(Sol_Zen,MAXVAL(Sol_Zen_Vec_LUT)))
Sat_Zen = MAX(0.,MIN(Sat_Zen,MAXVAL(Sat_Zen_Vec_LUT)))
Rel_Azi = MAX(0.,MIN(Rel_Azi,MAXVAL(Rel_Azi_Vec_LUT)))
  
CALL GET_INTERPOLATION_WEIGHT(Num_Sol_Zen,Sol_Zen,Sol_Zen_Vec_LUT,Sol_Wgt,Sol_Pos)              
CALL GET_INTERPOLATION_WEIGHT(Num_Sol_Zen,Sat_Zen,Sol_Zen_Vec_LUT,Sol2_Wgt,Sol2_Pos)             
CALL GET_INTERPOLATION_WEIGHT(Num_Sat_Zen,Sat_Zen,Sat_Zen_Vec_LUT,Sat_Wgt,Sat_Pos)
CALL GET_INTERPOLATION_WEIGHT(Num_Rel_Azi,Rel_Azi,Rel_Azi_Vec_LUT,Azi_Wgt,Azi_Pos)

Sol_Zen_Vec_LUT => NULL()
Sat_Zen_Vec_LUT => NULL()
Rel_Azi_Vec_LUT => NULL()  
         
END SUBROUTINE ASSIGN_LOCAL_VAR

!====================================================================
! SUBROUTINE Name: ATMOSPHERIC_CORRECTION
!
! Function:    Compute atmospheric correction
!
! Description: 
!
! Calling Sequence:
!
!      CALL ATMOSPHERIC_TRANSMISSION(atm_level,trans_vis,trans_IR)
!
! Inputs: 
!     Atm_Level:  a character, either "above" or "below"
!
! Output:
!
!     Trans_Vis : transmission in the visible channel
!     Trans_IR  : transmission in the (near)-IR channel
!     Refl_Sing_Scat : backscattered from air molecules (Rayleigh) or Aerosol above cloud
!
!
! History:  AW 2009
!
!===============================================================================
SUBROUTINE ATMOSPHERIC_CORRECTION( Atm_Level &
     , Total_Trans_VIS &
         , Total_Trans_IR &
         , Refl_Sing_Scat  )
  
IMPLICIT NONE

!-- input --
INTEGER(KIND=int1), INTENT(In):: Atm_Level     !- Below or above the cloud 

!-- output --
REAL(KIND=Real4), INTENT(Out):: Total_Trans_VIS  !- Transmission for VIS channel (ABI 2)
REAL(KIND=Real4), INTENT(Out):: Total_Trans_IR   !- Transmission for NIR channel (ABI 6)
REAL(KIND=Real4), INTENT(Out):: Refl_Sing_Scat !- Backscattered reflectance under single scattering assumption

!--- local -- 
REAL(KIND=Real4):: WV_Trans_VIS                    !- Water vapor transmission VIS channel
REAL(KIND=Real4):: Ozone_Trans_VIS         !- Ozone transmission  VIS
REAL(KIND=Real4):: Aer_Trans_VIS                   !- Aerosol tranmsittance VIS
REAL(KIND=Real4):: Rayleigh_Trans_VIS      !- Rayleigh transmittance  VIS
REAL(KIND=Real4):: Aer_OD_VIS                      !- Optical Depth aerosol VIS
REAL(KIND=Real4):: Rayleigh_OD_VIS         !- Rayleigh optical depth VIS
REAL(KINd=Real4):: Ozone_OD_VIS            !- Ozone optical depth VIS
REAL(KIND=Real4):: Total_OD_VIS            !- Total optical depth VIS
REAL(KIND=Real4):: WV_Trans_IR                     !- Water vapor transmission IR channel
REAL(KIND=Real4):: Phase_Function_Ray              !- Rayleigh Phase function 
REAL(KIND=Real4):: Phase_Function_Aer              !- Aerosol phase function
REAL(KIND=Real4):: Phase_Function                          !- Total phase function
REAL(KIND=Real4):: Single_Scatter_Albedo           !- Total single scatter Albedo
REAL(KIND=Real4):: Single_Scatter_Aa               !- backscattering term AA
REAL(KIND=Real4):: Single_Scatter_Ac               !- backscattering term AC
REAL(KIND=Real4):: Single_Scatter_Ca               !- backscattering term CA
REAL(KIND=Real4):: FACTOR_MULTIPLE_SCATTER  = 0.84 !- backscattering term 

REAL(KIND=Real4),DIMENSION(:,:,:),POINTER:: Cld_Alb_3D_LUT
REAL(KIND=Real4),DIMENSION(:,:,:,:,:), POINTER:: Refl_5D_LUT
REAL(KIND=Real4):: Cloud_Albedo_Sat !- Cloud albedo in satellite direction
REAL(KIND=Real4):: Cloud_Albedo_Sol !- Cloud albedo in solar direction
REAL(KIND=Real4):: Cos_Scat_Zen !- Cosinos scattering angle

REAL(KIND=Real4)::Air_Mass_Factor                  !- Air mass factor
REAL(KIND=Real4)::COD_Estimated                    !- Estimated optical depth for a apriori CPS assumption
REAL(KIND=Real4)::CPS_Estimated                    !- Apriori CPS
REAL(KIND=Real4),DIMENSION(:), POINTER  :: CPS_Vec_LUT !- CPS vector of LUT entrees
REAL(KIND=Real4),DIMENSION(:), POINTER  :: COD_Vec_LUT !- COD vector of LUT entrees
REAL(KIND=Real4)::Estimated_Rfl_VIS  ! - Reflectance in visible channel from CPS apriori

INTEGER :: CPS_Pos     !- LUT position in CPS_Vec_LUT 
INTEGER :: COD_Pos     !- LUT position in COD_Vec_LUT 
INTEGER, DIMENSION(1) :: Dum_COD_Pos !- Dummy variable
INTEGER, DIMENSION(1) :: Dum_CPS_Pos !- Dummy variable

REAL(KIND=real4),DIMENSION(3)::Ozone_Coeff
 
!------------------------------------------------------------------------------
!-- executable
!------------------------------------------------------------------------------
   
Air_Mass_Factor = 1.0/Cos_Sat_Zen + 1.0/Cos_Sol_Zen
 
 !- compute transmission from water vapor and other gases with airmass correction 
CALL COMPUTE_WATER_TRANS (Atm_Level, WV_Trans_VIS, WV_Trans_IR)
 
! === VIS channel ============================================================
! -- ozone optical depth computed from pre-computed oefficients 
  
CALL GET_LUT_DATA(sc_id_new,Cld_phase,Ozone_Coeff = Ozone_Coeff)

Ozone_OD_VIS =   Ozone_Coeff(1) &
                           + Ozone_Coeff(2) *  Ozone_Path_Nwp &
                           + Ozone_Coeff(3) *  Ozone_Path_Nwp**2
                  
Ozone_Trans_VIS = EXP(-Air_Mass_Factor * Ozone_OD_VIS)


! -- optical depth aerosol ; amount above cloud is estimated as a function of cloud top pressure -- 
IF (Atm_Level .EQ. LAYER_BELOW_CLOUD) THEN
  Aer_OD_VIS = OD_AEROSOL_BACKGROUND_VIS * ( (Sfc_Press_Nwp - Cld_Top_Press) &
                                          / Sfc_Press_Nwp)**4  !--below cloud
ELSE
  Aer_OD_VIS = OD_AEROSOL_BACKGROUND_VIS * (Cld_Top_Press / Sfc_Press_Nwp)**4   !--above cloud
ENDIF

! --- optical depth Rayleigh  -----
IF (Atm_Level .EQ. LAYER_BELOW_CLOUD) THEN
  Rayleigh_OD_VIS = 0.
ELSE
  Rayleigh_OD_VIS = OD_RAYLEIGH_BACKGROUND * (Cld_Top_Press / Sfc_Press_Nwp)
ENDIF

!-- compute transmisson from OD
Aer_Trans_VIS      = EXP(-Air_Mass_Factor * Aer_OD_VIS &
                                                        * FACTOR_MULTIPLE_SCATTER)
Rayleigh_Trans_VIS = EXP(-Air_Mass_Factor * Rayleigh_OD_VIS &
                                                        * FACTOR_MULTIPLE_SCATTER)

Total_Trans_VIS = Rayleigh_Trans_VIS * Aer_Trans_VIS * WV_Trans_VIS &
                                                   * Ozone_Trans_VIS 


! === IR channel ============================================================

Total_Trans_IR =  WV_Trans_IR 


! === rayleigh + aerosol single scattering for VIS channel ===
!  see Wang&King 1997
!

Cos_Scat_Zen = Cos(Scat_Zen * DTOR)

!--- compute phase functions for rayleigh and aerosol(Henyey-Greenstein)
Phase_Function_Ray = 0.75*(1.0 + Cos_Scat_Zen**2)

Phase_Function_Aer = (1.0 - G_AEROSOL_BACKGROUND_VIS**2) / &
                                         ( (1.0 + G_AEROSOL_BACKGROUND_VIS**2 -  &
                                                2.0*G_AEROSOL_BACKGROUND_VIS*Cos_Scat_Zen)**(1.5) )

!--- compute total optical depth needed for backscattered part
Total_OD_VIS = -1.0 * ALOG(Total_Trans_VIS)                                              

!-- total effective phase function                                                       
Phase_Function = (WO_AEROSOL_BACKGROUND_VIS * Aer_OD_VIS * Phase_Function_Aer &
                                           + Rayleigh_OD_VIS * Phase_Function_Ray) &
                                                        /Total_OD_VIS
                           
!--- single scatter albedo
Single_Scatter_Albedo = (WO_AEROSOL_BACKGROUND_VIS * Aer_OD_VIS + Rayleigh_OD_VIS) &
                                                           / ( Aer_OD_VIS + Rayleigh_OD_VIS )

IF ((Aer_OD_VIS + Rayleigh_OD_VIS) .EQ. 0 ) Single_Scatter_Albedo = 1.
  
!==cloud albedo  

!- get data
CALL GET_LUT_DATA(VIS_CHN_NUMBER,Cld_Phase &
   ,COD_Vec_LUT = COD_Vec_LUT  &
   ,Cld_Alb_3D_LUT = Cld_Alb_3D_LUT &
   ,CPS_Vec_LUT = CPS_Vec_LUT)

!- set an apriori CPS estimate
IF (Cld_Phase == WATER_PHASE) THEN
  CPS_Estimated = CPS_APRIORI_WATER
ELSEIF (Cld_Phase == ICE_PHASE) THEN
  CPS_Estimated = CPS_APRIORI_ICE
END IF

!- Determine the nearest LUT entree of CPD_Estimate
Dum_CPS_Pos = MINLOC(ABS(CPS_Vec_LUT - CPS_Estimated ))
CPS_Pos = Dum_CPS_Pos(1)

!- Rough reflectance estimate at TOC 
Estimated_Rfl_Vis = Refl_Vis /Total_Trans_VIS

! -- estimate Cld_OD for fixed effective radius
COD_estimated = COD_FIX_CPS(CPS_Estimated,Estimated_Rfl_Vis)

Dum_COD_Pos = MINLOC(ABS(COD_Vec_LUT - COD_Estimated ))
COD_Pos = Dum_COD_Pos(1)

                          
Cloud_Albedo_Sol = Cld_Alb_3D_LUT(CPS_Pos,COD_Pos,Sol_Pos)  
Cloud_Albedo_Sat = Cld_Alb_3D_LUT(CPS_Pos,COD_Pos,Sat_Pos)
          
!- backscattered light above cloud 
!- "ca" : cloud -> atmosphere -> TOA
!- "ac" : atmosphere -> cloud -> TOA  
!- "aa" : only atmosphere  -> TOA                 
!- reference : Wang and King 1997                 
Single_Scatter_Ca  = ( Single_Scatter_Albedo * Rayleigh_OD_VIS &
                                          * Cloud_Albedo_Sat * Exp(-Rayleigh_OD_VIS/Cos_Sat_Zen))&
                                        / (2.*Cos_Sol_Zen) 
          
Single_Scatter_Ac  = ( Single_Scatter_Albedo * Rayleigh_OD_VIS &
                                          * Cloud_Albedo_Sol * Exp(-Rayleigh_OD_VIS/Cos_Sol_Zen)) &
                                        / (2.*Cos_Sat_Zen) 

Single_Scatter_Aa  =  Single_Scatter_Albedo * Rayleigh_OD_VIS &
                                          * Phase_Function / (4.0 * Cos_Sol_Zen * Cos_Sat_Zen) 
  
Refl_Sing_Scat = Ozone_Trans_VIS * &
                                  (Single_Scatter_Ca + Single_Scatter_Ac + Single_Scatter_Aa)

END SUBROUTINE ATMOSPHERIC_CORRECTION


!===============================================================================
! SUBROUTINE Name:  COD_FIX_CPS
!
! Function:    Returns a estimate COD for a fix CPS from Reflectance in VIS Channel 
!                       
!
! Description:         1. Find the nearest LUT entree of input CPS
!                      2. Extract Reflectance vector (29 elements) for this CPS 
!                          in VIS channel
!                      3. Find the nearest entree of this vector to sate vectore in VIS channel
!                      4. Take this as an estimate for COD and return it.  
!
! Calling Sequence:
!                      COD_estimated = CALL(CPS_Fix,Rfl_Vis)
!
! Inputs: 
!                     CPS_Fix  -> Cloud Particle Size
!                     Rfl_VIS  -> Reflectance at TOC  
!
! Output:             The COD Entree in LUT, which reflectance in LUT is nearest to Rfl_VIS
!
!
!===============================================================================
FUNCTION COD_FIX_CPS (CPS_Fix , Rfl_VIS) RESULT(COD_FIX)
 
! - input
REAL(KIND=REAL4),INTENT(in)::CPS_Fix
REAL(KIND=REAL4),INTENT(in)::Rfl_VIS
 
! - output
REAL(KIND=REAL4)::COD_Fix
 
! - locals
REAL(KIND=Real4),DIMENSION(:), POINTER :: CPS_Vec_LUT
REAL(KIND=Real4),DIMENSION(:), POINTER :: COD_Vec_LUT
REAL(KIND=Real4),DIMENSION(:,:,:,:,:), POINTER:: Refl_5D_LUT
 
INTEGER, DIMENSION(1) :: Dum
REAL(Kind=Real4), DIMENSION(29):: Ref_Cld_Vector
REAL(Kind=Real4) , DIMENSION(1):: Dummy 
 
INTEGER :: CPS_Pos     ! tau and ref position in LUT


! - executable

CALL GET_LUT_DATA(VIS_CHN_NUMBER,Cld_phase,CPS_Vec_LUT = CPS_Vec_LUT)           

Dum = MINLOC(ABS(CPS_Vec_LUT - CPS_Fix ))
CPS_Pos = Dum(1)
        
! -- estimate Cld_OD for fixed effective radius
 
CALL GET_LUT_DATA(VIS_CHN_NUMBER,Cld_phase,Refl_5D_LUT = Refl_5D_LUT)
  
Ref_Cld_Vector = Refl_5D_LUT(CPS_Pos,:,Sol_Pos,Sat_Pos,Azi_Pos)
  
CALL GET_LUT_DATA(VIS_CHN_NUMBER,Cld_Phase,COD_Vec_LUT = COD_Vec_LUT)
  
! - 
Dummy = COD_Vec_LUT (MINLOC(ABS(Ref_Cld_Vector-Rfl_VIS )))
COD_Fix = Dummy(1)                                      

END FUNCTION COD_FIX_CPS


!====================================================================
! SUBROUTINE Name:  COMPUTE_WATER_TRANS
!
! Function:    Computes gas transmission, which is mainly determine by water vapor extinction 
!
! Description: Computes transmission of water vapor (and other trace gases)
!              by means of pre-computed coefficients.
!              
!
! Calling Sequence:
!
!               CALL(Atm_Level, Trans_WV_Chn_VIS, Trans_WV_Chn_IR)
!
! Inputs:  
!              Atm_Level --> level above or below cloud
!
! Output:
!             Trans_WV_Chn_VIS --> Transmission in visible channel
!             Trans_WV_Chn_IR  --> Transmission in infrared channel           
!
! References:
!
! Dependencies: 
!
! Restrictions:
!
! Side Effects:
!
! History:
!
! Local Variables:
!
!====================================================================
RECURSIVE SUBROUTINE COMPUTE_WATER_TRANS (Atm_Level, Trans_WV_Chn_VIS, Trans_WV_Chn_IR)

  IMPLICIT NONE
  
  !--input
  INTEGER(KIND=int1), INTENT(In):: Atm_Level

  !--output
  REAL(KIND=Real4), INTENT(Out):: Trans_WV_Chn_VIS
  REAL(KIND=Real4), INTENT(Out):: Trans_WV_Chn_IR
 
  !--local
  REAL::  Pc, Zc                     ! dummies needed 
  REAL(KIND=Real4):: Water_Path_Above_Cld          !  total water path above cloud
  REAL(KIND=Real4):: Prof_Wgt        ! position weight for NWP profile 
  INTEGER(KIND=Int4):: Idx_Lev       !level index
  INTEGER(KIND=int1)::Local_Which_Level
 
  REAL(KIND=Real4)::Trans_WV_Chn_VIS_All
  REAL(KIND=Real4)::Trans_WV_Chn_IR_All
  REAL(KIND=Real4)::Trans_WV_Chn_VIS_Above_Cld
  REAL(KIND=Real4)::Trans_WV_Chn_IR_Above_Cld
  
  REAL(KIND=Real4),DIMENSION(3) :: Gas_Coeff_VIS
  REAL(KIND=Real4),DIMENSION(3) :: Gas_Coeff_IR
 
  REAL(KIND=Real4) :: Air_Mass_Factor
 
 !------------------------------------------------------------------------------
 !-- executable
 !------------------------------------------------------------------------------
  
!  CALL GET_LUT_DATA(VIS_CHN_NUMBER,Cld_Phase,Gas_Coeff=Gas_Coeff_VIS)
!  CALL GET_LUT_DATA(IR_CHN_NUMBER,Cld_Phase,Gas_Coeff=Gas_Coeff_IR)
 
 Air_Mass_Factor = 1./Cos_Sol_Zen + 1./Cos_Sat_Zen
 
   Gas_Coeff_VIS =  h2o_tau_coef(sc_id_new,1,:)

  IF (IR_CHN_NUMBER .EQ. IR_CHN_IDENT_3a) THEN
    Gas_Coeff_IR = h2o_tau_coef(sc_id_new,3,:)
  ENDIF
      
  SELECT CASE (Atm_Level)

   CASE(LAYER_ALL_COLUMN)
     Trans_WV_Chn_VIS = TRANS_WV_FROM_COEFF(Wv_Path_Nwp,Gas_Coeff_VIS)
	 
    IF (IR_CHN_NUMBER .EQ. IR_CHN_IDENT_3a) THEN
	  Trans_WV_Chn_IR  = TRANS_WV_FROM_COEFF(Wv_Path_Nwp,Gas_Coeff_IR)
    ELSE 
	  Trans_WV_Chn_IR = RTM_Trans_Full_IR ** Air_mass_Factor
	ENDIF
	
	
	     
   CASE(LAYER_ABOVE_CLOUD) 
      ! --  use prof_lookup_using_t from geocat subroutine ----
          CALL Prof_Lookup_Using_T(Hgt_Prof_Nwp, Press_Prof_Nwp, Temp_Prof_Nwp &
                                   ,Zc , Pc, Cld_Top_Temp, Idx_Lev, Prof_Wgt)
                                                           
      Water_Path_Above_Cld = Wv_Prof_Nwp(Idx_Lev) &
                                 + Prof_Wgt*(Wv_Prof_Nwp(Idx_Lev+1) &
                                             - Wv_Prof_Nwp(Idx_Lev))
          
          ! -- set water vapor path to zero for negative values
      Water_Path_Above_Cld = MAX(0.,Water_Path_Above_Cld)
          
      Trans_WV_Chn_VIS =  TRANS_WV_FROM_COEFF(Water_Path_Above_Cld &
                                    ,Gas_Coeff_VIS)
       
	   IF (IR_CHN_NUMBER .EQ. IR_CHN_IDENT_3a) THEN                                                          
          Trans_WV_Chn_IR  =  TRANS_WV_FROM_COEFF(Water_Path_Above_Cld &
                                   ,Gas_Coeff_IR)
       ELSE 
	   
	      Trans_WV_Chn_IR = RTM_Trans_AC_Zen_IR ** Air_mass_Factor
	   ENDIF						   
                
   CASE(LAYER_BELOW_CLOUD)
      Local_Which_Level = LAYER_ABOVE_CLOUD
      CALL COMPUTE_WATER_TRANS (Local_Which_Level, Trans_WV_Chn_VIS_Above_Cld &
                                        ,Trans_WV_Chn_IR_Above_Cld)
      
          Local_Which_Level =  LAYER_ALL_COLUMN
      CALL COMPUTE_WATER_TRANS (Local_Which_Level, Trans_WV_Chn_VIS_All &
                                        ,Trans_WV_Chn_IR_All)
      
          Trans_WV_Chn_VIS = Trans_WV_Chn_VIS_All / Trans_WV_Chn_VIS_Above_Cld
      Trans_WV_Chn_IR  = Trans_WV_Chn_IR_All  / Trans_WV_Chn_IR_Above_Cld
          
   CASE Default 
      
      Trans_WV_Chn_VIS = 1.
          Trans_WV_Chn_IR = 1.
        
 END SELECT

END SUBROUTINE COMPUTE_WATER_TRANS

!===============================================================================
! Function Name: TRANS_WV_FROM_COEFF
!
! Function:            Calculates gas transmission from coefficients
!
!  INPUT:
!          TPW_ac -> Total water content 
!          Coef   -> Transmission coefficients
!
!
!===============================================================================
 
FUNCTION TRANS_WV_FROM_COEFF(Tpw_Ac,Coef) RESULT(Trans_Solar_TOA)
 
  IMPLICIT NONE
  ! -- Input
  REAL(KIND=Real4), INTENT(In):: Tpw_Ac
  REAL(KIND=Real4), INTENT(In), DIMENSION(:):: Coef
  
  ! -- Local
  REAL(KIND=Real4) :: Trans_Solar_TOA
  REAL(KIND=Real4) :: Tau
  REAL(KIND=Real4) :: Air_Mass_Factor
 
 !------------------------------------------------------------------------------
 !-- executable
 !------------------------------------------------------------------------------

  Air_Mass_Factor = 1.0/Cos_Sat_Zen + 1.0/Cos_Sol_Zen
 
  Tau = Coef(1) + Coef(2)*Tpw_Ac + Coef(3)*(Tpw_Ac**2) 
  Tau = MAX(0.,Tau)
 
  Trans_Solar_TOA = Exp ( -Air_Mass_Factor * Tau) 

END FUNCTION TRANS_WV_FROM_COEFF




!=============================================================================================================
! SUBROUTINE Name: OPTIMAL_ESTIMATION
!
! Function:      Proceeds the optimal estimation inverse technique for one pixel
!
! Description: 
!
! Calling Sequence:
!
!            CALL OPTIMAL_ESTIMATION(Obs_Vector,State_Vector, Quality)
!               called in the main program
!
! Inputs: 
!              Obs_Vector :  Observation vector with reflections [0,1] 
!
! Output:
!              State_Vector : Atmospheric state vector with COD and CPS values
!
! References:
!
! Dependencies: 
!
! Restrictions:
!
! Side Effects:
!
! History:              created in 2008 AW
!                      major revison 03/2010 (AW)
!
! Local Variables:
!              see definition part of code
!
!====================================================================
SUBROUTINE OPTIMAL_ESTIMATION (Obs_Vec,State_Vec, Quality,diag,Conv_Test,Sx)

IMPLICIT NONE

!----input
REAL(KIND=Real4), DIMENSION(Num_Chn), INTENT(In) :: Obs_Vec   !  observation vector y

!--- output
REAL(KIND=Real4), DIMENSION(Num_Chn), INTENT(Out):: State_Vec   ! state vector x
REAL(KIND=Real4), INTENT(Out):: Quality

!----local
REAL, DIMENSION(Num_Chn):: Fwd_Model_Vec  !  -- Forward model
REAL:: Fm_Err                       ! - Estimated forward model error 
REAL, DIMENSION(Num_Chn,Num_Chn):: Kernel    !  -- Jacobians (Kernel)

REAL, DIMENSION(:),POINTER :: COD_Vec_LUT  ! - COD vectore from LUT
REAL, DIMENSION(:),POINTER :: CPS_Vec_LUT  ! - CPS vectore from LUT
 
! a priori values
REAL, DIMENSION(Num_Chn,Num_Chn):: Error_Cov_Matrix_Obs     ! error covariance matrix of obs y
REAL, DIMENSION(Num_Chn,Num_Chn):: Inv_Error_Cov_Matrix_Obs ! inverse of error covariance matrix of obs y  
REAL, DIMENSION(Num_Obs,Num_Obs):: Error_Cov_Matrix_State   ! error covariance of state vector x
REAL, DIMENSION(Num_Obs,Num_Obs):: Inv_Error_Cov_Matrix_State !  inverse of Error_Cov_Matrix_State
REAL, DIMENSION(Num_Obs,Num_Obs):: Error_Cov_Matrix_Api     ! error covariance of background x_apriori
REAL, DIMENSION(Num_Chn,Num_Chn):: Inv_Error_Cov_Matrix_Api ! inverse error covariance of background x_apriori
REAL, DIMENSION(Num_Obs) :: State_Vec_Api    ! apriori state vector
REAL, DIMENSION(Num_Obs) :: Delta_X             ! iteration step 

REAL, DIMENSION(2)::Sx

REAL(KIND=REAL4)  :: COD_Estimated     ! - COD for fixed CPS
INTEGER(KIND=Int4):: Iteration_Idx  ! - Iteration loop index
REAL(Kind=Real4)  :: Conv_Test      ! - Convergence metric (tested againist conv_crit)
INTEGER(KIND=Int4):: Isingular      ! - Flag for Invert routine
INTEGER(KIND=Int4):: Ifail          ! - Flag noting a failed retrieval
  REAL(KIND=real4),DIMENSION(2)::diag
!  -- executable
  
  ! -- Forward model error
IF (Cld_Phase == WATER_PHASE) THEN
  Fm_Err = FM_ERR_WATER    
ELSE
  Fm_Err = FM_ERR_ICE
END IF

!--- VIS channel error assumption  
Error_Cov_Matrix_Obs(1,1) = ((CAL_ERR + Fm_Err + PP_ERR * Refl_Variab) &
                                                                * Obs_Vec(1) + ERROR_OFFSET_VIS)**2

! - give VIS channel no trust over snow 
IF (Is_Snow .EQ. sym%yes) then
   Error_Cov_Matrix_Obs(1,1) = ERROR_SNOW_VIS
ENDIF 

!--- IR channel error  
Error_Cov_Matrix_Obs(2,2) = ((Cal_Err + Fm_Err + Pp_Err * Refl_Variab) &
                                                                * Obs_Vec(2) + ERROR_OFFSET_IR)**2
 

 
!--- Invert error covariance matrix
CALL Invert_2X2(Error_Cov_Matrix_Obs,Inv_Error_Cov_Matrix_Obs,Isingular)
IF (Isingular == Sym%YES) THEN
  Ifail = Sym%YES
  RETURN
END IF



!--- particle size apriori
IF (Cld_Phase == WATER_PHASE) THEN
  State_Vec_Api(2) = CPS_APRIORI_WATER
ELSEIF (Cld_Phase == ICE_PHASE) THEN
  State_Vec_Api(2) = CPS_APRIORI_ICE
END IF

! --- optical depth apriori -------------------------------------------------- 
! AW20100326-- Estimate Cld_OD for fixed effective radius using CPS apriori value  
! AW20100326-- using VIS reflectance
! AW20100326-- of 10 micron(CPS_log10 = 1) 

COD_estimated = COD_FIX_CPS(State_Vec_Api(2),Obs_Vec(1))
State_Vec_Api(1) = COD_estimated

! AW20100326-- outdated stuff
State_Vec_Api(1) = 0.7*(100.*Obs_Vec(1))**(0.9) 
!State_Vector_Api(1) = 10.
State_Vec_Api(1) = Log10(MAX(0.1,State_Vec_Api(1)))
    
! -- Background error covariance matrix
Error_Cov_Matrix_Api = 0.0
Error_Cov_Matrix_Api(1,1) = (MAX(0.2,State_Vec_Api(1)))**2

IF (Cld_Phase == WATER_PHASE) THEN
  Error_Cov_Matrix_Api(2,2) = (CPS_APRIORI_ERROR_WATER)**2
ELSE
  Error_Cov_Matrix_Api(2,2) = (CPS_APRIORI_ERROR_ICE)**2
END IF

! -- Invert background error covariance matrix     
CALL Invert_2X2(Error_Cov_Matrix_Api,Inv_Error_Cov_Matrix_Api,Isingular)
IF (Isingular == Sym%YES) THEN
  Ifail = Sym%YES
  RETURN
END IF
 

!--- set first guess to a priori
State_Vec = State_Vec_Api

!---------------------------------------------------------------------
!---BEGIN Retrieval Loop
!---------------------------------------------------------------------

Iteration_Idx = 0
Retrieval_Loop: DO
  ! -- Update iteration counter
  Iteration_Idx = Iteration_Idx + 1
    
  CALL FORWARD_MODEL (State_Vec, Fwd_Model_Vec , Kernel)
  
  
    IF (Iteration_Idx .eq. 1)  diag = Fwd_Model_Vec
  !--compute Sx error covariance of solution x 
  Inv_Error_Cov_Matrix_State = Inv_Error_Cov_Matrix_Api + MATMUL(TRANSPOSE(Kernel) &
                                                                , MATMUL(Inv_Error_Cov_Matrix_Obs,Kernel))


  CALL Invert_2X2 (Inv_Error_Cov_Matrix_State,Error_Cov_Matrix_State,Isingular)
  IF (Isingular == 1) THEN
        Fail_Flag = Sym%YES
    EXIT
  END IF
  
  Sx(1) = Error_Cov_Matrix_State(1,1)
  Sx(2) = Error_Cov_Matrix_State(2,2)
 
  
  !--- compute next iteration step
  Delta_X = MATMUL(Error_Cov_Matrix_State, &
                  (MATMUL(TRANSPOSE(Kernel), &
                   MATMUL(Inv_Error_Cov_Matrix_Obs,(Obs_Vec-Fwd_Model_Vec))) +  &
                           MATMUL(Inv_Error_Cov_Matrix_Api,State_Vec_Api-State_Vec) ))
                           
                   
          
  !--- check for convergence
  Conv_Test = ABS(SUM(Delta_X * MATMUL(Inv_Error_Cov_Matrix_State,Delta_X)))
  
  !--- control step size maximal value is max_Step_Size
  IF (MAXVAL(ABS(Delta_X)) .GT. Max_Step_Size) THEN
  
    Delta_X = MAX_STEP_SIZE * Delta_X/(MAXVAL(ABS(Delta_X)))
  ENDIF
  
 
  
  !--- update retrieved vector
  State_Vec = State_Vec + Delta_X

  !-- is the new state vector outside LUT range ?
  ! - if yes set it to the edge
  CALL GET_LUT_DATA(VIS_CHN_NUMBER,Cld_Phase &
           ,COD_Vec_LUT = COD_Vec_LUT &
           ,CPS_Vec_LUT = CPS_Vec_LUT )
 
  State_Vec(1) = MIN(MAXVAL(COD_Vec_LUT), &
                        MAX( State_Vec(1) , COD_Vec_LUT(1))  )

  State_Vec(2) = MIN(MAXVAL(CPS_Vec_LUT ), &
                        MAX( State_Vec(2) , CPS_Vec_LUT(1))  )

  
  IF (Conv_Test .LT. CONV_CRIT .AND. Delta_X(2) .LT. 0.05 .AND. Delta_X(1) .LT. 0.05 ) THEN
    Fail_Flag = Sym%NO

    CALL FORWARD_MODEL (State_Vec , Fwd_Model_Vec , Kernel)
    
    Quality = 3
	 
         EXIT
  END IF
 
  ! check maximal number of iterations                    
  
  IF (Iteration_Idx > MAX_ITER) THEN
        Quality = 1
   
    IF (Rel_Azi .GT. 170) Quality = 1
    Fail_Flag = Sym%YES
        
    EXIT
  ENDIF
END DO Retrieval_Loop

END SUBROUTINE OPTIMAL_ESTIMATION



!====================================================================
! SUBROUTINE Name: FORWARD_MODEL
!
! Function:     Forward model calculation for a atmospheric state vector
!
! Description:   
!
! Calling Sequence:
!
! Inputs: 
!      x =  atmospheric state vector 
!
! Output:
!     f =  observation vector 
!     K  = Jacobians
!
! References:
!
! Dependencies: 
!
! Restrictions:
!
! Side Effects:
!
! History:                 Andi Walther 2009
!
! Local Variables:
!
!====================================================================
SUBROUTINE FORWARD_MODEL (State_Vector, Fwd_Model_Vec , Kernel)

IMPLICIT NONE

! -- input
REAL, DIMENSION(NUM_CHN) , INTENT(In) :: State_Vector   !--state vector

! -- output
REAL, DIMENSION(NUM_OBS)         , INTENT(Out) :: Fwd_Model_Vec    !--- Forward model output
REAL, DIMENSION(NUM_OBS,NUM_OBS) , INTENT(Out) :: Kernel           !--- Kernel (Jacobians)

! -- local
INTEGER:: COD_Pos          ! position in LUT
INTEGER:: CPS_Pos          ! position in LUT
REAL(KIND=Real4):: COD_Wgt ! position weights in LUT
REAL(KIND=Real4):: CPS_Wgt ! position weights in LUT
INTEGER:: num_CPS              ! number of CPS values in LUT 
INTEGER:: num_COD              ! number of COD values in LUT 
REAL (Kind=Real4), DIMENSION(:),POINTER :: CPS_Vec_LUT  !- CPS vector in LUT
REAL (Kind=Real4), DIMENSION(:),POINTER :: COD_Vec_LUT  !- COD vector in LUT

REAL, DIMENSION(NUM_OBS)          :: Fwd_Model_Vec_Solar    !--- Forward model output
REAL, DIMENSION(NUM_OBS,NUM_OBS)  :: Kernel_Solar           !--- Kernel (Jacobians)
REAL, DIMENSION(NUM_OBS)          :: Fwd_Model_Vec_Terr    !--- Forward model output
REAL, DIMENSION(NUM_OBS,NUM_OBS)  :: Kernel_Terr           !--- Kernel (Jacobians)


 
! --  executable

!--- interpolate within tables - assume the same for all channels and tables
  
!-- get needed LUT values
CALL GET_LUT_DATA(VIS_CHN_NUMBER,Cld_Phase &
        , CPS_Vec_LUT  = CPS_Vec_LUT &
        , COD_Vec_LUT  = COD_Vec_LUT )
       
Num_CPS = size(CPS_Vec_LUT)
Num_COD = size(COD_Vec_LUT)
   
CALL GET_INTERPOLATION_WEIGHT(Num_CPS,State_Vector(2)&
                                  ,CPS_Vec_LUT,CPS_Wgt,CPS_Pos)   
                          
CALL GET_INTERPOLATION_WEIGHT(Num_COD,State_Vector(1)&
                                 ,COD_Vec_LUT,COD_Wgt,COD_Pos)
								 
								 
	IF (CPS_Pos .GE. UBOUND(CPS_Vec_LUT,DIM=1)) THEN
	  CPS_Pos = UBOUND(CPS_Vec_LUT,DIM=1) - 1
	  CPS_Wgt = 1.
	ENDIF			
	
    IF (COD_Pos .GE. UBOUND(COD_Vec_LUT,DIM=1)) THEN
	  COD_Pos = UBOUND(COD_Vec_LUT,DIM=1) - 1
	  COD_Wgt = 1.
	ENDIF						 

!--- CALL forward model for each channel used

!--- Visible channel forward model
CALL FORWARD_MODEL_SINGLE_CHANNEL(&
                                           VIS_CHN_NUMBER,Cld_Phase, &
                                           CPS_Wgt,COD_Wgt, &
                                   CPS_Pos,COD_Pos, &
                                           Alb_Sfc_Chn_VIS, &
                                           Fwd_Model_Vec(1), &
                                   Kernel(1,1),Kernel(1,2))
!--- IR forward model
IF (IR_CHN_NUMBER .EQ. IR_CHN_IDENT_3A) THEN
   CALL FORWARD_MODEL_SINGLE_CHANNEL(&
                                           IR_CHN_NUMBER,Cld_Phase, &
                                           CPS_Wgt,COD_Wgt, &
                                   CPS_Pos,COD_Pos, &
                                           Alb_Sfc_Chn_IR, &
                                           Fwd_Model_Vec(2), &
                                   Kernel(2,1),Kernel(2,2))
  
ELSE

  !--- IR solar
 
  CALL FORWARD_MODEL_SINGLE_CHANNEL(&
                                           IR_CHN_NUMBER,Cld_Phase, &
                                           CPS_Wgt,COD_Wgt, &
                                   CPS_Pos,COD_Pos, &
                                           Alb_Sfc_Chn_IR, &
                                           Fwd_Model_Vec_Solar(2), &
                                   Kernel_Solar(2,1),Kernel_Solar(2,2))
  !--- IR terrestric                                       
  CALL FORWARD_MODEL_SINGLE_CHANNEL_EMISSIVITY(&
                                           IR_CHN_NUMBER,Cld_Phase, &
                                           CPS_Wgt,COD_Wgt, &
                                   CPS_Pos,COD_Pos, &                                      
                                           Fwd_Model_Vec_Terr(2), &
                                   Kernel_Terr(2,1),Kernel_Terr(2,2))                                      
 
   Fwd_Model_Vec(2) = Fwd_Model_Vec_Solar(2) + Fwd_Model_Vec_Terr(2)
   Kernel(2,1) = Kernel_Solar(2,1) + Kernel_Terr(2,1)
   Kernel(2,2) = Kernel_Solar(2,2) + Kernel_Terr(2,2)
   
  
ENDIF 

END SUBROUTINE FORWARD_MODEL 


!===============================================================================
! SUBROUTINE Name: CLOUD_FORWARD_REFLECTANCE_MODEL_SINGLE_LAYER
!
! Function:       Forward Model for a Reflectance Channel
!
! Description:  this routine is appropriate for single layer cloud placed in an 
!               absorbing atmosphere over a Lambertian Surface.  The accounting
!               for the surface contribution is given by Eq 11 in 
!                King, Michael, 1987: JAS, vol44, 1734-1751 
!
! Calling Sequence:
!
! Inputs: 
!
! Output:
!
! References:
!
! Dependencies: 
!
! Restrictions:
!
! Side Effects:
!
! History:                   
!
! Local Variables:
!
!===============================================================================
SUBROUTINE FORWARD_MODEL_SINGLE_CHANNEL(&
                         Idx_chn,idx_phase, &
                         CPS_Wgt,COD_Wgt, &
                                                 CPS_Pos,COD_Pos,  &
                         Alb_Sfc, &
                         Refl,Drefl_Dcod,Drefl_Dcps)
  
IMPLICIT NONE

!---input
INTEGER, INTENT(In):: Idx_chn             ! - Channel index
INTEGER, INTENT(In):: Idx_phase           ! - Phase index
INTEGER, INTENT(In):: CPS_Pos     ! - Position  in LUT
INTEGER, INTENT(In):: COD_Pos     ! - Position  in LUT
REAL, INTENT(In):: CPS_Wgt        ! - Weight beween positionn and next index in LUT
REAL, INTENT(In):: COD_Wgt        ! - Weight beween positionn and next index in LUT
REAL, INTENT(In):: Alb_Sfc                ! - Surface albedo; value between 0 and 1

!--output
REAL, INTENT(Out):: Refl                         ! - Reflectance at TOA under consideration of surface impact
REAL, INTENT(Out):: Drefl_Dcod           ! - Derivation of reflectance to delta cloud optical depth
REAL, INTENT(Out):: Drefl_Dcps           ! - Derivation of reflectance to delta cloud particle size

!--local
REAL:: Trans_sol             ! - Cloud transmission for radiance coming from solar angle
REAL:: Dtrans_sol_Dcod   ! - Derivation transmission solar angle to COD
REAL:: Dtrans_sol_Dcps   ! - Derivation transmission solar angle to CPS
REAL:: Trans_sat                 ! - Cloud transmission for radiance coming from satellite angle
REAL:: Dtrans_sat_Dcod   ! - Derivation transmission satellite angle to COD
REAL:: Dtrans_sat_Dcps   ! - Derivation transmission satellite angle to CPS
REAL:: Sph_alb                   ! - Spherical Albedo
REAL:: Dsph_alb_Dcod     ! - Derivation to cod
REAL:: Dsph_alb_Dcps     ! - Derivation to cps
REAL:: Alb_Sfc_Term      ! - Term which accounts for albedo surface 
REAL:: Cld_Refl                  ! - Cloud reflectance after interpolation

REAL, DIMENSION(2,2):: Cld_refl_2d    ! - Cloud reflectance 2D with dim CPS and COD array before interpolation
REAL, DIMENSION(2,2):: Trans_sol_2d   ! - Transmission sol 2D with dim CPS and COD array before interpolation
REAL, DIMENSION(2,2):: Trans_sat_2d   ! - Transmission sat 2D with dim CPS and COD array before interpolation

! - local sub arrays
REAL, DIMENSION(2,2)::   Sph_Alb_LUT_Sub   ! - sub table
REAL, DIMENSION(2,2,2):: Trans_Sol_LUT_Sub ! - sub table
REAL, DIMENSION(2,2,2):: Trans_Sat_LUT_Sub ! - sub table
REAL, DIMENSION(2,2,2,2,2):: Refl_LUT_Sub  ! - sub table

! - data which comes from LUT
REAL, DIMENSION(:,:), POINTER::  Sph_Alb_2D_Lut
REAL, DIMENSION(:,:,:), POINTER:: Trans_3D_Lut
REAL(KIND=real4), DIMENSION(:,:,:,:,:), POINTER:: Refl_5D_Lut

!-------------------------------------------------------------------------------
!   executable
!-------------------------------------------------------------------------------

CALL GET_LUT_DATA(Idx_chn, idx_phase &
                          , Refl_5D_LUT    = REFL_5D_LUT &
                  , Trans_3D_LUT   = Trans_3D_LUT &
                  , Sph_Alb_2D_LUT = Sph_Alb_2D_LUT)

! - build sub tables                       
Refl_LUT_Sub      = REFL_5D_LUT(CPS_Pos:CPS_Pos+1 &
                                        ,COD_Pos:COD_Pos+1 &
                                ,Sol_Pos:Sol_Pos+1 &
                                ,Sat_Pos:Sat_Pos+1 &
                                ,Azi_Pos:Azi_Pos+1)
                                
Trans_sol_LUT_Sub = Trans_3D_LUT(CPS_Pos:CPS_Pos+1 &
                                        ,COD_Pos:COD_Pos+1 &
                                        ,Sol_Pos:Sol_Pos+1)
                                
Trans_sat_LUT_Sub = Trans_3D_LUT(CPS_Pos:CPS_Pos+1 &
                                        ,COD_Pos:COD_Pos+1 &
                                        ,Sol2_Pos:Sol2_Pos+1)
                                 
Sph_Alb_LUT_Sub   = Sph_Alb_2D_LUT(CPS_Pos:CPS_Pos+1&
                                        ,COD_Pos:COD_Pos+1)

! - interpolate within the tables with weights calculated in GET_INTERPOLATION_WEIGHT
! - angular weights are stored in module variables 
Cld_Refl  = INTERPOLATE_5D(Refl_Lut_Sub, CPS_Wgt, COD_Wgt &
                                  ,Sol_Wgt, Sat_Wgt, Azi_Wgt, Cld_refl_2d)
                          
Sph_alb   = INTERPOLATE_2D(Sph_Alb_Lut_Sub, CPS_Wgt, COD_Wgt)

Trans_sol = INTERPOLATE_3D(Trans_Sol_Lut_Sub, CPS_Wgt, COD_Wgt &
                                 ,Sol_Wgt,  Trans_Sol_2d)
                         
Trans_sat = INTERPOLATE_3D(Trans_Sat_Lut_Sub, CPS_Wgt, COD_Wgt &
                                 ,Sol2_Wgt, Trans_sat_2d)

! -  calculate the derivations 
Drefl_Dcps      = DERIVATION(Cld_refl_2d, 1 , COD_Wgt, 0.2)
Drefl_Dcod      = DERIVATION(Cld_refl_2d, 2 , CPS_Wgt, 0.1)
Dtrans_sol_Dcps = DERIVATION(Trans_sol_2d, 1 , COD_Wgt, 0.2)
Dtrans_sol_Dcod = DERIVATION(Trans_sol_2d, 2 , CPS_Wgt, 0.1)
Dtrans_sat_Dcps = DERIVATION(Trans_sat_2d, 1 , COD_Wgt, 0.2)
Dtrans_sat_Dcod = DERIVATION(Trans_sat_2d, 2 , CPS_Wgt, 0.1)
Dsph_alb_Dcps   = DERIVATION(Sph_Alb_Lut_Sub, 1 , COD_Wgt, 0.2)
Dsph_alb_Dcod   = DERIVATION(Sph_Alb_Lut_Sub, 2 , CPS_Wgt, 0.1)

! -  
Trans_sol = MAX(0.,Trans_sol)
Trans_sat = MAX(0.,Trans_sat)

! - Account for surface reflection and atmospheric transmission

! - The next two terms are described in ATBD eq. 23 
Alb_Sfc_Term = MAX(0. , Alb_Sfc / (1.0 - Alb_Sfc * sph_alb)) 

Refl = Cld_Refl + Alb_Sfc_Term * Trans_sol * Trans_sat 


! - derivations of equation above
Drefl_Dcod = Drefl_Dcod  &
                           + Alb_Sfc_Term * Trans_sol  * Dtrans_sat_Dcod &
                   + Alb_Sfc_Term * Trans_sat  * Dtrans_sol_Dcod &
                   +  ((Trans_sol * Trans_sat  * Alb_Sfc * Alb_Sfc * Dsph_alb_Dcod) &
                                           /(( 1 - Alb_Sfc * sph_alb)**2)) 
                                           
Drefl_Dcps = Drefl_Dcps  &
                                + Alb_Sfc_Term * Trans_Sol  * Dtrans_sat_Dcps &
                        + Alb_Sfc_Term * Trans_Sat  * Dtrans_sol_Dcps  &
                                +  ((Trans_sol * Trans_Sat  * Alb_Sfc * Alb_Sfc * Dsph_alb_Dcps) &
                                         /(( 1 - Alb_Sfc * Sph_alb)**2 ))
                                           
END SUBROUTINE FORWARD_MODEL_SINGLE_CHANNEL


!======================================================================
! Forward Model for a Thermal Channel
!
! this routine is appropriate for single layer cloud placed in an 
! absorbing atmosphere over a Lambertian Surface. 
!======================================================================
SUBROUTINE FORWARD_MODEL_SINGLE_CHANNEL_EMISSIVITY(&
                         idx_Chn,Idx_Phase, &
                         CPS_wgt,COD_wgt, &
                                                 CPS_pos,COD_pos,  &
                         Refl_Terr,Drefl_Dcod_Terr,Drefl_Dcps_Terr)
                                                 
                                                 
                                                
INTEGER, INTENT(in):: CPS_pos
INTEGER, INTENT(in):: COD_pos
  
INTEGER, INTENT(in):: Idx_Phase
INTEGER, INTENT(in):: Idx_chn
  
REAL, INTENT(in):: CPS_wgt,COD_wgt
  
 
REAL, INTENT(out):: Refl_Terr
REAL, INTENT(out):: Drefl_Dcod_Terr
REAL, INTENT(out):: Drefl_Dcps_Terr
  

REAL, DIMENSION(:,:,:), POINTER:: Trans_3D_Lut
REAL, DIMENSION(:,:,:), POINTER:: EMS_3D_Lut

! - local sub arrays

REAL, DIMENSION(2,2,2):: Trans_Sat_LUT_Sub ! - sub table
REAL, DIMENSION(2,2,2):: Ems_LUT_Sub  ! - sub table

REAL, DIMENSION(2,2):: Trans_sat_2d   ! - Transmission sat 2D with dim CPS and COD array before interpolation
REAL, DIMENSION(2,2):: Ems_2d   

!- clavrx emissivty terms
REAL(KIND=real4):: Rad
REAL(KIND=real4):: Drad_Dcod
REAL(KIND=real4):: Drad_Dcps
REAL(KIND=real4):: B_Temp
REAL(KIND=real4):: DB_Temp_DTemp  

REAL(KIND=real4):: Cld_Emiss
REAL(KIND=real4):: Dems_Dcod
REAL(KIND=real4):: Dems_Dcps
REAL(KIND=real4):: Cld_Trans
REAL(KIND=real4):: Dtrans_Dcod
REAL(KIND=real4):: Dtrans_Dcps
REAL(KIND=real4):: Planck_Rad
REAL(KIND=real4):: Planck_Temp

!----------------------------------------------------------------------
! construct sub-tables from appropriate lookup tables
!----------------------------------------------------------------------

CALL GET_LUT_DATA(Idx_Chn, Idx_Phase &
      ,EMS_3D_LUT = EMS_3D_LUT &   
          ,TRANS_ems_3d_LUT = TRANS_3D_LUT)
          
! - build sub tables
Ems_LUT_Sub   = EMS_3D_LUT(CPS_Pos:CPS_Pos+1 &
                                              ,COD_Pos:COD_Pos+1 &
                                              ,Sat_Pos:Sat_Pos+1 )
                                                  
Trans_Sat_LUT_Sub = Trans_3D_LUT(CPS_Pos:CPS_Pos+1 &
                                        ,COD_Pos:COD_Pos+1 &
                                        ,Sat_Pos:Sat_Pos+1)

Cld_Emiss = INTERPOLATE_3D(Ems_LUT_Sub, CPS_Wgt, COD_Wgt ,Sat_Wgt,Ems_2D)
Cld_Trans = INTERPOLATE_3D(Trans_Sat_LUT_Sub,   CPS_Wgt, COD_Wgt ,Sat_Wgt,Trans_Sat_2D)                         
          
Dems_Dcps       =   DERIVATION(Ems_2d, 1 , COD_Wgt, 0.2)
Dems_Dcod       =   DERIVATION(Ems_2d, 2 , CPS_Wgt, 0.1)
Dtrans_Dcps = DERIVATION(Trans_sat_2d, 1 , COD_Wgt, 0.2)
Dtrans_Dcod = DERIVATION(Trans_sat_2d, 2 , CPS_Wgt, 0.1)


!--- compute planck emission at cloud temperature
 Planck_rad  = planck_rad_fast(idx_chn,Cld_Top_Temp)

!--- compute toa radiance and its Jacobian
 rad = (1-Cld_Trans) * RTM_Rad_AC_Zen_IR  &
       + RTM_Trans_AC_IR * Cld_Emiss * Planck_Rad &
           +  Cld_Trans * RTM_Rad_Clr_IR 
           


! print*,'RAD ==> ',rad
 
 drad_dcod =  - RTM_Rad_AC_Zen_IR  * Dtrans_Dcod &
              + RTM_Trans_AC_IR * Planck_Rad * Dems_Dcod + &
                RTM_Rad_Clr_IR * Dtrans_Dcod

 drad_dcps =   - RTM_Rad_AC_Zen_IR*Dtrans_Dcps &
              + RTM_Trans_AC_IR * Planck_Rad * Dems_Dcps + &
                RTM_Rad_Clr_IR  *Dtrans_Dcps 

! Planck_Temp = planck_temp_fast(idx_chn,rad,dB_dT = dB_dT)

 refl_terr = rad * Rad_To_Refl_Fac
 drefl_dcod_terr = drad_dcod * Rad_To_Refl_Fac
 drefl_dcps_terr = drad_dcps *  Rad_To_Refl_Fac

end SUBROUTINE FORWARD_MODEL_SINGLE_CHANNEL_EMISSIVITY


! ---------------------------------------------------------------------------------
!
! DERIVATION
!
! Returns derivation for one dimension in a 2D array 
! 
!  INPUT: 
!     table:      2D array with 2 elements for each dimension of CPS and COD
!     dim :       set 1 (CPS) or 2 (COD): which dimension will be used for derivation
!     wgt:        the weight for the other dimension
!    diff_dim:    discrete distance between start and end point  
!
! ----------------------------------------------------------------------------------
FUNCTION DERIVATION(table, dim, wgt, diff_dim) RESULT(output)
REAL,DIMENSION(2,2), INTENT(in) :: table
INTEGER,INTENT(in)::dim
REAL,INTENT(in)::diff_dim
REAL,INTENT(in)::wgt
REAL::output
REAL,DIMENSION(2)::table_1d

! - executable
! - interpol the other doimesnion to a 1D array
IF ( dim .eq. 1 ) then
  table_1d(1) = (1.0 - Wgt) * table(1,1) + ( Wgt) * table(1,2)
  table_1d(2) = (1.0 - Wgt) * table(2,1) + ( Wgt) * table(2,2)
ELSE
  table_1d(1) = (1.0 - Wgt) * table(1,1) + ( Wgt) * table(2,1)
  table_1d(2) = (1.0 - Wgt) * table(1,2) + ( Wgt) * table(2,2)
ENDIF

! - Discret derivation 
output =   (table_1d(2) - table_1d(1)) &
                       / diff_dim
END FUNCTION DERIVATION


! ---------------------------------------------------------------------------------
!
! INTERPOLATE_2D
!
! Linear Interpolation for a 2 x 2 
! 
!  Returns interpolated value of a 2D array with 2 elements for each dimension
!
!  INPUT: 
!     table:      3D array
!     Wgt_Dim1,Wgt_Dim2 : weights for each dimension
!
! ----------------------------------------------------------------------------------
FUNCTION INTERPOLATE_2D(table,Wgt_Dim1, Wgt_Dim2) RESULT(output)
REAL, DIMENSION(2,2), INTENT(in):: table
REAL, INTENT(in):: Wgt_Dim1,Wgt_Dim2

REAL::output

!- locals
INTEGER::i
REAL, DIMENSION(2) :: table_1d

! - executable

DO  i = 1, 2
  table_1d(i) = (1.0 - Wgt_Dim2) * table(i,1) + ( Wgt_Dim2) * table(i,2)
ENDDO

output = (1.0 - Wgt_Dim1) * table_1d(1) + ( Wgt_Dim1) * table_1d(2)
                  
END FUNCTION INTERPOLATE_2D

! ---------------------------------------------------------------------------------
!
! INTERPOLATE_3D
!
! Linear Interpolation for a 2 x 2 x 2  
! 
!  Returns interpolated value of a 3D array with 2 elements for each dimension
!
!  INPUT: 
!     table:      3D array
!     Wgt_Dim1,Wgt_Dim2,Wgt_Dim3 : weights for each dimension
!
!  OUTPUT:
!     table_2d: interpolated array of first two dimension 
!
!
!
! ----------------------------------------------------------------------------------
FUNCTION INTERPOLATE_3D (table,Wgt_Dim1, Wgt_Dim2, Wgt_Dim3 , table_2d) RESULT(output)

REAL, DIMENSION(2,2,2), INTENT(in):: table
REAL, INTENT(in):: Wgt_Dim1,Wgt_Dim2,Wgt_Dim3
REAL, DIMENSION(2,2), INTENT(out) :: table_2d
REAL :: output 

! - locals
INTEGER::i,j

! - executable

DO i = 1, 2 
  DO j = 1, 2
         Table_2d(i,j) = (1.0-Wgt_Dim3)*table(i,j,1) + (Wgt_Dim3)*table(i,j,2)
  ENDDO
ENDDO

output =  INTERPOLATE_2D(table_2d,Wgt_Dim1, Wgt_Dim2)

END FUNCTION INTERPOLATE_3D


! ---------------------------------------------------------------------------------
!
! INTERPOLATE_4D
!
! Linear Interpolation for a 2 x 2 x 2 x 2 
! 
!  Returns interpolated value of a 4D array with 2 elements for each dimension
!
!  INPUT: 
!     table:      4D array
!     Wgt_Dim1,Wgt_Dim2,Wgt_Dim3,Wgt_Dim4 : weights for each dimension
!
!  OUTPUT:
!     table_2d: interpolated array of first two dimension 
!
!
!
! ----------------------------------------------------------------------------------
FUNCTION INTERPOLATE_4D (table,Wgt_Dim1, Wgt_Dim2, Wgt_Dim3, Wgt_Dim4 , table_2d) RESULT(output)

REAL, DIMENSION(2,2,2,2), INTENT(in):: table
REAL, INTENT(in):: Wgt_Dim1,Wgt_Dim2,Wgt_Dim3, Wgt_Dim4
REAL,DIMENSION(2,2):: table_2d
REAL :: output 

! - locals
INTEGER::i,j,k
REAL, DIMENSION(2,2,2) :: table_3d

! - executable  

DO i = 1 , 2
  DO j = 1, 2
    DO k = 1, 2
      Table_3d(i,j,k) = (1.0-Wgt_Dim4)*table(i,j,k,1) + (Wgt_Dim4)*table(i,j,k,2)
    ENDDO
  ENDDO
ENDDO


output =  INTERPOLATE_3D(table_3d,Wgt_Dim1, Wgt_Dim2, Wgt_Dim3 , table_2d )

END FUNCTION INTERPOLATE_4D

! ---------------------------------------------------------------------------------
!
! INTERPOLATE_5D
!
! Linear Interpolation for a 2 x 2 x 2 x 2 x 2
! 
!  Returns interpolated value of a 5D array with 2 elements for each dimension
!
!  INPUT: 
!     table:      5D array
!     Wgt_Dim1,Wgt_Dim2,Wgt_Dim3,Wgt_Dim4,Wgt_Dim5 : weights for each dimension
!
!  OUTPUT:
!     table_2d: interpolated array of first two dimension 
!
!
!
! ----------------------------------------------------------------------------------
FUNCTION INTERPOLATE_5D (table,Wgt_Dim1, Wgt_Dim2, Wgt_Dim3, Wgt_Dim4, Wgt_Dim5 , table_2d) &
RESULT(output)

REAL, DIMENSION(2,2,2,2,2), INTENT(in):: table
REAL, INTENT(in):: Wgt_Dim1,Wgt_Dim2,Wgt_Dim3, Wgt_Dim4, Wgt_Dim5
REAL, DIMENSION(2,2) :: table_2d
REAL :: output 

! - locals
INTEGER::i,j,k,m
REAL, DIMENSION(2,2,2,2) :: table_4d

! - executable    
DO i = 1 , 2
  DO j = 1, 2
    DO k = 1, 2
      DO m = 1, 2
        Table_4d(i,j,k,m) = (1.0-Wgt_Dim5)*table(i,j,k,m,1) + (Wgt_Dim5)*table(i,j,k,m,2)
      ENDDO  
    ENDDO
  ENDDO
ENDDO

output =  INTERPOLATE_4D(table_4d,Wgt_Dim1, Wgt_Dim2, Wgt_Dim3, Wgt_Dim4, table_2d)

END FUNCTION INTERPOLATE_5D

! ----------------------------------------------------------------------------------
!
!     CLAVRX TOOL TO RETURN CLOUD ALBEDO and TRANSMISSION VALUES FROM STATE VECTOR
!
! ----------------------------------------------------------------------------------
SUBROUTINE GET_ALBEDO_TRANS_VIS( Cld_Phase &
                                     , State_Vec   &
                                         , Cld_Alb &
                                                                 , Trans_sol &
                                                                 , Trans_sat &
                                                                 , Sph_alb )    

INTEGER(KIND=Int4),INTENT(in) :: Cld_Phase
REAL, DIMENSION(NUM_CHN):: State_Vec

REAL(KIND=real4),INTENT(out)::Cld_Alb
REAL(KIND=real4),INTENT(out)::Trans_sol
REAL(KIND=real4),INTENT(out)::Trans_sat
REAL(KIND=real4),INTENT(out)::Sph_alb


! -- local
INTEGER:: COD_Pos          ! position in LUT
INTEGER:: CPS_Pos          ! position in LUT
REAL(KIND=Real4):: COD_Wgt ! position weights in LUT
REAL(KIND=Real4):: CPS_Wgt ! position weights in LUT
INTEGER:: num_CPS              ! number of CPS values in LUT 
INTEGER:: num_COD              ! number of COD values in LUT 
REAL (Kind=Real4), DIMENSION(:),POINTER :: CPS_Vec_LUT  !- CPS vector in LUT
REAL (Kind=Real4), DIMENSION(:),POINTER :: COD_Vec_LUT  !- COD vector in LUT


! - local sub arrays
REAL, DIMENSION(2,2)::   Sph_Alb_LUT_Sub   ! - sub table
REAL, DIMENSION(2,2,2):: Trans_Sol_LUT_Sub ! - sub table
REAL, DIMENSION(2,2,2):: Trans_Sat_LUT_Sub ! - sub table
REAL, DIMENSION(2,2,2):: Cld_Alb_LUT_Sub   ! - sub table


! - data which comes from LUT
REAL, DIMENSION(:,:), POINTER::  Sph_Alb_2D_LUT
REAL, DIMENSION(:,:,:), POINTER:: Trans_3D_LUT
REAL, DIMENSION(:,:,:), POINTER:: Cld_Alb_3D_LUT

! - dummies
REAL, DIMENSION(2,2):: Cld_refl_2d    
REAL, DIMENSION(2,2):: Trans_sol_2d   
REAL, DIMENSION(2,2):: Trans_sat_2d   

!-- get needed LUT values
CALL GET_LUT_DATA(VIS_CHN_NUMBER,Cld_Phase &
        , CPS_Vec_LUT  = CPS_Vec_LUT &
        , COD_Vec_LUT  = COD_Vec_LUT )
       
Num_CPS = size(CPS_Vec_LUT)
Num_COD = size(COD_Vec_LUT)
   
CALL GET_INTERPOLATION_WEIGHT(Num_CPS,State_Vec(2)&
                                  ,CPS_Vec_LUT,CPS_Wgt,CPS_Pos)   
                          
CALL GET_INTERPOLATION_WEIGHT(Num_COD,State_Vec(1)&
                                 ,COD_Vec_LUT,COD_Wgt,COD_Pos)
                                 
	IF (CPS_Pos .GE. UBOUND(CPS_Vec_LUT,DIM=1)) THEN
	  CPS_Pos = UBOUND(CPS_Vec_LUT,DIM=1) - 1
	  CPS_Wgt = 1.
	ENDIF			
	
    IF (COD_Pos .GE. UBOUND(COD_Vec_LUT,DIM=1)) THEN
	  COD_Pos = UBOUND(COD_Vec_LUT,DIM=1) - 1
	  COD_Wgt = 1.
	ENDIF						
	
CALL GET_LUT_DATA(VIS_CHN_NUMBER, Cld_Phase &
                  , Trans_3D_LUT   = Trans_3D_LUT &
                  , Sph_Alb_2D_LUT = Sph_Alb_2D_LUT &
              , Cld_Alb_3D_LUT = Cld_Alb_3D_LUT )

Trans_sol_LUT_Sub = Trans_3D_LUT(CPS_Pos:CPS_Pos+1 &
                                        ,COD_Pos:COD_Pos+1 &
                                        ,Sol_Pos:Sol_Pos+1)
                                
Trans_sat_LUT_Sub = Trans_3D_LUT(CPS_Pos:CPS_Pos+1 &
                                        ,COD_Pos:COD_Pos+1 &
                                        ,Sol2_Pos:Sol2_Pos+1)
                                 
Sph_Alb_LUT_Sub   = Sph_Alb_2D_LUT(CPS_Pos:CPS_Pos+1&
                                        ,COD_Pos:COD_Pos+1)


Cld_Alb_LUT_Sub = Cld_Alb_3D_LUT(CPS_Pos:CPS_Pos+1 &
                                        ,COD_Pos:COD_Pos+1 &
                                        ,Sol_Pos:Sol_Pos+1)                                     

! - interpolate within the tables with weights calculated in GET_INTERPOLATION_WEIGHT
! - angular weights are stored in module variables 

                          
Sph_alb   = INTERPOLATE_2D(Sph_Alb_Lut_Sub, CPS_Wgt, COD_Wgt)

Trans_sol = INTERPOLATE_3D(Trans_Sol_Lut_Sub, CPS_Wgt, COD_Wgt &
                                 ,Sol_Wgt,  Trans_Sol_2d)
                         
Trans_sat = INTERPOLATE_3D(Trans_Sat_Lut_Sub, CPS_Wgt, COD_Wgt &
                                 ,Sol2_Wgt, Trans_sat_2d)                                

Cld_Alb =  INTERPOLATE_3D(Cld_Alb_LUT_Sub, CPS_Wgt, COD_Wgt &
                                 ,Sol_Wgt, Trans_sat_2d)        


END SUBROUTINE GET_ALBEDO_TRANS_VIS

END MODULE CLOUD_TAU_RE_SOLAR_MODULE

