! $Id: comp_asc_des_level2b.f90,v 1.41 2010/09/23 21:35:52 heidinger Exp $
!---------------------------------------------------------------------
!@***h* PATMOSX/comp_asc_des_level2b.f90
! 
! NAME
!   comp_asc_des_level2b.f90
! 
! LOCATION
!   /patmosx/src/comp_asc_des_level2b.f90
! 
! PURPOSE
!   This code takes the orbit level2 files for one day from one satellite
!   and writes separate level2b files for the ascending and descending nodes
! 
! DESCRIPTION
!   This code takes the level2 files created for each orbit
!   for one day from one satellite and writes separate level2b 
!   files for the ascending and descending nodes. This code runs
!   after clavrxorb has processed the level-1b files for one day.
!   This program reads input (directories and filenames) from a file
!   called comp_asc_des_level2b_input.  Currently, there are no
!   command line arguments. 
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   May 2009
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!  Format of Required Input File:
!    line 1: directory of level2 files (input)
!    line 2: directory of level2b files (output)
!    line 3: year
!    line 4: julian day
!    line 5: satellite number (i.e. 18 = NOAA-18)
!    line 6: longitudes (west, east, spacing)
!    line 7: latitudes (south, north, spacing)
!    line 8+: level2 file name (one per line)
!
! Reference:  Level2 and Level2b file contents are given on
!             http://cimss.ssec.wisc.edu/patmosx
!
! Calling Sequence:  
!    comp_asc_des_level2 node nav_flag geo_flag
!
!    where:   
!          node = asc or des
!          nav_flag = 0 (compute pixel lat,lon from level2b)
!                     1 (read pixel lat, lat from nav file)
!          geo_flag = 0 (compute pixel angles from level2b)
!                     1 (read pixel angles from nav file)
!
! EXTERNALS
!  Dependencies:  (The following are names of modules)
!    CONSTANTS
!    HDF
!    HDF_PARAMS
!    NUMERICAL_ROUTINES
!    SCALING_PARAMETERS
!    LEVEL2B_ROUTINES
!    FILE_UTILITY
!
!
! MODIFICATION HISTORY
!     June 2010: Added options to use geo and nav files for speed   
!     August 2010: Added FCDR attributes
!     September 2010: Added descriptive global attributes
!
!@*****
!------------------------------------------------------------------------
!
! Source Code Name:
!    comp_asc_des_level2b.f90
!
! Program Name:
!    COMPILE_ASC_DES_LEVEL2B
!
! Function:
!    A main code that generates one of the executables in the CLAVR-x processing
!    system
!
! Descripton:
!    this code takes the level2 files created for each orbit
!    for one day from one satellite and writes separate level2b 
!    files for the ascending and descending nodes. This code runs
!    after clavrxorb has processed the level-1b files for one day.
!    This program reads input (directories and filenames) from a file
!    called comp_asc_des_level2b_input.  Currently, there are no
!    command line arguments.  
!
! Format of Required Input File:
!    line 1: directory of level2 files (input)
!    line 2: directory of level2b files (output)
!    line 3: year
!    line 4: julian day
!    line 5: satellite number (i.e. 18 = NOAA-18)
!    line 6: longitudes (west, east, spacing)
!    line 7: latitudes (south, north, spacing)
!    line 8+: level2 file name (one per line)
!
!
! Reference:  Level2 and Level2b file contents are given on
!             http://cimss.ssec.wisc.edu/patmosx
!
! Dependencies:  (The following are names of modules)
!    CONSTANTS
!    HDF
!    HDF_PARAMS
!    NUMERICAL_ROUTINES
!    SCALING_PARAMETERS
!    LEVEL2B_ROUTINES
!    FILE_UTILITY
!
! Calling Sequence:  
!    comp_asc_des_level2 node nav_flag geo_flag
!
!    where:   
!          node = asc or des
!          nav_flag = 0 (compute pixel lat,lon from level2b)
!                     1 (read pixel lat, lat from nav file)
!          geo_flag = 0 (compute pixel angles from level2b)
!                     1 (read pixel angles from nav file)
!
! Public Routines Within This Program:
!     None
!
! Author: Andrew K. Heidinger, National Oceanic and Atmospheric Administration
! (c) This code is copyrighted by the author and all NOAA restrictions apply
! Date: May 2009
!
! Revision History:
!     June 2010: Added options to use geo and nav files for speed   
!     August 2010: Added FCDR attributes
!     September 2010: Added descriptive global attributes
!----------------------------------------------------------------------
program COMPILE_ASC_DES_LEVEL2B
 use CONSTANTS
 use HDF
 use HDF_PARAMS
 use NUMERICAL_ROUTINES
 use SCALING_PARAMETERS
 use LEVEL2B_ROUTINES
 use FILE_UTILITY

 implicit none

 integer, parameter:: n_files_max = 100
 integer(kind=int4):: n_command_line_args

 integer(kind=int4):: first_valid_input
 integer(kind=int4):: sd_id_input
 integer(kind=int4):: sd_id_output
 integer(kind=int4):: sds_id_temp
 integer(kind=int4):: nav_sd_id_input
 integer(kind=int4):: geo_sd_id_input
 integer(kind=int4), dimension(2):: sds_dims_output_xy
 integer(kind=int4), dimension(1):: sds_dims_output_x
 integer(kind=int4), dimension(1):: sds_dims_output_y
 integer(kind=int4), dimension(1):: sds_output_start
 integer(kind=int4), dimension(1):: sds_output_stride
 integer(kind=int4), dimension(2):: sds_dims
 integer(kind=int4), dimension(2):: sds_chunk_size
 real(kind=real4), dimension(:), allocatable:: input_array_1d
 real(kind=real4), dimension(:), allocatable:: output_array_1d

 real(kind=real4), dimension(:,:), allocatable:: scaled_sds_data_input
 real(kind=real4), dimension(:,:), allocatable:: unscaled_sds_data_input
 real(kind=real4), dimension(:,:), allocatable:: scaled_sds_data_anchors_input
 real(kind=real4), dimension(:,:), allocatable:: unscaled_sds_data_anchors_input
 real(kind=real4), dimension(:,:), allocatable:: scaled_sds_data_output
 real(kind=real4), dimension(:,:), allocatable:: unscaled_sds_data_output
 real(kind=real4), dimension(:,:,:), allocatable:: scaled_sds_data_output_full
 character(len=100),dimension(n_files_max):: file_input
 character(len=100),dimension(n_files_max):: nav_file_input
 character(len=100),dimension(n_files_max):: geo_file_input
 character(len=100):: file_output
 character(len=100):: dir_in
 character(len=100):: dir_nav_in
 character(len=100):: dir_geo_in
 character(len=100):: dir_out
 character(len=100):: root_name
 integer:: comp_asc_des_input_lun
 integer:: ielem
 integer:: iline
 integer:: ilon
 integer:: ilat
 integer:: ifile
 integer:: isds
 integer:: ipoint

 integer:: n_files
 integer:: asc_des_node
 integer(kind=int2):: sc_id_output
 integer(kind=int2):: sc_id_input
 integer:: istatus_sum
 integer:: ios
 integer(kind=int4):: jday
 integer(kind=int4):: year
 real(kind=real4):: start_time_input
 real(kind=real4):: end_time_input
 integer(kind=int2):: start_day_input
 integer(kind=int2):: end_day_input
 integer(kind=int2):: start_year_input
 integer(kind=int2):: end_year_input
 integer(kind=int4):: num_anchors_input
 integer(kind=int4):: num_elements_input
 integer(kind=int4):: num_lines_input
 integer(kind=int4):: num_sds_input
 integer(kind=int4):: num_global_attrs
 character(len=40), dimension(2):: sds_2d_dim_string

 real(kind=real4):: lon_west
 real(kind=real4):: lon_east
 real(kind=real4):: dlon_output
 real(kind=real4):: lat_north
 real(kind=real4):: lat_south
 real(kind=real4):: dlat_output
 integer(kind=int4):: nlon_output
 integer(kind=int4):: nlat_output
 integer(kind=int4):: dateline_flag
 real(kind=real4), dimension(:), allocatable:: lon_output_1d
 real(kind=real4), dimension(:), allocatable:: lat_output_1d

 type(sds_struct), dimension(0:100) :: sds
 type(sds_struct) :: sds_lat
 type(sds_struct) :: sds_lon
 type(sds_struct) :: sds_senzen
 type(sds_struct) :: sds_solzen
 type(sds_struct) :: sds_relaz
 type(sds_struct) :: sds_solaz
 type(sds_struct) :: sds_time
 type(sds_struct) :: sds_ch4
 type(sds_struct) :: sds_bad_scan_flag
 type(sds_struct) :: sds_asc_des_flag
 type(sds_struct) :: sds_post_cld_prob

 character(len=4):: year_string
 character(len=3):: jday_string
 character(len=3):: sat_string
 character(len=3):: node_string
 character(len=1):: flag_string


 integer(kind=int4), dimension(:), allocatable:: input_update_index
 integer(kind=int4), dimension(:), allocatable:: output_update_index

 integer(kind=int1), dimension(:,:), allocatable:: update_output_mask

 real(kind=real4), dimension(:,:), allocatable:: lon_anchors_input
 real(kind=real4), dimension(:,:), allocatable:: lat_anchors_input
 real(kind=real4), dimension(:,:), allocatable:: senzen_anchors_input
 real(kind=real4), dimension(:,:), allocatable:: relaz_anchors_input
 real(kind=real4), dimension(:,:), allocatable:: solaz_anchors_input
 real(kind=real4), dimension(:,:), allocatable:: solzen_anchors_input
 real(kind=real4), dimension(:), allocatable:: time_anchors_input
 real(kind=real4), dimension(:), allocatable:: bad_scan_flag_input
 real(kind=real4), dimension(:), allocatable:: asc_des_flag_input

 real(kind=real4), dimension(:,:), allocatable:: time_input
 real(kind=real4), dimension(:,:), allocatable:: ch4_input

 real(kind=real4), dimension(:,:), allocatable:: solzen_input
 real(kind=real4), dimension(:,:), allocatable:: senzen_input
 real(kind=real4), dimension(:,:), allocatable:: relaz_input
 real(kind=real4), dimension(:,:), allocatable:: solaz_input
 real(kind=real4), dimension(:,:), allocatable:: post_cld_prob_input
 real(kind=real4), dimension(:,:), allocatable:: senzen_output
 real(kind=real4), dimension(:,:), allocatable:: solzen_output
 real(kind=real4), dimension(:,:), allocatable:: relaz_output
 real(kind=real4), dimension(:,:), allocatable:: solaz_output
 real(kind=real4), dimension(:,:), allocatable:: time_output
 real(kind=real4), dimension(:,:), allocatable:: post_cld_prob_output

 integer:: use_nav_file 
 integer:: use_geo_file 

 real:: processing_time_start_hours
 real:: processing_time_end_hours
 integer:: sfstart
 integer:: sfn2index
 integer:: sffattr
 integer:: sfrnatt
 integer:: sfsnatt
 integer:: sfscatt
 integer:: sffinfo
 integer:: sfginfo
 integer:: sfselect
 integer:: sfcreate
 integer:: sfwdata
 integer:: sfendacc
 integer:: sfend

 real(kind=real4):: domain_data_fraction     !percentage of domain with data
 integer(kind=int4):: count_valid            !number of domain points with data
 real(kind=real4):: domain_day_fraction      !percentage of domain with daytime data
 integer(kind=int4):: count_day              !number of domain points with daytime data
 real(kind=real4):: domain_cloud_fraction    !percentage of domain with cloudy data
 integer(kind=int4):: count_cloud            !number of domain points with cloudy data

!----------------------------------------------------------------------
! Begin Executable Code
!----------------------------------------------------------------------
processing_time_start_hours = COMPUTE_TIME_HOURS()

!----------------------------------------------------------------------
! command line arguments
!----------------------------------------------------------------------
 n_command_line_args = iargc()

 if (n_command_line_args == 3) then 
     call getarg(1, node_string)
     call getarg(2, flag_string)
     read(flag_string,'(i1)') use_nav_file
     call getarg(3, flag_string)
     read(flag_string,'(i1)') use_geo_file
 else
     print *, "COMP_ASC_DES_LEVEL2b ERROR:: Expecting Three Command Line Arguments"
     stop
 endif
 if (node_string /= "asc" .and. node_string /= "des") then
     print *, "COMP_ASC_DES_LEVEL2b ERROR:: Node argument is not asc or des"
     stop 
 endif
 if (use_nav_file /= 0 .and. use_nav_file /= 1) then
     print *, "COMP_ASC_DES_LEVEL2b ERROR:: Use_Nav_File argument is not 0 or 1"
     stop 
 endif
 if (use_geo_file /= 0 .and. use_geo_file /= 1) then
     print *, "COMP_ASC_DES_LEVEL2b ERROR:: Use_Geo_File argument is not 0 or 1"
     stop 
 endif

 sds_2d_dim_string(1) = "longitude_index"
 sds_2d_dim_string(2) = "latitude_index"


!-----------------------------------------------------
! open input file
!----------------------------------------------------
comp_asc_des_input_lun = GET_LUN()
open(unit=comp_asc_des_input_lun,file="comp_asc_des_level2b_input",status="old",action="read",position="rewind",iostat=ios)
if (ios /= 0) then
  print *, "error opening comp_asc_dec control file, ios = ", ios
  stop 1
else
  read(unit=comp_asc_des_input_lun,fmt="(A)") dir_in
  read(unit=comp_asc_des_input_lun,fmt="(A)") dir_nav_in
  read(unit=comp_asc_des_input_lun,fmt="(A)") dir_geo_in
  read(unit=comp_asc_des_input_lun,fmt="(A)") dir_out
  read(unit=comp_asc_des_input_lun,fmt=*) year
  read(unit=comp_asc_des_input_lun,fmt=*) jday
  read(unit=comp_asc_des_input_lun,fmt=*) sc_id_output
  read(unit=comp_asc_des_input_lun,fmt=*) lon_west, lon_east, dlon_output
  read(unit=comp_asc_des_input_lun,fmt=*) lat_south, lat_north, dlat_output
endif

!--- read in file names
do ifile = 1, n_files_max
    read(unit=comp_asc_des_input_lun,fmt=*,iostat=ios) file_input(ifile)
    if (ios /= 0) then
       exit
    endif
end do
n_files = ifile - 1
if (n_files == 0) then
   print *, "No files to process, stopping"
   stop
endif

!------------------------------------------------------------------------
! construct nav and geo files
!------------------------------------------------------------------------
do ifile = 1, n_files
  root_name = trim(file_input(ifile))
  root_name = root_name(1:len(trim(file_input(ifile)))-11)
  nav_file_input(ifile) = trim(root_name)//".nav.hdf"
  geo_file_input(ifile) = trim(root_name)//".geo.hdf"
enddo

!------------------------------------------------------------------------
! determine if this grid spans the dateline
!------------------------------------------------------------------------
dateline_flag = sym%NO
if (lon_west > 0 .and. lon_east < 0.0) dateline_flag = sym%YES

!------------------------------------------------------------------------
! compute map parameters based on input
!------------------------------------------------------------------------
if (dateline_flag == sym%NO) then
  nlon_output = int(lon_east - lon_west) / dlon_output + 1
  lon_east = lon_west + dlon_output * (nlon_output-1)
else
  nlon_output = int((360.0+lon_east) - lon_west) / dlon_output + 1
  lon_east = lon_west + dlon_output * (nlon_output-1) - 360.0
endif

nlat_output = int(lat_north - lat_south) / dlat_output + 1
lat_north = lat_south + dlat_output * (nlat_output-1)

!--- make string needed for output filename
write (year_string,  '(I4.4)') year
write (jday_string,  '(I3.3)') jday
write (sat_string,   '(I2.2)') sc_id_output

if (sc_id_output > 4) then
   sat_string = "n"//sat_string
else
   sat_string = "m"//sat_string
endif

!--- store output dimensions
sds_dims_output_xy = (/nlon_output,nlat_output/)
sds_dims_output_x = (/nlon_output/)
sds_dims_output_y = (/nlat_output/)
sds_output_start = (/0/)
sds_output_stride = (/1/)

    !--- set node integer value
    if (node_string == "asc") then
     asc_des_node = 0
    else
     asc_des_node = 1
    endif

    !--------------------------------------------------------------------
    ! create name for composite
    !--------------------------------------------------------------------

    !--- construct output file name for this node
    file_output = "patmosx_"//trim(sat_string)//"_"//trim(node_string)//"_"// &
               trim(year_string)//"_"//trim(jday_string)// &
               ".level2b.hdf"

    print *, "Creating ", trim(file_output)

    !--- open file for reading and writing
    sd_id_output = sfstart(trim(dir_out)//trim(file_output),DFACC_CREATE)

    if (sd_id_output <  0) then
       print *, "output HDF file creation failed. Exiting...", asc_des_node
       stop 38
    endif

    !--------------------------------------------------------------------------------
    ! allocate and initialize arrays for composite
    !--------------------------------------------------------------------------------
     allocate(time_output(nlon_output,nlat_output))
     allocate(senzen_output(nlon_output,nlat_output))
     allocate(relaz_output(nlon_output,nlat_output))
     allocate(solaz_output(nlon_output,nlat_output))
     allocate(solzen_output(nlon_output,nlat_output))
     allocate(post_cld_prob_output(nlon_output,nlat_output))
     allocate(scaled_sds_data_output(nlon_output,nlat_output))
     allocate(unscaled_sds_data_output(nlon_output,nlat_output))
     allocate(lon_output(nlon_output,nlat_output))
     allocate(lat_output(nlon_output,nlat_output))
     allocate(ielem_output(nlon_output,nlat_output))
     allocate(iline_output(nlon_output,nlat_output))
     allocate(update_output_mask(nlon_output,nlat_output))
     allocate(lon_output_1d(nlon_output))
     allocate(lat_output_1d(nlat_output))

     allocate(output_update_index(nlon_output*nlat_output))
     allocate(output_array_1d(nlon_output*nlat_output))

     output_array_1d = missing_value_real4
     time_output = missing_value_real4
     senzen_output = missing_value_real4
     relaz_output = missing_value_real4
     solaz_output = missing_value_real4
     solzen_output = missing_value_real4
     post_cld_prob_output = missing_value_real4
     scaled_sds_data_output = missing_value_real4
     unscaled_sds_data_output = missing_value_real4
     lon_output = missing_value_real4
     lat_output = missing_value_real4
     ielem_output = missing_value_int4
     iline_output = missing_value_int4
     update_output_mask = sym%NO
     output_update_index = 0

     first_valid_input = sym%YES
    !---------------------------------------------------------------------------------
    ! loop through orbit files
    !---------------------------------------------------------------------------------
    file_loop: do ifile = 1, n_files

        print *, "processing file ", ifile, " of ", n_files, " ", trim(file_input(ifile))

        istatus_sum = 0

        !---- determine if this file should be skipped
        sd_id_input = sfstart(trim(dir_in)//trim(file_input(ifile)), DFACC_READ)

        if (use_nav_file == sym%YES) then
          nav_sd_id_input = sfstart(trim(dir_nav_in)//trim(nav_file_input(ifile)), DFACC_READ)
        endif

        if (use_geo_file == sym%YES) then
          geo_sd_id_input = sfstart(trim(dir_geo_in)//trim(geo_file_input(ifile)), DFACC_READ)
        endif

        !--- determine number of sds's in these files (assume the same)
        istatus_sum = sffinfo(sd_id_input,num_sds_input,num_global_attrs)

        if (sd_id_input == FAIL) cycle
        if (use_nav_file == sym%YES .and. nav_sd_id_input == FAIL) cycle
        if (use_geo_file == sym%YES .and. geo_sd_id_input == FAIL) cycle

          !--- read attributes of this file
          istatus_sum = sfrnatt(sd_id_input,sffattr(sd_id_input,"NUMBER_OF_ANCHORS"),num_anchors_input) + istatus_sum
          istatus_sum = sfrnatt(sd_id_input,sffattr(sd_id_input,"NUMBER_OF_ELEMENTS"),num_elements_input) + istatus_sum
          istatus_sum = sfrnatt(sd_id_input,sffattr(sd_id_input,"NUMBER_OF_SCANS_LEVEL2"),num_lines_input) + istatus_sum
          istatus_sum = sfrnatt(sd_id_input,sffattr(sd_id_input,"START_YEAR"),start_year_input) + istatus_sum
          istatus_sum = sfrnatt(sd_id_input,sffattr(sd_id_input,"END_YEAR"),end_year_input) + istatus_sum
          istatus_sum = sfrnatt(sd_id_input,sffattr(sd_id_input,"START_DAY"),start_day_input) + istatus_sum
          istatus_sum = sfrnatt(sd_id_input,sffattr(sd_id_input,"END_DAY"),end_day_input) + istatus_sum
          istatus_sum = sfrnatt(sd_id_input,sffattr(sd_id_input,"START_TIME"),start_time_input) + istatus_sum
          istatus_sum = sfrnatt(sd_id_input,sffattr(sd_id_input,"END_TIME"),end_time_input) + istatus_sum
          istatus_sum = sfrnatt(sd_id_input,sffattr(sd_id_input,"SPACECRAFT_ID"),sc_id_input) + istatus_sum

          !--- based on attributes, see if this file should analyzed further
          if ((start_year_input /= year) .and. (end_year_input /= year)) cycle
          if ((start_day_input /= jday) .and. (end_day_input /= jday)) cycle
          if (sc_id_output /= sc_id_input) cycle

          !--- copy global attributes of first valid file to output file
          if (first_valid_input == sym%YES) then

             call COPY_GLOBAL_ATTRIBUTES(sd_id_input,sd_id_output)

             !--- add in attributes that define the output grid
             istatus_sum = sfsnatt(sd_id_output,"NUMBER_OF_LONGITUDES",DFNT_INT32,1,nlon_output) + istatus_sum
             istatus_sum = sfsnatt(sd_id_output,"NUMBER_OF_LATITUDES",DFNT_INT32,1,nlat_output) + istatus_sum
             istatus_sum = sfsnatt(sd_id_output,"LONGITUDE_SPACING",DFNT_FLOAT32,1,dlon_output) + istatus_sum
             istatus_sum = sfsnatt(sd_id_output,"LATITUDE_SPACING",DFNT_FLOAT32,1,dlat_output) + istatus_sum
             istatus_sum = sfsnatt(sd_id_output,"WESTERN_MOST_LONGITUDE",DFNT_FLOAT32,1,lon_west) + istatus_sum
             istatus_sum = sfsnatt(sd_id_output,"EASTERN_MOST_LONGITUDE",DFNT_FLOAT32,1,lon_east) + istatus_sum
             istatus_sum = sfsnatt(sd_id_output,"NORTHERN_MOST_LATITUDE",DFNT_FLOAT32,1,lat_north) + istatus_sum
             istatus_sum = sfsnatt(sd_id_output,"SOUTHERN_MOST_LATITUDE",DFNT_FLOAT32,1,lat_south) + istatus_sum

          endif

          !--- allocate memory for input data
          allocate(scaled_sds_data_input(num_elements_input,num_lines_input))
          allocate(lon_input(num_elements_input,num_lines_input))
          allocate(lat_input(num_elements_input,num_lines_input))
          allocate(senzen_input(num_elements_input,num_lines_input))
          allocate(relaz_input(num_elements_input,num_lines_input))
          allocate(solaz_input(num_elements_input,num_lines_input))
          allocate(solzen_input(num_elements_input,num_lines_input))
          allocate(post_cld_prob_input(num_elements_input,num_lines_input))

          if (use_nav_file == sym%NO) then

            allocate(lon_anchors_input(num_anchors_input,num_lines_input))
            allocate(lat_anchors_input(num_anchors_input,num_lines_input))
     
            !--- read longitude
      
            allocate(scaled_sds_data_anchors_input(num_anchors_input,num_lines_input))

            sds_lon%variable_name = "anchor_longitude"
      
            call READ_SDS(sd_id_input,scaled_sds_data_anchors_input,sds_lon)
            call UNSCALE_SDS(sds_lon,scaled_sds_data_anchors_input, lon_anchors_input)
     
            !--- read latiitude
            sds_lat%variable_name = "anchor_latitude"
     
            call READ_SDS(sd_id_input,scaled_sds_data_anchors_input,sds_lat)
            call UNSCALE_SDS(sds_lat,scaled_sds_data_anchors_input, lat_anchors_input)
          
            deallocate(scaled_sds_data_anchors_input)
      
            do iline = 1, num_lines_input

               if (maxval(abs(lat_anchors_input(:,iline))) > 85.0) then
                 call GNOMIC_ANCHOR_INTERP(num_elements_input,lon_anchors_input(:,iline),lat_anchors_input(:,iline), &
                                   lon_input(:,iline),lat_input(:,iline))
               else
                 call LAGRANGIAN_ANCHOR_INTERP(num_elements_input,lat_anchors_input(:,iline),lat_input(:,iline))
                 call LAGRANGIAN_ANCHOR_INTERP(num_elements_input,lon_anchors_input(:,iline),lon_input(:,iline))
               endif

            enddo
      
            deallocate(lon_anchors_input)
            deallocate(lat_anchors_input)

           else

            sds_lon%variable_name = "pixel_longitude"
            call READ_SDS(nav_sd_id_input,scaled_sds_data_input,sds_lon)
            call UNSCALE_SDS(sds_lon,scaled_sds_data_input, lon_input)
            sds_lon%variable_name = "anchor_longitude"

            sds_lat%variable_name = "pixel_latitude"
            call READ_SDS(nav_sd_id_input,scaled_sds_data_input,sds_lat)
            call UNSCALE_SDS(sds_lat,scaled_sds_data_input, lat_input)
            sds_lat%variable_name = "anchor_latitude"

           endif


          !--- compute remapping arrays for this orbit
          call regrid( lat_south, &
                       dlat_output,      &
                       nlat_output,      &
                       lon_west,  &
                       dlon_output,      &
                       nlon_output)
             
         !--- if first valid file, write output lon and lat vectors to output file
         if (first_valid_input == sym%YES) then

              !--- longitude (first row only)
              lon_output_1d = lon_output(:,1)
              sds_id_temp = sfcreate(sd_id_output,"longitude",DFNT_FLOAT32,1,sds_dims_output_x)
              istatus_sum = sfsnatt(sds_id_temp, "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
              istatus_sum = sfscatt(sds_id_temp, "UNITS", DFNT_CHAR8, len_trim("degrees"), "degrees") + istatus_sum
              istatus_sum = sfsnatt(sds_id_temp, "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum
              istatus_sum = sfwdata(sds_id_temp, sds_output_start, sds_output_stride,  &
                                sds_dims_output_x, lon_output_1d) + istatus_sum 
              istatus_sum = sfendacc(sds_id_temp) + istatus_sum

              !--- latitude (first column only)
              lat_output_1d = lat_output(1,:)
              sds_id_temp = sfcreate(sd_id_output,"latitude",DFNT_FLOAT32,1,sds_dims_output_y)
              istatus_sum = sfsnatt(sds_id_temp, "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
              istatus_sum = sfscatt(sds_id_temp, "UNITS", DFNT_CHAR8, len_trim("degrees"), "degrees") + istatus_sum
              istatus_sum = sfsnatt(sds_id_temp, "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum
              istatus_sum = sfwdata(sds_id_temp, sds_output_start, sds_output_stride,  &
                                sds_dims_output_y, lat_output_1d) + istatus_sum 
              istatus_sum = sfendacc(sds_id_temp) + istatus_sum
         endif

         allocate(input_update_index(num_elements_input*num_lines_input))
         allocate(input_array_1d(num_elements_input*num_lines_input))

         !--- allocate memory for output on first_valid_input
          if (first_valid_input == sym%YES) then
            allocate(scaled_sds_data_output_full(nlon_output, nlat_output, num_sds_input))
          endif

         !--------------------------------------------------------------
         !--- bad scan flag (assumed no scaling applied)
         !--------------------------------------------------------------
          sds_bad_scan_flag%variable_name = "bad_scan_line_flag"
          allocate(bad_scan_flag_input(num_lines_input))
          call READ_SDS(sd_id_input,bad_scan_flag_input,sds_bad_scan_flag)
          sds_bad_scan_flag%rank = 1

         !--------------------------------------------------------------
         !--- ascending / descending flag (assumed no scaling applied)
         !--------------------------------------------------------------
          sds_asc_des_flag%variable_name = "asc_des_flag"
          allocate(asc_des_flag_input(num_lines_input))
          call READ_SDS(sd_id_input,asc_des_flag_input,sds_asc_des_flag)
          sds_asc_des_flag%rank = 1

         !--------------------------------------------------------------
         !--- read time (assumed no scaling applied)
         !--------------------------------------------------------------
          sds_time%variable_name = "scan_line_time"
          allocate(time_anchors_input(num_lines_input))
          call READ_SDS(sd_id_input,time_anchors_input,sds_time)
          sds_time%rank = 2

          !---- extrapolate scan-line time to each pixel
          allocate(time_input(num_elements_input,num_lines_input))
          do iline = 1, num_lines_input
              time_input(:,iline) = time_anchors_input(iline)
          enddo
          deallocate(time_anchors_input)

          !--- define output or read in output values
          if (first_valid_input == sym%YES) then   !define sds in output

                 call DEFINE_SDS(sd_id_output, &
                                 sds_dims_output_xy, &
                                 sds_time)
                 time_output(:,:) = missing_value_real4

          endif

         !--------------------------------------------------------------
         !--- sensor zenith
         !--------------------------------------------------------------
         if (use_geo_file == sym%NO) then
           allocate(senzen_anchors_input(num_anchors_input,num_lines_input))
           allocate(scaled_sds_data_anchors_input(num_anchors_input,num_lines_input))
     
           sds_senzen%variable_name = "anchor_sensor_zenith"
           call READ_SDS(sd_id_input,scaled_sds_data_anchors_input,sds_senzen)
           call UNSCALE_SDS(sds_senzen,scaled_sds_data_anchors_input, senzen_anchors_input)
         
           !--- extrapolate  senzen
           do iline = 1, num_lines_input
             call LINEAR_ANCHOR_INTERP(num_elements_input,senzen_anchors_input(:,iline),senzen_input(:,iline))
           enddo

           deallocate(scaled_sds_data_anchors_input)
           deallocate(senzen_anchors_input)

         else
           sds_senzen%variable_name = "sensor_zenith"
           call READ_SDS(geo_sd_id_input,scaled_sds_data_input,sds_senzen)
           call UNSCALE_SDS(sds_senzen,scaled_sds_data_input, senzen_input)
           sds_senzen%variable_name = "anchor_sensor_zenith"  !needed to ensure skipping in sds loop
         endif

         !--- define output or read in output values
         if (first_valid_input == sym%YES) then   !define sds in output
                 call DEFINE_SDS(sd_id_output,sds_dims_output_xy,sds_senzen)
                 senzen_output(:,:) = missing_value_real4
         endif

         !--------------------------------------------------------------
         !--- solar zenith
         !--------------------------------------------------------------
         if (use_geo_file == sym%NO) then
           allocate(solzen_anchors_input(num_anchors_input,num_lines_input))
           allocate(scaled_sds_data_anchors_input(num_anchors_input,num_lines_input))
     
           sds_solzen%variable_name = "anchor_solar_zenith"
           call READ_SDS(sd_id_input,scaled_sds_data_anchors_input,sds_solzen)
           call UNSCALE_SDS(sds_solzen,scaled_sds_data_anchors_input, solzen_anchors_input)
         
           !--- extrapolate  solzen
           do iline = 1, num_lines_input
             call LINEAR_ANCHOR_INTERP(num_elements_input,solzen_anchors_input(:,iline),solzen_input(:,iline))
           enddo

           deallocate(scaled_sds_data_anchors_input)
           deallocate(solzen_anchors_input)

         else
           sds_solzen%variable_name = "solar_zenith"
           call READ_SDS(geo_sd_id_input,scaled_sds_data_input,sds_solzen)
           call UNSCALE_SDS(sds_solzen,scaled_sds_data_input, solzen_input)
           sds_solzen%variable_name = "anchor_solar_zenith"  !needed to ensure skipping in sds loop
         endif

         !--- define output or read in output values
         if (first_valid_input == sym%YES) then   !define sds in output
                 call DEFINE_SDS(sd_id_output,sds_dims_output_xy,sds_solzen)
                 solzen_output(:,:) = missing_value_real4
         endif


         !--------------------------------------------------------------
         !--- posterior probability of cloud
         !--------------------------------------------------------------
         sds_post_cld_prob%variable_name = "cloud_probability"
         call READ_SDS(sd_id_input,scaled_sds_data_input,sds_post_cld_prob)
         call UNSCALE_SDS(sds_post_cld_prob,scaled_sds_data_input, post_cld_prob_input)

         !--- define output or read in output values
         if (first_valid_input == sym%YES) then   !define sds in output
                 call DEFINE_SDS(sd_id_output,sds_dims_output_xy,sds_post_cld_prob)
                 post_cld_prob_output(:,:) = missing_value_real4
         endif

         !--------------------------------------------------------------
         !--- relative azimuth
         !--------------------------------------------------------------
         if (use_geo_file == sym%NO) then
           allocate(relaz_anchors_input(num_anchors_input,num_lines_input))
           allocate(scaled_sds_data_anchors_input(num_anchors_input,num_lines_input))
     
           sds_relaz%variable_name = "anchor_relative_azimuth"
           call READ_SDS(sd_id_input,scaled_sds_data_anchors_input,sds_relaz)
           call UNSCALE_SDS(sds_relaz,scaled_sds_data_anchors_input, relaz_anchors_input)
         
           !--- extrapolate relaz 
           do iline = 1, num_lines_input
             call LINEAR_ANCHOR_INTERP(num_elements_input,relaz_anchors_input(:,iline),relaz_input(:,iline))
           enddo

           deallocate(scaled_sds_data_anchors_input)
           deallocate(relaz_anchors_input)

         else
           sds_relaz%variable_name = "relative_azimuth"
           call READ_SDS(geo_sd_id_input,scaled_sds_data_input,sds_relaz)
           call UNSCALE_SDS(sds_relaz,scaled_sds_data_input, relaz_input)
           sds_relaz%variable_name = "anchor_relative_azimuth"
         endif

         !--- define output or read in output values
         if (first_valid_input == sym%YES) then   !define sds in output
                 call DEFINE_SDS(sd_id_output,sds_dims_output_xy,sds_relaz)
                 relaz_output(:,:) = missing_value_real4
         endif
     
         !--------------------------------------------------------------
         !--- solar azimuth
         !--------------------------------------------------------------
         if (use_geo_file == sym%NO) then
           allocate(solaz_anchors_input(num_anchors_input,num_lines_input))
           allocate(scaled_sds_data_anchors_input(num_anchors_input,num_lines_input))
     
           sds_solaz%variable_name = "anchor_solar_azimuth"
           call READ_SDS(sd_id_input,scaled_sds_data_anchors_input,sds_solaz)
           call UNSCALE_SDS(sds_solaz,scaled_sds_data_anchors_input, solaz_anchors_input)
         
           !--- extrapolate solaz 
           do iline = 1, num_lines_input
             call LINEAR_ANCHOR_INTERP(num_elements_input,solaz_anchors_input(:,iline),solaz_input(:,iline))
           enddo

           deallocate(scaled_sds_data_anchors_input)
           deallocate(solaz_anchors_input)

         else
           sds_solaz%variable_name = "solar_azimuth"
           call READ_SDS(geo_sd_id_input,scaled_sds_data_input,sds_solaz)
           call UNSCALE_SDS(sds_solaz,scaled_sds_data_input, solaz_input)
           sds_solaz%variable_name = "anchor_solar_azimuth"
         endif

         !--- define output or read in output values
         if (first_valid_input == sym%YES) then   !define sds in output
                 call DEFINE_SDS(sd_id_output,sds_dims_output_xy,sds_solaz)
                 solaz_output(:,:) = missing_value_real4
         endif
     
         !-----------------------------------------------------------
         !--- ch4
         !--------------------------------------------------------------
         allocate(ch4_input(num_elements_input,num_lines_input))
         
     
         sds(1)%variable_name = "ch4_temperature"
         call READ_SDS(sd_id_input,scaled_sds_data_input,sds(1))
         call UNSCALE_SDS(sds(1),scaled_sds_data_input, ch4_input)

         !-------------------------------------------------------------
         !  determine which output grid_points are to be updated by
         !  based on time, date and sensor zenth from the input file
         !-------------------------------------------------------------
         update_output_mask = sym%NO
         do ilon = 1, nlon_output
           do ilat = 1, nlat_output

              ielem = ielem_output(ilon,ilat)
              iline = iline_output(ilon,ilat)

              if ((ielem > 0) .and. (iline > 0)) then  !valid ielem, iline check

               if (bad_scan_flag_input(iline) == sym%NO) then

                if (asc_des_flag_input(iline) == asc_des_node) then

                 if ((senzen_input(ielem,iline) /= missing_value_real4) .and. &
                    (ch4_input(ielem,iline) /= missing_value_real4) .and. &
                    ((senzen_input(ielem,iline) < senzen_output(ilon,ilat)) .or. &
                     (senzen_output(ilon,ilat) == missing_value_real4)))  then

                  !--- check time to exclude data from wrong days for orbits that span midnight
                  if ((jday == start_day_input) .and. (time_input(ielem,iline) > start_time_input)) then   
                    update_output_mask(ilon,ilat) = sym%YES
                  endif

                  if ((jday == end_day_input) .and. (time_input(ielem,iline) < end_time_input)) then   
                    update_output_mask(ilon,ilat) = sym%YES
                  endif

                endif  !check for more nadir view
               endif   !check asc_des_node
              else
              endif    !check for bad scan
             endif    !check for valid ielem and iline
            end do
          end do
      
         deallocate(ch4_input)

         !----- compute 1d input to output index mapping 
         ipoint = 0
         do ilon = 1, nlon_output
           do ilat = 1, nlat_output
                if (update_output_mask(ilon,ilat) == sym%YES) then
                  ipoint = ipoint + 1
                  ielem = ielem_output(ilon,ilat)
                  iline = iline_output(ilon,ilat)
                  input_update_index(ipoint) = ielem + (iline-1)*num_elements_input
                  output_update_index(ipoint) = ilon + (ilat-1)*nlon_output
                endif
           enddo
          enddo

         !--- update results

        
          !---new
          if (ipoint > 0) then
           !--- update time
           input_array_1d = reshape(time_input, (/num_elements_input * num_lines_input/))  
           output_array_1d = reshape(time_output, (/nlon_output*nlat_output/))  
           output_array_1d(output_update_index(1:ipoint)) = input_array_1d(input_update_index(1:ipoint))
           time_output = reshape(output_array_1d, (/nlon_output,nlat_output/))

           !--- update senzen
           input_array_1d = reshape(senzen_input, (/num_elements_input * num_lines_input/))  
           output_array_1d = reshape(senzen_output, (/nlon_output*nlat_output/))  
           output_array_1d(output_update_index(1:ipoint)) = input_array_1d(input_update_index(1:ipoint))
           senzen_output = reshape(output_array_1d, (/nlon_output,nlat_output/))

           !--- update solzen
           input_array_1d = reshape(solzen_input, (/num_elements_input * num_lines_input/))  
           output_array_1d = reshape(solzen_output, (/nlon_output*nlat_output/))  
           output_array_1d(output_update_index(1:ipoint)) = input_array_1d(input_update_index(1:ipoint))
           solzen_output = reshape(output_array_1d, (/nlon_output,nlat_output/))

           !--- update relaz
           input_array_1d = reshape(relaz_input, (/num_elements_input * num_lines_input/))  
           output_array_1d = reshape(relaz_output, (/nlon_output*nlat_output/))  
           output_array_1d(output_update_index(1:ipoint)) = input_array_1d(input_update_index(1:ipoint))
           relaz_output = reshape(output_array_1d, (/nlon_output,nlat_output/))
 
           !--- update solaz
           input_array_1d = reshape(solaz_input, (/num_elements_input * num_lines_input/))  
           output_array_1d = reshape(solaz_output, (/nlon_output*nlat_output/))  
           output_array_1d(output_update_index(1:ipoint)) = input_array_1d(input_update_index(1:ipoint))
           solaz_output = reshape(output_array_1d, (/nlon_output,nlat_output/))

           !--- update post_cld_prob
           input_array_1d = reshape(post_cld_prob_input, (/num_elements_input * num_lines_input/))  
           output_array_1d = reshape(post_cld_prob_output, (/nlon_output*nlat_output/))  
           output_array_1d(output_update_index(1:ipoint)) = input_array_1d(input_update_index(1:ipoint))
           post_cld_prob_output = reshape(output_array_1d, (/nlon_output,nlat_output/))

         endif



         deallocate(senzen_input)
         deallocate(solzen_input)
         deallocate(relaz_input)
         deallocate(solaz_input)
         deallocate(time_input)
         deallocate(lon_input)
         deallocate(lat_input)
         deallocate(post_cld_prob_input)

         !---------------------------------------------------------------------
         ! loop through SDS's and store the information about them
         !---------------------------------------------------------------------
         if (first_valid_input == sym%YES) then
           sds_loop_1: do isds = 0, num_sds_input-1
            
            sds%id_input = sfselect(sd_id_input,isds) !note, sds index starts at zero

            istatus_sum = sfginfo(sds(isds)%id_input,  &
                              sds(isds)%variable_name,  &
                              sds(isds)%rank,  &
                              sds_dims,  &
                              sds(isds)%data_type,  &
                              sds(isds)%num_attrs) + istatus_sum

            enddo sds_loop_1
          endif
         !---------------------------------------------------------------------
         ! loop through SDS's and store the information about them
         !---------------------------------------------------------------------
         sds_loop_2: do isds = 0, num_sds_input-1

            !--- do not composite lat or lon or other array that are already processed
            if (trim(sds(isds)%variable_name) == "scan_line_number") cycle    
            if (trim(sds(isds)%variable_name) == trim(sds_lon%variable_name)) cycle    
            if (trim(sds(isds)%variable_name) == trim(sds_lat%variable_name)) cycle    
            if (trim(sds(isds)%variable_name) == trim(sds_time%variable_name)) cycle    
            if (trim(sds(isds)%variable_name) == trim(sds_senzen%variable_name)) cycle    
            if (trim(sds(isds)%variable_name) == trim(sds_solzen%variable_name)) cycle    
            if (trim(sds(isds)%variable_name) == trim(sds_relaz%variable_name)) cycle    
            if (trim(sds(isds)%variable_name) == trim(sds_solaz%variable_name)) cycle    
            if (trim(sds(isds)%variable_name) == trim(sds_bad_scan_flag%variable_name)) cycle    
            if (trim(sds(isds)%variable_name) == trim(sds_asc_des_flag%variable_name)) cycle    
            if (trim(sds(isds)%variable_name) == trim(sds_post_cld_prob%variable_name)) cycle    
       
            !-----------------------------------------------------------
            !--- read from input input variable
            !-----------------------------------------------------------
            if (index(sds(isds)%variable_name,"anchor") > 0)  then 
           
                 allocate(unscaled_sds_data_anchors_input(num_anchors_input,num_lines_input))
                 allocate(scaled_sds_data_anchors_input(num_anchors_input,num_lines_input))
                 allocate(unscaled_sds_data_input(num_elements_input,num_lines_input))
         
                 call READ_SDS(sd_id_input,scaled_sds_data_anchors_input,sds(isds))
                 call UNSCALE_SDS(sds(isds),scaled_sds_data_anchors_input,unscaled_sds_data_anchors_input)
                 
                 do iline = 1, num_lines_input
                    call LINEAR_ANCHOR_INTERP(num_elements_input,unscaled_sds_data_anchors_input(:,iline), &
                                              unscaled_sds_data_input(:,iline))
                 enddo
         
                 deallocate(unscaled_sds_data_anchors_input)
                 deallocate(scaled_sds_data_anchors_input)
         
                 call SCALE_SDS(sds(isds),unscaled_sds_data_input,scaled_sds_data_input)
         
                 deallocate(unscaled_sds_data_input)

            else
                 call READ_SDS(sd_id_input,scaled_sds_data_input,sds(isds))

            endif

            !-----------------------------------------------------------
            !--- read from or define  output variable
            !-----------------------------------------------------------
            if (first_valid_input == sym%YES) then 

                call DEFINE_SDS(sd_id_output, sds_dims_output_xy, sds(isds))

                !--- initialize output to missing
                if (sds(isds)%data_type == DFNT_INT8) then
                    scaled_sds_data_output = missing_value_int1
                    scaled_sds_data_output_full(:,:,isds) = missing_value_int1
                elseif (sds(isds)%data_type == DFNT_INT16) then
                    scaled_sds_data_output = missing_value_int2
                    scaled_sds_data_output_full(:,:,isds) = missing_value_int2
                elseif (sds(isds)%data_type == DFNT_INT32) then
                    scaled_sds_data_output = missing_value_int4
                    scaled_sds_data_output_full(:,:,isds) = missing_value_int4
                elseif (sds(isds)%data_type == DFNT_FLOAT32) then
                    scaled_sds_data_output = missing_value_real4
                    scaled_sds_data_output_full(:,:,isds) = missing_value_real4
                endif

            endif

            !-----------------------------------------------------------
            !--- update composite
            !--- switching between 2d to 3d arrays necessitated 
            !----by seg faults on ifort
            !-----------------------------------------------------------
            if (ipoint > 0) then
             input_array_1d = reshape(scaled_sds_data_input, (/num_elements_input * num_lines_input/))  
             scaled_sds_data_output = scaled_sds_data_output_full(:,:,isds)
             output_array_1d = reshape(scaled_sds_data_output, (/nlon_output*nlat_output/))  
             output_array_1d(output_update_index(1:ipoint)) = input_array_1d(input_update_index(1:ipoint))
             scaled_sds_data_output = reshape(output_array_1d, (/nlon_output,nlat_output/))
             scaled_sds_data_output_full(:,:,isds) = scaled_sds_data_output
            endif

           !-------------------------------------------------------------
           !---- write to output files
           !-------------------------------------------------------------
           call WRITE_SDS(sd_id_output,scaled_sds_data_output,sds(isds))

         end do sds_loop_2

         !----------------------------------------------------------------------
         ! after last file, write output
         !----------------------------------------------------------------------
         if (ifile == n_files) then

            !---- write time to output files
            call SCALE_SDS(sds_time,time_output,scaled_sds_data_output)
            call WRITE_SDS(sd_id_output,scaled_sds_data_output,sds_time)

            !---- write senzen output files
            call SCALE_SDS(sds_senzen,senzen_output,scaled_sds_data_output)
            call WRITE_SDS(sd_id_output,scaled_sds_data_output,sds_senzen)

            !---- write solzen output files
            call SCALE_SDS(sds_solzen,solzen_output,scaled_sds_data_output)
            call WRITE_SDS(sd_id_output,scaled_sds_data_output,sds_solzen)

            !---- write relaz output files
            call SCALE_SDS(sds_relaz,relaz_output,scaled_sds_data_output)
            call WRITE_SDS(sd_id_output,scaled_sds_data_output,sds_relaz)

            !---- write solaz output files
            call SCALE_SDS(sds_solaz,solaz_output,scaled_sds_data_output)
            call WRITE_SDS(sd_id_output,scaled_sds_data_output,sds_solaz)

            !---- write solzen output files
            call SCALE_SDS(sds_post_cld_prob,post_cld_prob_output,scaled_sds_data_output)
            call WRITE_SDS(sd_id_output,scaled_sds_data_output,sds_post_cld_prob)

            sds_loop_3: do isds = 1, num_sds_input

               !--- do not send these to output
               if (trim(sds(isds)%variable_name) == "scan_line_number") cycle    
               if (trim(sds(isds)%variable_name) == trim(sds_lon%variable_name)) cycle    
               if (trim(sds(isds)%variable_name) == trim(sds_lat%variable_name)) cycle    
               if (trim(sds(isds)%variable_name) == trim(sds_time%variable_name)) cycle    
               if (trim(sds(isds)%variable_name) == trim(sds_senzen%variable_name)) cycle    
               if (trim(sds(isds)%variable_name) == trim(sds_solzen%variable_name)) cycle    
               if (trim(sds(isds)%variable_name) == trim(sds_relaz%variable_name)) cycle    
               if (trim(sds(isds)%variable_name) == trim(sds_solaz%variable_name)) cycle    
               if (trim(sds(isds)%variable_name) == trim(sds_bad_scan_flag%variable_name)) cycle    
               if (trim(sds(isds)%variable_name) == trim(sds_asc_des_flag%variable_name)) cycle    
               if (trim(sds(isds)%variable_name) == trim(sds_post_cld_prob%variable_name)) cycle    

               !--- write to output
               scaled_sds_data_output = scaled_sds_data_output_full(:,:,isds)
               call WRITE_SDS(sd_id_output,scaled_sds_data_output,sds(isds))

               !--- add in attributes to FCDR variables (for NCDC)
               if (trim(sds(isds)%variable_name) == "ch1_reflectance") then   
                  istatus_sum = sfscatt(sds(isds)%id_output, "FCDR", DFNT_CHAR8, 4, "none") + istatus_sum
               endif
               if (trim(sds(isds)%variable_name) == "ch2_reflectance") then   
                  istatus_sum = sfscatt(sds(isds)%id_output, "FCDR", DFNT_CHAR8, 4, "none") + istatus_sum
               endif


            enddo sds_loop_3
         endif

         deallocate(bad_scan_flag_input)
         deallocate(asc_des_flag_input)
         deallocate(scaled_sds_data_input)
         deallocate(input_update_index)
         deallocate(input_array_1d)
         

         !----- close input hdf file
         istatus_sum = sfend(sd_id_input) + istatus_sum 

         first_valid_input = sym%NO

    end do file_loop


    !--- compute some metrics to include as attributes
    count_valid = count(senzen_output /= missing_value_real4)
    domain_data_fraction = real(count_valid) /  &
                           real(nlon_output * nlat_output)

    count_day = count(solzen_output /= missing_value_real4 .and. &
                      solzen_output < 90.0)
    domain_day_fraction = real(count_day) /  &
                           real(nlon_output * nlat_output)

    count_cloud = count(post_cld_prob_output /= missing_value_real4 .and. &
                        post_cld_prob_output >= 0.5)
    domain_cloud_fraction = real(count_cloud) /  &
                           real(nlon_output * nlat_output)

    istatus_sum = sfsnatt(sd_id_output,"FRACTION_WITH_VALID_DATA", &
                      DFNT_FLOAT32,1,domain_data_fraction) + istatus_sum

    istatus_sum = sfsnatt(sd_id_output,"FRACTION_WITH_DAYTIME_DATA", &
                      DFNT_FLOAT32,1,domain_day_fraction) + istatus_sum

    istatus_sum = sfsnatt(sd_id_output,"FRACTION_WITH_CLOUDY_DATA", &
                      DFNT_FLOAT32,1,domain_cloud_fraction) + istatus_sum

    !--- deallocate output fields
     deallocate(time_output)
     deallocate(senzen_output)
     deallocate(post_cld_prob_output)
     deallocate(solzen_output)
     deallocate(relaz_output)
     deallocate(solaz_output)
     deallocate(scaled_sds_data_output)
     deallocate(unscaled_sds_data_output)
     deallocate(lon_output)
     deallocate(lat_output)
     deallocate(ielem_output)
     deallocate(iline_output)
     deallocate(update_output_mask)
     deallocate(output_update_index)
     deallocate(output_array_1d)
     deallocate(lon_output_1d)
     deallocate(lat_output_1d)
     deallocate(scaled_sds_data_output_full)

    !----- close output hdf file
    istatus_sum = sfend(sd_id_output) + istatus_sum

    !--- notify if accumulated hdf call istatus is non-zero
    if (istatus_sum /= 0) then
           print *, "Non- zero accumulated error codes on hdf calls"
    endif


    processing_time_end_hours = COMPUTE_TIME_HOURS()

    print *, 'Total COMP_ASC_DES_LEVEL2b Procesing Time (minutes) = ',  &
          60.0*(processing_time_end_hours - processing_time_start_hours)
!----------------------------------------------------------------------
! End Executabe Code
!----------------------------------------------------------------------

end program COMPILE_ASC_DES_LEVEL2B
