! $Id: equal_area.f90,v 1.2 2010/09/23 20:45:54 heidinger Exp $
!------------------------------------------------------------------
!@***h* PATMOSX/equal_area.f90
! 
! NAME
!   equal_area.f90
! 
! LOCATION
!   /patmosx/src/equal_area.f90
! 
! PURPOSE
!   This set of routines in this modules is used when the equal area
!   options selected for the clavr-x gridcell
! 
! DESCRIPTION
!   This set of routines in this modules is used when the equal area
!   options selected for the clavr-x gridcell
!   This cell-ordering is the same as the PATMOS ordering
!   first cell is at 0 longitude (the prime meridian) and at
!   the south pole. Cells go east then north.
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! EXTERNALS
!  Dependencies: (The following are names of modules used in CLAVR-x)
!  CONSTANTS
!
!  Calling Sequence:
!  use EQUAL_AREA_GRID
!
!  Public Routines within this module:
!  SETUP_EQUAL_AREA
!  PLACE_EQUAL_AREA_CELL 
!  FIND_EQUAL_AREA_CELL
!  DESTROY_EQUAL_AREA
!
!@*****
!-------------------------------------------------------------------
!
! Source Code Name:
!  equal_area.f90
!
! Module Name:
!  EQUAL_AREA_GRID
!
! Function:
!  a module within the CLAVR-x processing system
!
! Descripion:
!  This set of routines in this modules is used when the equal area
!  options selected for the clavr-x gridcell.
!  The cell-ordering is the same as the PATMOS ordering
!  first cell is at 0 longitude (the prime meridian) and at
!  the south pole. Cells go east then north.
!
! Reference: None
!
! Dependencies: (The following are names of modules used in CLAVR-x)
!  CONSTANTS
!
! Calling Sequence:
!  use EQUAL_AREA_GRID
!
! Public Routines within this module:
!  SETUP_EQUAL_AREA
!  PLACE_EQUAL_AREA_CELL 
!  FIND_EQUAL_AREA_CELL
!  DESTROY_EQUAL_AREA
!
! Author: Andrew K. Heidinger, National Oceanic and Atmospheric Administration
! (c) This code is copyrighted by the author and all NOAA restrictions apply
!-------------------------------------------------------------------
 module EQUAL_AREA_GRID
  use CONSTANTS
  implicit none
  private
  public:: SETUP_EQUAL_AREA, &
           PLACE_EQUAL_AREA_CELL, &
           FIND_EQUAL_AREA_CELL, &
           DESTROY_EQUAL_AREA
  
  real(kind=real4), dimension(:),allocatable, private, save:: delta_lon_cells
  integer(kind=int4), dimension(:),allocatable,  &
                                  private, save:: num_zonal_cells
  integer(kind=int4), dimension(:),allocatable,  &
                                  private, save:: num_cells_spole
  integer(kind=int4), public, save:: ncells_area
  integer, private, save:: num_lat
  integer, private, save:: num_lon
  real(kind=real8), private, save:: delta_lat_deg
  real(kind=real8), parameter, private:: r_earth = 6371.2

  contains

!----------------------------------------------------------------------
! setup the equal-area grid cells - this uses the patmos convention
! where cell numbering starts at the south pole 0 longitude
! and goes west and north
!
! this routine should be called before any other in this module
!
!-----------------------------------------------------------------------
  subroutine SETUP_EQUAL_AREA(dlat)
    real(kind=real4), intent(in):: dlat
    integer:: ilat,ilat1,ilat2
    real(kind=real8):: delta_lat,eq_zonal_area,eq_cell_area,delta_sin,zonal_area

       delta_lat_deg = dlat

       delta_lat   = (delta_lat_deg * pi) / 180.0

       eq_zonal_area    = 2.0 * pi * r_earth**2 * sin(delta_lat)

       eq_cell_area   = (eq_zonal_area * delta_lat_deg) / 360.0

       num_lat = int(180.0/delta_lat_deg)    !number of latitude cells
       num_lon = int(360.0/delta_lat_deg)    !number of longitude cells

       allocate(num_zonal_cells(num_lat),delta_lon_cells(num_lat), &
                num_cells_spole(num_lat))
       num_zonal_cells = 0
       delta_lon_cells = 0.0
       num_cells_spole = 0

!
!...COMPUTE EQUAL - AREA LONGITUDE INTERVAL FOR EACH LATITUDE ZONE
       do ilat = 1, num_lat/2
          delta_sin = sin(delta_lat*ilat) - sin(delta_lat * (ilat-1))
          zonal_area   = 2.0 * pi * r_earth**2 * delta_sin
          ilat1 = ilat + num_lat/2
          ilat2 = (num_lat/2) + 1 - ilat
          num_zonal_cells(ilat1) = nint(zonal_area / eq_cell_area)
          num_zonal_cells(ilat2) = num_zonal_cells(ilat1)
          delta_lon_cells(ilat1) = 360.0/num_zonal_cells(ilat1)
          delta_lon_cells(ilat2) = delta_lon_cells(ilat1)
       enddo
       ncells_area = sum(num_zonal_cells)
       num_cells_spole = 0.0
       do ilat = 2,num_lat
          num_cells_spole(ilat) = sum(num_zonal_cells(1:ilat-1))
       enddo

  end subroutine SETUP_EQUAL_AREA

!------------------------------------------------------------------------
! deallocate the memory allocated in SETUP_EQUAL_AREA
!------------------------------------------------------------------------  
  subroutine DESTROY_EQUAL_AREA()
       deallocate(num_zonal_cells, num_cells_spole, delta_lon_cells)
  end subroutine DESTROY_EQUAL_AREA
!-------------------------------------------------------------------------
! Given a equal area cell index, find its central lat and lon
!-------------------------------------------------------------------------
  subroutine PLACE_EQUAL_AREA_CELL(num_box,lat_center,lon_center, &
                                   lat_width,lon_width)
      integer(kind=int4), intent(in):: num_box
      real(kind=real4), intent(out):: lat_center,lon_center,lat_width,lon_width
      integer:: latbin,lonbin

      do latbin = 1,num_lat-1
         if (num_cells_spole(latbin+1) >= num_box) then
             exit
         endif
      enddo
      lat_center = -90.0 + (latbin -1 + 0.5)*delta_lat_deg
      lonbin = num_box - num_cells_spole(latbin)
      lon_center = (lonbin - 1 + 0.5)*delta_lon_cells(latbin)
      if (lon_center > 180.0) then
        lon_center = lon_center - 360.0
      endif
      lat_width = delta_lat_deg
      lon_width = delta_lon_cells(latbin)
  end subroutine PLACE_EQUAL_AREA_CELL
!-------------------------------------------------------------------------
! given a latitude (-90 to 90) and a longitude (-180 to 180) determine
! which 1 deg equal area box this falls into.  These boxes are the
! 41258 boxes used in PATMOS mapping.
! This code is mainly taken from routines written by V. Rao Anne
!
!-------------------------------------------------------------------------
  subroutine FIND_EQUAL_AREA_CELL(latitude,longitude,num_box)
      real(kind=real4), intent(in):: latitude,longitude
      integer, intent(out):: num_box
      real(kind=real4):: rrlat,rrlon
      integer:: latbin,lonbin

!-------------------------------------------------------------------------
! determine equal area box
!-------------------------------------------------------------------------
    rrlat = latitude + 90.0
    rrlon = longitude
    if (rrlon < 0.00) then
       rrlon = 360.0 + rrlon
    endif
!   latbin = min(int(rrlat/delta_lat_deg) + 1,num_lat)
!   lonbin = min(int(rrlon/delta_lon_cells(latbin)) + 1,num_zonal_cells(latbin))
    latbin = min(int(rrlat/delta_lat_deg + 1.0),num_lat)
    lonbin = min(int(rrlon/delta_lon_cells(latbin)+1.0) ,num_zonal_cells(latbin))
    num_box = num_cells_spole(latbin) + lonbin

  end subroutine FIND_EQUAL_AREA_CELL

end module EQUAL_AREA_GRID
