!$Id: naive_bayesian_cloud_mask_module.f90,v 1.54 2010/09/23 15:46:09 wstraka Exp $
!----------------------------------------------------------------------
!@***h* PATMOSX/naive_bayesian_cloud_mask_module.f90
! 
! NAME
!   naive_bayesian_cloud_mask_module.f90
! 
! LOCATION
!   /patmosx/src/naive_bayesian_cloud_mask_module.f90
! 
! PURPOSE
!   Routines for the determination of the baysiean cloud mask
! 
! DESCRIPTION
!   Routines for the determination of the baysiean cloud mask
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!   Bayesian Surface Types:
!   1 - Deep_Water
!   2 - Shallow_Water
!   3 - Unfrozen_Land
!   4 - Frozen_Land
!   5 - Arctic
!   6 - Antarctic
!   7 - Desert
!
!   SHALLOW_OCEAN = 0
!   LAND = 1
!   COASTLINE = 2
!   SHALLOW_INLAND_WATER = 3
!   EPHEMERAL_WATER = 4
!   DEEP_INLAND_WATER = 5
!   MODERATE_OCEAN = 6
!   DEEP_OCEAN = 7
!
! EXTERNALS
!    use CONSTANTS
!    use AVHRR_PIXEL_COMMON
!    use NUMERICAL_ROUTINES
!    use FILE_UTILITY
!    use CLOUD_MASK_ROUTINES
!    
!    private:: COMPUTE_BAYES_SFC_TYPE
!    public:: READ_NAIVE_BAYES, &
!             CLOUD_MASK_NAIVE_BAYES
!
!@*****
!------------------------------------------------------------------------
! MODULE name: NAIVE_BAYESIAN_CLOUD_MASK
! 
! Routines for the determination of the baysiean cloud mask
!
! Author: Andrew Heidinger, NOAA/NESDIS
!
!----------------------------------------------------------------------
module NAIVE_BAYESIAN_CLOUD_MASK
 
 use CONSTANTS
 use AVHRR_PIXEL_COMMON
 use NUMERICAL_ROUTINES
 use FILE_UTILITY
 use CLOUD_MASK_ROUTINES
 
 implicit none
 private
 private:: COMPUTE_BAYES_SFC_TYPE
 public:: READ_NAIVE_BAYES, &
          CLOUD_MASK_NAIVE_BAYES

 !---- module-wide variables


 !--- manually choose which classifier to use
 character(len=72), parameter, private:: filename = 'naive_bayes_ir_only_avhrr_mask.txt'  !maximum number of classes
 integer, parameter, private:: N_class = 6  !maximum number of classes

 integer, parameter, private:: N_sfc_bayes = 7   !maximum number of surface types
 integer, parameter, private:: N_bounds = 101
 
 real, dimension(N_sfc_bayes):: prior_yes = missing_value_real4
 real, dimension(N_sfc_bayes):: prior_no = missing_value_real4
 real, dimension(N_sfc_bayes):: optimal_posterior_prob = missing_value_real4
 real, dimension(N_class,N_sfc_bayes), private, save:: Classifier_Bounds_min
 real, dimension(N_class,N_sfc_bayes), private, save:: Classifier_Bounds_max
 real, dimension(N_class,N_sfc_bayes), private, save:: delta_Classifier_Bounds
 integer, dimension(N_class,N_sfc_bayes), private, save:: first_valid_Classifier_Bounds = 0
 integer, dimension(N_class,N_sfc_bayes), private, save:: last_valid_Classifier_Bounds = 0
 real, dimension(N_bounds-1,N_class,N_sfc_bayes), private, save:: class_cond_yes
 real, dimension(N_bounds-1,N_class,N_sfc_bayes), private, save:: class_cond_no 
 character (len=20), dimension(N_class,N_sfc_bayes), private, save:: Classifier_Value_Name

 real, dimension(:,:,:), allocatable, save:: cond_yes
 real, dimension(:,:,:), allocatable, save:: cond_no
 real, dimension(:,:,:), allocatable, save:: posterior_cld_probability_by_class
 real, dimension(:,:,:), allocatable, save:: Classifier_Value
 real, dimension(:,:), allocatable, save:: fmft
 real, dimension(:,:), allocatable, save:: rgct 
 real, dimension(:,:), allocatable, save:: T_11um_stddev_sqrt
 real, dimension(:,:), allocatable, save:: bayes_sfc_type
 real, dimension(:,:), allocatable, save:: delta_z

 real:: temp_posterior

 INCLUDE "awg_cloud_mask_tests_thresholds.inc"
 
 contains 

!====================================================================
! SUBROUTINE Name: READ_NAIVE_BAYES
!
! Function:
!   Allocate and Read in the LUTs needed for Bayesian cloud mask tables
!
!====================================================================

 subroutine READ_NAIVE_BAYES()

   integer:: lun
   integer:: iclass
   integer:: iclass_temp
   integer:: isfc
   integer:: isfc_file
   integer:: ios
   integer:: ios_sum
   character(len=72):: header
   
   lun = GET_LUN()
   ios_sum = 0
   
   open(unit=lun,file=trim(ancil_data_dir)//"bayes/"//trim(filename), &
        action="read",form="formatted",status="old",iostat=ios)
   if (ios /= 0) then
     print *, EXE_PROMPT , 'ERROR: Bayesian Cloud Mask Classifier Open Failed '
     print *, EXE_PROMPT , 'Bayesian Cloud Mask Turned Off'
     cloud_mask_bayesian_flag = sym%NO
     return
   endif
   ios_sum = ios_sum + ios

   !--- skip first two lines which is a meta-data
   read (unit=lun,fmt=*) 
   read (unit=lun,fmt=*) 

   do isfc = 1, N_sfc_bayes

   read (unit=lun,fmt=*,iostat=ios) header
   ios_sum = ios_sum + ios

   read (unit=lun,fmt=*,iostat=ios) isfc_file
   ios_sum = ios_sum + ios

   isfc_file = isfc_file + 1          !note idl generated file is 0-based

   !--- check if file has the expected classifiers
   if (isfc_file /= isfc) then
         print *, EXE_PROMPT," ERROR: Confused on Naive Bayesian Cloud Mask Classifiers"
         print *, EXE_PROMPT , 'Bayesian Cloud Mask Turned Off'
         cloud_mask_bayesian_flag = sym%NO
         return
   endif

   read(unit=lun,fmt=*,iostat=ios) prior_yes(isfc),prior_no(isfc)
   ios_sum = ios_sum + ios
   read(unit=lun,fmt=*,iostat=ios) optimal_posterior_prob(isfc)
   ios_sum = ios_sum + ios

   do iclass = 1,N_class

            read(unit=lun,fmt=*,iostat=ios)  &
                                 iclass_temp, first_valid_Classifier_Bounds(iclass,isfc), &
                                 last_valid_Classifier_Bounds(iclass,isfc),Classifier_Value_Name(iclass,isfc)
            ios_sum = ios_sum + ios

            Classifier_Value_Name(iclass,isfc) = trim(Classifier_Value_Name(iclass,isfc))

            read(unit=lun,fmt=*,iostat=ios)  &
                                 Classifier_Bounds_min(iclass,isfc), &
                                 Classifier_Bounds_max(iclass,isfc), &
                                 delta_Classifier_Bounds(iclass,isfc)
            ios_sum = ios_sum + ios

            read(unit=lun,fmt=*,iostat=ios) class_cond_yes(:,iclass,isfc)
            ios_sum = ios_sum + ios

            read(unit=lun,fmt=*,iostat=ios) class_cond_no(:,iclass,isfc)
            ios_sum = ios_sum + ios

    end do

   end do 

   !--- close file
   close(unit=lun,iostat=ios)

   if (ios_sum == 0) then
           print *, EXE_PROMPT, ' Bayesian Cloud Mask Data Read in Successfully'
   else
           print *, EXE_PROMPT, 'ERROR:  Bayesian Cloud Mask Data Not Read in Successfully'
           print *, EXE_PROMPT , 'Bayesian Cloud Mask Turned Off'
           cloud_mask_bayesian_flag = sym%NO
   endif

 end subroutine READ_NAIVE_BAYES


!====================================================================
! SUBROUTINE Name: CLOUD_MASK_NAIVE_BAYES
!
! Function:
!   Calculates the bayesian cloud mask. The bayesian cloud mask is
!   determined by utilizing the following surface types:
!
! Bayesian Surface Types
! 1 - Deep_Water
! 2 - Shallow_Water
! 3 - Unfrozen_Land
! 4 - Frozen_Land
! 5 - Arctic
! 6 - Antarctic
! 7 - Desert
! SHALLOW_OCEAN = 0
! LAND = 1
! COASTLINE = 2
! SHALLOW_INLAND_WATER = 3
! EPHEMERAL_WATER = 4
! DEEP_INLAND_WATER = 5
! MODERATE_OCEAN = 6
! DEEP_OCEAN = 7
!
!====================================================================
 subroutine CLOUD_MASK_NAIVE_BAYES(Line_Start,Num_Lines,Segment_Number)
   integer, intent(in):: Line_Start
   integer, intent(in):: Num_Lines
   integer, intent(in):: Segment_Number
   integer:: Line_End
   integer:: ielem
   integer:: iline
   integer:: iclass
   integer:: isfc
   integer:: ibin
   integer:: turn_off_this_class

   Line_End = Line_Start + Num_Lines - 1

   if (Segment_Number == 1) then
      if (ALLOCATED(cond_yes) .eqv. .true.) then
          deallocate(cond_yes)
          deallocate(cond_no)
          deallocate(posterior_cld_probability_by_class)
          deallocate(Classifier_Value)
          deallocate(fmft)
          deallocate(delta_z)
          deallocate(rgct)
          deallocate(T_11um_stddev_sqrt)
          deallocate(bayes_sfc_type)
      endif
      allocate(cond_yes(num_pix,j_min:j_max,N_Class))
      allocate(cond_no(num_pix,j_min:j_max,N_Class)) 
      allocate(posterior_cld_probability_by_class(num_pix,j_min:j_max,N_Class)) 
      allocate(Classifier_Value(num_pix,j_min:j_max,N_Class)) 
      allocate(fmft(num_pix,j_min:j_max)) 
      allocate(delta_z(num_pix,j_min:j_max)) 
      allocate(rgct(num_pix,j_min:j_max)) 
      allocate(T_11um_stddev_sqrt(num_pix,j_min:j_max)) 
      allocate(bayes_sfc_type(num_pix,j_min:j_max)) 
   endif


   Line_End = Line_Start + Num_Lines - 1

   !--- compute some needed metrics
   fmft  = (bt_clear_ch4_rtm - bt_clear_ch5_rtm) * (bt4 - 260.0) / &
           (bt_clear_ch4_rtm - 260.0)
   where(bt_clear_ch4_rtm <= 265.0)
      fmft = 0.0
   endwhere
   fmft = (bt4 - bt5) - fmft

   delta_z = Zc_opaque_cloud - zsfc/1000.0    !km
   where(Zc_opaque_cloud == missing_value_real4)
     delta_z = missing_value_real4
   endwhere

   where(bt4_std_3x3 >= 0.0)
     T_11um_stddev_sqrt = sqrt(bt4_std_3x3)
   elsewhere
     T_11um_stddev_sqrt = 0.0
   endwhere

   rgct = alb1 - alb1_clear

   !--- setup classifier matrices
   do iclass = 1, N_class
       if (trim(Classifier_Value_Name(iclass,1)) == "T_11um") then
        Classifier_Value(:,:,iclass) = bt4
       elseif (trim(Classifier_Value_Name(iclass,1)) == "Emiss_tropo") then
        Classifier_Value(:,:,iclass) = emiss_11_tropo_rtm
       elseif (trim(Classifier_Value_Name(iclass,1)) == "Zc_opaque") then
        Classifier_Value(:,:,iclass) = delta_z
       elseif (trim(Classifier_Value_Name(iclass,1)) == "FMFT") then
        Classifier_Value(:,:,iclass) = fmft
       elseif (trim(Classifier_Value_Name(iclass,1)) == "T_max-T") then
        Classifier_Value(:,:,iclass) = bt4_max_3x3 - bt4
       elseif (trim(Classifier_Value_Name(iclass,1)) == "Emiss_375") then
        Classifier_Value(:,:,iclass) = (ems3b_median_3x3-ems3b_clear_solar_rtm)/ems3b_clear_solar_rtm
       elseif (trim(Classifier_Value_Name(iclass,1)) == "Emiss_375_Day") then
        Classifier_Value(:,:,iclass) = (ems3b_median_3x3-ems3b_clear_solar_rtm)/ems3b_clear_solar_rtm
       elseif (trim(Classifier_Value_Name(iclass,1)) == "Emiss_375_Night") then
       !Classifier_Value(:,:,iclass) = (ems3b_median_3x3-ems3b_clear_solar_rtm)/ems3b_clear_solar_rtm
        Classifier_Value(:,:,iclass) = ems3b_median_3x3
       elseif (trim(Classifier_Value_Name(iclass,1)) == "RFMFT") then
        Classifier_Value(:,:,iclass) = bt45 - bt45_bt4_max_3x3 
       elseif (trim(Classifier_Value_Name(iclass,1)) == "Ref_063_Day") then
        Classifier_Value(:,:,iclass) = rgct
       elseif (trim(Classifier_Value_Name(iclass,1)) == "T_stddev_sqrt") then
        Classifier_Value(:,:,iclass) = T_11um_stddev_sqrt
       elseif (trim(Classifier_Value_Name(iclass,1)) == "T_med-T") then
        Classifier_Value(:,:,iclass) = bt4_median_3x3 - bt4
       elseif (trim(Classifier_Value_Name(iclass,1)) == "T_11um-T_11um_clr") then
        Classifier_Value(:,:,iclass) = (bt_clear_ch4_rtm - bt4)/(bt4-180.0)
       else
        print *, "unknown bayesian classifier, stopping"
       !print *, isfc,iclass, Classifier_Value_Name(iclass,1)
        stop
       endif
   enddo

   !---- compute sfc type
   do iline = Line_Start, Line_End
      do ielem = 1, num_pix

         bayes_sfc_type(ielem,iline) =  &
                    COMPUTE_BAYES_SFC_TYPE(land(ielem,iline), &
                                           snow(ielem,iline), &
                                           sfc_type(ielem,iline), &
                                           tsfc_rtm(ielem,iline), &
                                           lat(ielem,iline), &
                                           lon(ielem,iline), &
                                           sst_anal_uni(ielem,iline), &
                                           sfc_emiss_3b(ielem,iline), &
                                           zsfc(ielem,iline))
      enddo
   enddo

   !---- compute conditional probabilites for each pixel and each class
   do iline = Line_Start, Line_End
      do ielem = 1, num_pix

          if (bad_pixel_mask(ielem,iline) == sym%YES) then
              cycle
          endif

          do iclass = 1, N_class

             isfc = bayes_sfc_type(ielem,iline)

             ibin = int ((Classifier_Value(ielem,iline,iclass) - Classifier_Bounds_min(iclass,isfc))  /    &
                           delta_Classifier_Bounds(iclass,isfc)) + 1
             ibin = max(1,min(N_bounds-1,ibin))
             cond_yes(ielem,iline,iclass) = class_cond_yes(ibin,iclass,isfc)
             cond_no(ielem,iline,iclass) = class_cond_no(ibin,iclass,isfc)

             temp_posterior = &
                 (prior_yes(isfc)*cond_yes(ielem,iline,iclass)) / &
                 (prior_yes(isfc)*cond_yes(ielem,iline,iclass) +  &        
                  prior_no(isfc)*cond_no(ielem,iline,iclass))

!  if (iclass == 2) then 
!           write(unit=6,fmt="(2I5,9f7.3)"), iline,  ibin, &
!           Classifier_Value(ielem,iline,iclass), &
!           Classifier_Bounds_min(iclass,isfc) + (ibin-1)*delta_Classifier_Bounds(iclass,isfc), &
!           Classifier_Bounds_min(iclass,isfc) + ibin*delta_Classifier_Bounds(iclass,isfc), &
!           prior_yes(isfc), cond_yes(ielem,iline,iclass), &
!        prior_no(isfc), cond_no(ielem,iline,iclass), temp_posterior
!  endif

             !--- turn off some tests in some situation
             turn_off_this_class = sym%NO

             !--- missing values
             if (Classifier_Value(ielem,iline,iclass) == missing_value_real4) then
                      turn_off_this_class = sym%YES
             endif

             !--- mountanous situations
             if (zsfc(ielem,iline) > 1000.0) then

                 if ( (trim(Classifier_Value_Name(iclass,isfc)) == "T_max-T") .or. &
                      (trim(Classifier_Value_Name(iclass,isfc)) == "T_stddev_sqrt") .or. &
                      (trim(Classifier_Value_Name(iclass,isfc)) == "T_med-T")) then

                      turn_off_this_class = sym%YES

                 endif

             endif


             !--- coastal situations
             if (coast_mask(ielem,iline)==sym%YES) then
                 if ( (trim(Classifier_Value_Name(iclass,isfc)) == "T_max-T") .or. &
                      (trim(Classifier_Value_Name(iclass,isfc)) == "T_stddev_sqrt") .or. &
                      (trim(Classifier_Value_Name(iclass,isfc)) == "T_med-T") .or. &
                      (trim(Classifier_Value_Name(iclass,isfc)) == "RFMFT") ) then

                      turn_off_this_class = sym%YES
                    
                 endif
             endif

             !--- situations with highy heterogenous sst
             if (sst_anal_uni(ielem,iline) > 0.5) then
                 if ( (trim(Classifier_Value_Name(iclass,isfc)) == "T_max-T") .or. &
                      (trim(Classifier_Value_Name(iclass,isfc)) == "T_stddev_sqrt") .or. &
                      (trim(Classifier_Value_Name(iclass,isfc)) == "T_med-T") .or. &
                      (trim(Classifier_Value_Name(iclass,isfc)) == "RFMFT") ) then

                      turn_off_this_class = sym%YES
                    
                 endif
             endif

             !--- glint situations in shallow oceans
             if (glint_mask(ielem,iline)==sym%YES) then
                 if ((trim(Classifier_Value_Name(iclass,isfc)) == "Emiss_375") .or. &
                     (trim(Classifier_Value_Name(iclass,isfc)) == "Emiss_375_Day")) then
                      turn_off_this_class = sym%YES
                endif
                if (trim(Classifier_Value_Name(iclass,isfc)) == "Ref_063_Day") then
                      turn_off_this_class = sym%YES
                endif
             endif

             !--- forward scattering of sun situations
             if (scatangle(ielem,iline) < 90.0) then
                 if ((trim(Classifier_Value_Name(iclass,isfc)) == "Emiss_375") .or. &
                     (trim(Classifier_Value_Name(iclass,isfc)) == "Emiss_375_Day") .or. &
                     (trim(Classifier_Value_Name(iclass,isfc)) == "Ref_063_Day") ) then
                      turn_off_this_class = sym%YES
                endif
             endif

             !--- snow situations
             if (snow(ielem,iline) /= sym%NO_SNOW) then
                 if ((trim(Classifier_Value_Name(iclass,isfc)) == "Ref_063_Day")) then
                      turn_off_this_class = sym%YES
                endif
             endif


             !--- solar contamination
           ! if (solar_contamination_mask(ielem,iline)==sym%YES) then
           !     if ((trim(Classifier_Value_Name(iclass,isfc)) == "Emiss_375") .or. &
           !         (trim(Classifier_Value_Name(iclass,isfc)) == "Emiss_375_Day") .or. &
           !         (trim(Classifier_Value_Name(iclass,isfc)) == "Emiss_375_Night") ) then
           !          turn_off_this_class = sym%YES
           !    endif
           ! endif

             !--- Emiss_375 outside valid temperature range
          !  if ((bt3b(ielem,iline) > 320.0) .or. (bt3b(ielem,iline) < 240.0)) then
          !      if ((trim(Classifier_Value_Name(iclass,isfc)) == "Emiss_375_Day") .or. &
          !          (trim(Classifier_Value_Name(iclass,isfc)) == "Emiss_375_Night")) then
          !           turn_off_this_class = sym%YES
          !      endif
          !  endif


             !--- Emiss_375  when ch3b is off
             if ((ch3a_on(iline) == 1)) then
                 if ((trim(Classifier_Value_Name(iclass,isfc)) == "Emiss_375_Day") .or. &
                     (trim(Classifier_Value_Name(iclass,isfc)) == "Emiss_375_Night")) then
                      turn_off_this_class = sym%YES
                 endif
             endif

             !--- Emiss_375_Day at night
             if (trim(Classifier_Value_Name(iclass,isfc)) == "Emiss_375_Day") then
                if (solzen(ielem,iline) > 90) then
                 turn_off_this_class = sym%YES
                endif
             endif

             if (trim(Classifier_Value_Name(iclass,isfc)) == "Emiss_375_Night") then
                if (solzen(ielem,iline) < 90) then
                 turn_off_this_class = sym%YES
                endif
             endif

             !--- Ref_063_Day  during night
             if (trim(Classifier_Value_Name(iclass,isfc)) == "Ref_063_Day") then
                if (solzen(ielem,iline) > 70.0) then
                 turn_off_this_class = sym%YES
                endif
             endif

             !--- turn off all tests
          !  if (trim(Classifier_Value_Name(iclass,isfc)) == "Ref_063_Day") then
          !      turn_off_this_class = sym%YES
          !  endif


             !--- turn of classifier if necessary
             if (turn_off_this_class == sym%YES) then
                cond_yes(ielem,iline,iclass) = 1.0 
                cond_no(ielem,iline,iclass) =  1.0
             endif

        enddo  
      enddo
   enddo

   !--- compute prosterior probabilites for each pixel
   do iline = Line_Start, Line_End
      do ielem = 1, num_pix

        if (bad_pixel_mask(ielem,iline) == sym%YES) then
            cycle
        endif

        isfc = bayes_sfc_type(ielem,iline)

        if (prior_yes(isfc) == missing_value_real4) then
          posterior_cld_probability(ielem,iline) = missing_value_real4
          posterior_cld_probability_by_class(ielem,iline,:) = missing_value_real4
        endif

        if (minval(cond_yes(ielem,iline,:)) == 0.0) then
          posterior_cld_probability(ielem,iline) = 0.0
        elseif (minval(cond_no(ielem,iline,:)) == 0.0) then
          posterior_cld_probability(ielem,iline) = 1.0
        else
          posterior_cld_probability(ielem,iline) = &
                 (prior_yes(isfc)*product(cond_yes(ielem,iline,:))) / &
                 (prior_yes(isfc)*product(cond_yes(ielem,iline,:)) +  &        
                 prior_no(isfc)*product(cond_no(ielem,iline,:)))
        endif


        !---- make a cloud mask
        cld_mask_bayes(ielem,iline) = sym%CLEAR
        if (posterior_cld_probability(ielem,iline) >= 0.9) then
                cld_mask_bayes(ielem,iline) = sym%CLOUDY 
        endif
        if ((posterior_cld_probability(ielem,iline) >= 0.5) .and. &
            (posterior_cld_probability(ielem,iline) < 0.9)) then
                cld_mask_bayes(ielem,iline) = sym%PROB_CLOUDY 
        endif
        if ((posterior_cld_probability(ielem,iline) > 0.1) .and. &
            (posterior_cld_probability(ielem,iline) < 0.5)) then
                cld_mask_bayes(ielem,iline) = sym%PROB_CLEAR
        endif

!       if (posterior_cld_probability(ielem,iline) > optimal_posterior_prob(isfc)) then
!               cld_mask_bayes(ielem,iline) = sym%CLOUDY 
!       else
!          if (sum(cld_test_vector(First_Clr_Uni_Test:Last_Clr_Uni_Test,ielem,iline)) == &
!                  Num_Clr_Uni_Tests*sym%NO) then   !check uni tests
!                 cld_mask_bayes(ielem,iline) = sym%CLEAR
!          else
!                 cld_mask_bayes(ielem,iline) = sym%PROB_CLEAR
!         endif 
!       endif
          
        !--- now do by class
        do iclass = 1, N_class

         if (cond_yes(ielem,iline,iclass) == 0.0) then
             posterior_cld_probability_by_class(ielem,iline,iclass) = 0.0
         elseif (cond_no(ielem,iline,iclass) == 0.0) then
             posterior_cld_probability_by_class(ielem,iline,iclass) = 1.0
         else
             posterior_cld_probability_by_class(ielem,iline,iclass) = &
                 (prior_yes(isfc)*cond_yes(ielem,iline,iclass)) / &
                 (prior_yes(isfc)*cond_yes(ielem,iline,iclass) +  &        
                  prior_no(isfc)*cond_no(ielem,iline,iclass))
         endif

        enddo

      enddo
   enddo


!  pix_array_1 = bayes_sfc_type
!  pix_array_2 = glint_mask
!  pix_array_2 = posterior_cld_probability_by_class(:,:,2)

!  pix_array_1 = Classifier_Value(:,:,4)
!  pix_array_2 = posterior_cld_probability_by_class(:,:,4)

 end subroutine CLOUD_MASK_NAIVE_BAYES

!====================================================================
! SUBROUTINE Name: COMPUTE_BAYES_SFC_TYPE
!
! Function:
!   Computes the bayesian surface type given the ancillary sfc data
!
!====================================================================
 function COMPUTE_BAYES_SFC_TYPE(land_temp,snow_temp,sfc_type_temp, tsfc_rtm_temp, &
                                 lat_temp,lon_temp,sst_back_uni_temp,sfc_emiss_3b_temp, &
                                 zsfc_temp) &
                                 result(bayes_sfc_type_temp)
   integer(kind=int1), intent(in):: land_temp
   integer(kind=int1), intent(in):: snow_temp
   integer(kind=int1), intent(in):: sfc_type_temp
   real(kind=real4), intent(in):: tsfc_rtm_temp
   real(kind=real4), intent(in):: lat_temp
   real(kind=real4), intent(in):: lon_temp
   real(kind=real4), intent(in):: sst_back_uni_temp
   real(kind=real4), intent(in):: sfc_emiss_3b_temp
   real(kind=real4), intent(in):: zsfc_temp

   integer(kind=int4):: bayes_sfc_type_temp


   if (land_temp == sym%LAND) then
           bayes_sfc_type_temp = 3
   else
           bayes_sfc_type_temp = 1
   endif

   !--- #2 - Shallow Ocean
   if ((land_temp == sym%MODERATE_OCEAN) .or. &
       (land_temp == sym%DEEP_INLAND_WATER) .or. &
       (land_temp == sym%SHALLOW_INLAND_WATER) .or. &
       (land_temp == sym%SHALLOW_OCEAN)) then
           bayes_sfc_type_temp = 2
   endif
   if ((land_temp /= sym%LAND) .and. &
       (sst_back_uni_temp > 0.5)) then
           bayes_sfc_type_temp = 2
   endif

   !--- #3 Unfrozen_Land 
   if ((land_temp == sym%LAND) .or. &
       (land_temp == sym%COASTLINE) .or. &
       (land_temp == sym%EPHEMERAL_WATER)) then

           bayes_sfc_type_temp = 3

   endif

   !--- #4 - Snow Covered Land
   if ((lat_temp > -60.0) .and. (snow_temp == sym%SNOW)) then
           bayes_sfc_type_temp = 4
   endif

   !--- #5 - Arctic
   if ((lat_temp >= 0.0) .and. (snow_temp == sym%SEA_ICE)) then
           bayes_sfc_type_temp = 5
   endif
!  if ((lat_temp >= 50.0) .and. (tsfc_rtm_temp < 250.0)) then
!          bayes_sfc_type_temp = 5
!  endif

   !--- #6 - Antarctic
   if ((lat_temp <= -60.0) .and. (snow_temp == sym%SNOW)) then
           bayes_sfc_type_temp = 6
   endif
   if ((lat_temp <= 0.0) .and. (snow_temp == sym%SEA_ICE)) then
           bayes_sfc_type_temp = 6
   endif
   if ((lat_temp >= 60.0) .and.  &
       (lon_temp > -75.0) .and. (lon_temp < -10.0) .and. &
       (land_temp == sym%LAND .or. land_temp == sym%COASTLINE) .and. &
       (snow_temp /= sym%SNOW)) then
           bayes_sfc_type_temp = 6
   endif

   !--- #7 - Desert
   if ((sfc_emiss_3b_temp < 0.90 ) .and. (abs(lat_temp) < 60.0)) then
           bayes_sfc_type_temp = 7
   endif

  return 

 end function COMPUTE_BAYES_SFC_TYPE

end module
