! $Id: num_mod.f90,v 1.19 2010/09/23 21:30:41 mfoster Exp $
!------------------------------------------------------------------
!@***h* PATMOSX/num_mod.f90
! 
! NAME
!   num_mod.f90
! 
! LOCATION
!   /patmosx/src/num_mod.f90
! 
! PURPOSE
!   Library of useful numerical functions
! 
! DESCRIPTION
!   Library of useful numerical functions
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!   none
!
! EXTERNALS
!  private::
!   PIX_SWAP
!   PIX_SORT
!   OPT_MEDIAN9_INT8
!   OPT_MEDIAN9_FLOAT32
!
!  public:: 
!   LOCATE
!   POSSOL
!   JULIAN
!   COMPUTE_MONTH
!   COMPUTE_DAY
!   VAPOR
!   VAPOR_ICE
!   INVERT_2x2
!   INVERT_3x3
!   FIND_BOUNDS
!   PACK_BYTES
!   COMPUTE_TIME_HOURS
!   COMPUTE_SPATIAL_UNIFORMITY_NxN_WITH_INDICES
!   GRADIENT_MEANDER
!   COMPUTE_MEDIAN
!   COMPUTE_MEDIAN_SEGMENT
!   GRADIENT2d
!   GRADIENT2d_REVERSE
!   MEDIAN_FILTER
!   GREAT_CIRCLE
!   POLYNOMIAL
!   LOGNORMAL_MODE_RADIUS
!   LOGNORMAL_DIST
!   OPTICAL_DEPTH_TO_NUM_DEN
!   CALCULATE_MASS_LOADING
!   LEAP_YEAR_FCT
!   GET_REGULAR_LUT_INDEX
!   GET_IRREGULAR_LUT_INDEX
!   SDEV_FROM_SUMS
!   MEAN_FROM_SUM
!   PDF_TO_NORMALIZED_CDF
!   NORMALIZED_CDF_TO_TILE
!   BYTESCALE_VALUE
!
!@*****
!------------------------------------------------------------------------
!
! Module name: numerical_routines
!
! Description: library of useful numerical functions
!
! Routines:
!
! private::
!   PIX_SWAP
!   PIX_SORT
!   OPT_MEDIAN9_INT8
!   OPT_MEDIAN9_FLOAT32
!
! public:: 
!   LOCATE
!   POSSOL
!   JULIAN
!   COMPUTE_MONTH
!   COMPUTE_DAY
!   VAPOR
!   VAPOR_ICE
!   INVERT_2x2
!   INVERT_3x3
!   FIND_BOUNDS
!   PACK_BYTES
!   COMPUTE_TIME_HOURS
!   COMPUTE_SPATIAL_UNIFORMITY_NxN_WITH_INDICES
!   GRADIENT_MEANDER
!   COMPUTE_MEDIAN
!   COMPUTE_MEDIAN_SEGMENT
!   GRADIENT2d
!   GRADIENT2d_REVERSE
!   MEDIAN_FILTER
!   GREAT_CIRCLE
!   POLYNOMIAL
!   LOGNORMAL_MODE_RADIUS
!   LOGNORMAL_DIST
!   OPTICAL_DEPTH_TO_NUM_DEN
!   CALCULATE_MASS_LOADING
!   LEAP_YEAR_FCT
!   GET_REGULAR_LUT_INDEX
!   GET_IRREGULAR_LUT_INDEX
!   SDEV_FROM_SUMS
!   MEAN_FROM_SUM
!   PDF_TO_NORMALIZED_CDF
!   NORMALIZED_CDF_TO_TILE
!   BYTESCALE_VALUE
!
!-------------------------------------------------------------------
 module NUMERICAL_ROUTINES
  use CONSTANTS
  
  implicit none
  
  private:: PIX_SWAP, &
            PIX_SORT, &
            OPT_MEDIAN9_INT8, &
            OPT_MEDIAN9_FLOAT32
  
  public:: LOCATE, &
           POSSOL, &
           JULIAN, &
           COMPUTE_MONTH, &
           COMPUTE_DAY, &
           VAPOR, &
           VAPOR_ICE, &
           INVERT_2x2,  &
           INVERT_3x3,  &
           FIND_BOUNDS,  &
           PACK_BYTES, &
           COMPUTE_TIME_HOURS, &
           COMPUTE_SPATIAL_UNIFORMITY_NxN_WITH_INDICES,  &
           GRADIENT_MEANDER, &
           COMPUTE_MEDIAN, &
           COMPUTE_MEDIAN_SEGMENT, &
	   GRADIENT2d, &
	   GRADIENT2d_REVERSE, &
	   MEDIAN_FILTER, &
	   GREAT_CIRCLE, &
	   POLYNOMIAL, &
	   LOGNORMAL_MODE_RADIUS, &
	   LOGNORMAL_DIST, &
	   OPTICAL_DEPTH_TO_NUM_DEN, &
	   CALCULATE_MASS_LOADING, &
           LEAP_YEAR_FCT, &
           GET_REGULAR_LUT_INDEX, &
           GET_IRREGULAR_LUT_INDEX, &
           SDEV_FROM_SUMS, &
           MEAN_FROM_SUM, &
           PDF_TO_NORMALIZED_CDF, &
           NORMALIZED_CDF_TO_TILE, &
           BYTESCALE_VALUE

 interface PACK_BYTES
     module procedure  &
         PACK_BYTES_I1,  &
         PACK_BYTES_I2
 end interface
 
 interface median_filter
   module procedure &
     median_filter_int8, &
     median_filter_float32
 end interface
 
 interface median_filter9
   module procedure &
     median_filter9_int8, &
     median_filter9_float32
 end interface
 
 interface polynomial
   module procedure &
     polynomial_float32, &
     polynomial_float64
 end interface
 
 interface pix_swap
   module procedure &
     pix_swap_int8, &
     pix_swap_float32
 end interface
 
 interface pix_sort
   module procedure &
     pix_sort_int8, &
     pix_sort_float32
 end interface
 
 INTERFACE Sdev_From_Sums
   MODULE PROCEDURE &
     Sdev_From_Sums_Int32, &
     Sdev_From_Sums_Float32
 END INTERFACE
 
 INTERFACE Mean_From_Sum
   MODULE PROCEDURE &
     Mean_From_Sum_Int32, &
     Mean_From_Sum_Float32
 END INTERFACE
 
 REAL(kind=real4), PARAMETER, PRIVATE :: R_EARTH = 6378206.4

  contains

!----------------------------------------------------------------------
! subroutine COMPUTE_SPATIAL_UNIFORMITY_NxN_WITH_INDICES
!
! Routine to compute spatial uniformity for every pixel in an array
! based on its nxn neighborhood
!
! z - the input array
! n - the size of the box (1=3x3, 2=5x5, ...)
! bad_mask - mask array (only values with sym%YES will contribute)
! imin - starting x-index of array
! imax - ending x-index of array
!
! z_mean - mean of z over the (2n+1 x 2n+1) box
! z_min - minimum of z over the (2n+1 x 2n+1) box
! z_max - maximum of z over the (2n+1 x 2n+1) box
! z_std - standard deviation of z over the (2n+1 x 2n+1) box
! i_loc_of_max - i index of the maximum value
!----------------------------------------------------------------------
subroutine COMPUTE_SPATIAL_UNIFORMITY_NxN_WITH_INDICES( &
                                          z, &
                                          n, &
                                          uni_land_mask_flag, &
                                          bad_mask, &
                                          land_mask,  &
                                          imin, &
                                          imax, &
                                          jmin, &
                                          jmax, &
                                          z_mean, &
                                          z_min,  &
                                          z_max,  &
                                          z_std,  &
                                          i_loc_of_max,  &
                                          j_loc_of_max,  &
                                          i_loc_of_min,  &
                                          j_loc_of_min)

  real(kind=real4), dimension(:,:), intent(in):: z
  integer(kind=int1), dimension(:,:), intent(in):: bad_mask
  integer(kind=int1), dimension(:,:), intent(in):: land_mask
  integer, intent(in):: uni_land_mask_flag
  real(kind=real4), dimension(:,:), intent(out):: z_mean
  real(kind=real4), dimension(:,:), intent(out):: z_min
  real(kind=real4), dimension(:,:), intent(out):: z_max
  real(kind=real4), dimension(:,:), intent(out):: z_std
  integer, intent(in):: n
  integer, intent(in):: imin
  integer, intent(in):: imax
  integer, intent(in):: jmin
  integer, intent(in):: jmax
  integer, dimension(:,:), intent(out):: i_loc_of_min
  integer, dimension(:,:), intent(out):: j_loc_of_min
  integer, dimension(:,:), intent(out):: i_loc_of_max
  integer, dimension(:,:), intent(out):: j_loc_of_max
  integer:: i
  integer:: j
  integer:: i1
  integer:: i2
  integer:: j1
  integer:: j2
  integer:: ii
  integer:: jj
  integer:: n_good

  !--- initialize to missing
  z_mean = missing_value_real4
  z_min = missing_value_real4
  z_max = missing_value_real4
  z_std = missing_value_real4
  i_loc_of_max = missing_value_int1
  j_loc_of_max = missing_value_int1
  i_loc_of_min = missing_value_int1
  j_loc_of_min = missing_value_int1

  line_loop: do j=jmin, jmax - jmin + 1

  !--- set limits of NxN array in the j-direction
  j1 = max(jmin,j-n)   !top index of local array
  j2 = min(jmax,j+n)   !bottom index of local array

  element_loop: do i = imin, imax - imin + 1

     !--- set limits of NxN array in the i-direction
     i1 = max(imin,i-n)   !left index of local array
     i2 = min(imax,i+n)   !right index of local array

     !--- initialize
     z_mean(i,j) = 0.0
     z_std(i,j) = 0.0
     z_min(i,j) = huge(z_min(i,j))
     z_max(i,j) = -1.0*huge(z_min(i,j))
     n_good = 0

     !--- go through each element in NxN array
     sub_line_loop: do jj = j1,j2
      sub_element_loop: do ii = i1,i2

        if (bad_mask(ii,jj) == sym%YES) then
          cycle
        endif

        if (z(ii,jj) == missing_value_real4) then
          cycle
        endif
       
        if ((uni_land_mask_flag == sym%YES) .and. &
             (land_mask(i,j) /= land_mask(ii,jj))) then    
          cycle
        endif

        n_good = n_good + 1
        z_mean(i,j) = z_mean(i,j) + z(ii,jj)
        z_std(i,j) = z_std(i,j) + z(ii,jj)**2
 
        if (z(ii,jj) < z_min(i,j)) then
           z_min(i,j) = z(ii,jj)
           i_loc_of_min(i,j) = ii
           j_loc_of_min(i,j) = jj
        endif

        if (z(ii,jj) > z_max(i,j)) then
           z_max(i,j) = z(ii,jj)
           i_loc_of_max(i,j) = ii
           j_loc_of_max(i,j) = jj
        endif

       end do sub_element_loop
     end do sub_line_loop

     !--- if any good pixel found, compute mean and standard deviation
     if (n_good > 0) then 
       z_mean(i,j) = z_mean(i,j) / n_good
       z_std(i,j) = sqrt(max(0.0,(z_std(i,j)/n_good - z_mean(i,j)**2)))
     endif
      
 end do element_loop

end do line_loop

end subroutine COMPUTE_SPATIAL_UNIFORMITY_NxN_WITH_INDICES

!-------------------------------------------------------------------------
! subroutine LOCATE(xx, n, x, j)
! Numerical recipes bisection search - x will be between xx(j) and xx(j+1)
!--------------------------------------------------------------------------
  subroutine LOCATE(xx, n, x, j)

!   Arguments
    integer,                        intent(in)  :: n
    integer,                        intent(out) :: j
    real (kind=ipre),               intent(in)  :: x
    real (kind=ipre), dimension(:), intent(in)  :: xx

!   Local variables
    integer :: i, jl, jm, ju

    jl = 0
    ju = n + 1
    do i = 1, 2*n
       if (ju-jl <= 1) then
          exit
       endif
       jm = (ju + jl) / 2
       if ((xx(n) >= xx(1)) .eqv. (x >= xx(jm))) then
          jl = jm
       else
          ju = jm
       endif
    enddo
    if (x == xx(1)) then
       j=1
    else if (x == xx(n)) then
       j = n - 1
    else
       j = jl
    endif

  end subroutine LOCATE

!---------------------------------------------------------------------
! subroutine POSSOL(jday,tu,xlon,xlat,asol,phis)
! Description: compute solar angles
!
! input:
!         jday - julian day
!         tu -  time of day - fractional hours
!         lon - latitude in degrees
!         lat - latitude in degrees
!      
! output:
!         asol - solar zenith angle 
!         phi - solar azimuth angle
!
!----------------------------------------------------------------------
 subroutine POSSOL(jday,tu,xlon,xlat,asol,phis)
  integer, intent(in):: jday 
  real (kind=ipre), intent(in):: tu,xlon,xlat
  real (kind=ipre), intent(out):: asol, phis
  real (kind=ipre):: tsm, xlo, xla, xj, a1, a2, et, tsv, ah, a3, delta, amuzero, elev, &
         az, caz, azim, pi2
  real (kind=ipre), parameter:: pi = 3.14159265,dtor = pi / 180.0
!
! solar position (zenithal angle asol, azimuthal angle phi
! in degrees
! jday is the number of the day in the month
!
!
! mean solar time
!
      tsm = tu + xlon/15.0
      xlo = xlon*dtor
      xla = xlat*dtor
      xj = real(jday)
!
! time equation (mn.dec)
!
      a1 = (1.00554*xj-6.28306)*dtor
      a2 = (1.93946*xj+23.35089)*dtor
      et = -7.67825*sin(a1) - 10.09176*sin(a2)
!
! true solar time
!
      tsv = tsm + et/60.0
      tsv = (tsv-12.0)
!
! hour angle
!
      ah = tsv*15.0*dtor
!
! solar declination (in radian)
!
      a3 = (0.9683*xj-78.00878)*dtor
      delta = 23.4856*sin(a3)*dtor
!
! elevation, azimuth
!
      amuzero = sin(xla)*sin(delta)+cos(xla)*cos(delta)*cos(ah)
      elev=asin(amuzero)
      az = cos(delta)*sin(ah)/cos(elev)
      caz=(-cos(xla)*sin(delta)+sin(xla)*cos(delta)*cos(ah))/cos(elev)
!
      if (az >= 1.0) then
        azim = asin(1.0)
      elseif (az <= -1.0) then
        azim = asin(-1.0)
      else
        azim = asin(az)
      endif
!
      if (caz <= 0.0) then
        azim = pi - azim
      endif

      if ((caz > 0.0) .and. (az <= 0.0)) then
          azim = 2 * pi + azim
      endif
      azim = azim + pi
      pi2 = 2 * pi
      if (azim> pi2) then
          azim = azim - pi2
      endif
!
! conversion in degrees
!
      elev = elev / dtor
      asol = 90.0 - elev
      phis = azim / dtor

 end subroutine POSSOL
!-------------------------------------------------
! subroutine JULIAN(iday,imonth,iyear,jday)
! compute julian day
! input:
!         iday - integer day
!         imonth - integer month
!         iyear - integer year (2 or four digits)
!         
! output : jday - julian day
!--------------------------------------------------
 subroutine JULIAN(iday,imonth,iyear,jday)

!-- Computes julian day (1-365/366)
        integer, intent(in)::  iday,imonth,iyear
        integer, intent(out):: jday
        integer::  j
        integer, dimension(12)::  jmonth

        jmonth = reshape ((/31,28,31,30,31,30,31,31,30,31,30,31/),(/12/))

        jday = iday
        if (modulo(iyear,4) == 0) then
            jmonth(2)=29
        endif

        do j = 1,imonth-1
           jday = jday + jmonth(j)
        end do

   end subroutine JULIAN

!--------------------------------------------
! compute the month
!---------------------------------------------
 function COMPUTE_MONTH(jday,ileap) result(month)
   integer, intent(in):: ileap
   integer, intent(in):: jday
   integer:: month

   month = 0
   if (jday < 32) then
     month = 1
   elseif (jday < 60+ileap) then
     month = 2
   elseif (jday < 91+ileap) then
     month = 3
   elseif (jday < 121+ileap) then
     month = 4
   elseif (jday < 152+ileap) then
     month = 5
   elseif (jday < 182+ileap) then
     month = 6
   elseif (jday < 213+ileap) then
     month = 7
   elseif (jday < 244+ileap) then
     month = 8
   elseif (jday < 274+ileap) then
     month = 9
   elseif (jday < 305+ileap) then
     month = 10
   elseif (jday < 335+ileap) then
     month = 11
   else
     month = 12
   endif

 end function COMPUTE_MONTH
!--------------------------------------------
! compute the day
!---------------------------------------------
 function COMPUTE_DAY(jday,ileap) result(day)
   integer, intent(in):: ileap
   integer, intent(in):: jday
   integer:: day

   if (jday < 32) then
     day = jday
   elseif (jday < 60) then
     day = jday - 31
   elseif ((jday == 60).and.(ileap == 1)) then
     day = jday - 31
   elseif ((jday == 60).and.(ileap == 0)) then
     day = jday - 59
   elseif (jday < 91+ileap) then
     day = jday - (59 + ileap)
   elseif (jday < 121+ileap) then
     day = jday - (90 + ileap)
   elseif (jday < 152+ileap) then
     day = jday - (120 + ileap)
   elseif (jday < 182+ileap) then
     day = jday - (151 + ileap)
   elseif (jday < 213+ileap) then
     day = jday - (181 + ileap)
   elseif (jday < 244+ileap) then
     day = jday - (212 + ileap)
   elseif (jday < 274+ileap) then
     day = jday - (243 + ileap)
   elseif (jday < 305+ileap) then
     day = jday - (273 + ileap)
   elseif (jday < 335+ileap) then
     day = jday - (304 + ileap)
   else
     day = jday - (334 + ileap)
   endif

   return

 end function COMPUTE_DAY

 !---- function to return the system in fractional hours
 function COMPUTE_TIME_HOURS()   result(time_hours)
   character(len=8):: system_date
   character(len=10):: system_time
   character(len=5):: system_time_zone
   integer, dimension(8):: system_time_value

   real:: time_hours

   call DATE_AND_TIME(system_date,system_time,system_time_zone, system_time_value)

   time_hours = real(system_time_value(5)) +  &
                     (real(system_time_value(6)) + &
                      real(system_time_value(7) + &
                      real(system_time_value(8))/1000.0)/60.0)/60.0
   return

 end function COMPUTE_TIME_HOURS

!----------------------------------------------------------------
! functions to compute some needed water vapor parameters
!----------------------------------------------------------------
 function VAPOR(T) result(es)
                                                                     
!  T in Kelvin                                                          
!  es in mbar

  implicit none
  real, intent (in) :: T
  real :: es

   es = 6.112 * exp(17.67 * (T-273.16) / (T - 29.66))

  return 
end function VAPOR

!---- saturation vapor pressure for ice
function VAPOR_ICE(T) result(es)
   implicit none
   real, intent(in):: T
   real:: es
     es = 6.1078 * exp(21.8745584 * (T-273.16) / (T - 7.66))
  return 
end function VAPOR_ICE


!--------------------------------------------------------------------------
! subroutine INVERT_2x2(A,A_inv,ierr)
!
! Matrix Inversion for a 2x2 matrix
!
! A - input matrix
! A_inv - the inverse of A (output)
! ierr - output flag that report a singular matrix and a failure
!--------------------------------------------------------------------------
subroutine INVERT_2x2(A,A_inv,ierr)
  real, dimension(:,:), intent(in):: A
  real, dimension(:,:), intent(out):: A_inv
  real:: determinant
  integer, intent(out):: ierr

!--- compute determinant
  ierr = 0
  determinant = A(1,1)*A(2,2) - A(1,2)*A(2,1)
  if (determinant == 0.0) then
!       print *, "Singular Matrix in Invert 2x2"
        ierr = 1
  endif

!--- compute inverse
  A_inv(1,1) = A(2,2)
  A_inv(1,2) = -A(1,2)
  A_inv(2,1) = -A(2,1)
  A_inv(2,2) = A(1,1)
  A_inv = A_inv / determinant

end subroutine INVERT_2x2
!--------------------------------------------------------------------------
! subroutine INVERT_3x3(A,A_inv,ierr)
!
! Matrix Inversion for a 3x3 matrix
!--------------------------------------------------------------------------
subroutine INVERT_3x3(A,A_inv,ierr)
  real, dimension(:,:), intent(in):: A
  real, dimension(:,:), intent(out):: A_inv
  integer, intent(out):: ierr
  real:: determinant

  ierr = 0
!--- compute determinant
  determinant = A(1,1)*(A(2,2)*A(3,3)-A(3,2)*A(2,3)) - &
                A(1,2)*(A(2,1)*A(3,3)-A(3,1)*A(2,3)) + &
                A(1,3)*(A(2,1)*A(3,2)-A(3,1)*A(2,2))
  if (determinant == 0.0) then
!       print *, "Singular Matrix in Invert 3x3"
        ierr = 1
  endif

!--- compute inverse
  A_inv(1,1) = A(2,2)*A(3,3) - A(3,2)*A(2,3)
  A_inv(1,2) = A(1,3)*A(3,2) - A(3,3)*A(1,2)
  A_inv(1,3) = A(1,2)*A(2,3) - A(2,2)*A(1,3)
  A_inv(2,1) = A(2,3)*A(3,1) - A(3,3)*A(2,1)
  A_inv(2,2) = A(1,1)*A(3,3) - A(3,1)*A(1,3)
  A_inv(2,3) = A(1,3)*A(2,1) - A(2,3)*A(1,1)
  A_inv(3,1) = A(2,1)*A(3,2) - A(3,1)*A(2,2)
  A_inv(3,2) = A(1,2)*A(3,1) - A(3,2)*A(1,1)
  A_inv(3,3) = A(1,1)*A(2,2) - A(2,1)*A(1,2)
  A_inv = A_inv / determinant


end subroutine INVERT_3x3

!--------------------------------------------------------------------
! subroutine FIND_BOUNDS(lat,lon,wlon,elon,slat,nlat,dateline_flg)
!
! This subroutine reads in a satellite data set containing lat/lon
! values and finds the western/eastern-most longitude and
! southern/northern-most latitude values.
!--------------------------------------------------------------------

subroutine FIND_BOUNDS(lat,lon,wlon,elon,slat,nlat,dateline_flg)

!Inputs and outputs are as follows:

!Inputs:
! lat    - array of satellite latitudes
! lon    - array of satellite longitudes

!Outputs
! wlon   - western-most longitude value in satellite data
! elon   - eastern-most longitude value in satellite data
! slat   - southern-most latitude value in satellite data
! nlat   - northern-most latitude value in satellite data

!Declare calling parameters

  REAL(kind=real4),dimension(:,:),intent(in) :: lat,lon
  REAL(kind=real4),intent(out) :: wlon,elon,slat,nlat
  INTEGER(kind=int1), intent(out) :: dateline_flg

  INTEGER(kind=int4) :: astatus, nx, ny
  REAL(kind=real4), dimension(:,:), allocatable :: dum

  nx = size(lon,1)
  ny = size(lon,2)

  allocate(dum(nx,ny),stat=astatus)
  if (astatus /= 0) then
    print "(a,'Not enough memory to allocate dummy longitude array.')",EXE_PROMPT
    stop
  endif

  dum = lon

  nlat = missing_value_real4
  slat = missing_value_real4
  elon = missing_value_real4
  wlon = missing_value_real4

  nlat = maxval(lat, mask = lat >= -90.0 .and. lat <= 90.0)
  slat = minval(lat, mask = lat >= -90.0 .and. lat <= 90.0)

  elon = maxval(lon, mask = lon >= -180.0 .and. lon <= 180.0)
  wlon = minval(lon, mask = lon >= -180.0 .and. lon <= 180.0)

  !Check if intl. date line is crossed
  dateline_flg = 0
  if (elon > 160.0 .and. wlon < -160) then
    where(lon < 0.0 .and. lon > missing_value_real4)
      dum = lon + 360.0
    endwhere
    !write(*,*) "international date line is assumed to be crossed"
    elon = maxval(dum, mask = lon >= -180.0 .and. lon <= 180.0)
    wlon = minval(dum, mask = lon >= -180.0 .and. lon <= 180.0)
    dateline_flg = 1
  endif

  deallocate(dum, stat=astatus)
  if (astatus /= 0) then
    print "(a,'Error deallocating dummy longitude array.')",EXE_PROMPT
    stop
  endif

end subroutine FIND_BOUNDS

!------------------------------------------------------------------------
! subroutine PACK_BYTES_I1(input_bytes,bit_depth,output_bytes)
! subroutine PACK_BYTES_I2(input_bytes,bit_depth,output_bytes)
!
! Routines to pack individual bytes into a single byte
!
! input:
! input_bytes - vector of bytes to be packed into output_byte
! bit_start - vector of bit starting positions (1-7) for each input byte
! bit_depth - vector of bit depths (1-7) for each input byte (total can not exceed 8)
!
! output: 
! output_byte - byte variable that holds the bit values of the input_bytes - can be i1 or i2
!
! local
!  n_in - number of elements of input vectors
!  i_in - index of input_bytes (1-n_in)
!  n_out - number of elements of output vectors
!  i_out - index of output_bytes (1-n_out)
!
! Note:
! 1.  if the input byte has information in bits greater then bit depth - they are removed 
!
!
! Example, pack an input_byte wth  bit_start = 2 and bit depth 3 
!
! input byte
!           x x x
! _ _ _ _ _ _ _ _
!
! result of first ishft
! x x x
! _ _ _ _ _ _ _ _
!
! result of second ishft
!       x x x
! _ _ _ _ _ _ _ _
!
! Author: Andrew Heidinger
!
! Version History:  
! February 2006 - Created
!-----------------------------------------------------------------------------------

!--- This Version packs into one byte words
   subroutine PACK_BYTES_I1(input_bytes,bit_depth,output_bytes)
    integer(kind=int1), dimension(:), intent(in):: input_bytes
    integer(kind=int4), dimension(:), intent(in):: bit_depth
    integer(kind=int1), dimension(:), intent(out):: output_bytes
    integer(kind=int1):: bit_start, bit_end, bit_offset
    integer(kind=int1):: temp_byte
    integer:: n_in,i_in,n_out,i_out
    integer, parameter:: word_bit_depth = 8
  
!--- determine size of vectors
    n_in = size(input_bytes)
    n_out = size(output_bytes)

!--- reset output byte
   output_bytes = 0

!--- initialize
   bit_offset = 0
   bit_start = 0
   bit_end = 0
   i_out = 1

!--- loop through input bytes
   do i_in = 1, n_in

!--- determine starting and ending bit locations
     bit_start = bit_offset + 1
     bit_end = bit_start + bit_depth(i_in) - 1

!--- determine if this input byte will fit on current output byte, if not go to next
     if (bit_end > word_bit_depth) then
      i_out = i_out + 1
      bit_offset = 0
      bit_start = bit_offset + 1
      bit_end = bit_start + bit_depth(i_in) - 1
     endif

!--- check for exceeding the space allowed for the packed bytes
     if (i_out > n_out) then
       print *, "ERROR: Insufficient space for bit packing" 
       return
     endif

!--- place input byte into correct position
     temp_byte =0
     temp_byte = ishft(input_bytes(i_in),word_bit_depth-bit_depth(i_in))   !first ishft
     temp_byte = ishft(temp_byte,bit_end - word_bit_depth)                 !second ishft

!--- modify output byte
     output_bytes(i_out) = output_bytes(i_out) + temp_byte

!--- update bit offset
     bit_offset = bit_offset + bit_depth(i_in)

   enddo

  end subroutine  PACK_BYTES_I1

!--- This Version packs into two byte words
   subroutine PACK_BYTES_I2(input_bytes,bit_depth,output_bytes)
    integer(kind=int1), dimension(:), intent(in):: input_bytes
    integer(kind=int4), dimension(:), intent(in):: bit_depth
    integer(kind=int2), dimension(:), intent(out):: output_bytes
    integer(kind=int1):: bit_start, bit_end, bit_offset
    integer(kind=int2):: temp_byte                 
    integer:: n_in,i_in,n_out,i_out
    integer, parameter:: word_bit_depth = 16

!--- determine size of vectors
    n_in = size(input_bytes)
    n_out = size(output_bytes)

!--- reset output byte
   output_bytes = 0

!--- initialize
   bit_offset = 0
   bit_start = 0
   bit_end = 0
   i_out = 1

!--- loop through input bytes
   do i_in = 1, n_in

!--- determine starting and ending bit locations
     bit_start = bit_offset + 1
     bit_end = bit_start + bit_depth(i_in) - 1

!--- determine if this input byte will fit on current output byte, if not go to next
     if (bit_end > word_bit_depth) then
      i_out = i_out + 1
      bit_offset = 0
      bit_start = bit_offset + 1
      bit_end = bit_start + bit_depth(i_in) - 1
     endif

!--- check for exceeding the space allowed for the packed bytes
     if (i_out > n_out) then
       print *, "ERROR: Insufficient space for bit packing"
       return
     endif

!--- place input byte into correct position
     temp_byte =0
     temp_byte = ishft(INT(input_bytes(i_in),KIND=INT2),word_bit_depth-bit_depth(i_in))   !first ishft
     temp_byte = ishft(temp_byte,bit_end - word_bit_depth)                 !second ishft

!--- modify output byte
     output_bytes(i_out) = output_bytes(i_out) + temp_byte

!--- update bit offset
     bit_offset = bit_offset + bit_depth(i_in)

   enddo

  end subroutine  PACK_BYTES_I2
!-----------------------------------------------------------------
! subroutine GRADIENT_MEANDER(grid,nx,jmin,ny,grad_flag,mask,min_valid,
!                             max_valid,num_grad_dir_change_max,xlrc,ylrc)
!
! grid - the data to analyze
! nx - the x dimension
! ny - the y dimension
! mask - a mask that can be applied to the data
! num_grad_dir_change_max = maximum number of direction changes allowed
! min_valid - smallest value to attempt to find lrc
! max_valid - largest value to attempt to find lrc
! threshold_value - the smallest value allowed inorder to look for lrc,
!   pixels with values greater than this, are called missing.
! xlrc - x dimension index of lrc
! ylrc - y dimension index of lrc
!
! This version checks for the optimal gradient at each step
! another alternate is to only recheck gradient when it changes 
! sign 
!-----------------------------------------------------------------
subroutine GRADIENT_MEANDER(grid,                    &
                            nx,                      &
                            jmin,                    &
                            ny,                      &
                            grad_flag,               &
                            mask,                    &
                            min_valid,               &
                            max_valid,               &
                            num_grad_dir_change_max, &
                            xlrc,                    &
                            ylrc)

  REAL (kind=real4), dimension(:,:), intent(in) :: grid
  INTEGER (kind=int4), intent(in) :: nx
  INTEGER (kind=int4), intent(in) :: jmin
  INTEGER (kind=int4), intent(in) :: ny
  INTEGER (kind=int4), intent(in) :: grad_flag
  INTEGER (kind=int4), intent(in) :: num_grad_dir_change_max
  INTEGER (kind=int4) :: num_grad_dir_change
  INTEGER (kind=int1), dimension(:,:), intent(in) :: mask
  REAL (kind=real4), intent(in) :: min_valid
  REAL (kind=real4), intent(in) :: max_valid
  INTEGER (kind=int4), dimension(:,:), intent(inout) :: xlrc
  INTEGER (kind=int4), dimension(:,:), intent(inout) :: ylrc
  INTEGER (kind=int4), parameter :: max_step = 50
  real:: grad_max
  real:: grad
  real:: grad_temp
  real:: grad_threshold
  integer:: i
  integer:: j
  integer:: ii
  integer:: jj
  integer:: istep
  integer:: i_grad
  integer:: j_grad
  integer:: ix
  integer:: jx
  integer:: itemp
  integer:: jtemp
  integer:: istop
  integer:: i_grad_old
  integer:: j_grad_old

  xlrc = missing_value_int4
  ylrc = missing_value_int4
  grad_threshold = 0.0001

i_loop:  do i = 1, nx
j_loop:    do j = jmin, ny + jmin - 1

!i_loop:  do i = 2, nx-1
!j_loop:    do j = jmin+1, ny - 1 + jmin - 1

   !--- check to see that pixel is a valid one to process
   if (mask(i,j) == sym%NO) then
       cycle
   endif

   !--- initialize lrc as this pixel if it falls within valid bounds
   if ((grid(i,j) >= min_valid) .and. (grid(i,j) >= max_valid)) then
     xlrc(i,j) = i
     ylrc(i,j) = j
   endif

   !--- check to see if this pixel disqualifies by being below mininum
   !--- if grad_flag = 1, then these pixels are lrc's
   if (grid(i,j) <= min_valid) then
       if (grad_flag == 0) then
          cycle
       else
          xlrc(i,j) = i
          ylrc(i,j) = j
          cycle
       endif
   endif

   !--- check to see if this pixel disqualifies by being above maximum
   !--- if grad_flag = 0, then these pixels are lrc's
   if (grid(i,j) >= max_valid) then
       if (grad_flag == 0) then
          xlrc(i,j) = i
          ylrc(i,j) = j
          cycle
       else
          cycle
       endif
   endif

   !---- go along gradient and find pixel where it reverses 
   !---- value exceeds maximum value
   ii = i
   jj = j
   num_grad_dir_change = 0
   i_grad_old = 0
   j_grad_old = 0
   istop = 0

step_loop:    do istep = 1, max_step

   !--- compute gradient direction for the 
   !--- first step or after a gradient reversal found
   if ((istep == 1) .or. (istop == 1)) then

    i_grad = 0
    j_grad = 0

    if (grad_flag == 0) then
       grad_max = -999.0
    else
       grad_max = 999.0
    endif

    do ix = -1, 1
     do jx = -1, 1

      if ((ix == 0) .and. (jx == 0)) then 
         cycle
      endif

      if (ii+ix < 1 .or. ii+ix > nx .or. jj+jx <1 .or. jj+jx>ny+jmin-1) then
         cycle
      endif

      grad = grid(ii+ix,jj+jx) - grid(ii,jj)

      if ((grad_flag == 0) .and. (grad >= grad_max) .and. (grad > 0.0) .and.  &
          (abs(grad) >= grad_threshold)) then
        i_grad = ix
        j_grad = jx
        grad_max = grad
      endif

      if ((grad_flag == 1) .and. (grad <= grad_max) .and. (grad < 0.0) .and. &
          (abs(grad) >= grad_threshold)) then
        i_grad = ix
        j_grad = jx
        grad_max = grad
      endif

     enddo
    enddo

    !--- check to see if a gradient was not found
    if ((i_grad == 0) .and. (j_grad == 0)) then
        xlrc(i,j) = ii
        ylrc(i,j) = jj
       exit
    endif

    !--- increment direction change counter
    num_grad_dir_change  = num_grad_dir_change + 1

   endif

   !--- replace old copy with new
   i_grad_old = i_grad
   j_grad_old = j_grad

   !--- move forward along the gradient
   itemp = max(1,min(ii+i_grad,nx))  
   jtemp = max(jmin,min(jj+j_grad,jmin+ny-1))  

   ii = ii + i_grad
   jj = jj + j_grad 

   !--- check to see if point exceeds segment bound
   if ((ii < 1) .or. (ii > nx) .or. (jj < jmin) .or. (jj>jmin+ny-1)) then
        xlrc(i,j) = ii - i_grad 
        ylrc(i,j) = jj - j_grad
        exit
   endif

   !--- check to see if max/min value exceeded if grad_flag = 0/1
   if (((grad_flag == 0) .and. (grid(ii,jj) >=  max_valid)) .or.  &
       ((grad_flag == 1) .and. (grid(ii,jj) <=  min_valid))) then
         xlrc(i,j) = ii
         ylrc(i,j) = jj
         exit
   endif

   !--- check to see if next point exceeds segment bound along scan direction
   if ((ii+i_grad < 1).or.(ii+i_grad > nx).or.(jj+j_grad < jmin).or. &
       (jj+j_grad > jmin + ny - 1)) then
          xlrc(i,j) = ii
          ylrc(i,j) = jj
          exit
   endif

   !--- if next point in segment, compute gradient to next pixel
   grad = grid(ii+i_grad,jj+j_grad) - grid(ii,jj)

   !--- check for a gradient reversal, if so stop looking
   istop = 0
   if ((grad_flag == 0) .and. (grad < 0.0)) then
         xlrc(i,j) = min(nx,max(1,ii - i_grad))
         ylrc(i,j) = min(ny+jmin-1,max(jmin,jj - j_grad))
         istop = 1
   endif

   if ((grad_flag == 1) .and. (grad > 0.0)) then
         xlrc(i,j) = min(nx,max(1,ii - i_grad))
         ylrc(i,j) = min(ny+jmin-1,max(jmin,jj - j_grad))
         istop = 1 
   endif

   !--- check to see if another meander is allowed
   if ((istop == 1) .and.  &
          (num_grad_dir_change > num_grad_dir_change_max)) then
          exit
   endif

  end do step_loop

  end do j_loop
 end do i_loop

end subroutine GRADIENT_MEANDER

!==============================================================
! subroutine COMPUTE_MEDIAN(z,mask,z_median,z_mean,z_std_median)
!
! Median filter
!==============================================================
subroutine COMPUTE_MEDIAN(z,mask,z_median,z_mean,z_std_median)

! The purpose of this function is to find 
! median (emed), minimum (emin) and maximum (emax)
! for the array elem with nelem elements. 

 real, dimension(:,:), intent(in):: z
 real, intent(out):: z_median
 real, intent(out):: z_mean
 real, intent(out):: z_std_median
 integer(kind=int1), dimension(:,:), intent(in):: mask 
 integer:: i,j,k,nx,ny,nelem
 real, dimension(:), allocatable::x
 real(kind=real4):: u

 z_median = missing_value_real4
 z_std_median = missing_value_real4
 z_mean = missing_value_real4

 nx = size(z,1)
 ny = size(z,2)

 nelem = nx * ny

 allocate(x(nelem))
 x = 0.0

 k = 0
 do i = 1, nx
   do j = 1, ny
      if (mask(i,j) == sym%NO .and. z(i,j) /= missing_value_real4) then
           k = k + 1   
           x(k) = z(i,j)
      endif
  enddo
 enddo

 nelem = k
   
 if (nelem < 1) then
     if (allocated(x)) deallocate(x)
     return
 endif 
!--- sort the array into ascending order
  do i=1,nelem-1
   do j=i+1,nelem
    if(x(j)<x(i))then
     u=x(j)
     x(j)=x(i)
     x(i)=u
    end if   
   end do
  end do

!---- pick the median
  if(mod(nelem,2)==1)then
   i=nelem/2+1
   z_median=x(i)
  else  
   i=nelem/2
   z_median=(x(i)+x(i+1))/2
  end if

!--- compute standard deviation wrt median
  z_mean = sum(x(1:nelem))/nelem
  z_std_median = sqrt(sum((x(1:nelem) - z_median)**2) / nelem)


! if (z_std_median > 60.0) then 
!         print *, "big std median ", z_std_median, nelem, x(1:nelem)
!         print *, "z_nxn = ", z
! endif

  if (allocated(x)) deallocate(x)

end subroutine COMPUTE_MEDIAN

!----------------------------------------------------------------------
! subroutine COMPUTE_MEDIAN_SEGMENT(z,mask,n,imin,imax,jmin,jmax,
!                                   z_median,z_std_median)
!
! Compute standard deviaion of an array wrt to the median
!----------------------------------------------------------------------
subroutine COMPUTE_MEDIAN_SEGMENT(z,mask,n,imin,imax,jmin,jmax, &
                                  z_median, &
                                  z_std_median)
  real(kind=real4), dimension(:,:), intent(in):: z
  integer(kind=int1), dimension(:,:), intent(in):: mask
  real(kind=real4), dimension(:,:), intent(out):: z_std_median
  real(kind=real4), dimension(:,:), intent(out):: z_median
! real(kind=real4), dimension(:,:), intent(out):: z_mean
  integer, intent(in):: n
  integer, intent(in):: imin
  integer, intent(in):: imax
  integer, intent(in):: jmin
  integer, intent(in):: jmax
  integer:: i
  integer:: j
  integer:: i1
  integer:: i2
  integer:: j1
  integer:: j2
  real(kind=real4) :: z_mean

  do i = imin, imax
    do j = jmin, jmax

     j1 = max(jmin,j-n)   !top index of local array
     j2 = min(jmax,j+n)   !bottom index of local array
     i1 = max(imin,i-n)   !left index of local array
     i2 = min(imax,i+n)   !right index of local array

!--- compute median
     call COMPUTE_MEDIAN(z(i1:i2,j1:j2),mask(i1:i2,j1:j2),z_median(i,j), &
                         z_mean,z_std_median(i,j))

     enddo
  enddo

end subroutine COMPUTE_MEDIAN_SEGMENT

!-----------------------------------------------------------------
! SUBROUTINE gradient2d(grid, nx, ny, mask, min_valid, max_valid, threshold_value, xmax, ymax)
!-----------------------------------------------------------------

SUBROUTINE gradient2d(grid, nx, ny, mask, min_valid, max_valid, threshold_value, xmax, ymax)

  REAL (kind=real4), dimension(:,:), intent(in) :: grid
  INTEGER (kind=int4), intent(in) :: nx, ny
  INTEGER (kind=int1), dimension(:,:), intent(in) :: mask
  REAL (kind=real4), intent(in) :: min_valid, max_valid, threshold_value
  INTEGER (kind=int4), dimension(:,:), intent(inout) :: xmax, ymax
  
  INTEGER (kind=int4), parameter :: max_step = 150
  INTEGER (kind=int4) :: ielem, iline, im, jm, ip, jp, dx_start, dy_start, index
  INTEGER (kind=int4) :: direction, di, dj, i0, j0, i1, j1, ibad
  REAL (kind=real4) :: min_grad, ref_value
  INTEGER (kind=int4), dimension(8) :: icol, irow, di_default, dj_default
  REAL (kind=real4), dimension(8) :: grad
  
  dx_start = 2
  dy_start = 2
  
  di_default = (/0,1,1,1,0,-1,-1,-1/)
  dj_default = (/-1,-1,0,1,1,1,0,-1/)
  
  xmax = missing_value_int4
  ymax = missing_value_int4
  
  line_loop: do iline=1, ny
    
    jm = max(1,iline-dy_start)
    jp = min(ny,iline+dy_start)    
    
    element_loop: do ielem=1, nx
    
      if (mask(ielem,iline) == sym%YES .and. grid(ielem,iline) >= min_valid .and. grid(ielem,iline) <= max_valid) then
      
        im = max(1,ielem-dx_start)
        ip = min(nx,ielem+dx_start)
	
	icol = (/ielem,ip,ip,ip,ielem,im,im,im/)	
	irow = (/jm,jm,iline,jp,jp,jp,iline,jm/)
	
	min_grad = 99999.0
	do index=1, 8 
	  if (grid(icol(index),irow(index)) >= min_valid .and. grid(icol(index),irow(index)) <= max_valid) then
	    grad(index) = grid(ielem,iline) - grid(icol(index),irow(index))
	    if (grad(index) < min_grad) then
	      min_grad = grad(index)
	      direction = index
	    endif
	  endif
	end do
	
	di = di_default(direction)
	dj = dj_default(direction)
	
	do index = 1, max_step
	  i0 = max(1,min(ielem + di*index,nx))
	  j0 = max(1,min(iline + dj*index,ny))
	  i1 = max(1,min(ielem + di*index + di,nx))
	  j1 = max(1,min(iline + dj*index + dj,ny))
	  
	  ref_value = grid(i0,j0)
	  ibad = 1
	  if (grid(i1,j1) >= min_valid .and. grid(i1,j1) <= max_valid) then 
	    ibad = 0
	  endif
	  
	  if (grid(i1,j1) >= threshold_value .or. index == max_step .or. grid(i1,j1) < ref_value .or. ibad == 1) then
	    xmax(ielem,iline) = i0
	    ymax(ielem,iline) = j0
	    exit
	  endif
	end do
	
      endif
    
    end do element_loop    
  end do line_loop

END SUBROUTINE gradient2d

!-----------------------------------------------------------------
! SUBROUTINE gradient2d_reverse(grid, nx, ny, mask, min_valid, max_valid, threshold_value, xmin, ymin)
!-----------------------------------------------------------------

SUBROUTINE gradient2d_reverse(grid, nx, ny, mask, min_valid, max_valid, threshold_value, xmin, ymin)

  REAL (kind=real4), dimension(:,:), intent(in) :: grid
  INTEGER (kind=int4), intent(in) :: nx, ny
  INTEGER (kind=int1), dimension(:,:), intent(in) :: mask
  REAL (kind=real4), intent(in) :: min_valid, max_valid, threshold_value
  INTEGER (kind=int4), dimension(:,:), intent(inout) :: xmin, ymin
  
  INTEGER (kind=int4), parameter :: max_step = 150
  INTEGER (kind=int4) :: ielem, iline, im, jm, ip, jp, dx_start, dy_start, index
  INTEGER (kind=int4) :: direction, di, dj, i0, j0, i1, j1
  REAL (kind=real4) :: max_grad
  INTEGER (kind=int4), dimension(8) :: icol, irow, di_default, dj_default
  REAL (kind=real4), dimension(8) :: grad
  
  dx_start = 2
  dy_start = 2
  
  di_default = (/0,1,1,1,0,-1,-1,-1/)
  dj_default = (/-1,-1,0,1,1,1,0,-1/)
  
  xmin = missing_value_int4
  ymin = missing_value_int4
  
  line_loop: do iline=1, ny
    
    jm = max(1,iline-dy_start)
    jp = min(ny,iline+dy_start)    
    
    element_loop: do ielem=1, nx
    
      if (mask(ielem,iline) == sym%YES .and. grid(ielem,iline) >= min_valid .and. grid(ielem,iline) <= max_valid) then
      
        im = max(1,ielem-dx_start)
        ip = min(nx,ielem+dx_start)
	
	icol = (/ielem,ip,ip,ip,ielem,im,im,im/)	
	irow = (/jm,jm,iline,jp,jp,jp,iline,jm/)
	
	max_grad = -99999.0
	do index=1, 8 
	  if (grid(icol(index),irow(index)) >= min_valid .and. grid(icol(index),irow(index)) <= max_valid) then
	    grad(index) = grid(ielem,iline) - grid(icol(index),irow(index))
	    if (grad(index) > max_grad) then
	      max_grad = grad(index)
	      direction = index
	    endif
	  endif
	end do
	
	di = di_default(direction)
	dj = dj_default(direction)
	
	do index = 1, max_step
	  i0 = max(1,min(ielem + di*index,nx))
	  j0 = max(1,min(iline + dj*index,ny))
	  i1 = max(1,min(ielem + di*index + di,nx))
	  j1 = max(1,min(iline + dj*index + dj,ny))
	  if (grid(i1,j1) <= threshold_value .or. index == max_step) then
	    xmin(ielem,iline) = i0
	    ymin(ielem,iline) = j0
	    exit
	  endif
	end do
	
      endif
    
    end do element_loop    
  end do line_loop

END SUBROUTINE gradient2d_reverse

!-----------------------------------------------------------------
! SUBROUTINE median_filter_int8(image, width, mask)
!-----------------------------------------------------------------

SUBROUTINE median_filter_int8(image, width, mask)

  INTEGER (kind=int1), dimension(:,:), intent(inout) :: image
  INTEGER (kind=int4), intent(in) :: width
  INTEGER (kind=int1), dimension(:,:), intent(in) :: mask
  
  select case (width)
  case (3)
    call median_filter9(image, mask)
  case (5)
  !  call_median_filter25
  case default
  !  call_median_filterx
  end select  
  
END SUBROUTINE median_filter_int8

!-----------------------------------------------------------------
! SUBROUTINE median_filter_float32(image, width, mask)
!-----------------------------------------------------------------

SUBROUTINE median_filter_float32(image, width, mask)

  REAL (kind=real4), dimension(:,:), intent(inout) :: image
  INTEGER (kind=int4), intent(in) :: width
  INTEGER (kind=int1), dimension(:,:), intent(in) :: mask
  
  select case (width)
  case (3)
    call median_filter9(image, mask)
  case (5)
  !  call_median_filter25
  case default
  !  call_median_filterx
  end select  
  
END SUBROUTINE median_filter_float32

!-----------------------------------------------------------------
! SUBROUTINE median_filter9_int8(image, mask)
!-----------------------------------------------------------------

SUBROUTINE median_filter9_int8(image, mask)

  INTEGER (kind=int1), dimension(:,:), intent(inout) :: image
  INTEGER (kind=int1), dimension(:,:), intent(in) :: mask
  
  INTEGER (kind=int4), parameter :: width = 3
  INTEGER (kind=int4) :: nx, ny, iline, ielem, astatus
  INTEGER (kind=int4) :: jm, jp, im, ip, dpix, arr_size
  INTEGER (kind=int1), dimension(:,:), pointer :: arr
  
  dpix = (width - 1)/2
  arr_size = width*width
  
  nx = size(image,1)
  ny = size(image,2)
  
  allocate(arr(nx,ny),stat=astatus)
  if (astatus /= 0) then
    print "(a,'Not enough memory to allocate temporary array in median_filter.')",EXE_PROMPT
    stop
  endif
  
  arr = image
  
  line_loop: do iline=1, ny
    
    jm = iline - dpix
    jp = iline + dpix
    
    if (jm < 1 .or. jp > ny) cycle
    
    element_loop: do ielem=1, nx
    
      if (mask(ielem,iline) == sym%YES) cycle
    
      im = ielem - dpix
      ip = ielem + dpix
      
      if (im < 1 .or. ip > nx) cycle
      
      call opt_median9_int8(reshape(image(im:ip,jm:jp),(/arr_size/)), arr(ielem,iline))
    
    end do element_loop
  end do line_loop
  
  image = arr
  
  deallocate(arr,stat=astatus)
  if (astatus /= 0) then
    print "(a,'Error deallocating temporary array in median_filter.')",EXE_PROMPT
    stop
  endif
  
END SUBROUTINE median_filter9_int8

!-----------------------------------------------------------------
! SUBROUTINE median_filter9_float32(image, mask)
!-----------------------------------------------------------------

SUBROUTINE median_filter9_float32(image, mask)

  REAL (kind=real4), dimension(:,:), intent(inout) :: image
  INTEGER (kind=int1), dimension(:,:), intent(in) :: mask
  
  INTEGER (kind=int4), parameter :: width = 3
  INTEGER (kind=int4) :: nx, ny, iline, ielem, astatus
  INTEGER (kind=int4) :: jm, jp, im, ip, dpix, arr_size
  REAL (kind=real4), dimension(:,:), pointer :: arr
  
  dpix = (width - 1)/2
  arr_size = width*width
  
  nx = size(image,1)
  ny = size(image,2)
  
  allocate(arr(nx,ny),stat=astatus)
  if (astatus /= 0) then
    print "(a,'Not enough memory to allocate temporary array in median_filter.')",EXE_PROMPT
    stop
  endif
  
  arr = image
  
  line_loop: do iline=1, ny
    
    jm = iline - dpix
    jp = iline + dpix
    
    if (jm < 1 .or. jp > ny) cycle
    
    element_loop: do ielem=1, nx
    
      if (mask(ielem,iline) == sym%YES) cycle
    
      im = ielem - dpix
      ip = ielem + dpix
      
      if (im < 1 .or. ip > nx) cycle
      
      call opt_median9_float32(reshape(image(im:ip,jm:jp),(/arr_size/)), arr(ielem,iline))
    
    end do element_loop
  end do line_loop
  
  image = arr
  
  deallocate(arr,stat=astatus)
  if (astatus /= 0) then
    print "(a,'Error deallocating temporary array in median_filter.')",EXE_PROMPT
    stop
  endif
  
END SUBROUTINE median_filter9_float32

!-----------------------------------------------------------------
! FUNCTION great_circle(lon0, lat0, lon1, lat1) result(distance)
!
! Calculate the great circle distance on the Earth;s sphere
! in meters given a pair of latitudes and longitudes in degrees.
!-----------------------------------------------------------------

FUNCTION great_circle(lon0, lat0, lon1, lat1) result(distance)
  
  REAL(kind=real4), intent(in) :: lon0
  REAL(kind=real4), intent(in) :: lat0
  REAL(kind=real4), intent(in) :: lon1
  REAL(kind=real4), intent(in) :: lat1
  
  REAL(kind=real4) :: distance
  
  REAL(kind=real4) :: coslt1
  REAL(kind=real4) :: sinlt1
  REAL(kind=real4) :: coslt0
  REAL(kind=real4) :: sinlt0
  REAL(kind=real4) :: cosl0l1
  REAL(kind=real4) :: sinl0l1
  REAL(kind=real4) :: cosc
  REAL(kind=real4) :: sinc
  REAL(kind=real4) :: cosaz
  REAL(kind=real4) :: sinaz
  
  CHARACTER(*), parameter :: routine_name = "great_circle"
  
  if (lat0 > 90.0 .or. lat0 < -90.0 .or. &
      lat1 > 90.0 .or. lat1 < -90.0) then
    print*,"Invalid latitude",lat0,lat1
    distance = huge(distance)
    return
  endif
  
  coslt1 = cos(DTOR*lat1)
  sinlt1 = sin(DTOR*lat1)
  coslt0 = cos(DTOR*lat0)
  sinlt0 = sin(DTOR*lat0)

  cosl0l1 = cos(DTOR*(lon1-lon0))
  sinl0l1 = sin(DTOR*(lon1-lon0))

  cosc = sinlt0 * sinlt1 + coslt0 * coslt1 * cosl0l1 !Cos of angle between pnts
  ! Avoid roundoff problems by clamping cosine range to [-1,1].
  cosc = max(min(cosc,1.0),-1.0)
  sinc = sqrt(1.0 - cosc**2)

  if (abs(sinc) > 1.0e-7) then !Small angle?
    cosaz = (coslt0 * sinlt1 - sinlt0*coslt1*cosl0l1) / sinc !Azmuith
    sinaz = sinl0l1*coslt1/sinc
  else		!Its antipodal
    cosaz = 1.0
    sinaz = 0.0
  endif
  
  distance = acos(cosc) * R_EARTH
  
  return

END FUNCTION great_circle

!-----------------------------------------------------------------
! FUNCTION polynomial_float32(xval, coeffs, n_degrees) result(yval)
!
! Evaluate a polynomial function of a float 32 variable.
!-----------------------------------------------------------------

FUNCTION polynomial_float32(xval, coeffs, n_degrees) result(yval)
  
  REAL(kind=real4), intent(in) :: xval
  REAL(kind=real4), dimension(:), intent(in) :: coeffs
  INTEGER(kind=int4), intent(in) :: n_degrees  
  REAL(kind=real4) :: yval
  INTEGER(kind=int4) :: ideg
  
  yval = coeffs(1)
  do ideg=1, n_degrees
    yval = yval + coeffs(ideg+1)*(xval**ideg)
  end do
  
  return

END FUNCTION polynomial_float32

!-----------------------------------------------------------------
! FUNCTION polynomial_float64(xval, coeffs, n_degrees) result(yval)
!
! Evaluate a polynomial function of a float 64 variable.
!-----------------------------------------------------------------

FUNCTION polynomial_float64(xval, coeffs, n_degrees) result(yval)
  
  REAL(kind=real8), intent(in) :: xval
  REAL(kind=real8), dimension(:), intent(in) :: coeffs
  INTEGER(kind=int4), intent(in) :: n_degrees  
  REAL(kind=real8) :: yval
  INTEGER(kind=int4) :: ideg
  
  yval = coeffs(1)
  do ideg=1, n_degrees
    yval = yval + coeffs(ideg+1)*(xval**ideg)
  end do
  
  return

END FUNCTION polynomial_float64

!-----------------------------------------------------------------
! FUNCTION lognormal_mode_radius(reff, sigma) result(rmod)
!
! Calculate the mean radius of a lognormal distribution given
! an effective radius
!-----------------------------------------------------------------

FUNCTION lognormal_mode_radius(reff, sigma) result(rmod)
  
  REAL(kind=real4), intent(in) :: reff
  REAL(kind=real4), intent(in) :: sigma
  REAL(kind=real4) :: rmod
  REAL(kind=real4) :: exp_term
  
  exp_term = exp(2.5*sigma*sigma)
  rmod = reff/exp_term
  
  return

END FUNCTION lognormal_mode_radius

!-----------------------------------------------------------------
! FUNCTION lognormal_dist(r, rmod, sigma, num_den, normalize) result(status)
!
! Function that evaluates the lognormal distribution for a given
! radius, total number concentration, mean, and standard deviation.
!-----------------------------------------------------------------

FUNCTION lognormal_dist(r, rmod, sigma, num_den, normalize) result(status)

  REAL(kind=real4), dimension(:), intent(in) :: r
  REAL(kind=real4), intent(in) :: rmod
  REAL(kind=real4), intent(in) :: sigma
  REAL(kind=real4), dimension(:), intent(inout) :: num_den
  INTEGER(kind=int4), intent(in), optional :: normalize
  INTEGER(kind=int4) :: status
  REAL(kind=real4) :: delr
  INTEGER(kind=int4) :: npts
  INTEGER(kind=int4) :: index
  REAL(kind=real4) :: sum
  
  CHARACTER(*), parameter :: routine_name = "lognormal_dist"
  
  status = sym%SUCCESS
  
  npts = size(r)
  
  if (sigma <= 0.0) then
    print*,"sigma in lognormal_dist() must be larger than 0.0"
    status = sym%FAILURE
    return
  endif
  
  if (rmod <= 0) then
    print*,"rmode in lognormal_dist() must be larger than 0.0"
    status = sym%FAILURE
    return
  endif
  
  if (npts < 3) then
    print*,"less than 3 radii were passed into lognormal_dist()"
    status = sym%FAILURE
    return
  endif

  !Quick check to make sure the spacing between the radii are equal */
  delr = r(2) - r(1)
  do index = 2, npts
    if (abs(delr - (r(index)-r(index-1))) > 1e-5) then
      print*,"the delta between the radii was not constant in lognormal_dist()"
      status = sym%FAILURE
      return
    endif
   end do

  sum = 0.0
  
  !# of particles per size bin [um^(-1)]
  
  do index=1, npts
    num_den(index) = (1.0 / (sqrt(2.0*PI)*sigma*r(index))) * &
    			exp(-0.5 * (alog(r(index)/rmod) / sigma)**2)
    sum = sum + num_den(index)
  end do
  
  if (present(normalize)) then
    do index = 1, npts
      num_den(index) = num_den(index)/(sum*delr)
    end do
  endif
  
  return

END FUNCTION lognormal_dist

!-----------------------------------------------------------------
! FUNCTION optical_depth_to_num_den(tau, sigma_ext, dz_meters) result(N)
!
! Convert optical depth to number density in #/um^3
!-----------------------------------------------------------------

FUNCTION optical_depth_to_num_den(tau, sigma_ext, dz_meters) result(N)

  REAL(kind=real4), intent(in) :: tau
  REAL(kind=real4), intent(in) :: sigma_ext
  REAL(kind=real4), intent(in) :: dz_meters
  REAL(kind=real4) :: N
  REAL(kind=real4) :: dz_micron
  
  if (tau <= 0.0 .or. sigma_ext <= 0.0 .or. dz_meters <= 0.0) then
    N = missing_value_real4
    return
  endif
  
  dz_micron = dz_meters*1.0e6
  
  N = tau/(sigma_ext*dz_micron)
  
  return

END FUNCTION optical_depth_to_num_den

!-----------------------------------------------------------------
! FUNCTION calculate_mass_loading(tau, sigma_ext, r, num_den, 
!                                 particle_density) result(mass_loading)
!
! Calculate mass loading in kg/m^2 given reff, sigma_ext, tau,
! sigma (of distribution), and density of particle of interest.
! This assumes spherical particles.
!-----------------------------------------------------------------

FUNCTION calculate_mass_loading(tau, sigma_ext, &
                                r, num_den, particle_density) result(mass_loading)

  REAL(kind=real4), intent(in) :: tau
  REAL(kind=real4), intent(in) :: sigma_ext
  REAL(kind=real4), dimension(:), intent(in) :: r
  REAL(kind=real4), dimension(:), intent(in) :: num_den
  REAL(kind=real4), intent(in) :: particle_density
  REAL(kind=real4) :: mass_loading
  REAL(kind=real4) :: Ntot
  INTEGER(kind=int4) :: nr
  REAL(Kind=real4) :: delr
  INTEGER(kind=int4) :: index
  
  nr = size(r)
  
  Ntot = tau/sigma_ext
  delr = r(2) - r(1)
  
  mass_loading = 0.0
  
  do index=1, nr
    mass_loading = mass_loading + (4.0/3.0)*PI*(r(index)**3)*num_den(index)*Ntot*delr
  end do
  
  !tons/m^2
  mass_loading = mass_loading * particle_density * 1.0e-6
  
  return

END FUNCTION calculate_mass_loading

!-----------------------------------------------------------------
! SUBROUTINE pix_sort_int8(a, b)
!-----------------------------------------------------------------

SUBROUTINE pix_sort_int8(a, b)
  INTEGER(kind=int1), INTENT(inout) :: a
  INTEGER(kind=int1), INTENT(inout) :: b
  
  if (a > b) then
    call pix_swap(a,b)
  endif

END SUBROUTINE pix_sort_int8

!-----------------------------------------------------------------
! SUBROUTINE pix_swap_int8(a,b)
!-----------------------------------------------------------------

SUBROUTINE pix_swap_int8(a,b)
  INTEGER(kind=int1), INTENT(inout) :: a
  INTEGER(kind=int1), INTENT(inout) :: b
  INTEGER(kind=int1) :: temp
  
  temp = a
  a = b
  b = temp
  
END SUBROUTINE pix_swap_int8

!-----------------------------------------------------------------
! SUBROUTINE pix_sort_float32(a, b)
!-----------------------------------------------------------------

SUBROUTINE pix_sort_float32(a, b)
  REAL(kind=real4), INTENT(inout) :: a
  REAL(kind=real4), INTENT(inout) :: b
  
  if (a > b) then
    call pix_swap(a,b)
  endif

END SUBROUTINE pix_sort_float32

!-----------------------------------------------------------------
! SUBROUTINE pix_swap_float32(a,b)
!-----------------------------------------------------------------

SUBROUTINE pix_swap_float32(a,b)
  REAL(kind=real4), INTENT(inout) :: a
  REAL(kind=real4), INTENT(inout) :: b
  REAL(kind=real4) :: temp
  
  temp = a
  a = b
  b = temp
  
END SUBROUTINE pix_swap_float32

!-----------------------------------------------------------------
! SUBROUTINE opt_median9_int8(p_in, median)
!-----------------------------------------------------------------

SUBROUTINE opt_median9_int8(p_in, median)
  INTEGER(kind=int1), INTENT(in), DIMENSION(9) :: p_in
  INTEGER(kind=int1), INTENT(inout) :: median
  INTEGER(kind=int1), DIMENSION(9) :: p
  
  p = p_in
  
  call PIX_SORT(p(2), p(3))
  call PIX_SORT(p(5), p(6))
  call PIX_SORT(p(8), p(9))
  call PIX_SORT(p(1), p(2))
  call PIX_SORT(p(4), p(5))
  call PIX_SORT(p(7), p(8))
  call PIX_SORT(p(2), p(3))
  call PIX_SORT(p(5), p(6))
  call PIX_SORT(p(8), p(9))
  call PIX_SORT(p(1), p(4))
  call PIX_SORT(p(6), p(9))
  call PIX_SORT(p(5), p(8))
  call PIX_SORT(p(4), p(7))
  call PIX_SORT(p(2), p(5))
  call PIX_SORT(p(3), p(6))
  call PIX_SORT(p(5), p(8))
  call PIX_SORT(p(5), p(3))
  call PIX_SORT(p(7), p(5))
  call PIX_SORT(p(5), p(3)) 
    
  median = p(5)

END SUBROUTINE opt_median9_int8

!-----------------------------------------------------------------
! SUBROUTINE opt_median9_float32(p_in, median)
!-----------------------------------------------------------------

SUBROUTINE opt_median9_float32(p_in, median)
  REAL(kind=real4), INTENT(in), DIMENSION(9) :: p_in
  REAL(kind=real4), INTENT(inout) :: median
  REAL(kind=real4), DIMENSION(9) :: p
  
  p = p_in
  
  call PIX_SORT(p(2), p(3))
  call PIX_SORT(p(5), p(6))
  call PIX_SORT(p(8), p(9))
  call PIX_SORT(p(1), p(2))
  call PIX_SORT(p(4), p(5))
  call PIX_SORT(p(7), p(8))
  call PIX_SORT(p(2), p(3))
  call PIX_SORT(p(5), p(6))
  call PIX_SORT(p(8), p(9))
  call PIX_SORT(p(1), p(4))
  call PIX_SORT(p(6), p(9))
  call PIX_SORT(p(5), p(8))
  call PIX_SORT(p(4), p(7))
  call PIX_SORT(p(2), p(5))
  call PIX_SORT(p(3), p(6))
  call PIX_SORT(p(5), p(8))
  call PIX_SORT(p(5), p(3))
  call PIX_SORT(p(7), p(5))
  call PIX_SORT(p(5), p(3)) 
    
  median = p(5)

END SUBROUTINE opt_median9_float32



!---------------------------------------------------------------------
! FUNCTION leap_year_fct(yyyy) result(leap_flg)
!
! Function to determine if an input year is a leap year.
!---------------------------------------------------------------------

FUNCTION leap_year_fct(yyyy) result(leap_flg)
  INTEGER, intent(in) :: yyyy
  INTEGER :: leap_flg

  leap_flg = 0

  if ((modulo(yyyy,4) == 0 .and. modulo(yyyy,100) /= 0) .or. &
       modulo(yyyy,400) == 0) leap_flg = 1

  return

END FUNCTION leap_year_fct

!-----------------------------------------------------------------
! FUNCTION get_irregular_lut_index(data, data_bin_start_lut, 
!                                  data_nbins_lut) result(index)
!
! Find a particular data bin in an irregularly spaced look-up table.
!-----------------------------------------------------------------

FUNCTION get_irregular_lut_index(data, data_bin_start_lut, data_nbins_lut) result(index)

  REAL(KIND=real4), INTENT(IN) :: data
  REAL(KIND=real4), DIMENSION(:), INTENT(IN) :: data_bin_start_lut
  INTEGER(KIND=int4), INTENT(IN) :: data_nbins_lut
  INTEGER(KIND=int4) :: index
  
  CALL locate(data_bin_start_lut, data_nbins_lut, data, index)
  index = min(max(index, 1), data_nbins_lut)
  
  return

END FUNCTION get_irregular_lut_index

!-----------------------------------------------------------------
! FUNCTION get_regular_lut_index(data, data_min_lut, 
!                                data_increment_lut, 
!                                data_nbins_lut) result(index)
!
! Find a particular data bin in a regularly spaced look-up table.
!-----------------------------------------------------------------

FUNCTION get_regular_lut_index(data, data_min_lut, data_increment_lut, data_nbins_lut) result(index)

  REAL(KIND=real4), INTENT(IN) :: data
  REAL(KIND=real4), INTENT(IN) :: data_min_lut
  REAL(KIND=real4), INTENT(IN) :: data_increment_lut
  INTEGER(KIND=int4), INTENT(IN) :: data_nbins_lut
  INTEGER(KIND=int4) :: index
  
  index = int((data - data_min_lut) / data_increment_lut, kind=int4) + 1
  index = min(max(index, 1), data_nbins_lut)
  
  return

END FUNCTION get_regular_lut_index
!-----------------------------------------------------------------
! SUBROUTINE get_interpolation_weight(count, value, data, weight, index)
!
! Get interpolation weight.
!-----------------------------------------------------------------

SUBROUTINE get_interpolation_weight(count, value, data, weight, index)
  
  INTEGER(KIND=int4), INTENT(IN) :: count
  REAL(KIND=real4), INTENT(IN) :: value
  REAL(KIND=real4), DIMENSION(:), INTENT(IN) :: data
  REAL(KIND=real4), INTENT(OUT) :: weight
  INTEGER(KIND=int4), INTENT(OUT) :: index
  
  INTEGER(KIND=int4) :: index2
  
  call locate(data, count, value, index)
  
  index = max(1,min(count,index))
  index2 = max(1,min(count,index + 1))
        
  weight = (value - data(index))/(data(index2) - data(index))
  
END SUBROUTINE get_interpolation_weight

!-----------------------------------------------------------------
! FUNCTION get_interpolated_value_using_weight(index, weight, count, 
!                                              data) result(interp_value)
!
! Given a starting index and weight perform linear interpolation.
!-----------------------------------------------------------------

FUNCTION get_interpolated_value_using_weight(index, weight, count, data) result(interp_value)
  
  INTEGER(KIND=int4), INTENT(IN) :: index
  REAL(KIND=real4), INTENT(IN) :: weight
  INTEGER(KIND=int4), INTENT(IN) :: count
  REAL(KIND=real4), DIMENSION(:), INTENT(IN) :: data
  
  INTEGER(KIND=int4) :: index1
  INTEGER(KIND=int4) :: index2
  REAL(KIND=real4) :: interp_value
  
  interp_value = missing_value_real4
  
  index1 = max(1,min(count,index))
  index2 = max(1,min(count,index+1))
  
  interp_value = data(index1) + weight*(data(index2) - data(index1))
  
  return
  
END FUNCTION get_interpolated_value_using_weight

!-----------------------------------------------------------------
! FUNCTION get_interpolated_value_using_value(count, data, value) 
!                                             result(interp_value)
!
! Given a starting index and desired value perform linear
! interpolation.
!-----------------------------------------------------------------

FUNCTION get_interpolated_value_using_value(count, data, value) result(interp_value)
  
  INTEGER(KIND=int4), INTENT(IN) :: count
  REAL(KIND=real4), DIMENSION(:), INTENT(IN) :: data
  REAL(KIND=real4), INTENT(IN) :: value
  
  INTEGER(KIND=int4) :: index
  REAL(KIND=real4) :: weight
  REAL(KIND=real4) :: interp_value
  
  call get_interpolation_weight(count, value, data, weight, index)
  interp_value = get_interpolated_value_using_weight(index, weight, count, data)
  
  return
  
END FUNCTION get_interpolated_value_using_value

!-----------------------------------------------------------------
! FUNCTION Sdev_From_Sums_Int32(Sum_X, Sum_X_Sqd, N, Var) RESULT(Sdev)
!
! Calculate the standard deviation according to Wilks Eq. 3.25.
!-----------------------------------------------------------------

FUNCTION Sdev_From_Sums_Int32(Sum_X, Sum_X_Sqd, N, Var) RESULT(Sdev)

  INTEGER(KIND=INT4), INTENT(IN) :: Sum_X
  INTEGER(KIND=INT4), INTENT(IN) :: Sum_X_Sqd
  INTEGER(KIND=INT4), INTENT(IN) :: N
  REAL(KIND=REAL4), INTENT(OUT), OPTIONAL :: Var
  
  REAL(KIND=REAL4) :: N_Real
  REAL(KIND=REAL4) :: Sum_X_Real
  REAL(KIND=REAL4) :: Sum_X_Sqd_Real
  REAL(KIND=REAL4) :: Var_Tmp
  REAL(KIND=REAL4) :: Sdev
  
  Sdev = MISSING_VALUE_REAL4
  Var_Tmp = MISSING_VALUE_REAL4
  
  IF (N > 1) THEN
  
    N_Real = REAL(N, KIND=REAL4)
    Sum_X_Real = REAL(Sum_X, KIND=REAL4)
    Sum_X_Sqd_Real = REAL(Sum_X_Sqd, KIND=REAL4)
  
    Var_Tmp = (Sum_X_Sqd_Real - ((1.0/N_Real)*Sum_X_Real**2))/(N_Real - 1.0)
    IF (Var_Tmp > 0.0) Sdev = SQRT(Var_Tmp)
    
  ENDIF
  
  IF (PRESENT(Var)) Var = Var_Tmp
  
  RETURN
  
END FUNCTION Sdev_From_Sums_Int32

!-----------------------------------------------------------------
! FUNCTION Sdev_From_Sums_Float32(Sum_X, Sum_X_Sqd, N, Var) RESULT(Sdev)
!
! Calculate the standard deviation according to Wilks Eq. 3.25.
!-----------------------------------------------------------------

FUNCTION Sdev_From_Sums_Float32(Sum_X, Sum_X_Sqd, N, Var) RESULT(Sdev)

  REAL(KIND=REAL4), INTENT(IN) :: Sum_X
  REAL(KIND=REAL4), INTENT(IN) :: Sum_X_Sqd
  INTEGER(KIND=INT4), INTENT(IN) :: N
  REAL(KIND=REAL4), INTENT(OUT), OPTIONAL :: Var
  
  REAL(KIND=REAL4) :: N_Real
  REAL(KIND=REAL4) :: Var_Tmp
  REAL(KIND=REAL4) :: Sdev
  
  Sdev = MISSING_VALUE_REAL4
  Var_Tmp = MISSING_VALUE_REAL4
  
  IF (N > 1) THEN
  
    N_Real = REAL(N, KIND=REAL4)
  
    Var_Tmp = (Sum_X_Sqd - ((1.0/N_Real)*Sum_X**2))/(N_Real - 1.0)
    IF (Var_Tmp > 0.0) Sdev = SQRT(Var_Tmp)
    
  ENDIF
  
  IF (PRESENT(Var)) Var = Var_Tmp
  
  RETURN
  
END FUNCTION Sdev_From_Sums_Float32

!-----------------------------------------------------------------
! FUNCTION Mean_From_Sum_Int32(Sum_X, N) RESULT(Mean)
!
! Calculate the mean from a sum.
!-----------------------------------------------------------------

FUNCTION Mean_From_Sum_Int32(Sum_X, N) RESULT(Mean)

  INTEGER(KIND=INT4), INTENT(IN) :: Sum_X
  INTEGER(KIND=INT4), INTENT(IN) :: N
  
  REAL(KIND=REAL4) :: N_Real
  REAL(KIND=REAL4) :: Sum_X_Real
  REAL(KIND=REAL4) :: Mean
  
  Mean = MISSING_VALUE_REAL4
  
  IF (N > 0) THEN
    
    N_Real = REAL(N, KIND=REAL4)
    Sum_X_Real = REAL(Sum_X, KIND=REAL4)
    Mean = Sum_X_Real/N_Real
    
  ENDIF
  
  RETURN
  
END FUNCTION Mean_From_Sum_Int32

!-----------------------------------------------------------------
! FUNCTION Mean_From_Sum_Float32(Sum_X, N) RESULT(Mean)
!
! Calculate the mean from a sum.
!-----------------------------------------------------------------

FUNCTION Mean_From_Sum_Float32(Sum_X, N) RESULT(Mean)

  REAL(KIND=REAL4), INTENT(IN) :: Sum_X
  INTEGER(KIND=INT4), INTENT(IN) :: N
  
  REAL(KIND=REAL4) :: N_Real
  REAL(KIND=REAL4) :: Mean
  
  Mean = MISSING_VALUE_REAL4
  
  IF (N > 0) THEN
    
    N_Real = REAL(N, KIND=REAL4)
    Mean = Sum_X/N_Real
    
  ENDIF
  
  RETURN
  
END FUNCTION Mean_From_Sum_Float32

!-----------------------------------------------------------------
! SUBROUTINE Pdf_To_Normalized_Cdf(Pdf, Cdf, Nbins, Total_Count)
!
! Convert a PDF to a CDF.  It is assumed that the PDF is not
! normalized.
!-----------------------------------------------------------------

SUBROUTINE Pdf_To_Normalized_Cdf(Pdf, Cdf, Nbins, Total_Count)

  INTEGER(KIND=INT4), DIMENSION(:), INTENT(IN) :: Pdf
  REAL(KIND=REAL4), DIMENSION(:), INTENT(INOUT) :: Cdf
  INTEGER(KIND=INT4), INTENT(IN) :: Nbins
  INTEGER(KIND=INT4), INTENT(OUT), OPTIONAL :: Total_Count
  
  INTEGER(KIND=INT4), DIMENSION(Nbins) :: Cdf_Counts
  INTEGER(KIND=INT4) :: Index
  INTEGER(KIND=INT4) :: Count
  
  Cdf_Counts = 0
  Cdf = MISSING_VALUE_REAL4
  
  Cdf_Counts(1) = Pdf(1)
  
  DO Index = 2, Nbins
    Cdf_Counts(Index) = Cdf_Counts(Index-1) + Pdf(Index)
  END DO
  
  Count = Cdf_Counts(Nbins)
  
  IF (Count > 0) THEN
    Cdf(1:Nbins) = Cdf_Counts/REAL(Count, KIND=REAL4)
  ENDIF
  
  IF (PRESENT(Total_Count)) Total_Count = Count

END SUBROUTINE Pdf_To_Normalized_Cdf

!-----------------------------------------------------------------
! FUNCTION Normalized_Cdf_To_Tile(Cdf, N, Tile, Nbins, Bin_Min, Bin_Size) 
!                                 RESULT(Tile_Value)
!
! Compute a given percentile value given a normalized CDF.
!-----------------------------------------------------------------

FUNCTION Normalized_Cdf_To_Tile(Cdf, N, Tile, Nbins, Bin_Min, Bin_Size) RESULT(Tile_Value)

  CHARACTER(LEN=*), PARAMETER :: Routine_Name = &
    "NUM_MOD:NORMALIZED_CDF_TO_TILE"
  
  REAL(KIND=REAL4), DIMENSION(:), INTENT(INOUT) :: Cdf
  INTEGER(KIND=INT4), INTENT(IN) :: N
  REAL(KIND=REAL4), INTENT(IN) :: Tile
  INTEGER(KIND=INT4), INTENT(IN) :: Nbins
  REAL(KIND=REAL4), INTENT(IN) :: Bin_Min
  REAL(KIND=REAL4), INTENT(IN) :: Bin_Size
  
  INTEGER(KIND=INT4) :: Index1
  INTEGER(KIND=INT4) :: Index2
  REAL(KIND=REAL4) :: D_Cdf
  REAL(KIND=REAL4) :: Weight
  REAL(KIND=REAL4) :: Bin1
  REAL(KIND=REAL4) :: Bin2
  REAL(KIND=REAL4) :: Tile_Value
  
  Tile_Value = MISSING_VALUE_REAL4
  
  IF (N < 1) RETURN
  
  IF (Tile < 0.0 .OR. Tile > 1.0) THEN
    PRINT*,TRIM(Routine_Name)
    PRINT*,"Percentile values must range from 0.0 to 1.0 - returning missing"
    STOP
  ENDIF
  
  IF (Bin_Size <= 0.0) THEN
    PRINT*,TRIM(Routine_Name)
    PRINT*,"Binsize must be greater than zero - returning missing"
    STOP
  ENDIF
  
  IF (Tile <= Cdf(1)) THEN
    
    Tile_Value = Bin_Min
  
  ELSE IF (Tile >= Cdf(Nbins)) THEN
    
    Tile_Value = Bin_Min + Bin_Size*(Nbins-1)
  
  ELSE
 
    CALL Locate(Cdf(1:Nbins), Nbins, Tile, Index1)
    Index1 = MAX(MIN(Index1, Nbins),1)
    Index2 = MAX(MIN(Index1+1, Nbins),1)
    
    D_Cdf = (Cdf(Index2) - Cdf(Index1))
  
    Weight = 0.0
    IF (D_Cdf > 0.0) THEN
      Weight = (Tile - Cdf(Index1))/D_Cdf
    ENDIF
    
    Bin1 = Bin_Min + Bin_Size*(Index1-1) + Bin_Size/2.0
    Bin2 = Bin_Min + Bin_Size*(Index2-1) + Bin_Size/2.0
    
    Tile_Value = Bin1 + Weight*(Bin2 - Bin1)
  
  ENDIF
  
  RETURN

END FUNCTION Normalized_Cdf_To_Tile

!-----------------------------------------------------------------
! FUNCTION Bytescale_Value(Value, Data_Min, Data_Max, Negative) RESULT(Scaled_Value)
!
! Function used to scale a real4 number that ranges from min to max
! into a range bound by 0 to 255.  Modeled after
! the IDL routine bytscl.
!-----------------------------------------------------------------

FUNCTION Bytescale_Value(Value, Data_Min, Data_Max, Negative) RESULT(Scaled_Value)

  CHARACTER(LEN=*), PARAMETER :: Routine_Name = &
    "NUM_MOD:BYTESCALE_VALUE"
  
  REAL(KIND=Real4), INTENT(IN) :: Value
  REAL(KIND=Real4), INTENT(IN) :: Data_Min
  REAL(KIND=Real4), INTENT(IN) :: Data_Max
  INTEGER(KIND=INT4), INTENT(IN), OPTIONAL :: Negative
  
  INTEGER(KIND=Int2) :: Scaled_Value
  INTEGER(KIND=Int2), PARAMETER :: Top = 255
  
  IF (Data_Max <= Data_Min) THEN
    PRINT*,TRIM(Routine_Name)
    PRINT*,"Data max must be greater than data min"
    STOP
  ENDIF
  
  IF (Value < Data_Min) THEN
  
    Scaled_Value = 0
    
  ELSE IF (Value > Data_Max) THEN
  
    Scaled_Value = Top
    
  ELSE
  
    Scaled_Value = INT((Top+0.9999)*(Value-Data_Min)/(Data_Max-Data_Min),KIND=INT1)
    
  ENDIF
  
  IF (PRESENT(Negative)) THEN
    Scaled_Value = Top - Scaled_Value
  ENDIF
  
  RETURN

END FUNCTION Bytescale_Value

end module NUMERICAL_ROUTINES
