!$Id: nwp_common.f90,v 1.22 2010/09/23 21:49:39 mfoster Exp $
!-----------------------------------------------------------------------
!@***h* PATMOSX/nwp_common.f90
! 
! NAME
!   nwp_common.f90
! 
! LOCATION
!   /patmosx/src/nwp_common.f90
! 
! PURPOSE
!   This module holds the radiative transfer quantities needed for
!   the algorithms
! 
! DESCRIPTION
!   This module holds the radiative transfer quantities needed for
!   the algorithms
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!   note, there two type of nwp data
!    1- the pressure level data
!    2- the data on different surface grid
!
!    the only data assumed to be a on the surface grid are
!      - surface temperature
!      - weasd depth
!      - u and v wind speed at 10m
!
!   the surface and pressure level grid may be different
!   i_nwp, j_nwp points to a cell in the pressure level data
!
!   In the GFS data, the pressure and surface grids are the same, in the
!   NCEP reanalysis, they differ
!
! EXTERNALS
!   public::
!   CREATE_NWP_ARRAYS
!   DESTROY_NWP_ARRAYS
!   FIND_NWP_GRID_CELL
!   MAP_PIXEL_NWP
!   KNOWING_P_COMPUTE_T_Z_NWP
!   KNOWING_Z_COMPUTE_T_P_NWP
!   KNOWING_T_COMPUTE_P_Z_NWP
!   FIND_NWP_LEVELS
!   INTERPOLATE_NWP
!   INTERPOLATE_PROFILE
!   INTERPOLATE_NWP_TZ_PROFILES
!   COMPUTE_COAST_MASK_NWP
!   QC_NWP
!   COMPUTE_NWP_CLOUD_PARAMETERS
!   COMPUTE_TSFC_NWP
!   PROF_LOOKUP_USING_P
!   PROF_LOOKUP_USING_T
!   PROF_LOOKUP_USING_T_LAPSE
!   PROF_LOOKUP_USING_T_PROF
!   PROF_LOOKUP_USING_Z
!   TEMPORAL_INTERP_TMPSFC_NWP
!
!@*****
!------------------------------------------------------------------------
!  MODULE name : NWP_COMMON
!
!  This module holds the radiative transfer quantities needed for
!  the algorithms
!
! note, there two type of nwp data
!  1- the pressure level data
!  2- the data on different surface grid
!
!  the only data assumed to be a on the surface grid are
!    - surface temperature
!    - weasd depth
!    - u and v wind speed at 10m
!
! the surface and pressure level grid may be different
! i_nwp, j_nwp points to a cell in the pressure level data
!
! In the GFS data, the pressure and surface grids are the same, in the
! NCEP reanalysis, they differ.

! ROUTINES:
!
! public::
!   CREATE_NWP_ARRAYS
!   DESTROY_NWP_ARRAYS
!   FIND_NWP_GRID_CELL
!   MAP_PIXEL_NWP
!   KNOWING_P_COMPUTE_T_Z_NWP
!   KNOWING_Z_COMPUTE_T_P_NWP
!   KNOWING_T_COMPUTE_P_Z_NWP
!   FIND_NWP_LEVELS
!   INTERPOLATE_NWP
!   INTERPOLATE_PROFILE
!   INTERPOLATE_NWP_TZ_PROFILES
!   COMPUTE_COAST_MASK_NWP
!   QC_NWP
!   COMPUTE_NWP_CLOUD_PARAMETERS
!   COMPUTE_TSFC_NWP
!   PROF_LOOKUP_USING_P
!   PROF_LOOKUP_USING_T
!   PROF_LOOKUP_USING_T_LAPSE
!   PROF_LOOKUP_USING_T_PROF
!   PROF_LOOKUP_USING_Z
!   TEMPORAL_INTERP_TMPSFC_NWP
!
!-----------------------------------------------------------------------
module NWP_COMMON

  use CONSTANTS
  use AVHRR_PIXEL_COMMON
  use NUMERICAL_ROUTINES

  implicit none

  private

  public:: CREATE_NWP_ARRAYS,  &
           DESTROY_NWP_ARRAYS,  &
           FIND_NWP_GRID_CELL,  &
           MAP_PIXEL_NWP, &
           KNOWING_P_COMPUTE_T_Z_NWP,  &
           KNOWING_Z_COMPUTE_T_P_NWP,  &
           KNOWING_T_COMPUTE_P_Z_NWP,  &
           FIND_NWP_LEVELS, &
           INTERPOLATE_NWP,  &
           INTERPOLATE_PROFILE,  &
           INTERPOLATE_NWP_TZ_PROFILES,  &
           COMPUTE_COAST_MASK_NWP, &
           QC_NWP, &
           COMPUTE_NWP_CLOUD_PARAMETERS, &
           COMPUTE_TSFC_NWP, &
           PROF_LOOKUP_USING_P, &
           PROF_LOOKUP_USING_T, &
           PROF_LOOKUP_USING_T_LAPSE, &
           PROF_LOOKUP_USING_T_PROF, &
           PROF_LOOKUP_USING_Z, &
           TEMPORAL_INTERP_TMPSFC_NWP
!----------------------------------------------------------------------
!--- set this parameter to 1 when reading GFS hdf files that have
!--- x as the first index, not z
!----------------------------------------------------------------------
  integer, public, save:: REFORMAT_GFS_ZXY

! NWP array declarations
  integer (kind=int4), save, public :: npoints, nlevels_nwp
  integer (kind=int4), save, public :: nlat_nwp, nlon_nwp
  real (kind=real4),   save, public :: dlat_nwp, dlon_nwp
  real (kind=real4), public :: lat1_nwp
  real (kind=real4), public :: lon1_nwp
  real (kind=real4), save, public :: missing_nwp
  real, public, parameter :: psfc_max_nwp = 1100.0
  real (kind=real8), public, save :: ncep_time_before
  real (kind=real8), public, save :: ncep_time_after
  integer,           dimension(:,:), allocatable, public, save :: mask_nwp
  integer,           dimension(:,:), allocatable, public, save :: bad_nwp_mask
  integer,           dimension(:,:), allocatable, public, save :: sfc_type_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: satzen_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: solzen_nwp

  real (kind=real4), dimension(:,:), allocatable, public, save :: psfc_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: zsfc_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: tmpsfc_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: tmpsfc_nwp_before
  real (kind=real4), dimension(:,:), allocatable, public, save :: tmpsfc_nwp_after
  real (kind=real4), dimension(:,:), allocatable, public, save :: tmpair_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: tmpair_uni_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: tmpsfc_uni_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: t_trop_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: p_trop_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: rhsfc_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: tpw_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: uth_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: hght500_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: ozone_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: weasd_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: u_wnd_10m_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: v_wnd_10m_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: wnd_spd_10m_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: wnd_dir_10m_nwp
  integer (kind=int1), dimension(:,:), allocatable, public, save :: land_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: ice_nwp
  integer (kind=int4), dimension(:,:), allocatable, public, save :: tropo_level_nwp
  integer (kind=int4), dimension(:,:), allocatable, public, save :: level850_nwp
  integer (kind=int4), dimension(:,:), allocatable, public, save :: sfc_level_nwp
  integer (kind=int4), dimension(:,:), allocatable, public, save :: inversion_level_nwp
  integer (kind=int4), dimension(:,:), allocatable, public, save :: strato_level_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: Pc_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: lwp_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: iwp_nwp
  real (kind=real4), dimension(:,:), allocatable, public, save :: ncld_layers_nwp
  real (kind=real4), dimension(:), allocatable, public, save :: lat_nwp
  real (kind=real4), dimension(:), allocatable, public, save :: lon_nwp
  real (kind=real4), dimension(:), allocatable, target, public, save :: P_std_nwp
  real (kind=real4), dimension(:,:,:), allocatable, target, save, public :: Z_prof_nwp
  real (kind=real4), dimension(:,:,:), allocatable, target, save, public :: T_prof_nwp
  real (kind=real4), dimension(:,:,:), allocatable, save, public :: rh_prof_nwp
  real (kind=real4), dimension(:,:,:), allocatable, save, public :: ozone_prof_nwp
  real (kind=real4), dimension(:,:,:), allocatable, target, save, public :: tpw_prof_nwp
  real (kind=real4), dimension(:,:,:), allocatable, target, save, public :: clwmr_prof_nwp
  real (kind=real4), dimension(:,:), allocatable, save, public :: temp2d_nwp_1
  real (kind=real4), dimension(:,:), allocatable, save, public :: temp2d_nwp_2
  real (kind=real4), dimension(:,:,:), allocatable, save, public :: temp3d_nwp_1
  real (kind=real4), dimension(:,:,:), allocatable, save, public :: temp3d_nwp_2
  real (kind=real4), dimension(:,:,:), allocatable, save, public :: temp3d

  integer(kind=int4), save, public:: nwp_start_hour
  integer(kind=int4), save, public:: nwp_end_hour

!--- nwp profiles interpolated to the pixel level
  real (kind=real4), dimension(:), allocatable, public, save :: t_prof_nwp_pix
  real (kind=real4), dimension(:), allocatable, public, save :: z_prof_nwp_pix

!--- local parameters
  real(kind=real4), public, parameter :: p_trop_max = 300.0
  real(kind=real4), public, parameter :: p_trop_min = 25.0
  real(kind=real4), public, parameter :: p_inversion_min = 700.0
  real(kind=real4), public, parameter :: delta_t_inversion = 0.5

contains
!-------------------------------------------------------------
! subroutine QC_NWP()
!
! Subroutine to quality control NWP data
!
! Check the values of some fields and set bad_nwp_mask
! accordingly
!-------------------------------------------------------------
 subroutine QC_NWP()

  integer:: ilon, ilat

  
  do ilon = 1, nlon_nwp
     do ilat = 1, nlat_nwp

        if ((p_trop_nwp(ilon,ilat) <= 0.0) .or. &
            (t_trop_nwp(ilon,ilat) <= 0.0) .or. &
            (tmpsfc_nwp(ilon,ilat) <= 0.0)) then

            bad_nwp_mask(ilon,ilat) = sym%YES

        endif

     enddo
  enddo

end subroutine QC_NWP

!----------------------------------------------------------------------
! subroutine COMPUTE_TSFC_NWP(i1,nx,j1,ny,smooth_nwp)
!
! compute a pixel level surface temperature from the NWP fields
! and smooth if option chosen
!
! i1 = first element index
! nx = number of element indices
! j1 = first element index
! ny = number of element indices
! smooth_nwp = flag to smooth nwp
!
! This must be called after MAP_PIXEL_NWP
!----------------------------------------------------------------------
subroutine COMPUTE_TSFC_NWP(i1,nx,j1,ny,smooth_nwp)

  integer(kind=int4), intent(in):: i1
  integer(kind=int4), intent(in):: nx
  integer(kind=int4), intent(in):: j1
  integer(kind=int4), intent(in):: ny
  integer(kind=int4), intent(in):: smooth_nwp
  integer(kind=int4) :: i2
  integer(kind=int4) :: j2
  integer(kind=int4) :: ielem
  integer(kind=int4) :: iline

  i2 = i1 + nx - 1
  j2 = j1 + ny - 1

  do ielem = i1, i2
    do iline = j1, j2

     if (bad_pixel_mask(ielem,iline) == sym%YES) then
             tsfc_rtm(ielem,iline) = missing_value_real4
             tpw_rtm(ielem,iline) = missing_value_real4
             cycle
     endif

     !--- if smoothing turned off or in a region of coastal nwp, ignore
     if (smooth_nwp == sym%NO) then

      tsfc_rtm(ielem,iline) = tmpsfc_nwp(i_nwp(ielem,iline),j_nwp(ielem,iline))
      tpw_rtm(ielem,iline) = tpw_nwp(i_nwp(ielem,iline),j_nwp(ielem,iline))

     !--- if not, perform interpolation
     else

      tsfc_rtm(ielem,iline) = INTERPOLATE_NWP( &
                       tmpsfc_nwp(i_nwp(ielem,iline),j_nwp(ielem,iline)), &
                       tmpsfc_nwp(i_nwp_x(ielem,iline),j_nwp(ielem,iline)), &
                       tmpsfc_nwp(i_nwp(ielem,iline),j_nwp_x(ielem,iline)), &
                       tmpsfc_nwp(i_nwp_x(ielem,iline),j_nwp_x(ielem,iline)), &
                       lon_nwp_fac(ielem,iline), &
                       lat_nwp_fac(ielem,iline))

      tpw_rtm(ielem,iline) = INTERPOLATE_NWP( &
                       tpw_nwp(i_nwp(ielem,iline),j_nwp(ielem,iline)), &
                       tpw_nwp(i_nwp_x(ielem,iline),j_nwp(ielem,iline)), &
                       tpw_nwp(i_nwp(ielem,iline),j_nwp_x(ielem,iline)), &
                       tpw_nwp(i_nwp_x(ielem,iline),j_nwp_x(ielem,iline)), &
                       lon_nwp_fac(ielem,iline), &
                       lat_nwp_fac(ielem,iline))

     endif

    enddo
  enddo   

end subroutine COMPUTE_TSFC_NWP

!-------------------------------------------------------------
! subroutine MAP_PIXEL_NWP(j1,j2)
!
! Subroutine to find nwp cell where each in a segment lies
!-------------------------------------------------------------
 subroutine MAP_PIXEL_NWP(j1,j2)

  integer, intent(in):: j1, j2
  integer:: i,j,ierr

 do j = j1,j1+j2-1
     do i = 1, num_pix
                                                                                                                                         
!--- check for bad scans
      if (bad_pixel_mask(i,j) == sym%YES) then
        i_nwp(i,j) = missing_value_int1
        j_nwp(i,j) = missing_value_int1
        i_nwp_x(i,j) = missing_value_int1
        j_nwp_x(i,j) = missing_value_int1
        lon_nwp_fac(i,j) = missing_value_real4
        lat_nwp_fac(i,j) = missing_value_real4
        cycle
      endif
                                                                                                                                         
!--- compute NWP cell to pixel mapping
      call FIND_NWP_GRID_CELL(lon(i,j),lat(i,j),i_nwp(i,j),j_nwp(i,j),i_nwp_x(i,j),j_nwp_x(i,j),  &
                              lon_nwp_fac(i,j), lat_nwp_fac(i,j),ierr)

!-- if there is an error, flag pixel as bad
      if (ierr == 1) then
         bad_pixel_mask(i,j) = sym%YES     !this pixel won't be processed further
         i_nwp(i,j) = missing_value_int1
         j_nwp(i,j) = missing_value_int1
         i_nwp_x(i,j) = missing_value_int1
         j_nwp_x(i,j) = missing_value_int1
         lon_nwp_fac(i,j) = missing_value_real4
         lat_nwp_fac(i,j) = missing_value_real4
      endif

!--- if this cell has bad nwp data, treat its pixels as being bad
      if (bad_nwp_mask(i_nwp(i,j),j_nwp(i,j)) == sym%YES) then
         bad_pixel_mask(i,j) = sym%YES
      endif

     enddo
  enddo

 end subroutine MAP_PIXEL_NWP
!------------------------------------------------------------------
! subroutine FIND_NWP_GRID_CELL(lon, lat, ilon, ilat, ilonx, ilatx, lonfac, latfac, ierror)
!
! Subroutine to convert lat, lon into NWP grid-cell
!
! input:
!   lon - longitude (-180 to 180)
!   lat - latitude (-90 to 90)
! output:
!   ilat - nwp latitude index of the nearest nwp latitude
!   ilon - nwp longitude index of the nearest nwp longitude
!   ilatx - nwp latitude index of the nearest nwp latitude diagnol
!   ilonx - nwp longitude index of the nearest nwp longitude diagnol
!   latfac - latitude weight between ilat(0.0) and ilatx(1.0)
!   lonfac - longitude weight between ilon(0.0) and ilonx(1.0)
!  
!
! imagine a pixel, x, surrounded by nwp vertices (O)
!
!        O            O            O
!                     -   x
!
!        O            O            O
!                                 --- 
!
!  point O is (ilon,ilat) and O is (ilonx,ilatx)
!        -                   ---
!
! modified to return information needed to spatially interpolate
!------------------------------------------------------------------
  subroutine FIND_NWP_GRID_CELL(lon, lat, ilon, ilat, ilonx, ilatx, lonfac, latfac, ierror)

    real (kind=real4), intent(in) :: lon, lat
    integer (kind=int4), intent(out) :: ilon, ilat, ilonx, ilatx, ierror
    real (kind=real4), intent(out) :: latfac
    real (kind=real4), intent(out) :: lonfac

    real (kind=real4) :: rlat
    real (kind=real4) :: rlon
    real (kind=real4) :: rlon_nwp
    real (kind=real4) :: rlon_nwpx
    integer:: Is_Dateline

    ierror = 0
    rlon = lon
    rlat = lat
    Is_Dateline = 0

    !--- convert negative lons to go from 180 to 360 degrees
    if (rlon < 0.0) then
       rlon = rlon + 360.0
    endif

    !--- Find Position in NWP grid
    if (rlon < 0.0 .or. rlon > 360.0 .or. rlat < -90.0 .or. rlat > 90.0) then
       ierror = 1
       ilat = 0
       ilon = 0
    else
       ilat = max(1, min(nlat_nwp, nint( (rlat-lat1_nwp) / dlat_nwp + 1.0) ))
       ilon = max(1, min(nlon_nwp, nint( (rlon-lon1_nwp) / dlon_nwp + 1.0) ))
    endif

    rlon_nwp = lon_nwp(ilon)
    if (lon_nwp(ilon) < 0.0) then
       rlon_nwp = rlon_nwp + 360.0
    endif

    !---  latitude interpolation information
    if (ilat > 1 .and. ilat < nlat_nwp) then
      if (sign(1.0,lat-lat_nwp(ilat)) == sign(1.0,dlat_nwp)) then
         ilatx = ilat + 1
      else
         ilatx = ilat - 1
      endif
      ilatx = min(nlat_nwp,max(1,ilatx))
      latfac = (lat - lat_nwp(ilat))/(lat_nwp(ilatx)-lat_nwp(ilat))
    endif

    !---- determine dateline flag
    if (abs(lon - lon_nwp(ilon)) > abs(dlon_nwp)) then
       Is_Dateline = 1
    endif

    !---  longitude interpolation information
    if (ilon > 1 .and. ilon < nlon_nwp) then
      if (sign(1.0,lon-lon_nwp(ilon)) == sign(1.0,dlon_nwp)) then
          if (Is_Dateline == 0) then
            ilonx = ilon + 1
          else
            ilonx = ilon - 1
          endif
      else
          if (Is_Dateline == 0) then
            ilonx = ilon - 1
          else
            ilonx = ilon + 1
          endif
      endif
    endif
    ilonx = min(nlon_nwp,max(1,ilonx))

    !--- make a positive definite value of lon at ilonx
    rlon_nwpx = lon_nwp(ilonx)
    if (lon_nwp(ilonx) < 0.0) then
       rlon_nwpx = rlon_nwpx + 360.0
    endif

    !--- recompute date line flag including ilonx point
    if (abs(lon - lon_nwp(ilon)) > abs(dlon_nwp)) then
       Is_Dateline = 1
    endif
    if (abs(lon - lon_nwp(ilonx)) > abs(dlon_nwp)) then
       Is_Dateline = 1
    endif

    if (Is_Dateline == 0) then
       lonfac = (lon - lon_nwp(ilon))/(lon_nwp(ilonx)-lon_nwp(ilon))
    else
       lonfac = abs((rlon - rlon_nwp)/(rlon_nwp-rlon_nwpx))
    endif

   !--- constrain
   lonfac = min(0.5,max(0.0,lonfac))
   latfac = min(0.5,max(0.0,latfac))
   ilonx = min(nlon_nwp,max(1,ilonx))
   ilatx = min(nlat_nwp,max(1,ilatx))

  end subroutine FIND_NWP_GRID_CELL


!----------------------------------------------------------------------------
! Function INTERPOLATE_NWP
! 
! general interpolation routine for nwp fields
!
! description of arguments
! ilon, ilat - nwp indices of closest nwp cell
! ilonx,ilatx - nwp indices of nwp cells of diagnoal of bounding box
! lonx - longitude weighting factor 
! latx = latitude weighting factor
! z1 = data(ilon, ilat)
! z2 = data(ilonx,ilat)
! z3 = data(ilon,ilatx)
! z4 = data(ilonx,ilatx)
!---------------------------------------------------------------------------
 function INTERPOLATE_NWP(z1,z2,z3,z4,lonx,latx) result(z)

  real, intent(in):: z1
  real, intent(in):: z2
  real, intent(in):: z3
  real, intent(in):: z4
  real, intent(in):: lonx
  real, intent(in):: latx
  real:: z

  !--- linear inteprpolation scheme
  z =  (1.0-lonx) * ((1.0-latx) * z1 + (latx)* z3) + &
           (lonx) * ((1.0-latx) * z2 + (latx)* z4)

 end function INTERPOLATE_NWP

!---------------------------------------------------------------------------
! subroutine INTERPOLATE_PROFILE(z1,z2,z3,z4,lonx,latx,z)
!---------------------------------------------------------------------------

 subroutine INTERPOLATE_PROFILE(z1,z2,z3,z4,lonx,latx,z)

  real, dimension(:), intent(in):: z1
  real, dimension(:), intent(in):: z2
  real, dimension(:), intent(in):: z3
  real, dimension(:), intent(in):: z4
  real, intent(in):: lonx
  real, intent(in):: latx
  real, dimension(:), intent(out):: z

  !--- linear inteprpolation scheme
  z =  (1.0-lonx) * ((1.0-latx) * z1 + (latx)* z3) + &
           (lonx) * ((1.0-latx) * z2 + (latx)* z4)

 end subroutine INTERPOLATE_PROFILE


!--------------------------------------------------------------------------
! subroutine INTERPOLATE_NWP_TZ_PROFILES(i,j)
!
! spatially interpolate TZ profiles
!
!--------------------------------------------------------------------------
 subroutine INTERPOLATE_NWP_TZ_PROFILES(i,j)

   integer, intent(in):: i, j
   integer:: ilev



   do ilev = 1,nlevels_nwp

      t_prof_nwp_pix(ilev) = INTERPOLATE_NWP(t_prof_nwp(ilev,i_nwp(i,j),j_nwp(i,j)), &
                                             t_prof_nwp(ilev,i_nwp(i,j),j_nwp_x(i,j)), &
                                             t_prof_nwp(ilev,i_nwp_x(i,j),j_nwp(i,j)), &
                                             t_prof_nwp(ilev,i_nwp_x(i,j),j_nwp_x(i,j)), &
                                             lon_nwp_fac(i,j), lat_nwp_fac(i,j))

      z_prof_nwp_pix(ilev) = INTERPOLATE_NWP(z_prof_nwp(ilev,i_nwp(i,j),j_nwp(i,j)), &
                                             z_prof_nwp(ilev,i_nwp(i,j),j_nwp_x(i,j)), &
                                             z_prof_nwp(ilev,i_nwp_x(i,j),j_nwp(i,j)), &
                                             z_prof_nwp(ilev,i_nwp_x(i,j),j_nwp_x(i,j)), &
                                             lon_nwp_fac(i,j), lat_nwp_fac(i,j))
   enddo


 end subroutine INTERPOLATE_NWP_TZ_PROFILES

!-----------------------------------------------------------------------------
! subroutine CREATE_NWP_ARRAYS()
!
! allocate and initialize the memory needed for the nwp arrays
!-----------------------------------------------------------------------------
  subroutine CREATE_NWP_ARRAYS()

    integer:: ilon, ilat, k

!   Allocate arrays

    allocate(mask_nwp(nlon_nwp, nlat_nwp))

    allocate(bad_nwp_mask(nlon_nwp, nlat_nwp))

    allocate(sfc_type_nwp(nlon_nwp, nlat_nwp))
    allocate(satzen_nwp(nlon_nwp, nlat_nwp))
    allocate(solzen_nwp(nlon_nwp, nlat_nwp))

    allocate(psfc_nwp(nlon_nwp, nlat_nwp))
    allocate(zsfc_nwp(nlon_nwp, nlat_nwp))
    allocate(tmpsfc_nwp(nlon_nwp, nlat_nwp))
    allocate(tmpsfc_nwp_before(nlon_nwp, nlat_nwp))
    allocate(tmpsfc_nwp_after(nlon_nwp, nlat_nwp))
    allocate(tmpair_nwp(nlon_nwp, nlat_nwp))
    allocate(tmpair_uni_nwp(nlon_nwp, nlat_nwp))
    allocate(tmpsfc_uni_nwp(nlon_nwp, nlat_nwp))
    allocate(t_trop_nwp(nlon_nwp, nlat_nwp))
    allocate(p_trop_nwp(nlon_nwp, nlat_nwp))
    allocate(rhsfc_nwp(nlon_nwp, nlat_nwp))
    allocate(uth_nwp(nlon_nwp, nlat_nwp))
    allocate(hght500_nwp(nlon_nwp, nlat_nwp))
    allocate(tpw_nwp(nlon_nwp, nlat_nwp))
    allocate(ozone_nwp(nlon_nwp, nlat_nwp))
    allocate(weasd_nwp(nlon_nwp, nlat_nwp))
    allocate(u_wnd_10m_nwp(nlon_nwp, nlat_nwp))
    allocate(v_wnd_10m_nwp(nlon_nwp, nlat_nwp))
    allocate(wnd_spd_10m_nwp(nlon_nwp, nlat_nwp))
    allocate(wnd_dir_10m_nwp(nlon_nwp, nlat_nwp))
    allocate(P_std_nwp(nlevels_nwp))
    allocate(lat_nwp(nlat_nwp))
    allocate(lon_nwp(nlon_nwp))
    allocate(z_prof_nwp(nlevels_nwp, nlon_nwp, nlat_nwp))
    allocate(t_prof_nwp(nlevels_nwp, nlon_nwp, nlat_nwp))
    allocate(ozone_prof_nwp(nlevels_nwp, nlon_nwp, nlat_nwp))
    allocate(rh_prof_nwp(nlevels_nwp, nlon_nwp, nlat_nwp))
    allocate(tpw_prof_nwp(nlevels_nwp, nlon_nwp, nlat_nwp))
    allocate(clwmr_prof_nwp(nlevels_nwp, nlon_nwp, nlat_nwp))
    allocate(land_nwp(nlon_nwp, nlat_nwp))
    allocate(ice_nwp(nlon_nwp, nlat_nwp))
    allocate(sfc_level_nwp(nlon_nwp, nlat_nwp))
    allocate(tropo_level_nwp(nlon_nwp, nlat_nwp))
    allocate(inversion_level_nwp(nlon_nwp, nlat_nwp))
    allocate(strato_level_nwp(nlon_nwp, nlat_nwp))
    allocate(level850_nwp(nlon_nwp, nlat_nwp))
    allocate(Pc_nwp(nlon_nwp, nlat_nwp))
    allocate(lwp_nwp(nlon_nwp, nlat_nwp))
    allocate(iwp_nwp(nlon_nwp, nlat_nwp))
    allocate(ncld_layers_nwp(nlon_nwp, nlat_nwp))

    allocate(temp2d_nwp_1(nlon_nwp, nlat_nwp))
    allocate(temp2d_nwp_2(nlon_nwp, nlat_nwp))
    allocate(temp3d_nwp_1(nlevels_nwp, nlon_nwp, nlat_nwp))
    allocate(temp3d_nwp_2(nlevels_nwp, nlon_nwp, nlat_nwp))

    allocate(t_prof_nwp_pix(nlevels_nwp))
    allocate(z_prof_nwp_pix(nlevels_nwp))

    if (REFORMAT_GFS_ZXY == 1) then
     allocate(temp3d(nlon_nwp, nlat_nwp,nlevels_nwp))
    else
     allocate(temp3d(nlevels_nwp,nlon_nwp, nlat_nwp))
    endif

    mask_nwp = 0
    bad_nwp_mask = sym%NO
    sfc_type_nwp = 0
    satzen_nwp = 0
    solzen_nwp = 0
    psfc_nwp = 0
    zsfc_nwp = 0
    tmpsfc_nwp = 0
    tmpsfc_nwp_before = 0
    tmpsfc_nwp_after = 0
    tmpair_nwp = 0
    tmpair_uni_nwp = 0
    tmpsfc_uni_nwp = 0
    t_trop_nwp = 0
    p_trop_nwp = 0
    rhsfc_nwp = 0
    uth_nwp = 0
    hght500_nwp = 0
    tpw_nwp = 0
    ozone_nwp = 0
    weasd_nwp = 0
    u_wnd_10m_nwp = 0
    v_wnd_10m_nwp = 0
    wnd_spd_10m_nwp = 0
    wnd_dir_10m_nwp = 0
    lon_nwp = 0
    lat_nwp = 0
    P_std_nwp = 0
    z_prof_nwp = 0
    t_prof_nwp = 0
    rh_prof_nwp = 0.0
    ozone_prof_nwp = 0.0
    tpw_prof_nwp = 0.0
    clwmr_prof_nwp = 0.0
    land_nwp = 0
    ice_nwp = 0.0
    temp3d_nwp_1 = 0
    temp3d_nwp_2 = 0
    temp3d = 0
    sfc_level_nwp = 0
    strato_level_nwp = 0
    tropo_level_nwp = 0
    level850_nwp = 0
    inversion_level_nwp = 0
    Pc_nwp = 0.0
    iwp_nwp = 0.0
    lwp_nwp = 0.0
    ncld_layers_nwp = 0.0

    t_prof_nwp_pix = 0
    z_prof_nwp_pix = 0

!--- create nwp lat and lon vectors
    do ilat = 1, nlat_nwp
       lat_nwp(ilat) = lat1_nwp + (ilat-1) * dlat_nwp
    end do

    do ilon = 1, nlon_nwp
       lon_nwp(ilon) = lon1_nwp + (ilon-1) * dlon_nwp
       if (lon_nwp(ilon) > 180.0) then
         lon_nwp(ilon) = lon_nwp(ilon) - 360.0
       endif
    end do

end subroutine CREATE_NWP_ARRAYS


!-----------------------------------------------------------------------------
! subroutine DESTROY_NWP_ARRAYS
!
! deallocate the memory needed for the nwp arrays
!-----------------------------------------------------------------------------
subroutine DESTROY_NWP_ARRAYS

    if (allocated(mask_nwp))          deallocate(mask_nwp)
    if (allocated(bad_nwp_mask))      deallocate(bad_nwp_mask)
    if (allocated(sfc_type_nwp))      deallocate(sfc_type_nwp)
    if (allocated(satzen_nwp))        deallocate(satzen_nwp)
    if (allocated(solzen_nwp))        deallocate(solzen_nwp)

    if (allocated(land_nwp))          deallocate(land_nwp)
    if (allocated(ice_nwp))          deallocate(ice_nwp)
    if (allocated(sfc_level_nwp))     deallocate(sfc_level_nwp)
    if (allocated(tropo_level_nwp))    deallocate(tropo_level_nwp)
    if (allocated(level850_nwp))       deallocate(level850_nwp)
    if (allocated(strato_level_nwp))   deallocate(strato_level_nwp)
    if (allocated(inversion_level_nwp)) deallocate(inversion_level_nwp)
    if (allocated(psfc_nwp))          deallocate(psfc_nwp)
    if (allocated(zsfc_nwp))          deallocate(zsfc_nwp)
    if (allocated(tmpsfc_nwp))        deallocate(tmpsfc_nwp)
    if (allocated(tmpsfc_nwp_before)) deallocate(tmpsfc_nwp_before)
    if (allocated(tmpsfc_nwp_after))  deallocate(tmpsfc_nwp_after)
    if (allocated(tmpair_nwp))        deallocate(tmpair_nwp)
    if (allocated(tmpair_uni_nwp))    deallocate(tmpair_uni_nwp)
    if (allocated(tmpsfc_uni_nwp))    deallocate(tmpsfc_uni_nwp)
    if (allocated(t_trop_nwp))        deallocate(t_trop_nwp)
    if (allocated(p_trop_nwp))        deallocate(p_trop_nwp)
    if (allocated(rhsfc_nwp))         deallocate(rhsfc_nwp)
    if (allocated(hght500_nwp))       deallocate(hght500_nwp)
    if (allocated(uth_nwp))           deallocate(uth_nwp)
    if (allocated(tpw_nwp))           deallocate(tpw_nwp)
    if (allocated(ozone_nwp))         deallocate(ozone_nwp)
    if (allocated(weasd_nwp))         deallocate(weasd_nwp)
    if (allocated(u_wnd_10m_nwp))     deallocate(u_wnd_10m_nwp)
    if (allocated(v_wnd_10m_nwp))     deallocate(v_wnd_10m_nwp)
    if (allocated(wnd_spd_10m_nwp))   deallocate(wnd_spd_10m_nwp)
    if (allocated(wnd_dir_10m_nwp))   deallocate(wnd_dir_10m_nwp)
    if (allocated(lat_nwp))           deallocate(lat_nwp)
    if (allocated(lon_nwp))           deallocate(lon_nwp)
    if (allocated(P_std_nwp))         deallocate(P_std_nwp)
    if (allocated(z_prof_nwp))        deallocate(z_prof_nwp)
    if (allocated(t_prof_nwp))        deallocate(t_prof_nwp)
    if (allocated(rh_prof_nwp))       deallocate(rh_prof_nwp)
    if (allocated(ozone_prof_nwp))    deallocate(ozone_prof_nwp)
    if (allocated(tpw_prof_nwp))      deallocate(tpw_prof_nwp)
    if (allocated(clwmr_prof_nwp))    deallocate(clwmr_prof_nwp)
    if (allocated(temp2d_nwp_1))      deallocate(temp2d_nwp_1)
    if (allocated(temp2d_nwp_2))      deallocate(temp2d_nwp_2)
    if (allocated(temp3d_nwp_1))      deallocate(temp3d_nwp_1)
    if (allocated(temp3d_nwp_2))      deallocate(temp3d_nwp_2)
    if (allocated(temp3d))            deallocate(temp3d)
    if (allocated(t_prof_nwp_pix))    deallocate(t_prof_nwp_pix)
    if (allocated(z_prof_nwp_pix))    deallocate(z_prof_nwp_pix)
    if (allocated(Pc_nwp))            deallocate(Pc_nwp)
    if (allocated(lwp_nwp))           deallocate(lwp_nwp)
    if (allocated(iwp_nwp))           deallocate(iwp_nwp)
    if (allocated(ncld_layers_nwp))   deallocate(ncld_layers_nwp)

  end subroutine DESTROY_NWP_ARRAYS

!----------------------------------------------------------------------
! subroutine KNOWING_P_COMPUTE_T_Z_NWP(inwp,jnwp,P,T,Z,ilev)
!
! derive height and temperature from a profile knowing pressure
!----------------------------------------------------------------------
 subroutine KNOWING_P_COMPUTE_T_Z_NWP(inwp,jnwp,P,T,Z,ilev)
  integer, intent(in):: inwp, jnwp
  real, intent(in):: P
  real, intent(out):: T,Z
  integer, intent(out):: ilev
  real:: dp, dt, dz


!--- interpolate pressure profile
  call LOCATE(P_std_nwp,nlevels_nwp,P,ilev)
  ilev = max(1,min(nlevels_nwp-1,ilev))

  dp = P_std_nwp(ilev+1) - P_std_nwp(ilev)
  dt = T_prof_nwp(ilev+1,inwp,jnwp) - T_prof_nwp(ilev,inwp,jnwp)
  dz = Z_prof_nwp(ilev+1,inwp,jnwp) - z_prof_nwp(ilev,inwp,jnwp)

!--- perform interpolation
  if (dp /= 0.0) then
   T = T_prof_nwp(ilev, inwp, jnwp) + dt/dp * (P - P_std_nwp(ilev))
   Z = Z_prof_nwp(ilev, inwp, jnwp) + dz/dp * (P - P_std_nwp(ilev))
  else
   T = T_prof_nwp(ilev, inwp, jnwp) 
   Z = Z_prof_nwp(ilev, inwp, jnwp)
  endif
 end subroutine KNOWING_P_COMPUTE_T_Z_NWP

!----------------------------------------------------------------------
! subroutine KNOWING_Z_COMPUTE_T_P_NWP(inwp,jnwp,P,T,Z,ilev)
!
! derive pressure and temperature from a profile knowing height
!----------------------------------------------------------------------
 subroutine KNOWING_Z_COMPUTE_T_P_NWP(inwp,jnwp,P,T,Z,ilev)
  integer, intent(in):: inwp, jnwp
  real, intent(in):: Z
  real, intent(out):: T,P
  integer, intent(out):: ilev
  real:: dp
  real:: dt
  real:: dz
  integer:: kstart
  integer:: kend
  integer:: nlevels_temp

!---
    kstart = tropo_level_nwp(inwp,jnwp)
    kend = sfc_level_nwp(inwp,jnwp)
    nlevels_temp = kend - kstart + 1

!--- interpolate pressure profile
  call LOCATE(Z_prof_nwp(kstart:kend,inwp,jnwp),nlevels_temp,Z,ilev)
  ilev = max(1,min(nlevels_temp-1,ilev))
  ilev = ilev + kstart - 1

  dp = P_std_nwp(ilev+1) - P_std_nwp(ilev)
  dt = T_prof_nwp(ilev+1,inwp,jnwp) - T_prof_nwp(ilev,inwp,jnwp)
  dz = Z_prof_nwp(ilev+1,inwp,jnwp) - z_prof_nwp(ilev,inwp,jnwp)

!--- perform interpolation
  if (dp /= 0.0) then
   T = T_prof_nwp(ilev, inwp, jnwp) + dt/dz * (Z - Z_prof_nwp(ilev,inwp,jnwp))
   P = P_std_nwp(ilev) + dp/dz * (Z - Z_prof_nwp(ilev,inwp,jnwp))
  else
   T = T_prof_nwp(ilev, inwp, jnwp) 
   P = P_std_nwp(ilev)
  endif

 end subroutine KNOWING_Z_COMPUTE_T_P_NWP

!----------------------------------------------------------------------------
! subroutine KNOWING_T_COMPUTE_P_Z_NWP(inwp,jnwp,P,T,Z,klev,i,j,ierr)
!
! Interpolate P and Z from a profile knowing T
!
! klev = result is between klev and llev + 1
! 
! note, the highest level allowed is the tropopause
!
!----------------------------------------------------------------------------
 subroutine KNOWING_T_COMPUTE_P_Z_NWP(inwp,jnwp,P,T,Z,klev,i,j,ierr)

  integer, intent(in):: inwp, jnwp
  real, intent(in):: T
  integer, intent(in):: i,j
  real, intent(out):: P,Z
  integer, intent(out):: klev,ierr
  integer:: nlevels_temp,kstart,kend,level_within_inversion_flag
  real:: dp, dt, dz,z1,z2,t1,t2

  ierr = sym%NO
  Z = missing_value_real4
  P = missing_value_real4

!--- test for existence of a valid solution with troposphere
    kstart = tropo_level_nwp(inwp,jnwp)
    kend = sfc_level_nwp(inwp,jnwp)
    nlevels_temp = kend - kstart + 1

!--- interpolate temperature profile

!--- check to see if warmer than max, than assume at surface
  if (T > maxval(T_prof_nwp(kstart:kend,inwp,jnwp))) then
    P = P_std_nwp(kend)
    Z = Z_prof_nwp(kend,inwp,jnwp)
    klev = kend - 1
    ierr = sym%NO
    return
  endif

!--- check to see if colder than min, than assume tropopause
  if (T < minval(T_prof_nwp(kstart:kend,inwp,jnwp))) then
    P = P_std_nwp(kstart)
    Z = Z_prof_nwp(kstart,inwp,jnwp)
    klev = kstart + 1
    ierr = sym%NO
    return
  endif
  
!--- if there is an inversion, look below first
  level_within_inversion_flag = 0
  if (inversion_level_nwp(inwp, jnwp) > 0) then
   kstart = inversion_level_nwp(inwp,jnwp)
   kend = sfc_level_nwp(inwp,jnwp)
   nlevels_temp = kend - kstart + 1
   call LOCATE(T_prof_nwp(kstart:kend,inwp,jnwp),nlevels_temp,T,klev)
   if ((klev > 0) .and. (klev < nlevels_temp -1)) then
     klev = klev + kstart - 1
     level_within_inversion_flag = 1
   endif
  endif

!--- if no solution within an inversion, look above
  if (level_within_inversion_flag == 0) then

    kstart = tropo_level_nwp(inwp,jnwp)
    kend = sfc_level_nwp(inwp,jnwp)
    nlevels_temp = kend - kstart + 1
    call LOCATE(T_prof_nwp(kstart:kend,inwp,jnwp),nlevels_temp,T,klev)
    klev = klev + kstart - 1
    klev = max(1,min(nlevels_nwp-1,klev))

  endif

!-- if solution is above trop, set to trop values
  if (klev < tropo_level_nwp(inwp,jnwp)) then

    P = P_std_nwp(tropo_level_nwp(inwp,jnwp))
    Z = Z_prof_nwp(tropo_level_nwp(inwp,jnwp),inwp,jnwp)

  else
!--- determine derivatives
    dp = P_std_nwp(klev+1) - P_std_nwp(klev)

    if (smooth_nwp == sym%NO) then
     z2 = Z_prof_nwp(klev+1,inwp,jnwp)
     z1 = Z_prof_nwp(klev,inwp,jnwp)
     t2 = T_prof_nwp(klev+1,inwp,jnwp)
     t1 = T_prof_nwp(klev,inwp,jnwp)
    else
     z1 = INTERPOLATE_NWP(z_prof_nwp(klev,i_nwp(i,j),j_nwp(i,j)), &
                          z_prof_nwp(klev,i_nwp_x(i,j),j_nwp(i,j)), &
                          z_prof_nwp(klev,i_nwp(i,j),j_nwp_x(i,j)), &
                          z_prof_nwp(klev,i_nwp_x(i,j),j_nwp_x(i,j)), &
                          lon_nwp_fac(i,j), lat_nwp_fac(i,j))
     z2 = INTERPOLATE_NWP(z_prof_nwp(klev+1,i_nwp(i,j),j_nwp(i,j)), &
                          z_prof_nwp(klev+1,i_nwp_x(i,j),j_nwp(i,j)), &
                          z_prof_nwp(klev+1,i_nwp(i,j),j_nwp_x(i,j)), &
                          z_prof_nwp(klev+1,i_nwp_x(i,j),j_nwp_x(i,j)), &
                          lon_nwp_fac(i,j), lat_nwp_fac(i,j))
     t1 = INTERPOLATE_NWP(t_prof_nwp(klev,i_nwp(i,j),j_nwp(i,j)), &
                          t_prof_nwp(klev,i_nwp_x(i,j),j_nwp(i,j)), &
                          t_prof_nwp(klev,i_nwp(i,j),j_nwp_x(i,j)), &
                          t_prof_nwp(klev,i_nwp_x(i,j),j_nwp_x(i,j)), &
                          lon_nwp_fac(i,j), lat_nwp_fac(i,j))
     t2 = INTERPOLATE_NWP(t_prof_nwp(klev+1,i_nwp(i,j),j_nwp(i,j)), &
                          t_prof_nwp(klev+1,i_nwp_x(i,j),j_nwp(i,j)), &
                          t_prof_nwp(klev+1,i_nwp(i,j),j_nwp_x(i,j)), &
                          t_prof_nwp(klev+1,i_nwp_x(i,j),j_nwp_x(i,j)), &
                          lon_nwp_fac(i,j), lat_nwp_fac(i,j))

    endif

    dz = z2 - z1
    dt = t2 - t1

!--- perform interpolation
    if (dt /= 0.0) then
     P = P_std_nwp(klev) + dp/dt * (T - t1)
     Z = z1 + dz/dt * (T - t1)
    else
     P = P_std_nwp(klev) 
     Z = z1
    endif

    
!--- end above trop check
  endif

 end subroutine KNOWING_T_COMPUTE_P_Z_NWP

!-------------------------------------------------------------
! subroutine FIND_NWP_LEVELS(ilon,ilat)
!
! subroutine to find key levels in the profiles
!-------------------------------------------------------------
subroutine FIND_NWP_LEVELS(ilon,ilat)

 integer (kind=int4), intent(in):: ilon, ilat
 integer (kind=int4):: k
 real:: P_trop_temp

!--- find surface level - this is closest nwp level above the actual surface
   sfc_level_nwp(ilon,ilat) = nlevels_nwp
   do k = nlevels_nwp,1,-1
       if (P_std_nwp(k) < psfc_nwp(ilon,ilat)) then
            sfc_level_nwp(ilon,ilat) = k
            exit
       endif
    enddo


   !--- find 850 mb level
   do k = 1, sfc_level_nwp(ilon,ilat)-1
      if ((P_std_nwp(k) <= 850.0) .and. &
          (P_std_nwp(k+1) > 850.0)) then
        level850_nwp(ilon,ilat) = k
      endif
   enddo
!--------------------------------------------------------------------
!--- find tropopause level  based on tropopause pressure
!--- tropopause is between tropopause_level and tropopaue_level + 1
!--------------------------------------------------------------------
   !--- constrain tropopause pressure to be greater than 75 mb
   P_trop_temp = max(P_trop_nwp(ilon,ilat),75.0)

   do k = 1, sfc_level_nwp(ilon,ilat)-1
      if ((P_std_nwp(k) <= P_trop_temp) .and. &
          (P_std_nwp(k+1) > P_trop_temp)) then
        tropo_level_nwp(ilon,ilat) = k
      endif
   enddo

    !--- check if tropopause level found
    if (tropo_level_nwp(ilon,ilat) == 0) then
        tropo_level_nwp(ilon,ilat) = 1   !assume top level if no trop found
    endif         

!--------------------------------------------------------------------
!--- find stratopause level starting at 500 mb by looking for levels
!--- with the minimum temp
!--- Note, no point in doing this until higher vertical resolution
!--- profiles are used.
!--------------------------------------------------------------------
   strato_level_nwp(ilon,ilat) = 1
!  tmax = -999.0   ! initialize to a smaller number than expected
!  do k = tropo_level_nwp(ilon,ilat),1,-1
!     if ((t_prof_nwp(k,ilon,ilat) > tmax) .and. &
!         (P_std_nwp(k) < p_trop_min)) then
!          strato_level_nwp(ilon,ilat) = k
!          tmax = t_prof_nwp(k,ilon,ilat) 
!      endif
!   enddo

!--- check if stratopause level found
!   if (strato_level_nwp(ilon,ilat) == 0) then
!      print *, "Error, stratopause level not found, stopping "
!      stop
!   endif

!---------------------------------------------------------------------
! find inversion level - highest level inversion below trop
!---------------------------------------------------------------------
   inversion_level_nwp(ilon,ilat) = 0
   do k = tropo_level_nwp(ilon,ilat),sfc_level_nwp(ilon,ilat)-1
      if ((t_prof_nwp(k,ilon,ilat) - t_prof_nwp(k+1,ilon,ilat) > delta_t_inversion) .and. &
!         (t_prof_nwp(k+1,ilon,ilat) - t_prof_nwp(k+2,ilon,ilat) > delta_t_inversion) .and. &
          (P_std_nwp(k) >= p_inversion_min)) then
          inversion_level_nwp(ilon,ilat) = k
          exit 
      endif
   enddo

end subroutine FIND_NWP_LEVELS

!======================================================================
! subroutine COMPUTE_COAST_MASK_NWP(j1,j2)
!
! Compute Coast Mask for NWP data
!
! dependencies - land_mask and land_mask_nwp should be already computed
!======================================================================
 subroutine COMPUTE_COAST_MASK_NWP(j1,j2)

  integer, intent(in):: j1, j2
  integer:: ielem,iline,inwp,jnwp

!--- loop over all pixels in segment
  do iline = j1,j1+j2-1
     do ielem = 1, num_pix

!--- initialize to no
        coast_mask_nwp(ielem,iline) = sym%NO
       
!--- check for valid pixels
        if (bad_pixel_mask(ielem,iline) == sym%YES)  then
          cycle
        endif

!--- save nwp indices
        inwp = i_nwp(ielem,iline)
        jnwp = j_nwp(ielem,iline)

!--- derive nwp coast mask
        if (land_mask(ielem,iline) /= land_nwp(inwp,jnwp)) then
           coast_mask_nwp(ielem,iline) = sym%YES
        endif 

!--- end loop over all pixels in segment
     enddo
   enddo

 end subroutine COMPUTE_COAST_MASK_NWP
!======================================================================
! subroutine COMPUTE_NWP_CLOUD_PARAMETERS(clwmr_profile,
!    temperature_profile,pressure_profile,P_cld_top,iwp,lwp,
!    number_of_cloud_layers)
!======================================================================
 subroutine COMPUTE_NWP_CLOUD_PARAMETERS(clwmr_profile, &
                                         temperature_profile, &
                                         pressure_profile, &
                                         P_cld_top,iwp,lwp, &
                                         number_of_cloud_layers)

  real, dimension(:), intent(in):: clwmr_profile
  real, dimension(:), intent(in):: temperature_profile
  real, dimension(:), intent(in):: pressure_profile
  real, intent(out):: P_cld_top
  real, intent(out):: iwp
  real, intent(out):: lwp
  real, intent(out):: number_of_cloud_layers
  
  real:: clwmr_min
  real, parameter:: lwp_threshold  = 10.0
  real, parameter:: iwp_threshold  = 2.0
  real, parameter:: max_temperature_ice = 273.15
  real, parameter:: min_temperature_water  = 253.15
  integer:: number_of_levels_in_profile
  real:: factor
  real:: clwmr_ice_layer
  real:: clwmr_water_layer
  real:: ice_frac_top
  real:: ice_frac_bot
  integer:: ilay

  number_of_levels_in_profile = size(clwmr_profile)

  !---------------------------------------------------------------------
  ! convert clwmr profiles into optical depths
  ! iwp and lwp are in g/m^2
  !---------------------------------------------------------------------
     lwp = 0.0
     iwp = 0.0
     P_cld_top = missing_value_real4

     do ilay = 1, number_of_levels_in_profile-1

        ice_frac_top  = min(1.0,max(0.0,&
                           (max_temperature_ice - temperature_profile(ilay))/ &
                           (max_temperature_ice-min_temperature_water)))

        ice_frac_bot  = min(1.0,max(0.0,&
                           (max_temperature_ice - temperature_profile(ilay+1))/ &
                           (max_temperature_ice-min_temperature_water)))

        clwmr_ice_layer = 0.5 * (ice_frac_top*clwmr_profile(ilay) +  &
                                 ice_frac_bot*clwmr_profile(ilay+1))

        clwmr_water_layer = 0.5 * ((1.0-ice_frac_top)*clwmr_profile(ilay) + &
                                   (1.0-ice_frac_bot)*clwmr_profile(ilay+1))

        factor = 1000.0*10.0 * (pressure_profile(ilay+1) - pressure_profile(ilay)) / g

        iwp = iwp + clwmr_ice_layer * factor
        lwp = lwp + clwmr_water_layer * factor


        !--- compute P_cld_top
        if ((iwp + lwp >= iwp_threshold) .and. (P_cld_top == missing_value_real4)) then
            P_cld_top = P_std_nwp(ilay+1)
        endif

      enddo

  !----------------------------------------------------------------------
  ! compute number of distinct cloud layers in GFS profile
  !----------------------------------------------------------------------
      number_of_cloud_layers = 0

      if ((iwp > iwp_threshold) .and. (lwp > lwp_threshold)) then     !why and?

           do ilay = 3, number_of_levels_in_profile-1

             clwmr_min = 1.0e-05 * pressure_profile(ilay)/1000.0    !

              if (clwmr_profile(ilay) > clwmr_min .and.   &
                  clwmr_profile(ilay-1) < clwmr_min .and. &
                  clwmr_profile(ilay-2) < clwmr_min) then

                  number_of_cloud_layers = number_of_cloud_layers + 1

              endif
           enddo
       endif 


 end subroutine COMPUTE_NWP_CLOUD_PARAMETERS

!----------------------------------------------------------------------
! SUBROUTINE prof_lookup_using_p(zlev, plev, tlev, z, p, t, ilev, a) 
!
!----------------------------------------------------------------------
SUBROUTINE prof_lookup_using_p(zlev, plev, tlev, z, p, t, ilev, a)
   
  REAL (kind=real4), intent(in), dimension(:) :: zlev, plev, tlev
  REAL (kind=real4), intent(in) :: p
  REAL (kind=real4), intent(out) :: z, t
  INTEGER (kind=int4), intent(out) :: ilev
  REAL (kind=real4), intent(out) :: a
  INTEGER (kind=int4) :: nlev
  REAL (kind=real4) :: dp, dt, dz

  nlev = size(plev,1)
   
  call LOCATE(plev,nlev,p,ilev)
  ilev = max(1,min(nlev-1,ilev))

  dp = plev(ilev+1) - plev(ilev)
  dt = tlev(ilev+1) - tlev(ilev)
  dz = zlev(ilev+1) - zlev(ilev)

  if (dp /= 0.0) then
    a = (p - plev(ilev))/dp
    t = tlev(ilev) + a*dt
    z = zlev(ilev) + a*dz
  else
    a = 0.0
    t = tlev(ilev) 
    z = zlev(ilev)
  endif
 
END SUBROUTINE prof_lookup_using_p

!----------------------------------------------------------------------
! SUBROUTINE prof_lookup_using_t(zlev, plev, tlev, z, p, t, ilev, a)
!----------------------------------------------------------------------
SUBROUTINE prof_lookup_using_t(zlev, plev, tlev, z, p, t, ilev, a)
   
  REAL (kind=real4), intent(in), dimension(:) :: zlev, plev, tlev
  REAL (kind=real4), intent(in) :: t
  REAL (kind=real4), intent(out) :: p, z
  INTEGER (kind=int4), intent(out) :: ilev
  REAL (kind=real4), intent(out) :: a
  INTEGER (kind=int4) :: nlev

  nlev = size(plev,1)
   
  if (t < tlev(1)) then
    CALL prof_lookup_using_t_lapse(zlev, plev, tlev, z, p, t, ilev, a)
    !print*,zlev
    !print*,tlev
    !print*,plev
    !print*,z,p,t,ilev,a
    !stop
  else
    CALL prof_lookup_using_t_prof(zlev, plev, tlev, z, p, t, ilev, a)
  endif
 
END SUBROUTINE prof_lookup_using_t

!----------------------------------------------------------------------
! SUBROUTINE prof_lookup_using_t_lapse(zlev, plev, tlev, z, p, t, ilev, a)
!----------------------------------------------------------------------
SUBROUTINE prof_lookup_using_t_lapse(zlev, plev, tlev, z, p, t, ilev, a)
   
  REAL (kind=real4), intent(in), dimension(:) :: zlev, plev, tlev
  REAL (kind=real4), intent(in) :: t
  REAL (kind=real4), intent(out) :: p, z
  INTEGER (kind=int4), intent(out) :: ilev
  REAL (kind=real4), intent(out) :: a
  REAL (kind=real4) :: dt, dz, t_tmp
  REAL (kind=real4), PARAMETER :: DT_STRATO_LAPSE_RATE = -6.5 !k
  REAL (kind=real4), PARAMETER :: DZ_STRATO_LAPSE_RATE = 1000.0 !m

  dt = DT_STRATO_LAPSE_RATE
  dz = DZ_STRATO_LAPSE_RATE

  a = (t - tlev(1))/dt
  z = zlev(1) + a*dz
  
  !This is a hack, really should be using full profile
  CALL prof_lookup_using_z(zlev, plev, tlev, z, p, t_tmp, ilev, a)
 
END SUBROUTINE prof_lookup_using_t_lapse

!----------------------------------------------------------------------
! SUBROUTINE prof_lookup_using_t_prof(zlev, plev, tlev, z, p, t, ilev, a)
!----------------------------------------------------------------------

SUBROUTINE prof_lookup_using_t_prof(zlev, plev, tlev, z, p, t, ilev, a)
   
  REAL (kind=real4), intent(in), dimension(:) :: zlev, plev, tlev
  REAL (kind=real4), intent(in) :: t
  REAL (kind=real4), intent(out) :: p, z
  INTEGER (kind=int4), intent(out) :: ilev
  REAL (kind=real4), intent(out) :: a
  INTEGER (kind=int4) :: nlev
  REAL (kind=real4) :: dp, dt, dz

  nlev = size(plev,1)
   
  call LOCATE(tlev,nlev,t,ilev)
  ilev = max(1,min(nlev-1,ilev))

  dp = plev(ilev+1) - plev(ilev)
  dt = tlev(ilev+1) - tlev(ilev)
  dz = zlev(ilev+1) - zlev(ilev)

  if (dp /= 0.0) then
    a = (t - tlev(ilev))/dt
    p = plev(ilev) + a*dp
    z = zlev(ilev) + a*dz
  else
    a = 0.0
    p = plev(ilev) 
    z = zlev(ilev)
  endif
 
END SUBROUTINE prof_lookup_using_t_prof

!----------------------------------------------------------------------
! SUBROUTINE prof_lookup_using_z(zlev, plev, tlev, z, p, t, ilev, a)
!----------------------------------------------------------------------

SUBROUTINE prof_lookup_using_z(zlev, plev, tlev, z, p, t, ilev, a)
   
  REAL (kind=real4), intent(in), dimension(:) :: zlev, plev, tlev
  REAL (kind=real4), intent(in) :: z
  REAL (kind=real4), intent(out) :: p, t
  INTEGER (kind=int4), intent(out) :: ilev
  REAL (kind=real4), intent(out) :: a
  INTEGER (kind=int4) :: nlev
  REAL (kind=real4) :: dp, dt, dz

  nlev = size(plev,1)
   
  call LOCATE(zlev,nlev,z,ilev)
  ilev = max(1,min(nlev-1,ilev))

  dp = plev(ilev+1) - plev(ilev)
  dt = tlev(ilev+1) - tlev(ilev)
  dz = zlev(ilev+1) - zlev(ilev)

  if (dp /= 0.0) then
    a = (z - zlev(ilev))/dz
    p = plev(ilev) + a*dp
    t = tlev(ilev) + a*dt
  else
    a = 0.0
    p = plev(ilev) 
    t = tlev(ilev)
  endif
 
END SUBROUTINE prof_lookup_using_z

!----------------------------------------------------------------------
! subroutine TEMPORAL_INTERP_TMPSFC_NWP(start_itime, end_itime)
!----------------------------------------------------------------------
subroutine TEMPORAL_INTERP_TMPSFC_NWP(start_itime, end_itime)
integer(kind=int4), intent(in):: start_itime
integer(kind=int4), intent(in):: end_itime
real(kind=real4):: x_interp
real(kind=real4):: start_hour
integer(kind=int4):: mean_doy
real(kind=real4)::  mean_hours
real(kind=real4):: segment_start_time
real(kind=real4):: segment_end_time
real(kind=real4):: segment_mean_time
integer(kind=int4):: nwp_start_hour_segment
integer(kind=int4):: nwp_end_hour_segment

!--- check for valid times (assume positive)
if (start_itime < 0 .or. end_time < 0) then
  return
endif

!--- determine the bound day values for this orbit
start_hour = start_itime/60.0/60.0/1000.0

nwp_start_hour_segment = 6*int(start_hour/6)
nwp_end_hour_segment = nwp_start_hour_segment + 6

if (nwp_start_hour == 0 .and. nwp_end_hour_segment == 24) then
   nwp_end_hour_segment = 0
endif
if (nwp_end_hour == 24 .and. nwp_start_hour_segment == 0) then
   nwp_start_hour_segment = 24
endif

!--- initialize interpolation weight
x_interp = missing_value_real4

!--- 
if (nwp_end_hour_segment == nwp_start_hour) then
  x_interp = 0.0
elseif (nwp_start_hour_segment == nwp_end_hour) then
  x_interp = 1.0
endif

if ((nwp_start_hour_segment == nwp_start_hour) .and. & 
    (nwp_end_hour_segment == nwp_end_hour)) then

!----  determine mean time for this segment
segment_start_time = start_itime/86400000.0_real4
segment_end_time = end_itime/86400000.0_real4
segment_mean_time = 0.5*(segment_start_time + segment_end_time)
mean_hours = segment_mean_time*24.0

 !--- determine interpolation weight for 6 hrly ncep fields
 !
 x_interp = (mean_hours  - nwp_start_hour)/  &
            (nwp_end_hour - nwp_start_hour)
 x_interp = max(0.0,min(1.0,x_interp))

endif

 !--- apply interpolation
if (x_interp /= missing_value_real4) then 
 where ( (tmpsfc_nwp_before /= missing_value_real4) .and. (tmpsfc_nwp_after /= missing_value_real4) )
    tmpsfc_nwp = (1.0 - x_interp) * tmpsfc_nwp_before + x_interp * tmpsfc_nwp_after
 end where
endif

!print *, "in tempo interp ", nwp_start_hour, nwp_end_hour,  &
!          nwp_start_hour_segment, nwp_end_hour_segment, mean_hours, x_interp

end subroutine TEMPORAL_INTERP_TMPSFC_NWP

end module NWP_COMMON
